@@ -220,7 +220,7 @@ enum css_break_after_e {
 	CSS_BREAK_AFTER_COLUMN			= 0x7,
 	CSS_BREAK_AFTER_AVOID_PAGE		= 0x8,
 	CSS_BREAK_AFTER_AVOID_COLUMN		= 0x9
-};	
+};
 
 enum css_break_before_e {
 	CSS_BREAK_BEFORE_INHERIT		= CSS_BREAK_AFTER_INHERIT,
@@ -282,8 +282,8 @@ enum css_column_fill_e {
 
 enum css_column_gap_e {
 	CSS_COLUMN_GAP_INHERIT			= 0x0,
-	CSS_COLUMN_GAP_NORMAL			= 0x1,
-	CSS_COLUMN_GAP_SET			= 0x2
+	CSS_COLUMN_GAP_SET			= 0x1,
+	CSS_COLUMN_GAP_NORMAL			= 0x2
 };
 
 enum css_column_rule_color_e {
@@ -295,6 +295,7 @@ enum css_column_rule_color_e {
 enum css_column_rule_style_e {
 	CSS_COLUMN_RULE_STYLE_INHERIT		= CSS_BORDER_STYLE_INHERIT,
 	CSS_COLUMN_RULE_STYLE_NONE		= CSS_BORDER_STYLE_NONE,
+	CSS_COLUMN_RULE_STYLE_HIDDEN		= CSS_BORDER_STYLE_HIDDEN,
 	CSS_COLUMN_RULE_STYLE_DOTTED		= CSS_BORDER_STYLE_DOTTED,
 	CSS_COLUMN_RULE_STYLE_DASHED		= CSS_BORDER_STYLE_DASHED,
 	CSS_COLUMN_RULE_STYLE_SOLID		= CSS_BORDER_STYLE_SOLID,
@@ -321,8 +322,8 @@ enum css_column_span_e {
 
 enum css_column_width_e {
 	CSS_COLUMN_WIDTH_INHERIT		= 0x0,
-	CSS_COLUMN_WIDTH_AUTO			= 0x1,
-	CSS_COLUMN_WIDTH_SET			= 0x2
+	CSS_COLUMN_WIDTH_SET			= 0x1,
+	CSS_COLUMN_WIDTH_AUTO			= 0x2
 };
 
 enum css_content_e {
@@ -474,9 +475,9 @@ enum css_left_e {
 };
 
 enum css_letter_spacing_e {
-	CSS_LETTER_SPACING_INHERIT		= 0x0,
-	CSS_LETTER_SPACING_SET			= 0x1,
-	CSS_LETTER_SPACING_NORMAL		= 0x2
+	CSS_LETTER_SPACING_INHERIT		= CSS_COLUMN_GAP_INHERIT,
+	CSS_LETTER_SPACING_SET			= CSS_COLUMN_GAP_SET,
+	CSS_LETTER_SPACING_NORMAL		= CSS_COLUMN_GAP_NORMAL
 };
 
 enum css_line_height_e {
@@ -604,7 +605,7 @@ enum css_page_break_after_e {
 	CSS_PAGE_BREAK_AFTER_ALWAYS		= CSS_BREAK_AFTER_ALWAYS,
 	CSS_PAGE_BREAK_AFTER_LEFT		= CSS_BREAK_AFTER_LEFT,
 	CSS_PAGE_BREAK_AFTER_RIGHT		= CSS_BREAK_AFTER_RIGHT
-};	
+};
 
 enum css_page_break_before_e {
 	CSS_PAGE_BREAK_BEFORE_INHERIT		= CSS_BREAK_AFTER_INHERIT,
@@ -728,8 +729,8 @@ enum css_white_space_e {
 enum css_widows_e {
 	CSS_WIDOWS_INHERIT			= 0x0,
 	CSS_WIDOWS_SET				= 0x1
-};	
-	
+};
+
 enum css_width_e {
 	CSS_WIDTH_INHERIT			= 0x0,
 	CSS_WIDTH_SET				= 0x1,
@@ -737,9 +738,9 @@ enum css_width_e {
 };
 
 enum css_word_spacing_e {
-	CSS_WORD_SPACING_INHERIT		= 0x0,
-	CSS_WORD_SPACING_SET			= 0x1,
-	CSS_WORD_SPACING_NORMAL			= 0x2
+	CSS_WORD_SPACING_INHERIT		= CSS_COLUMN_GAP_INHERIT,
+	CSS_WORD_SPACING_SET			= CSS_COLUMN_GAP_SET,
+	CSS_WORD_SPACING_NORMAL			= CSS_COLUMN_GAP_NORMAL
 };
 
 enum css_writing_mode_e {
