@@ -46,24 +46,6 @@ static css_error compute_absolute_length(css_computed_style *style,
 				css_fixed *len, css_unit *unit),
 		css_error (*set)(css_computed_style *style, uint8_t type,
 				css_fixed len, css_unit unit));
-static css_error compute_absolute_length_auto(css_computed_style *style,
-		const css_hint_length *ex_size,
-		uint8_t (*get)(const css_computed_style *style, 
-				css_fixed *len, css_unit *unit),
-		css_error (*set)(css_computed_style *style, uint8_t type,
-				css_fixed len, css_unit unit));
-static css_error compute_absolute_length_none(css_computed_style *style,
-		const css_hint_length *ex_size,
-		uint8_t (*get)(const css_computed_style *style, 
-				css_fixed *len, css_unit *unit),
-		css_error (*set)(css_computed_style *style, uint8_t type,
-				css_fixed len, css_unit unit));
-static css_error compute_absolute_length_normal(css_computed_style *style,
-		const css_hint_length *ex_size,
-		uint8_t (*get)(const css_computed_style *style, 
-				css_fixed *len, css_unit *unit),
-		css_error (*set)(css_computed_style *style, uint8_t type,
-				css_fixed len, css_unit unit));
 static css_error compute_absolute_length_pair(css_computed_style *style,
 		const css_hint_length *ex_size,
 		uint8_t (*get)(const css_computed_style *style, 
@@ -89,12 +71,10 @@ css_error css_computed_style_create(css_computed_style **result)
 	if (result == NULL)
 		return CSS_BADPARM;
 
-	s = malloc(sizeof(css_computed_style));
+	s = calloc(sizeof(css_computed_style), 1);
 	if (s == NULL)
 		return CSS_NOMEM;
 
-	memset(s, 0, sizeof(css_computed_style));
-
 	*result = s;
 
 	return CSS_OK;
@@ -326,7 +306,14 @@ uint8_t css_computed_outline_color(const css_computed_style *style,
 uint8_t css_computed_outline_width(const css_computed_style *style,
 		css_fixed *length, css_unit *unit)
 {
-	return get_outline_width(style, length, unit);
+	/* This property is in the uncommon block, so we need to handle
+	 * absolute value calculation for initial value (medium) here. */
+	if (get_outline_width(style, length, unit) == CSS_BORDER_WIDTH_MEDIUM) {
+		*length = INTTOFIX(2);
+		*unit = CSS_UNIT_PX;
+	}
+
+	return CSS_BORDER_WIDTH_WIDTH;
 }
 
 uint8_t css_computed_border_spacing(const css_computed_style *style,
@@ -787,6 +774,80 @@ uint8_t css_computed_background_position(const css_computed_style *style,
 	return get_background_position(style, hlength, hunit, vlength, vunit);
 }
 
+uint8_t css_computed_break_after(const css_computed_style *style)
+{
+	return get_break_after(style);
+}
+
+uint8_t css_computed_break_before(const css_computed_style *style)
+{
+	return get_break_before(style);
+}
+
+uint8_t css_computed_break_inside(const css_computed_style *style)
+{
+	return get_break_inside(style);
+}
+
+uint8_t css_computed_column_count(const css_computed_style *style,
+		int32_t *column_count)
+{
+	return get_column_count(style, column_count);
+}
+
+uint8_t css_computed_column_fill(const css_computed_style *style)
+{
+	return get_column_fill(style);
+}
+
+uint8_t css_computed_column_gap(const css_computed_style *style,
+		css_fixed *length, css_unit *unit)
+{
+	return get_column_gap(style, length, unit);
+}
+
+uint8_t css_computed_column_rule_color(const css_computed_style *style,
+		css_color *color)
+{
+	/* This property is in the uncommon block, so we need to handle
+	 * absolute value calculation for initial value (currentColor) here. */
+	if (get_column_rule_color(style, color) ==
+			CSS_COLUMN_RULE_COLOR_CURRENT_COLOR) {
+		css_computed_color(style, color);
+	}
+	return CSS_COLUMN_RULE_COLOR_COLOR;
+}
+
+uint8_t css_computed_column_rule_style(const css_computed_style *style)
+{
+	return get_column_rule_style(style);
+}
+
+uint8_t css_computed_column_rule_width(const css_computed_style *style,
+		css_fixed *length, css_unit *unit)
+{
+	/* This property is in the uncommon block, so we need to handle
+	 * absolute value calculation for initial value (medium) here. */
+	if (get_column_rule_width(style, length, unit) ==
+			CSS_BORDER_WIDTH_MEDIUM) {
+		*length = INTTOFIX(2);
+		*unit = CSS_UNIT_PX;
+	}
+
+	return CSS_BORDER_WIDTH_WIDTH;
+}
+
+uint8_t css_computed_column_span(const css_computed_style *style)
+{
+	return get_column_span(style);
+}
+
+uint8_t css_computed_column_width(const css_computed_style *style,
+		css_fixed *length, css_unit *unit)
+{
+	return get_column_width(style, length, unit);
+}
+
 uint8_t css_computed_display(const css_computed_style *style,
 		bool root)
 {
@@ -1021,7 +1082,7 @@ css_error css__compute_absolute_values(const css_computed_style *parent,
 		return error;
 
 	/* Fix up height */
-	error = compute_absolute_length_auto(style, &ex_size.data.length, 
+	error = compute_absolute_length(style, &ex_size.data.length,
 			get_height, set_height);
 	if (error != CSS_OK)
 		return error;
@@ -1037,25 +1098,25 @@ css_error css__compute_absolute_values(const css_computed_style *parent,
 		return error;
 
 	/* Fix up max-height */
-	error = compute_absolute_length_none(style, &ex_size.data.length, 
+	error = compute_absolute_length(style, &ex_size.data.length,
 			get_max_height, set_max_height);
 	if (error != CSS_OK)
 		return error;
 
 	/* Fix up max-width */
-	error = compute_absolute_length_none(style, &ex_size.data.length, 
+	error = compute_absolute_length(style, &ex_size.data.length,
 			get_max_width, set_max_width);
 	if (error != CSS_OK)
 		return error;
 
 	/* Fix up min-height */
-	error = compute_absolute_length(style, &ex_size.data.length, 
+	error = compute_absolute_length(style, &ex_size.data.length,
 			get_min_height, set_min_height);
 	if (error != CSS_OK)
 		return error;
 
 	/* Fix up min-width */
-	error = compute_absolute_length(style, &ex_size.data.length, 
+	error = compute_absolute_length(style, &ex_size.data.length,
 			get_min_width, set_min_width);
 	if (error != CSS_OK)
 		return error;
@@ -1066,7 +1127,7 @@ css_error css__compute_absolute_values(const css_computed_style *parent,
 		return error;
 
 	/* Fix up text-indent */
-	error = compute_absolute_length(style, &ex_size.data.length, 
+	error = compute_absolute_length(style, &ex_size.data.length,
 			get_text_indent, set_text_indent);
 	if (error != CSS_OK)
 		return error;
@@ -1077,7 +1138,7 @@ css_error css__compute_absolute_values(const css_computed_style *parent,
 		return error;
 
 	/* Fix up width */
-	error = compute_absolute_length_auto(style, &ex_size.data.length, 
+	error = compute_absolute_length(style, &ex_size.data.length,
 			get_width, set_width);
 	if (error != CSS_OK)
 		return error;
@@ -1098,7 +1159,7 @@ css_error css__compute_absolute_values(const css_computed_style *parent,
 			return error;
 
 		/* Fix up letter-spacing */
-		error = compute_absolute_length_normal(style,
+		error = compute_absolute_length(style,
 				&ex_size.data.length,
 				get_letter_spacing, 
 				set_letter_spacing);
@@ -1120,13 +1181,37 @@ css_error css__compute_absolute_values(const css_computed_style *parent,
 		if (error != CSS_OK)
 			return error;
 
-		/* Fix up word spacing */
-		error = compute_absolute_length_normal(style,
+		/* Fix up word-spacing */
+		error = compute_absolute_length(style,
 				&ex_size.data.length,
 				get_word_spacing, 
 				set_word_spacing);
 		if (error != CSS_OK)
 			return error;
+
+		/* Fix up column-rule-width */
+		error = compute_absolute_border_side_width(style,
+				&ex_size.data.length,
+				get_column_rule_width,
+				set_column_rule_width);
+		if (error != CSS_OK)
+			return error;
+
+		/* Fix up column-width */
+		error = compute_absolute_length(style,
+				&ex_size.data.length,
+				get_column_width,
+				set_column_width);
+		if (error != CSS_OK)
+			return error;
+
+		/* Fix up column-gap */
+		error = compute_absolute_length(style,
+				&ex_size.data.length,
+				get_column_gap,
+				set_column_gap);
+		if (error != CSS_OK)
+			return error;
 	}
 
 	return CSS_OK;
@@ -1267,23 +1352,26 @@ css_error compute_absolute_border_side_width(css_computed_style *style,
 {
 	css_fixed length;
 	css_unit unit;
-	uint8_t type;
 
-	type = get(style, &length, &unit);
-	if (type == CSS_BORDER_WIDTH_THIN) {
+	switch (get(style, &length, &unit)) {
+	case CSS_BORDER_WIDTH_THIN:
 		length = INTTOFIX(1);
 		unit = CSS_UNIT_PX;
-	} else if (type == CSS_BORDER_WIDTH_MEDIUM) {
+		break;
+	case CSS_BORDER_WIDTH_MEDIUM:
 		length = INTTOFIX(2);
 		unit = CSS_UNIT_PX;
-	} else if (type == CSS_BORDER_WIDTH_THICK) {
+		break;
+	case CSS_BORDER_WIDTH_THICK:
 		length = INTTOFIX(4);
 		unit = CSS_UNIT_PX;
-	}
-
-	if (unit == CSS_UNIT_EX) {
-		length = FMUL(length, ex_size->value);
-		unit = ex_size->unit;
+		break;
+	case CSS_BORDER_WIDTH_WIDTH:
+		if (unit == CSS_UNIT_EX) {
+			length = FMUL(length, ex_size->value);
+			unit = ex_size->unit;
+		}
+		break;
 	}
 
 	return set(style, CSS_BORDER_WIDTH_WIDTH, length, unit);
@@ -1384,21 +1472,22 @@ css_error compute_absolute_sides(css_computed_style *style,
 	css_error error;
 
 	/* Calculate absolute lengths for sides */
-	error = compute_absolute_length_auto(style, ex_size, get_top, set_top);
+	error = compute_absolute_length(style, ex_size,
+			get_top, set_top);
 	if (error != CSS_OK)
 		return error;
 
-	error = compute_absolute_length_auto(style, ex_size,
+	error = compute_absolute_length(style, ex_size,
 			get_right, set_right);
 	if (error != CSS_OK)
 		return error;
 
-	error = compute_absolute_length_auto(style, ex_size,
+	error = compute_absolute_length(style, ex_size,
 			get_bottom, set_bottom);
 	if (error != CSS_OK)
 		return error;
 
-	error = compute_absolute_length_auto(style, ex_size,
+	error = compute_absolute_length(style, ex_size,
 			get_left, set_left);
 	if (error != CSS_OK)
 		return error;
@@ -1418,22 +1507,22 @@ css_error compute_absolute_margins(css_computed_style *style,
 {
 	css_error error;
 
-	error = compute_absolute_length_auto(style, ex_size,
+	error = compute_absolute_length(style, ex_size,
 			get_margin_top, set_margin_top);
 	if (error != CSS_OK)
 		return error;
 
-	error = compute_absolute_length_auto(style, ex_size,
+	error = compute_absolute_length(style, ex_size,
 			get_margin_right, set_margin_right);
 	if (error != CSS_OK)
 		return error;
 
-	error = compute_absolute_length_auto(style, ex_size,
+	error = compute_absolute_length(style, ex_size,
 			get_margin_bottom, set_margin_bottom);
 	if (error != CSS_OK)
 		return error;
 
-	error = compute_absolute_length_auto(style, ex_size,
+	error = compute_absolute_length(style, ex_size,
 			get_margin_left, set_margin_left);
 	if (error != CSS_OK)
 		return error;
@@ -1529,112 +1618,16 @@ css_error compute_absolute_length(css_computed_style *style,
 
 	type = get(style, &length, &unit);
 
-	if (unit == CSS_UNIT_EX) {
+	if (type == CSS_WIDTH_SET && unit == CSS_UNIT_EX) {
 		length = FMUL(length, ex_size->value);
 		unit = ex_size->unit;
+
+		return set(style, type, length, unit);
 	}
 
-	return set(style, type, length, unit);
+	return CSS_OK;
 }
 
-/**
- * Compute the absolute value of length or auto
- *
- * \param style      Style to process
- * \param ex_size    Ex size, in ems
- * \param get        Function to read length
- * \param set        Function to write length
- * \return CSS_OK on success
- */
-css_error compute_absolute_length_auto(css_computed_style *style,
-		const css_hint_length *ex_size,
-		uint8_t (*get)(const css_computed_style *style, 
-				css_fixed *len, css_unit *unit),
-		css_error (*set)(css_computed_style *style, uint8_t type,
-				css_fixed len, css_unit unit))
-{
-	css_fixed length;
-	css_unit unit;
-	uint8_t type;
-
-	type = get(style, &length, &unit);
-	if (type != CSS_BOTTOM_AUTO) {
-		if (unit == CSS_UNIT_EX) {
-			length = FMUL(length, ex_size->value);
-			unit = ex_size->unit;
-		}
-
-		return set(style, CSS_BOTTOM_SET, length, unit);
-	}
-
-	return set(style, CSS_BOTTOM_AUTO, 0, CSS_UNIT_PX);
-}
-
-/**
- * Compute the absolute value of length or none
- *
- * \param style      Style to process
- * \param ex_size    Ex size, in ems
- * \param get        Function to read length
- * \param set        Function to write length
- * \return CSS_OK on success
- */
-css_error compute_absolute_length_none(css_computed_style *style,
-		const css_hint_length *ex_size,
-		uint8_t (*get)(const css_computed_style *style, 
-				css_fixed *len, css_unit *unit),
-		css_error (*set)(css_computed_style *style, uint8_t type,
-				css_fixed len, css_unit unit))
-{
-	css_fixed length;
-	css_unit unit;
-	uint8_t type;
-
-	type = get(style, &length, &unit);
-	if (type != CSS_MAX_HEIGHT_NONE) {
-		if (unit == CSS_UNIT_EX) {
-			length = FMUL(length, ex_size->value);
-			unit = ex_size->unit;
-		}
-
-		return set(style, CSS_MAX_HEIGHT_SET, length, unit);
-	}
-
-	return set(style, CSS_MAX_HEIGHT_NONE, 0, CSS_UNIT_PX);
-}
-
-/**
- * Compute the absolute value of length or normal
- *
- * \param style      Style to process
- * \param ex_size    Ex size, in ems
- * \param get        Function to read length
- * \param set        Function to write length
- * \return CSS_OK on success
- */
-css_error compute_absolute_length_normal(css_computed_style *style,
-		const css_hint_length *ex_size,
-		uint8_t (*get)(const css_computed_style *style, 
-				css_fixed *len, css_unit *unit),
-		css_error (*set)(css_computed_style *style, uint8_t type,
-				css_fixed len, css_unit unit))
-{
-	css_fixed length;
-	css_unit unit;
-	uint8_t type;
-
-	type = get(style, &length, &unit);
-	if (type != CSS_LETTER_SPACING_NORMAL) {
-		if (unit == CSS_UNIT_EX) {
-			length = FMUL(length, ex_size->value);
-			unit = ex_size->unit;
-		}
-
-		return set(style, CSS_LETTER_SPACING_SET, length, unit);
-	}
-
-	return set(style, CSS_LETTER_SPACING_NORMAL, 0, CSS_UNIT_PX);
-}
 
 /**
  * Compute the absolute value of length pair
