@@ -150,42 +150,38 @@ css_error css__selector_hash_create(css_selector_hash **hash)
 	if (hash == NULL)
 		return CSS_BADPARM;
 
-	h = malloc(sizeof(css_selector_hash));
+	h = calloc(1, sizeof(css_selector_hash));
 	if (h == NULL)
 		return CSS_NOMEM;
 
 	/* Element hash */
-	h->elements.slots = malloc(DEFAULT_SLOTS * sizeof(hash_entry));
+	h->elements.slots = calloc(DEFAULT_SLOTS, sizeof(hash_entry));
 	if (h->elements.slots == NULL) {
 		free(h);
 		return CSS_NOMEM;
 	}
-	memset(h->elements.slots, 0, DEFAULT_SLOTS * sizeof(hash_entry));
 	h->elements.n_slots = DEFAULT_SLOTS;
 
 	/* Class hash */
-	h->classes.slots = malloc(DEFAULT_SLOTS * sizeof(hash_entry));
+	h->classes.slots = calloc(DEFAULT_SLOTS, sizeof(hash_entry));
 	if (h->classes.slots == NULL) {
 		free(h->elements.slots);
 		free(h);
 		return CSS_NOMEM;
 	}
-	memset(h->classes.slots, 0, DEFAULT_SLOTS * sizeof(hash_entry));
 	h->classes.n_slots = DEFAULT_SLOTS;
 
 	/* ID hash */
-	h->ids.slots = malloc(DEFAULT_SLOTS * sizeof(hash_entry));
+	h->ids.slots = calloc(DEFAULT_SLOTS, sizeof(hash_entry));
 	if (h->ids.slots == NULL) {
 		free(h->classes.slots);
 		free(h->elements.slots);
 		free(h);
 		return CSS_NOMEM;
 	}
-	memset(h->ids.slots, 0, DEFAULT_SLOTS * sizeof(hash_entry));
 	h->ids.n_slots = DEFAULT_SLOTS;
 
-	/* Universal chain */
-	memset(&h->universal, 0, sizeof(hash_entry));
+	/* Universal chain head already initiliased by calloc of `h`. */
 
 	h->hash_size = sizeof(css_selector_hash) + 
 			DEFAULT_SLOTS * sizeof(hash_entry) +
@@ -366,6 +362,7 @@ css_error css__selector_hash_find(css_selector_hash *hash,
 		const css_selector ***matched)
 {
 	uint32_t index, mask;
+	lwc_hash name_hash;
 	hash_entry *head;
 
 	if (hash == NULL || req == NULL || iterator == NULL || matched == NULL)
@@ -374,12 +371,11 @@ css_error css__selector_hash_find(css_selector_hash *hash,
 	/* Find index */
 	mask = hash->elements.n_slots - 1;
 
-	if (req->qname.name->insensitive == NULL &&
-			lwc__intern_caseless_string(
-			req->qname.name) != lwc_error_ok) {
+	if (lwc_string_caseless_hash_value(req->qname.name,
+			&name_hash) != lwc_error_ok) {
 		return CSS_NOMEM;
 	}
-	index = _hash_name(req->qname.name) & mask;
+	index = name_hash & mask;
 
 	head = &hash->elements.slots[index];
 
@@ -437,6 +433,7 @@ css_error css__selector_hash_find_by_class(css_selector_hash *hash,
 		const css_selector ***matched)
 {
 	uint32_t index, mask;
+	lwc_hash class_hash;
 	hash_entry *head;
 
 	if (hash == NULL || req == NULL || req->class == NULL ||
@@ -446,12 +443,11 @@ css_error css__selector_hash_find_by_class(css_selector_hash *hash,
 	/* Find index */
 	mask = hash->classes.n_slots - 1;
 
-	if (req->class->insensitive == NULL &&
-			lwc__intern_caseless_string(
-			req->class) != lwc_error_ok) {
+	if (lwc_string_caseless_hash_value(req->class,
+			&class_hash) != lwc_error_ok) {
 		return CSS_NOMEM;
 	}
-	index = _hash_name(req->class) & mask;
+	index = class_hash & mask;
 
 	head = &hash->classes.slots[index];
 
@@ -517,6 +513,7 @@ css_error css__selector_hash_find_by_id(css_selector_hash *hash,
 		const css_selector ***matched)
 {
 	uint32_t index, mask;
+	lwc_hash id_hash;
 	hash_entry *head;
 
 	if (hash == NULL || req == NULL || req->id == NULL ||
@@ -526,12 +523,11 @@ css_error css__selector_hash_find_by_id(css_selector_hash *hash,
 	/* Find index */
 	mask = hash->ids.n_slots - 1;
 
-	if (req->id->insensitive == NULL &&
-			lwc__intern_caseless_string(
-			req->id) != lwc_error_ok) {
+	if (lwc_string_caseless_hash_value(req->id,
+			&id_hash) != lwc_error_ok) {
 		return CSS_NOMEM;
 	}
-	index = _hash_name(req->id) & mask;
+	index = id_hash & mask;
 
 	head = &hash->ids.slots[index];
 
