@@ -17,22 +17,30 @@
 css_error css__cascade_break_inside(uint32_t opv, css_style *style, 
 		css_select_state *state)
 {
+	uint16_t value = CSS_BREAK_INSIDE_AUTO;
+
 	UNUSED(style);
 
 	if (isInherit(opv) == false) {
 		switch (getValue(opv)) {
 		case BREAK_INSIDE_AUTO:
+			value = CSS_BREAK_INSIDE_AUTO;
+			break;
 		case BREAK_INSIDE_AVOID:
+			value = CSS_BREAK_INSIDE_AVOID;
+			break;
 		case BREAK_INSIDE_AVOID_PAGE:
+			value = CSS_BREAK_INSIDE_AVOID_PAGE;
+			break;
 		case BREAK_INSIDE_AVOID_COLUMN:
-			/** \todo convert to public values */
+			value = CSS_BREAK_INSIDE_AVOID_COLUMN;
 			break;
 		}
 	}
 
 	if (css__outranks_existing(getOpcode(opv), isImportant(opv), state,
 			isInherit(opv))) {
-		/** \todo set computed elevation */
+		return set_break_inside(state->computed, value);
 	}
 
 	return CSS_OK;
@@ -41,27 +49,24 @@ css_error css__cascade_break_inside(uint32_t opv, css_style *style,
 css_error css__set_break_inside_from_hint(const css_hint *hint,
 		css_computed_style *style)
 {
-	UNUSED(hint);
-	UNUSED(style);
-
-	return CSS_OK;
+	return set_break_inside(style, hint->status);
 }
 
 css_error css__initial_break_inside(css_select_state *state)
 {
-	UNUSED(state);
-
-	return CSS_OK;
+	return set_break_inside(state->computed, CSS_BREAK_INSIDE_AUTO);
 }
 
 css_error css__compose_break_inside(const css_computed_style *parent,
 		const css_computed_style *child,
 		css_computed_style *result)
 {
-	UNUSED(parent);
-	UNUSED(child);
-	UNUSED(result);
+	uint8_t type = get_break_inside(child);
 
-	return CSS_OK;
+	if (type == CSS_BREAK_INSIDE_INHERIT) {
+		type = get_break_inside(parent);
+	}
+
+	return set_break_inside(result, type);
 }
 
