@@ -110,7 +110,7 @@ typedef struct css_select_rule_source {
 } css_select_rule_source;
 
 
-static css_error set_hint(css_select_state *state, uint32_t prop);
+static css_error set_hint(css_select_state *state, css_hint *hint);
 static css_error set_initial(css_select_state *state, 
 		uint32_t prop, css_pseudo_element pseudo,
 		void *parent);
@@ -230,12 +230,10 @@ css_error css_select_ctx_create(css_select_ctx **result)
 	if (result == NULL)
 		return CSS_BADPARM;
 
-	c = malloc(sizeof(css_select_ctx));
+	c = calloc(sizeof(css_select_ctx), 1);
 	if (c == NULL)
 		return CSS_NOMEM;
 
-	memset(c, 0, sizeof(css_select_ctx));
-
 	error = intern_strings(c);
 	if (error != CSS_OK) {
 		free(c);
@@ -435,12 +433,14 @@ css_error css_select_style(css_select_ctx *ctx, void *node,
 		css_select_handler *handler, void *pw,
 		css_select_results **result)
 {
-	uint32_t i, j;
+	uint32_t i, j, nhints;
 	css_error error;
 	css_select_state state;
 	void *parent = NULL;
+	css_hint *hints = NULL;
 	css_bloom *bloom = NULL;
 	css_bloom *parent_bloom = NULL;
+	lwc_hash hash;
 
 	if (ctx == NULL || node == NULL || result == NULL || handler == NULL ||
 	    handler->handler_version != CSS_SELECT_HANDLER_VERSION_1)
@@ -538,6 +538,29 @@ css_error css_select_style(css_select_ctx *ctx, void *node,
 	if (error != CSS_OK)
 		goto cleanup;
 
+	/* Apply presentational hints */
+	error = handler->node_presentational_hint(pw, node, &nhints, &hints);
+	if (error != CSS_OK)
+		goto cleanup;
+	if (nhints > 0) {
+		/* Ensure that the appropriate computed style exists */
+		struct css_computed_style *computed_style =
+				state.results->styles[CSS_PSEUDO_ELEMENT_NONE];
+		if (computed_style == NULL) {
+			error = css_computed_style_create(&computed_style);
+			if (error != CSS_OK)
+				goto cleanup;
+		}
+		state.results->styles[CSS_PSEUDO_ELEMENT_NONE] = computed_style;
+		state.computed = computed_style;
+
+		for (i = 0; i < nhints; i++) {
+			error = set_hint(&state, &hints[i]);
+			if (error != CSS_OK)
+				goto cleanup;
+		}
+	}
+
 	/* Iterate through the top-level stylesheets, selecting styles
 	 * from those which apply to our current media requirements and
 	 * are not disabled */
@@ -579,8 +602,7 @@ css_error css_select_style(css_select_ctx *ctx, void *node,
 		}
 	}
 
-	/* Take account of presentational hints and fix up any remaining
-	 * unset properties. */
+	/* Fix up any remaining unset properties. */
 
 	/* Base element */
 	state.current_pseudo = CSS_PSEUDO_ELEMENT_NONE;
@@ -589,17 +611,6 @@ css_error css_select_style(css_select_ctx *ctx, void *node,
 		const prop_state *prop = 
 				&state.props[i][CSS_PSEUDO_ELEMENT_NONE];
 
-		/* Apply presentational hints if the property is unset or 
-		 * the existing property value did not come from an author 
-		 * stylesheet or a user sheet using !important. */
-		if (prop->set == false ||
-				(prop->origin != CSS_ORIGIN_AUTHOR &&
-				prop->important == false)) {
-			error = set_hint(&state, i);
-			if (error != CSS_OK)
-				goto cleanup;
-		}
-
 		/* If the property is still unset or it's set to inherit 
 		 * and we're the root element, then set it to its initial 
 		 * value. */
@@ -649,43 +660,34 @@ css_error css_select_style(css_select_ctx *ctx, void *node,
 	}
 
 	/* Add node name to bloom */
-	if (state.element.name->insensitive == NULL) {
-		if (lwc__intern_caseless_string(
-				state.element.name) != lwc_error_ok) {
-			error = CSS_NOMEM;
-			goto cleanup;
-		}
+
+	if (lwc_string_caseless_hash_value(state.element.name,
+			&hash) != lwc_error_ok) {
+		error = CSS_NOMEM;
+		goto cleanup;
 	}
-	css_bloom_add_hash(bloom, lwc_string_hash_value(
-			state.element.name->insensitive));
+	css_bloom_add_hash(bloom, hash);
 
 	/* Add id name to bloom */
 	if (state.id != NULL) {
-		if (state.id->insensitive == NULL) {
-			if (lwc__intern_caseless_string(state.id) !=
-					lwc_error_ok) {
-				error = CSS_NOMEM;
-				goto cleanup;
-			}
+		if (lwc_string_caseless_hash_value(state.id,
+				&hash) != lwc_error_ok) {
+			error = CSS_NOMEM;
+			goto cleanup;
 		}
-		css_bloom_add_hash(bloom, lwc_string_hash_value(
-				state.id->insensitive));
+		css_bloom_add_hash(bloom, hash);
 	}
 
 	/* Add class names to bloom */
 	if (state.classes != NULL) {
-		lwc_string *s;
 		for (i = 0; i < state.n_classes; i++) {
-			s = state.classes[i];
-			if (s->insensitive == NULL) {
-				if (lwc__intern_caseless_string(s) !=
-						lwc_error_ok) {
-					error = CSS_NOMEM;
-					goto cleanup;
-				}
+			lwc_string *s = state.classes[i];
+			if (lwc_string_caseless_hash_value(s,
+					&hash) != lwc_error_ok) {
+				error = CSS_NOMEM;
+				goto cleanup;
 			}
-			css_bloom_add_hash(bloom, lwc_string_hash_value(
-					s->insensitive));
+			css_bloom_add_hash(bloom, hash);
 		}
 	}
 
@@ -1120,32 +1122,23 @@ void destroy_strings(css_select_ctx *ctx)
 		lwc_string_unref(ctx->after);
 }
 
-css_error set_hint(css_select_state *state, uint32_t prop)
+css_error set_hint(css_select_state *state, css_hint *hint)
 {
-	css_hint hint;
+	uint32_t prop = hint->prop;
+	prop_state *existing = &state->props[prop][CSS_PSEUDO_ELEMENT_NONE];
 	css_error error;
 
-	/* Initialise hint */
-	memset(&hint, 0, sizeof(css_hint));
-
-	/* Retrieve this property's hint from the client */
-	error = state->handler->node_presentational_hint(state->pw, 
-			state->node, prop, &hint);
-	if (error != CSS_OK)
-		return (error == CSS_PROPERTY_NOT_SET) ? CSS_OK : error;
-
 	/* Hint defined -- set it in the result */
-	error = prop_dispatch[prop].set_from_hint(&hint, state->computed);
+	error = prop_dispatch[prop].set_from_hint(hint, state->computed);
 	if (error != CSS_OK)
 		return error;
 
 	/* Keep selection state in sync with reality */
-	state->props[prop][CSS_PSEUDO_ELEMENT_NONE].set = 1;
-	state->props[prop][CSS_PSEUDO_ELEMENT_NONE].specificity = 0;
-	state->props[prop][CSS_PSEUDO_ELEMENT_NONE].origin = CSS_ORIGIN_AUTHOR;
-	state->props[prop][CSS_PSEUDO_ELEMENT_NONE].important = 0;
-	state->props[prop][CSS_PSEUDO_ELEMENT_NONE].inherit = 
-			(hint.status == 0);
+	existing->set = 1;
+	existing->specificity = 0;
+	existing->origin = CSS_ORIGIN_AUTHOR;
+	existing->important = 0;
+	existing->inherit = (hint->status == 0);
 
 	return CSS_OK;
 }
@@ -1175,26 +1168,25 @@ css_error set_initial(css_select_state *state,
 		 * accessors to return the initial values for the 
 		 * property.
 		 */
-		if (group == GROUP_NORMAL) {
-			error = prop_dispatch[prop].initial(state);
-			if (error != CSS_OK)
-				return error;
-		} else if (group == GROUP_UNCOMMON &&
-				state->computed->uncommon != NULL) {
-			error = prop_dispatch[prop].initial(state);
-			if (error != CSS_OK)
-				return error;
-		} else if (group == GROUP_PAGE &&
-				state->computed->page != NULL) {
-			error = prop_dispatch[prop].initial(state);
-			if (error != CSS_OK)
-				return error;
-		} else if (group == GROUP_AURAL &&
-				state->computed->aural != NULL) {
-			error = prop_dispatch[prop].initial(state);
-			if (error != CSS_OK)
-				return error;
+		switch (group) {
+		case GROUP_NORMAL:
+			break;
+		case GROUP_UNCOMMON:
+			if (state->computed->uncommon == NULL)
+				return CSS_OK;
+			break;
+		case GROUP_PAGE:
+			if (state->computed->page == NULL)
+				return CSS_OK;
+			break;
+		case GROUP_AURAL:
+			if (state->computed->aural == NULL)
+				return CSS_OK;
+			break;
 		}
+		error = prop_dispatch[prop].initial(state);
+		if (error != CSS_OK)
+			return error;
 	}
 
 	return CSS_OK;
