@@ -29,6 +29,7 @@ typedef struct node {
 	uint32_t n_attrs;
 	attribute *attrs;
 
+	css_select_results *sr;
 	void *libcss_node_data;
 
 	struct node *parent;
@@ -155,9 +156,9 @@ static css_error node_is_target(void *pw, void *node, bool *match);
 static css_error node_is_lang(void *pw, void *node,
 		lwc_string *lang, bool *match);
 static css_error node_presentational_hint(void *pw, void *node,
-		uint32_t property, css_hint *hint);
+		uint32_t *nhints, css_hint **hints);
 static css_error ua_default_for_property(void *pw, uint32_t property,
-		css_hint *hint);
+		css_hint *hints);
 static css_error compute_font_size(void *pw, const css_hint *parent,
 		css_hint *size);
 static css_error set_libcss_node_data(void *pw, void *n,
@@ -721,6 +722,73 @@ void css__parse_expected(line_ctx *ctx, const char *data, size_t len)
 	ctx->expused += len;
 }
 
+static void show_differences(size_t len, const char *exp, const char *res)
+{
+	const char *pos_exp, *opos_exp;
+	const char *pos_res, *opos_res;
+
+	opos_exp = pos_exp = exp;
+	opos_res = pos_res = res;
+
+	printf("Line differences:\n");
+	while (pos_exp < exp + len && pos_res < res + len) {
+		if (*pos_exp == '\n' && *pos_res == '\n') {
+			if (pos_exp - opos_exp != pos_res - opos_res ||
+					memcmp(opos_exp, opos_res,
+					pos_exp - opos_exp) != 0) {
+				printf("Expected:\t%.*s\n",
+						(int)(pos_exp - opos_exp),
+						opos_exp);
+				printf("  Result:\t%.*s\n",
+						(int)(pos_res - opos_res),
+						opos_res);
+				printf("\n");
+			}
+			opos_exp = ++pos_exp;
+			opos_res = ++pos_res;
+		} else if (*pos_exp == '\n') {
+			pos_res++;
+		} else if (*pos_res == '\n') {
+			pos_exp++;
+		} else {
+			pos_exp++;
+			pos_res++;
+		}
+	}
+}
+
+
+static void run_test_select_tree(css_select_ctx *select,
+		node *node, line_ctx *ctx,
+		char *buf, size_t *buflen)
+{
+	css_select_results *sr;
+	struct node *n = NULL;
+
+	assert(css_select_style(select, node, ctx->media, NULL, 
+			&select_handler, ctx, &sr) == CSS_OK);
+
+	if (node->parent != NULL) {
+		assert(css_computed_style_compose(
+				node->parent->sr->styles[ctx->pseudo_element],
+				sr->styles[ctx->pseudo_element],
+				compute_font_size, NULL,
+				sr->styles[ctx->pseudo_element]) == CSS_OK);
+	}
+
+	node->sr = sr;
+
+	if (node == ctx->target) {
+		dump_computed_style(sr->styles[ctx->pseudo_element],
+				buf, buflen);
+	}
+
+	for (n = node->children; n != NULL; n = n->next) {
+		run_test_select_tree(select, n, ctx, buf, buflen);
+	}
+}
+
+
 void run_test(line_ctx *ctx, const char *exp, size_t explen)
 {
 	css_select_ctx *select;
@@ -748,25 +816,24 @@ void run_test(line_ctx *ctx, const char *exp, size_t explen)
 
 	testnum++;
 
-	assert(css_select_style(select, ctx->target, ctx->media, NULL, 
-			&select_handler, ctx, &results) == CSS_OK);
+	run_test_select_tree(select, ctx->tree, ctx, buf, &buflen);
 
+	results = ctx->target->sr;
 	assert(results->styles[ctx->pseudo_element] != NULL);
 
-	dump_computed_style(results->styles[ctx->pseudo_element], buf, &buflen);
-
 	if (8192 - buflen != explen || memcmp(buf, exp, explen) != 0) {
+		size_t len = 8192 - buflen < explen ? 8192 - buflen : explen;
 		printf("Expected (%u):\n%.*s\n", 
 				(int) explen, (int) explen, exp);
 		printf("Result (%u):\n%.*s\n", (int) (8192 - buflen),
 			(int) (8192 - buflen), buf);
+
+		show_differences(len, exp, buf);
 		assert(0 && "Result doesn't match expected");
 	}
 
 	/* Clean up */
-	css_select_results_destroy(results);
 	css_select_ctx_destroy(select);
-
 	destroy_tree(ctx->tree);
 
 	for (i = 0; i < ctx->n_sheets; i++) {
@@ -796,6 +863,8 @@ void destroy_tree(node *root)
 
 		destroy_tree(n);
 	}
+
+	css_select_results_destroy(root->sr);
 	
 	for (i = 0; i < root->n_attrs; ++i) {
 		lwc_string_unref(root->attrs[i].name);
@@ -1497,14 +1566,15 @@ css_error node_is_lang(void *pw, void *n,
 }
 
 css_error node_presentational_hint(void *pw, void *node,
-		uint32_t property, css_hint *hint)
+		uint32_t *nhints, css_hint **hints)
 {
 	UNUSED(pw);
 	UNUSED(node);
-	UNUSED(property);
-	UNUSED(hint);
 
-	return CSS_PROPERTY_NOT_SET;
+	*nhints = 0;
+	*hints = NULL;
+
+	return CSS_OK;
 }
 
 css_error ua_default_for_property(void *pw, uint32_t property, css_hint *hint)
