@@ -1370,7 +1370,7 @@ dom_exception _dom_node_get_base(dom_node_internal *node,
 	struct dom_document *doc = node->owner;
 	assert(doc != NULL);
 
-	return dom_document_get_base(doc, result);
+	return _dom_document_get_uri(doc, result);
 }
 
 /**
@@ -2316,11 +2316,43 @@ dom_exception _dom_node_remove_event_listener_ns(dom_event_target *et,
 			namespace, type, listener, capture);
 }
 
+
+/** Helper for allocating/expanding array of event targets */
+static inline dom_exception _dom_event_targets_expand(
+		uint32_t *ntargets_allocated,
+		dom_event_target ***targets)
+{
+	dom_event_target **t = *targets;
+	uint32_t size = *ntargets_allocated;
+
+	if (t == NULL) {
+		/* Create the event target list */
+		size = 64;
+		t = calloc(sizeof(*t), size);
+		if (t == NULL) {
+			return DOM_NO_MEM_ERR;
+		}
+	} else {
+		/* Extend events target list */
+		dom_event_target **tmp = realloc(t, size * 2 * sizeof(*t));
+		if (tmp == NULL) {
+			return DOM_NO_MEM_ERR;
+		}
+		memset(tmp + size, 0, size * sizeof(*tmp));
+		t = tmp;
+		size *= 2;
+	}
+
+	*ntargets_allocated = size;
+	*targets = t;
+
+	return DOM_NO_ERR;
+}
+
 /**
  * Dispatch an event into the implementation's event model
  *
  * \param et       The EventTarget object
- * \param eti      Internal EventTarget
  * \param evt      The event object
  * \param success  Indicates whether any of the listeners which handled the 
  *                 event called Event.preventDefault(). If 
@@ -2341,7 +2373,7 @@ dom_exception _dom_node_dispatch_event(dom_event_target *et,
 	dom_exception err, ret = DOM_NO_ERR;
 	dom_node_internal *target = (dom_node_internal *) et;
 	dom_document *doc;
-	dom_document_event_internal *dei;
+	dom_document_event_internal *dei = NULL;
 	dom_event_target **targets;
 	uint32_t ntargets, ntargets_allocated, targetnr;
 	void *pw;
@@ -2360,9 +2392,6 @@ dom_exception _dom_node_dispatch_event(dom_event_target *et,
 		return DOM_UNSPECIFIED_EVENT_TYPE_ERR;
 	}
 
-	if (evt->doc == NULL)
-		return DOM_NOT_SUPPORTED_ERR;
-	
 	doc = dom_node_get_owner(et);
 	if (doc == NULL) {
 		/* TODO: In the progress of parsing, many Nodes in the DTD has
@@ -2372,31 +2401,46 @@ dom_exception _dom_node_dispatch_event(dom_event_target *et,
 	
 	*success = true;
 
-	/* Compose the event target list */
+	/* Initialise array of targets for capture/bubbling phases */
+	targets = NULL;
+	ntargets_allocated = 0;
 	ntargets = 0;
-	ntargets_allocated = 64;
-	targets = calloc(sizeof(*targets), ntargets_allocated);
-	if (targets == NULL) {
-		/** \todo Report memory exhaustion? */
-		return DOM_NO_ERR;
-	}
-	targets[ntargets++] = (dom_event_target *)dom_node_ref(et);
-	target = target->parent;
 
-	while (target != NULL) {
+	/* Add interested event listeners to array */
+	for (; target != NULL; target = target->parent) {
+		struct listener_entry *le = target->eti.listeners;
+		bool target_has_listener = false;
+
+		if (le == NULL) {
+			/* This event target isn't listening to anything */
+			continue;
+		}
+
+		/* Check whether the event target is listening for this
+		 * event type */
+		do {
+			if (dom_string_isequal(le->type, evt->type)) {
+				target_has_listener = true;
+				break;
+			}
+			le = (struct listener_entry *) le->list.next;
+		} while (le != target->eti.listeners);
+
+		if (target_has_listener == false) {
+			continue;
+		}
+
+		/* The event target is listening for this event type,
+		 * so add it to the array. */
 		if (ntargets == ntargets_allocated) {
-			dom_event_target **newtargets = realloc(
-				targets,
-				ntargets_allocated * 2 * sizeof(*targets));
-			if (newtargets == NULL)
+			err = _dom_event_targets_expand(&ntargets_allocated,
+					&targets);
+			if (err != DOM_NO_ERR) {
+				ret = err;
 				goto cleanup;
-			memset(newtargets + ntargets_allocated,
-			       0, ntargets_allocated * sizeof(*newtargets));
-			targets = newtargets;
-			ntargets_allocated *= 2;
+			}
 		}
 		targets[ntargets++] = (dom_event_target *)dom_node_ref(target);
-		target = target->parent;
 	}
 
 	/* Fill the target of the event */
@@ -2464,19 +2508,12 @@ dom_exception _dom_node_dispatch_event(dom_event_target *et,
 	if (dei->actions == NULL)
 		goto cleanup;
 
-	/* The end callback of default action */
-	if (evt->prevent_default != true) {
-		dom_default_action_callback cb = dei->actions(evt->type,
-				DOM_DEFAULT_ACTION_END, &pw);
-		if (cb != NULL) {
-			cb(evt, pw);
-		}
-	}
-
-	/* The prevented callback of default action */
-	if (evt->prevent_default != true) {
-		dom_default_action_callback cb = dei->actions(evt->type,
-				DOM_DEFAULT_ACTION_PREVENTED, &pw);
+	if (dei->actions != NULL) {
+		dom_default_action_phase phase = DOM_DEFAULT_ACTION_END;
+		if (evt->prevent_default == true)
+			phase = DOM_DEFAULT_ACTION_PREVENTED;
+		dom_default_action_callback cb = dei->actions(
+				evt->type, phase, &pw);
 		if (cb != NULL) {
 			cb(evt, pw);
 		}
@@ -2490,7 +2527,17 @@ cleanup:
 	while (ntargets--) {
 		dom_node_unref(targets[ntargets]);
 	}
-	free(targets);
+	if (targets != NULL) {
+		free(targets);
+	}
+
+	if (dei != NULL && dei->actions != NULL) {
+		dom_default_action_callback cb = dei->actions(evt->type,
+				DOM_DEFAULT_ACTION_FINISHED, &pw);
+		if (cb != NULL) {
+			cb(evt, pw);
+		}
+	}
 
 	return ret;
 }
