@@ -22,14 +22,13 @@ struct dom_mouse_wheel_event {
 };
 
 /* Constructor */
-dom_exception _dom_mouse_wheel_event_create(struct dom_document *doc, 
-		struct dom_mouse_wheel_event **evt);
+dom_exception _dom_mouse_wheel_event_create(struct dom_mouse_wheel_event **evt);
 
 /* Destructor */
 void _dom_mouse_wheel_event_destroy(struct dom_mouse_wheel_event *evt);
 
 /* Initialise function */
-dom_exception _dom_mouse_wheel_event_initialise(struct dom_document *doc,
+dom_exception _dom_mouse_wheel_event_initialise(
 		struct dom_mouse_wheel_event *evt);
 
 /* Finalise function */
