@@ -8,12 +8,16 @@
 #include <assert.h>
 #include <stdlib.h>
 
+#include <dom/html/html_elements.h>
+
 #include "html/html_document.h"
 #include "html/html_element.h"
 #include "html/html_collection.h"
 #include "html/html_html_element.h"
 #include "html/html_head_element.h"
 #include "html/html_body_element.h"
+#include "html/html_base_element.h"
+#include "html/html_div_element.h"
 #include "html/html_link_element.h"
 #include "html/html_title_element.h"
 #include "html/html_meta_element.h"
@@ -73,7 +77,7 @@ static struct dom_html_document_vtable html_document_vtable = {
 			{
 				DOM_NODE_EVENT_TARGET_VTABLE
 			},
-			DOM_NODE_VTABLE,
+			DOM_NODE_VTABLE_DOCUMENT,
 		},
 		DOM_DOCUMENT_VTABLE_HTML
 	},
@@ -134,10 +138,16 @@ dom_exception _dom_html_document_initialise(dom_html_document *doc,
 		error = DOM_NO_MEM_ERR;
 		goto out;
 	}
+	doc->elements = calloc(sizeof(dom_string *),
+			DOM_HTML_ELEMENT_TYPE__COUNT);
+	if (doc->elements == NULL) {
+		error = DOM_NO_MEM_ERR;
+		goto out;
+	}
 
-#define HTML_DOCUMENT_STRINGS_ACTION(attr,str)                             \
+#define HTML_DOCUMENT_STRINGS_ACTION(attr,str)				\
 	error = dom_string_create_interned((const uint8_t *) #str,	\
-			SLEN(#str), &doc->memoised[hds_##attr]); \
+			SLEN(#str), &doc->memoised[hds_##attr]);	\
 	if (error != DOM_NO_ERR) {					\
 		goto out;						\
 	}
@@ -145,15 +155,38 @@ dom_exception _dom_html_document_initialise(dom_html_document *doc,
 #include "html_document_strings.h"
 #undef HTML_DOCUMENT_STRINGS_ACTION
 
+#define DOM_HTML_ELEMENT_STRINGS_ENTRY(tag)				\
+	error = dom_string_create_interned((const uint8_t *) #tag,	\
+			SLEN(#tag),					\
+			&doc->elements[DOM_HTML_ELEMENT_TYPE_##tag]);	\
+	if (error != DOM_NO_ERR) {					\
+		goto out;						\
+	}
+
+#include <dom/html/html_elements.h>
+#undef DOM_HTML_ELEMENT_STRINGS_ENTRY
+
 out:
-	if (doc->memoised != NULL && error != DOM_NO_ERR) {
-		for(sidx = 0; sidx < hds_COUNT; ++sidx) {
-			if (doc->memoised[sidx] != NULL) {
-				dom_string_unref(doc->memoised[sidx]);
+	if (error != DOM_NO_ERR) {
+		if (doc->memoised != NULL) {
+			for(sidx = 0; sidx < hds_COUNT; ++sidx) {
+				if (doc->memoised[sidx] != NULL) {
+					dom_string_unref(doc->memoised[sidx]);
+				}
 			}
+			free(doc->memoised);
+			doc->memoised = NULL;
+		}
+		if (doc->elements != NULL) {
+			for(sidx = 0; sidx < DOM_HTML_ELEMENT_TYPE__COUNT;
+					++sidx) {
+				if (doc->elements[sidx] != NULL) {
+					dom_string_unref(doc->elements[sidx]);
+				}
+			}
+			free(doc->elements);
+			doc->elements = NULL;
 		}
-		free(doc->memoised);
-		doc->memoised = NULL;
 	}
 	return error;
 }
@@ -184,6 +217,16 @@ bool _dom_html_document_finalise(dom_html_document *doc)
 		doc->memoised = NULL;
 	}
 	
+	if (doc->elements != NULL) {
+		for(sidx = 0; sidx < DOM_HTML_ELEMENT_TYPE__COUNT; ++sidx) {
+			if (doc->elements[sidx] != NULL) {
+				dom_string_unref(doc->elements[sidx]);
+			}
+		}
+		free(doc->elements);
+		doc->elements = NULL;
+	}
+
 	return _dom_document_finalise(&doc->base);
 }
 
@@ -205,189 +248,598 @@ dom_exception _dom_html_document_copy(dom_node_internal *old,
 	return DOM_NOT_SUPPORTED_ERR;
 }
 
+#define RETURN_IF_MATCH(_x, _y) \
+	if (dom_string_isequal(_x, html->elements[_y])) { return _y; }
+
+/**
+ * Get html element type from an uppercased tag name string
+ *
+ * \param html            The html document that the html element belongs to
+ * \param upper  Upper cased string containing tag name
+ */
+static inline dom_html_element_type _dom_html_document_get_element_type(
+		dom_html_document *html, dom_string *upper)
+{
+	size_t len = dom_string_byte_length(upper);
+
+	switch (len) {
+	case 1:
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_P)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_A)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_B)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_I)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_S)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_U)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_Q)
+		break;
+	case 2:
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_EM)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_LI)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_H1)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_H2)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_H3)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_H4)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_H5)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_H6)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_HR)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_DD)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_DT)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_DL)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_UL)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_OL)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_TD)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_TH)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_TR)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_BR)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_RP)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_RT)
+		break;
+	case 3:
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_DIV)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_IMG)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_PRE)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_DEL)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_INS)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_COL)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_MAP)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_DIR)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_SUB)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_SUP)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_DFN)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_KBD)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_NAV)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_VAR)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_BDI)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_BDO)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_WBR)
+		break;
+	case 4:
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_META)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_CODE)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_SPAN)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_FORM)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_LINK)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_MENU)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_FONT)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_ABBR)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_CITE)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_DATA)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_HTML)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_BODY)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_HEAD)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_AREA)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_MAIN)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_MARK)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_RUBY)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_SAMP)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_BASE)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_TIME)
+		break;
+	case 5:
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_INPUT)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_TABLE)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_STYLE)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_THEAD)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_TBODY)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_TFOOT)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_LABEL)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_PARAM)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_TITLE)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_FRAME)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_AUDIO)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_ASIDE)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_EMBED)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_METER)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_SMALL)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_TRACK)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_VIDEO)
+		break;
+	case 6:
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_BUTTON)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_OPTION)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_SCRIPT)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_IFRAME)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_LEGEND)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_SELECT)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_OBJECT)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_APPLET)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_CANVAS)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_DIALOG)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_CENTER)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_FIGURE)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_FOOTER)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_HEADER)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_HGROUP)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_KEYGEN)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_OUTPUT)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_SOURCE)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_STRONG)
+		break;
+	case 7:
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_CAPTION)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_ISINDEX)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_ADDRESS)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_ARTICLE)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_DETAILS)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_PICTURE)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_SECTION)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_SUMMARY)
+		break;
+	case 8:
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_TEXTAREA)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_OPTGROUP)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_BASEFONT)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_FIELDSET)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_COLGROUP)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_FRAMESET)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_DATALIST)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_MENUITEM)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_NOSCRIPT)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_PROGRESS)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_TEMPLATE)
+		break;
+	case 10:
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_BLOCKQUOTE)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_FIGCAPTION)
+		break;
+	}
+
+	return DOM_HTML_ELEMENT_TYPE__UNKNOWN;
+}
+
+#undef RETURN_IF_MATCH
+
 /* Overloaded methods inherited from super class */
 
 /** Internal method to support both kinds of create method */
 static dom_exception
-_dom_html_document_create_element_internal(dom_html_document *html,
-					   dom_string *in_tag_name,
-					   dom_string *namespace,
-					   dom_string *prefix,
-					   dom_html_element **result)
+_dom_html_document_create_element_internal(
+		dom_html_document *html,
+		dom_string *in_tag_name,
+		dom_string *namespace,
+		dom_string *prefix,
+		dom_html_element **result)
 {
 	dom_exception exc;
-	dom_string *tag_name;
+	struct dom_html_element_create_params params;
 
-	exc = dom_string_toupper(in_tag_name, true, &tag_name);
+	exc = dom_string_toupper(in_tag_name, true, &params.name);
 	if (exc != DOM_NO_ERR)
 		return exc;
 
-	if (dom_string_caseless_isequal(tag_name, html->memoised[hds_HTML])) {
-		exc = _dom_html_html_element_create(html, namespace, prefix,
+	params.type = _dom_html_document_get_element_type(html, params.name);
+	params.doc = html;
+	params.namespace = namespace;
+	params.prefix = prefix;
+
+	switch(params.type) {
+	case DOM_HTML_ELEMENT_TYPE__COUNT:
+		assert(params.type != DOM_HTML_ELEMENT_TYPE__COUNT);
+		break;
+	case DOM_HTML_ELEMENT_TYPE_HTML:
+		exc = _dom_html_html_element_create(&params,
 				(dom_html_html_element **) result);
-	} else if (dom_string_caseless_isequal(tag_name, html->memoised[hds_HEAD])) {
-		exc = _dom_html_head_element_create(html, namespace, prefix,
+		break;
+	case DOM_HTML_ELEMENT_TYPE_HEAD:
+		exc = _dom_html_head_element_create(&params,
 				(dom_html_head_element **) result);
-	} else if (dom_string_caseless_isequal(tag_name, html->memoised[hds_TITLE])) {
-		exc = _dom_html_title_element_create(html, namespace, prefix,
+		break;
+	case DOM_HTML_ELEMENT_TYPE_META:
+		exc = _dom_html_meta_element_create(&params,
+				(dom_html_meta_element **) result);
+		break;
+	case DOM_HTML_ELEMENT_TYPE_BASE:
+		exc = _dom_html_base_element_create(&params,
+				(dom_html_base_element **) result);
+		break;
+	case DOM_HTML_ELEMENT_TYPE_TITLE:
+		exc = _dom_html_title_element_create(&params,
 				(dom_html_title_element **) result);
-	} else if (dom_string_caseless_isequal(tag_name, html->memoised[hds_BODY])) {
-		exc = _dom_html_body_element_create(html, namespace, prefix,
+		break;
+	case DOM_HTML_ELEMENT_TYPE_BODY:
+		exc = _dom_html_body_element_create(&params,
 				(dom_html_body_element **) result);
-	} else if (dom_string_caseless_isequal(tag_name, html->memoised[hds_FORM])) {
-		exc = _dom_html_form_element_create(html, namespace, prefix,
+		break;
+	case DOM_HTML_ELEMENT_TYPE_DIV:
+		exc = _dom_html_div_element_create(&params,
+				(dom_html_div_element **) result);
+		break;
+	case DOM_HTML_ELEMENT_TYPE_FORM:
+		exc = _dom_html_form_element_create(&params,
 				(dom_html_form_element **) result);
-	} else if (dom_string_caseless_isequal(tag_name, html->memoised[hds_LINK])) {
-		exc = _dom_html_link_element_create(html, namespace, prefix,
+		break;
+	case DOM_HTML_ELEMENT_TYPE_LINK:
+		exc = _dom_html_link_element_create(&params,
 				(dom_html_link_element **) result);
-	} else if (dom_string_caseless_isequal(tag_name, html->memoised[hds_BUTTON])) {
-		exc = _dom_html_button_element_create(html, namespace, prefix,
+		break;
+	case DOM_HTML_ELEMENT_TYPE_BUTTON:
+		exc = _dom_html_button_element_create(&params,
 				(dom_html_button_element **) result);
-	} else if (dom_string_caseless_isequal(tag_name, html->memoised[hds_INPUT])) {
-		exc = _dom_html_input_element_create(html, namespace, prefix,
+		break;
+	case DOM_HTML_ELEMENT_TYPE_INPUT:
+		exc = _dom_html_input_element_create(&params,
 				(dom_html_input_element **) result);
-	} else if (dom_string_caseless_isequal(tag_name, html->memoised[hds_TEXTAREA])) {
-		exc = _dom_html_text_area_element_create(html, namespace, prefix,
+		break;
+	case DOM_HTML_ELEMENT_TYPE_TEXTAREA:
+		exc = _dom_html_text_area_element_create(&params,
 				(dom_html_text_area_element **) result);
-	} else if (dom_string_caseless_isequal(tag_name, html->memoised[hds_OPTGROUP])) {
-		exc = _dom_html_opt_group_element_create(html, namespace, prefix,
+		break;
+	case DOM_HTML_ELEMENT_TYPE_OPTGROUP:
+		exc = _dom_html_opt_group_element_create(&params,
 				(dom_html_opt_group_element **) result);
-	} else if (dom_string_caseless_isequal(tag_name, html->memoised[hds_OPTION])) {
-		exc = _dom_html_option_element_create(html, namespace, prefix,
+		break;
+	case DOM_HTML_ELEMENT_TYPE_OPTION:
+		exc = _dom_html_option_element_create(&params,
 				(dom_html_option_element **) result);
-	} else if (dom_string_caseless_isequal(tag_name, html->memoised[hds_SELECT])) {
-		exc = _dom_html_select_element_create(html, namespace, prefix,
+		break;
+	case DOM_HTML_ELEMENT_TYPE_SELECT:
+		exc = _dom_html_select_element_create(&params,
 				(dom_html_select_element **) result);
-	} else if (dom_string_caseless_isequal(tag_name, html->memoised[hds_HR])) {
-		exc = _dom_html_hr_element_create(html, namespace, prefix,
+		break;
+	case DOM_HTML_ELEMENT_TYPE_HR:
+		exc = _dom_html_hr_element_create(&params,
 				(dom_html_hr_element **) result);
-	} else if (dom_string_caseless_isequal(tag_name, html->memoised[hds_DL])) {
-		exc = _dom_html_d_list_element_create(html, namespace, prefix,
-				(dom_html_d_list_element **) result);
-	} else if (dom_string_caseless_isequal(tag_name, html->memoised[hds_DIRECTORY])) {
-		exc = _dom_html_directory_element_create(html, namespace, prefix,
+		break;
+	case DOM_HTML_ELEMENT_TYPE_DL:
+		exc = _dom_html_dlist_element_create(&params,
+				(dom_html_dlist_element **) result);
+		break;
+	case DOM_HTML_ELEMENT_TYPE_DIR:
+		exc = _dom_html_directory_element_create(&params,
 				(dom_html_directory_element **) result);
-	} else if (dom_string_caseless_isequal(tag_name, html->memoised[hds_MENU])) {
-		exc = _dom_html_menu_element_create(html, namespace, prefix,
+		break;
+	case DOM_HTML_ELEMENT_TYPE_MENU:
+		exc = _dom_html_menu_element_create(&params,
 				(dom_html_menu_element **) result);
-	} else if (dom_string_caseless_isequal(tag_name, html->memoised[hds_FIELDSET])) {
-		exc = _dom_html_field_set_element_create(html, namespace, prefix,
+		break;
+	case DOM_HTML_ELEMENT_TYPE_FIELDSET:
+		exc = _dom_html_field_set_element_create(&params,
 				(dom_html_field_set_element **) result);
-	} else if (dom_string_caseless_isequal(tag_name, html->memoised[hds_LEGEND])) {
-		exc = _dom_html_legend_element_create(html, namespace, prefix,
+		break;
+	case DOM_HTML_ELEMENT_TYPE_LEGEND:
+		exc = _dom_html_legend_element_create(&params,
 				(dom_html_legend_element **) result);
-	} else if (dom_string_caseless_isequal(tag_name, html->memoised[hds_P])) {
-		exc = _dom_html_paragraph_element_create(html, namespace, prefix,
+		break;
+	case DOM_HTML_ELEMENT_TYPE_P:
+		exc = _dom_html_paragraph_element_create(&params,
 				(dom_html_paragraph_element **) result);
-	} else if (dom_string_caseless_isequal(tag_name, html->memoised[hds_H1]) ||
-			dom_string_caseless_isequal(tag_name, html->memoised[hds_H2]) ||
-			dom_string_caseless_isequal(tag_name, html->memoised[hds_H3]) ||
-			dom_string_caseless_isequal(tag_name, html->memoised[hds_H4]) ||
-			dom_string_caseless_isequal(tag_name, html->memoised[hds_H5]) ||
-			dom_string_caseless_isequal(tag_name, html->memoised[hds_H6])
-			) {
-		exc = _dom_html_heading_element_create(html, tag_name, namespace, prefix,
+		break;
+	case DOM_HTML_ELEMENT_TYPE_H1:
+	case DOM_HTML_ELEMENT_TYPE_H2:
+	case DOM_HTML_ELEMENT_TYPE_H3:
+	case DOM_HTML_ELEMENT_TYPE_H4:
+	case DOM_HTML_ELEMENT_TYPE_H5:
+	case DOM_HTML_ELEMENT_TYPE_H6:
+		exc = _dom_html_heading_element_create(&params,
 				(dom_html_heading_element **) result);
-	} else if (dom_string_caseless_isequal(tag_name, html->memoised[hds_Q])) {
-		exc = _dom_html_quote_element_create(html, namespace, prefix,
+		break;
+	case DOM_HTML_ELEMENT_TYPE_BLOCKQUOTE:
+	case DOM_HTML_ELEMENT_TYPE_Q:
+		exc = _dom_html_quote_element_create(&params,
 				(dom_html_quote_element **) result);
-	} else if (dom_string_caseless_isequal(tag_name, html->memoised[hds_PRE])) {
-		exc = _dom_html_pre_element_create(html, namespace, prefix,
+		break;
+	case DOM_HTML_ELEMENT_TYPE_PRE:
+		exc = _dom_html_pre_element_create(&params,
 				(dom_html_pre_element **) result);
-	} else if (dom_string_caseless_isequal(tag_name, html->memoised[hds_BR])) {
-		exc = _dom_html_br_element_create(html, namespace, prefix,
+		break;
+	case DOM_HTML_ELEMENT_TYPE_BR:
+		exc = _dom_html_br_element_create(&params,
 				(dom_html_br_element **) result);
-	} else if (dom_string_caseless_isequal(tag_name, html->memoised[hds_LABEL])) {
-		exc = _dom_html_label_element_create(html, namespace, prefix,
+		break;
+	case DOM_HTML_ELEMENT_TYPE_LABEL:
+		exc = _dom_html_label_element_create(&params,
 				(dom_html_label_element **) result);
-	} else if (dom_string_caseless_isequal(tag_name, html->memoised[hds_UL])) {
-		exc = _dom_html_u_list_element_create(html, namespace, prefix,
+		break;
+	case DOM_HTML_ELEMENT_TYPE_UL:
+		exc = _dom_html_u_list_element_create(&params,
 				(dom_html_u_list_element **) result);
-	} else if (dom_string_caseless_isequal(tag_name, html->memoised[hds_OL])) {
-		exc = _dom_html_o_list_element_create(html, namespace, prefix,
-				(dom_html_o_list_element **) result);
-	} else if (dom_string_caseless_isequal(tag_name, html->memoised[hds_LI])) {
-		exc = _dom_html_li_element_create(html, namespace, prefix,
+		break;
+	case DOM_HTML_ELEMENT_TYPE_OL:
+		exc = _dom_html_olist_element_create(&params,
+				(dom_html_olist_element **) result);
+		break;
+	case DOM_HTML_ELEMENT_TYPE_LI:
+		exc = _dom_html_li_element_create(&params,
 				(dom_html_li_element **) result);
-	} else if (dom_string_caseless_isequal(tag_name, html->memoised[hds_FONT])) {
-		exc = _dom_html_font_element_create(html, namespace, prefix,
+		break;
+	case DOM_HTML_ELEMENT_TYPE_FONT:
+		exc = _dom_html_font_element_create(&params,
 				(dom_html_font_element **) result);
-	} else if (dom_string_caseless_isequal(tag_name, html->memoised[hds_DEL]) ||
-			dom_string_caseless_isequal(tag_name, html->memoised[hds_INS])) {
-		exc = _dom_html_mod_element_create(html, tag_name, namespace, 
-				prefix, (dom_html_mod_element **) result);
-	} else if (dom_string_caseless_isequal(tag_name, html->memoised[hds_A])) {
-		exc = _dom_html_anchor_element_create(html, namespace, prefix,
+		break;
+	case DOM_HTML_ELEMENT_TYPE_DEL:
+	case DOM_HTML_ELEMENT_TYPE_INS:
+		exc = _dom_html_mod_element_create(&params,
+				(dom_html_mod_element **) result);
+		break;
+	case DOM_HTML_ELEMENT_TYPE_A:
+		exc = _dom_html_anchor_element_create(&params,
 				(dom_html_anchor_element **) result);
-	} else if (dom_string_caseless_isequal(tag_name, html->memoised[hds_BASEFONT])) {
-		exc = _dom_html_base_font_element_create(html, namespace, prefix,
+		break;
+	case DOM_HTML_ELEMENT_TYPE_BASEFONT:
+		exc = _dom_html_base_font_element_create(&params,
 				(dom_html_base_font_element **) result);
-	} else if (dom_string_caseless_isequal(tag_name, html->memoised[hds_IMG])) {
-		exc = _dom_html_image_element_create(html, namespace, prefix,
+		break;
+	case DOM_HTML_ELEMENT_TYPE_IMG:
+		exc = _dom_html_image_element_create(&params,
 				(dom_html_image_element **) result);
-	} else if (dom_string_caseless_isequal(tag_name, html->memoised[hds_OBJECT])) {
-		exc = _dom_html_object_element_create(html, namespace, prefix,
+		break;
+	case DOM_HTML_ELEMENT_TYPE_OBJECT:
+		exc = _dom_html_object_element_create(&params,
 				(dom_html_object_element **) result);
-	} else if (dom_string_caseless_isequal(tag_name, html->memoised[hds_PARAM])) {
-		exc = _dom_html_param_element_create(html, namespace, prefix,
+		break;
+	case DOM_HTML_ELEMENT_TYPE_PARAM:
+		exc = _dom_html_param_element_create(&params,
 				(dom_html_param_element **) result);
-	} else if (dom_string_caseless_isequal(tag_name, html->memoised[hds_APPLET])) {
-		exc = _dom_html_applet_element_create(html, namespace, prefix,
+		break;
+	case DOM_HTML_ELEMENT_TYPE_APPLET:
+		exc = _dom_html_applet_element_create(&params,
 				(dom_html_applet_element **) result);
-	} else if (dom_string_caseless_isequal(tag_name, html->memoised[hds_MAP])) {
-		exc = _dom_html_map_element_create(html, namespace, prefix,
+		break;
+	case DOM_HTML_ELEMENT_TYPE_MAP:
+		exc = _dom_html_map_element_create(&params,
 				(dom_html_map_element **) result);
-	} else if (dom_string_caseless_isequal(tag_name, html->memoised[hds_AREA])) {
-		exc = _dom_html_area_element_create(html, namespace, prefix,
+		break;
+	case DOM_HTML_ELEMENT_TYPE_AREA:
+		exc = _dom_html_area_element_create(&params,
 				(dom_html_area_element **) result);
-	} else if (dom_string_caseless_isequal(tag_name, html->memoised[hds_SCRIPT])) {
-		exc = _dom_html_script_element_create(html, namespace, prefix,
+		break;
+	case DOM_HTML_ELEMENT_TYPE_SCRIPT:
+		exc = _dom_html_script_element_create(&params,
 				(dom_html_script_element **) result);
-	} else if (dom_string_caseless_isequal(tag_name, html->memoised[hds_CAPTION])) {
-		exc = _dom_html_table_caption_element_create(html, namespace, prefix,
+		break;
+	case DOM_HTML_ELEMENT_TYPE_CAPTION:
+		exc = _dom_html_table_caption_element_create(&params,
 				(dom_html_table_caption_element **) result);
-	} else if (dom_string_caseless_isequal(tag_name, html->memoised[hds_TD]) ||
-			dom_string_caseless_isequal(tag_name, html->memoised[hds_TH])
-			) {
-		exc = _dom_html_table_cell_element_create(html, tag_name, namespace, prefix,
+		break;
+	case DOM_HTML_ELEMENT_TYPE_TD:
+	case DOM_HTML_ELEMENT_TYPE_TH:
+		/* TODO: Should be:
+		 * HTMLTableDataCellElement : HTMLTableCellElement : HTMLElement
+		 * HTMLTableHeaderCellElement : HTMLTableCellElement : HTMLElement
+		 */
+		exc = _dom_html_table_cell_element_create(&params,
 				(dom_html_table_cell_element **) result);
-	} else if (dom_string_caseless_isequal(tag_name, html->memoised[hds_COL])||
-			dom_string_caseless_isequal(tag_name, html->memoised[hds_COLGROUP])
-			) {
-		exc = _dom_html_table_col_element_create(html, tag_name, namespace, prefix,
+		break;
+	case DOM_HTML_ELEMENT_TYPE_COL:
+	case DOM_HTML_ELEMENT_TYPE_COLGROUP:
+		exc = _dom_html_table_col_element_create(&params,
 				(dom_html_table_col_element **) result);
-	} else if (dom_string_caseless_isequal(tag_name, html->memoised[hds_THEAD])||
-			dom_string_caseless_isequal(tag_name, html->memoised[hds_TBODY])||
-			dom_string_caseless_isequal(tag_name, html->memoised[hds_TFOOT])) {
-		exc = _dom_html_table_section_element_create(html, tag_name, namespace, prefix,
+		break;
+	case DOM_HTML_ELEMENT_TYPE_THEAD:
+	case DOM_HTML_ELEMENT_TYPE_TBODY:
+	case DOM_HTML_ELEMENT_TYPE_TFOOT:
+		exc = _dom_html_table_section_element_create(&params,
 				(dom_html_table_section_element **) result);
-	} else if (dom_string_caseless_isequal(tag_name, html->memoised[hds_TABLE])) {
-		exc = _dom_html_table_element_create(html, namespace, prefix,
+		break;
+	case DOM_HTML_ELEMENT_TYPE_TABLE:
+		exc = _dom_html_table_element_create(&params,
 				(dom_html_table_element **) result);
-	} else if (dom_string_caseless_isequal(tag_name, html->memoised[hds_TD])) {
-		exc = _dom_html_table_row_element_create(html, namespace, prefix,
+		break;
+	case DOM_HTML_ELEMENT_TYPE_TR:
+		exc = _dom_html_table_row_element_create(&params,
 				(dom_html_table_row_element **) result);
-	} else if (dom_string_caseless_isequal(tag_name, html->memoised[hds_STYLE])) {
-		exc = _dom_html_style_element_create(html,
+		break;
+	case DOM_HTML_ELEMENT_TYPE_STYLE:
+		exc = _dom_html_style_element_create(&params,
 				(dom_html_style_element **) result);
-	} else if (dom_string_caseless_isequal(tag_name, html->memoised[hds_FRAMESET])) {
-		exc = _dom_html_frame_set_element_create(html, namespace, prefix,
+		break;
+	case DOM_HTML_ELEMENT_TYPE_FRAMESET:
+		exc = _dom_html_frame_set_element_create(&params,
 				(dom_html_frame_set_element **) result);
-	} else if (dom_string_caseless_isequal(tag_name, html->memoised[hds_FRAME])) {
-		exc = _dom_html_frame_element_create(html, namespace, prefix,
+		break;
+	case DOM_HTML_ELEMENT_TYPE_FRAME:
+		exc = _dom_html_frame_element_create(&params,
 				(dom_html_frame_element **) result);
-	} else if (dom_string_caseless_isequal(tag_name, html->memoised[hds_IFRAME])) {
-		exc = _dom_html_iframe_element_create(html, namespace, prefix,
+		break;
+	case DOM_HTML_ELEMENT_TYPE_IFRAME:
+		exc = _dom_html_iframe_element_create(&params,
 				(dom_html_iframe_element **) result);
-	} else if (dom_string_caseless_isequal(tag_name, html->memoised[hds_ISINDEX])) {
-		exc = _dom_html_isindex_element_create(html, namespace, prefix,
+		break;
+	case DOM_HTML_ELEMENT_TYPE_ISINDEX:
+		exc = _dom_html_isindex_element_create(&params,
 				(dom_html_isindex_element **) result);
-	} else {
-		exc =  _dom_html_element_create(html, tag_name, namespace,
-						prefix, result);
+		break;
+	case DOM_HTML_ELEMENT_TYPE_DATA:
+	case DOM_HTML_ELEMENT_TYPE_SPAN:
+	case DOM_HTML_ELEMENT_TYPE_TIME:
+	case DOM_HTML_ELEMENT_TYPE_AUDIO:
+	case DOM_HTML_ELEMENT_TYPE_EMBED:
+	case DOM_HTML_ELEMENT_TYPE_METER:
+	case DOM_HTML_ELEMENT_TYPE_TRACK:
+	case DOM_HTML_ELEMENT_TYPE_VIDEO:
+	case DOM_HTML_ELEMENT_TYPE_CANVAS:
+	case DOM_HTML_ELEMENT_TYPE_DIALOG:
+	case DOM_HTML_ELEMENT_TYPE_KEYGEN:
+	case DOM_HTML_ELEMENT_TYPE_OUTPUT:
+	case DOM_HTML_ELEMENT_TYPE_SOURCE:
+	case DOM_HTML_ELEMENT_TYPE_DETAILS:
+	case DOM_HTML_ELEMENT_TYPE_PICTURE:
+	case DOM_HTML_ELEMENT_TYPE_DATALIST:
+	case DOM_HTML_ELEMENT_TYPE_MENUITEM:
+	case DOM_HTML_ELEMENT_TYPE_PROGRESS:
+	case DOM_HTML_ELEMENT_TYPE_TEMPLATE:
+		/* TODO: LibDOM implementation of these specialisations */
+		/* For now, fall through to HTMLElement */
+	case DOM_HTML_ELEMENT_TYPE_B:
+	case DOM_HTML_ELEMENT_TYPE_I:
+	case DOM_HTML_ELEMENT_TYPE_S:
+	case DOM_HTML_ELEMENT_TYPE_U:
+	case DOM_HTML_ELEMENT_TYPE_DD:
+	case DOM_HTML_ELEMENT_TYPE_DT:
+	case DOM_HTML_ELEMENT_TYPE_EM:
+	case DOM_HTML_ELEMENT_TYPE_RP:
+	case DOM_HTML_ELEMENT_TYPE_RT:
+	case DOM_HTML_ELEMENT_TYPE_BDI:
+	case DOM_HTML_ELEMENT_TYPE_BDO:
+	case DOM_HTML_ELEMENT_TYPE_DFN:
+	case DOM_HTML_ELEMENT_TYPE_KBD:
+	case DOM_HTML_ELEMENT_TYPE_NAV:
+	case DOM_HTML_ELEMENT_TYPE_SUB:
+	case DOM_HTML_ELEMENT_TYPE_SUP:
+	case DOM_HTML_ELEMENT_TYPE_VAR:
+	case DOM_HTML_ELEMENT_TYPE_WBR:
+	case DOM_HTML_ELEMENT_TYPE_ABBR:
+	case DOM_HTML_ELEMENT_TYPE_CITE:
+	case DOM_HTML_ELEMENT_TYPE_CODE:
+	case DOM_HTML_ELEMENT_TYPE_MAIN:
+	case DOM_HTML_ELEMENT_TYPE_MARK:
+	case DOM_HTML_ELEMENT_TYPE_RUBY:
+	case DOM_HTML_ELEMENT_TYPE_SAMP:
+	case DOM_HTML_ELEMENT_TYPE_ASIDE:
+	case DOM_HTML_ELEMENT_TYPE_SMALL:
+	case DOM_HTML_ELEMENT_TYPE_CENTER:
+	case DOM_HTML_ELEMENT_TYPE_FIGURE:
+	case DOM_HTML_ELEMENT_TYPE_FOOTER:
+	case DOM_HTML_ELEMENT_TYPE_HEADER:
+	case DOM_HTML_ELEMENT_TYPE_HGROUP:
+	case DOM_HTML_ELEMENT_TYPE_STRONG:
+	case DOM_HTML_ELEMENT_TYPE_ADDRESS:
+	case DOM_HTML_ELEMENT_TYPE_ARTICLE:
+	case DOM_HTML_ELEMENT_TYPE_SECTION:
+	case DOM_HTML_ELEMENT_TYPE_SUMMARY:
+	case DOM_HTML_ELEMENT_TYPE_NOSCRIPT:
+	case DOM_HTML_ELEMENT_TYPE_FIGCAPTION:
+		/* These have no specialisation: use HTMLElement */
+		/* Fall through */
+	case DOM_HTML_ELEMENT_TYPE__UNKNOWN:
+		/* Create generic HTMLElement */
+		exc = _dom_html_element_create(&params, result);
+		break;
 	}
 
-	dom_string_unref(tag_name);
+	dom_string_unref(params.name);
 
 	return exc;
 }
@@ -576,8 +1028,8 @@ dom_exception _dom_html_document_get_title(dom_html_document *doc,
 		uint32_t len;
 		
 		exc = dom_document_get_elements_by_tag_name(doc,
-							    doc->memoised[hds_TITLE],
-							    &nodes);
+				doc->elements[DOM_HTML_ELEMENT_TYPE_TITLE],
+				&nodes);
 		if (exc != DOM_NO_ERR) {
 			return exc;
 		}
@@ -654,7 +1106,7 @@ dom_exception _dom_html_document_get_body(dom_html_document *doc,
 		uint32_t len;
 
 		exc = dom_document_get_elements_by_tag_name(doc,
-				doc->memoised[hds_BODY],
+				doc->elements[DOM_HTML_ELEMENT_TYPE_BODY],
 				&nodes);
 		if (exc != DOM_NO_ERR) {
 			return exc;
@@ -668,7 +1120,7 @@ dom_exception _dom_html_document_get_body(dom_html_document *doc,
 
 		if (len == 0) {
 			exc = dom_document_get_elements_by_tag_name(doc,
-					doc->memoised[hds_FRAMESET],
+					doc->elements[DOM_HTML_ELEMENT_TYPE_FRAMESET],
 					&nodes);
 			if (exc != DOM_NO_ERR) {
 				return exc;
@@ -713,9 +1165,10 @@ dom_exception _dom_html_document_set_body(dom_html_document *doc,
  */
 bool images_callback(struct dom_node_internal *node, void *ctx)
 {
+	dom_html_document *doc = ctx;
 	if(node->type == DOM_ELEMENT_NODE &&
 			dom_string_caseless_isequal(node->name,
-				((dom_html_document *)ctx)->memoised[hds_IMG])) {
+				doc->elements[DOM_HTML_ELEMENT_TYPE_IMG])) {
 		return true;
 	}
 	return false;
@@ -736,9 +1189,10 @@ dom_exception _dom_html_document_get_images(dom_html_document *doc,
 
 bool applet_callback(struct dom_node_internal * node, void *ctx)
 {
+	dom_html_document *doc = ctx;
 	if(node->type == DOM_ELEMENT_NODE &&
 			dom_string_caseless_isequal(node->name,
-				((dom_html_document *)ctx)->memoised[hds_APPLET])) {
+				doc->elements[DOM_HTML_ELEMENT_TYPE_APPLET])) {
 		return true;
 	}
 	return false;
@@ -748,19 +1202,21 @@ bool applet_callback(struct dom_node_internal * node, void *ctx)
  *
  * \param node          The dom_node_internal object
  * \param ctx           The dom_html_document object (void *)
- * \return DOM_NO_ERR on success, appropriate dom_exception on failure.
+ * \return true if node is an applet object
  */
 bool applets_callback(struct dom_node_internal *node, void *ctx)
 {
+	dom_html_document *doc = ctx;
 	if(node->type == DOM_ELEMENT_NODE &&
 			dom_string_caseless_isequal(node->name,
-				((dom_html_document *)ctx)->memoised[hds_OBJECT])) {
+				doc->elements[DOM_HTML_ELEMENT_TYPE_OBJECT])) {
 		uint32_t len = 0;
 		dom_html_collection *applets;
-		_dom_html_collection_create(ctx, node,
-				applet_callback, ctx, &applets);
-
+		if (_dom_html_collection_create(ctx, node,
+				applet_callback, ctx, &applets) != DOM_NO_ERR)
+			return false;
 		dom_html_collection_get_length(applets, &len);
+		dom_html_collection_unref(applets);
 		if(len != 0)
 			return true;
 	}
@@ -789,17 +1245,18 @@ dom_exception _dom_html_document_get_applets(dom_html_document *doc,
  */
 bool links_callback(struct dom_node_internal *node, void *ctx)
 {
+	dom_html_document *doc = ctx;
 	if(node->type == DOM_ELEMENT_NODE &&
 			(dom_string_caseless_isequal(node->name,
-				((dom_html_document *)ctx)->memoised[hds_A]) ||
+				doc->elements[DOM_HTML_ELEMENT_TYPE_A]) ||
 			dom_string_caseless_isequal(node->name,
-				((dom_html_document *)ctx)->memoised[hds_AREA]))
+				doc->elements[DOM_HTML_ELEMENT_TYPE_AREA]))
 			 ) {
 		bool has_value = false;
 		dom_exception err;
 
 		err = dom_element_has_attribute(node,
-				((dom_html_document *)ctx)->memoised[hds_href], &has_value);
+				doc->memoised[hds_href], &has_value);
 		if(err !=DOM_NO_ERR)
 			return err;
 
@@ -830,7 +1287,7 @@ static bool __dom_html_document_node_is_form(dom_node_internal *node,
 	UNUSED(ctx);
 
 	return dom_string_caseless_isequal(node->name,
-			doc->memoised[hds_FORM]);
+			doc->elements[DOM_HTML_ELEMENT_TYPE_FORM]);
 }
 
 dom_exception _dom_html_document_get_forms(dom_html_document *doc,
@@ -867,14 +1324,15 @@ dom_exception _dom_html_document_get_forms(dom_html_document *doc,
  */
 bool anchors_callback(struct dom_node_internal *node, void *ctx)
 {
+	dom_html_document *doc = ctx;
 	if(node->type == DOM_ELEMENT_NODE &&
 			dom_string_caseless_isequal(node->name,
-				((dom_html_document *)ctx)->memoised[hds_A])) {
+				doc->elements[DOM_HTML_ELEMENT_TYPE_A])) {
 		bool has_value = false;
 		dom_exception err;
 
 		err = dom_element_has_attribute(node,
-				((dom_html_document *)ctx)->memoised[hds_name], &has_value);
+				doc->memoised[hds_name], &has_value);
 		if(err !=DOM_NO_ERR)
 			return err;
 
