@@ -31,12 +31,12 @@ static struct dom_element_protected_vtable _protect_vtable = {
 /**
  * Create a dom_html_table_element object
  *
- * \param doc  The document object
- * \param ele  The returned element object
+ * \param params  The html element creation parameters
+ * \param ele     The returned element object
  * \return DOM_NO_ERR on success, appropriate dom_exception on failure.
  */
-dom_exception _dom_html_table_element_create(struct dom_html_document *doc,
-		dom_string *namespace, dom_string *prefix,
+dom_exception _dom_html_table_element_create(
+		struct dom_html_element_create_params *params,
 		struct dom_html_table_element **ele)
 {
 	struct dom_node_internal *node;
@@ -50,23 +50,21 @@ dom_exception _dom_html_table_element_create(struct dom_html_document *doc,
 	node->base.vtable = &_dom_html_element_vtable;
 	node->vtable = &_protect_vtable;
 
-	return _dom_html_table_element_initialise(doc, namespace, prefix, *ele);
+	return _dom_html_table_element_initialise(params, *ele);
 }
 
 /**
  * Initialise a dom_html_table_element object
  *
- * \param doc  The document object
- * \param ele  The dom_html_table_element object
+ * \param params  The html element creation parameters
+ * \param ele     The dom_html_table_element object
  * \return DOM_NO_ERR on success, appropriate dom_exception on failure.
  */
-dom_exception _dom_html_table_element_initialise(struct dom_html_document *doc,
-		dom_string *namespace, dom_string *prefix,
+dom_exception _dom_html_table_element_initialise(
+		struct dom_html_element_create_params *params,
 		struct dom_html_table_element *ele)
 {
-	return _dom_html_element_initialise(doc, &ele->base,
-			doc->memoised[hds_TABLE],
-			namespace, prefix);
+	return _dom_html_element_initialise(params, &ele->base);
 }
 
 /**
@@ -115,10 +113,39 @@ void _dom_virtual_html_table_element_destroy(dom_node_internal *node)
 }
 
 /* The virtual copy function, see src/core/node.c for detail */
-dom_exception _dom_html_table_element_copy(dom_node_internal *old,
-		dom_node_internal **copy)
+dom_exception _dom_html_table_element_copy(
+		dom_node_internal *old, dom_node_internal **copy)
 {
-	return _dom_html_element_copy(old, copy);
+	dom_html_table_element *new_node;
+	dom_exception err;
+
+	new_node = malloc(sizeof(dom_html_table_element));
+	if (new_node == NULL)
+		return DOM_NO_MEM_ERR;
+
+	err = dom_html_table_element_copy_internal(old, new_node);
+	if (err != DOM_NO_ERR) {
+		free(new_node);
+		return err;
+	}
+
+	*copy = (dom_node_internal *) new_node;
+
+	return DOM_NO_ERR;
+}
+
+dom_exception _dom_html_table_element_copy_internal(
+		dom_html_table_element *old,
+		dom_html_table_element *new)
+{
+	dom_exception err;
+
+	err = dom_html_element_copy_internal(old, new);
+	if (err != DOM_NO_ERR) {
+		return err;
+	}
+
+	return DOM_NO_ERR;
 }
 
 /*-----------------------------------------------------------------------*/
@@ -183,7 +210,9 @@ dom_exception dom_html_table_element_get_caption(
 
 	for (node_tmp = node_tmp->first_child; node_tmp != NULL; node_tmp = node_tmp->next) {
 		if((node_tmp->type == DOM_ELEMENT_NODE) &&
-				dom_string_caseless_isequal(doc->memoised[hds_CAPTION],node_tmp->name)) {
+				dom_string_caseless_isequal(
+				doc->elements[DOM_HTML_ELEMENT_TYPE_CAPTION],
+				node_tmp->name)) {
 			break;
 		}
 	}
@@ -207,11 +236,14 @@ dom_exception dom_html_table_element_set_caption(
 	dom_node_internal *check_node = ((dom_node_internal *)caption);
 	dom_html_document *doc = (dom_html_document *)(((dom_node_internal *)table)->owner);
 	dom_exception exp;
-	if(check_node == NULL) {
+	dom_node *new_caption;
+
+	if (check_node == NULL) {
 		return DOM_HIERARCHY_REQUEST_ERR;
 	}
-	if(!dom_string_caseless_isequal(doc->memoised[hds_CAPTION],
-				check_node->name)) {
+	if (!dom_string_caseless_isequal(
+			doc->elements[DOM_HTML_ELEMENT_TYPE_CAPTION],
+			check_node->name)) {
 		return DOM_HIERARCHY_REQUEST_ERR;
 	}
 
@@ -220,8 +252,6 @@ dom_exception dom_html_table_element_set_caption(
 		return exp;
 
 	/* Create a new caption */
-        dom_node *new_caption;
-
 	return dom_node_append_child(table, caption,
 			&new_caption);
 }
@@ -239,7 +269,9 @@ dom_exception dom_html_table_element_get_t_head(
 
 	for (node_tmp = node_tmp->first_child; node_tmp != NULL; node_tmp = node_tmp->next) {
 		if((node_tmp->type == DOM_ELEMENT_NODE) &&
-				dom_string_caseless_isequal(doc->memoised[hds_THEAD],node_tmp->name)) {
+				dom_string_caseless_isequal(
+				doc->elements[DOM_HTML_ELEMENT_TYPE_THEAD],
+				node_tmp->name)) {
 			break;
 		}
 	}
@@ -262,11 +294,14 @@ dom_exception dom_html_table_element_set_t_head(
 	dom_node_internal *check_node = ((dom_node_internal *)t_head);
 	dom_html_document *doc = (dom_html_document *)(((dom_node_internal *)table)->owner);
 	dom_exception exp;
+	dom_node *new_t_head;
 
 	if (check_node == NULL) {
 		return DOM_HIERARCHY_REQUEST_ERR;
 	}
-	if (!dom_string_caseless_isequal(doc->memoised[hds_CAPTION],check_node->name)) {
+	if (!dom_string_caseless_isequal(
+			doc->elements[DOM_HTML_ELEMENT_TYPE_CAPTION],
+			check_node->name)) {
 		return DOM_HIERARCHY_REQUEST_ERR;
 	}
 
@@ -274,8 +309,6 @@ dom_exception dom_html_table_element_set_t_head(
 	if(exp != DOM_NO_ERR)
 		return exp;
 
-	dom_node *new_t_head;
-
 	return dom_node_append_child(table,
 			t_head, &new_t_head);
 
@@ -294,8 +327,9 @@ dom_exception dom_html_table_element_get_t_foot(
 
 	for (node_tmp = node_tmp->first_child; node_tmp != NULL; node_tmp = node_tmp->next) {
 		if ((node_tmp->type == DOM_ELEMENT_NODE) &&
-				dom_string_caseless_isequal(doc->memoised[hds_TFOOT],
-					node_tmp->name)) {
+				dom_string_caseless_isequal(
+				doc->elements[DOM_HTML_ELEMENT_TYPE_TFOOT],
+				node_tmp->name)) {
 			break;
 		}
 	}
@@ -318,12 +352,15 @@ dom_exception dom_html_table_element_set_t_foot(
 	dom_node_internal *check_node = ((dom_node_internal *)t_foot); /*< temporary node to check for raised exceptions */
 	dom_html_document *doc = (dom_html_document *)(((dom_node_internal *)table)->owner);
 	dom_exception exp;
+	dom_node *new_t_foot;
 
 	if(check_node == NULL) {
 		return DOM_HIERARCHY_REQUEST_ERR;
 	}
 
-	if(!dom_string_caseless_isequal(doc->memoised[hds_TFOOT],check_node->name)) {
+	if(!dom_string_caseless_isequal(
+			doc->elements[DOM_HTML_ELEMENT_TYPE_TFOOT],
+			check_node->name)) {
 		return DOM_HIERARCHY_REQUEST_ERR;
 	}
 
@@ -331,8 +368,6 @@ dom_exception dom_html_table_element_set_t_foot(
 	if(exp != DOM_NO_ERR)
 		return exp;
 
-	dom_node *new_t_foot;
-
 	return dom_node_append_child(table, t_foot,
 				&new_t_foot);
 
@@ -345,11 +380,12 @@ dom_exception dom_html_table_element_set_t_foot(
  * \param ctx		The dom_html_document object (void *)
  * \return DOM_NO_ERR on success, appropriate dom_exception on failure.
  */
-bool table_rows_callback(struct dom_node_internal *node, void *ctx)
+static bool table_rows_callback(struct dom_node_internal *node, void *ctx)
 {
+	dom_html_document *doc = ctx;
 	if(node->type == DOM_ELEMENT_NODE &&
 			dom_string_caseless_isequal(node->name,
-				((dom_html_document *)ctx)->memoised[hds_TR])) {
+				doc->elements[DOM_HTML_ELEMENT_TYPE_TR])) {
 		return true;
 	}
 	return false;
@@ -378,11 +414,12 @@ dom_exception dom_html_table_element_get_rows(
  * \param ctx		The dom_html_document object (void *)
  * \return DOM_NO_ERR on success, appropriate dom_exception on failure.
  */
-bool table_t_bodies_callback(struct dom_node_internal *node, void *ctx)
+static bool table_t_bodies_callback(struct dom_node_internal *node, void *ctx)
 {
+	dom_html_document *doc = ctx;
 	if(node->type == DOM_ELEMENT_NODE && 
 			dom_string_caseless_isequal(node->name,
-				((dom_html_document *)ctx)->memoised[hds_TBODY])) {
+				doc->elements[DOM_HTML_ELEMENT_TYPE_TBODY])) {
 		return true;
 	}
 	return false;
@@ -395,6 +432,7 @@ bool table_t_bodies_callback(struct dom_node_internal *node, void *ctx)
  * \param t_bodies	The Status
  * \return DOM_NO_ERR on success, appropriate dom_exception on failure.
  */
+
 dom_exception dom_html_table_element_get_t_bodies(
 		dom_html_table_element *element,
 		dom_html_collection **t_bodies)
@@ -422,13 +460,19 @@ dom_exception dom_html_table_element_create_caption(
 		return exp;
 	}
 	if((*caption) == NULL) {
-		dom_html_document *doc = (dom_html_document *) 
-			((dom_node_internal *) element)->owner;
 		dom_node *new_caption;
+		dom_html_document *doc = (dom_html_document *)
+				((dom_node_internal *) element)->owner;
 
-		exp = _dom_html_table_caption_element_create(doc,
-				((dom_node_internal *)element)->namespace,
-				((dom_node_internal *)element)->prefix,
+		struct dom_html_element_create_params params = {
+			.type = DOM_HTML_ELEMENT_TYPE_CAPTION,
+			.doc = doc,
+			.name = doc->elements[DOM_HTML_ELEMENT_TYPE_CAPTION],
+			.namespace = ((dom_node_internal *)element)->namespace,
+			.prefix = ((dom_node_internal *)element)->prefix
+		};
+
+		exp = _dom_html_table_caption_element_create(&params,
 				(dom_html_table_caption_element **)caption);
 		if(exp != DOM_NO_ERR) {
 			dom_node_unref(*caption);
@@ -489,14 +533,19 @@ dom_exception dom_html_table_element_create_t_foot(
 		return exp;
 
 	if ((*t_foot) == NULL) {
-		dom_html_document *doc = (dom_html_document *) 
-			((dom_node_internal *) element)->owner;
 		dom_node *new_t_foot;
+		dom_html_document *doc = (dom_html_document *)
+				((dom_node_internal *) element)->owner;
 
-		exp = _dom_html_table_section_element_create(doc,
-				doc->memoised[hds_TFOOT],
-				((dom_node_internal *)element)->namespace,
-				((dom_node_internal *)element)->prefix,
+		struct dom_html_element_create_params params = {
+			.type = DOM_HTML_ELEMENT_TYPE_TFOOT,
+			.doc = doc,
+			.name = doc->elements[DOM_HTML_ELEMENT_TYPE_TFOOT],
+			.namespace = ((dom_node_internal *)element)->namespace,
+			.prefix = ((dom_node_internal *)element)->prefix
+		};
+
+		exp = _dom_html_table_section_element_create(&params,
 				(dom_html_table_section_element **)t_foot);
 		if (exp != DOM_NO_ERR) {
 			dom_node_unref(*t_foot);
@@ -562,14 +611,19 @@ dom_exception dom_html_table_element_create_t_head(
 	}
 	if((*t_head) == NULL) {
 		dom_exception exp;
-		dom_html_document *doc = (dom_html_document *)
-			((dom_node_internal *) element)->owner;
 		dom_node *new_t_head;
+		dom_html_document *doc = (dom_html_document *)
+				((dom_node_internal *) element)->owner;
 
-		exp = _dom_html_table_section_element_create(doc,
-				doc->memoised[hds_THEAD],
-				((dom_node_internal *)element)->namespace,
-				((dom_node_internal *)element)->prefix,
+		struct dom_html_element_create_params params = {
+			.type = DOM_HTML_ELEMENT_TYPE_THEAD,
+			.doc = doc,
+			.name = doc->elements[DOM_HTML_ELEMENT_TYPE_THEAD],
+			.namespace = ((dom_node_internal *)element)->namespace,
+			.prefix = ((dom_node_internal *)element)->prefix
+		};
+
+		exp = _dom_html_table_section_element_create(&params,
 				(dom_html_table_section_element **)t_head);
 		if(exp != DOM_NO_ERR) {
 			dom_node_unref(*t_head);
@@ -621,7 +675,7 @@ dom_exception dom_html_table_element_delete_t_head(
  * \param t_head	The Status
  * \return DOM_NO_ERR on success, appropriate dom_exception on failure.
  */
-dom_exception dom_html_table_element_create_t_body(
+static dom_exception dom_html_table_element_create_t_body(
 		dom_html_table_element *element,
 		dom_html_table_section_element **t_body)
 {
@@ -631,7 +685,6 @@ dom_exception dom_html_table_element_create_t_body(
 	exp = dom_html_table_element_get_t_bodies(element,
 			&t_bodies);
 	if(exp != DOM_NO_ERR) {
-		dom_html_collection_unref(t_bodies);
 		return exp;
 	}
 	exp = dom_html_collection_get_length(t_bodies,
@@ -641,17 +694,22 @@ dom_exception dom_html_table_element_create_t_body(
 		return exp;
 	}
 	if(len == 0) {
-		dom_html_document *doc = (dom_html_document *)
-			((dom_node_internal *) element)->owner;
 		dom_node *new_t_body;
+		dom_html_document *doc = (dom_html_document *)
+				((dom_node_internal *) element)->owner;
 
-		exp = _dom_html_table_section_element_create(doc,
-				doc->memoised[hds_TBODY],
-				((dom_node_internal *)element)->namespace,
-				((dom_node_internal *)element)->prefix,
-				t_body);
+		struct dom_html_element_create_params params = {
+			.type = DOM_HTML_ELEMENT_TYPE_TBODY,
+			.doc = doc,
+			.name = doc->elements[DOM_HTML_ELEMENT_TYPE_TBODY],
+			.namespace = ((dom_node_internal *)element)->namespace,
+			.prefix = ((dom_node_internal *)element)->prefix
+		};
+
+		exp = _dom_html_table_section_element_create(&params, t_body);
 		if(exp != DOM_NO_ERR) {
 			dom_node_unref(*t_body);
+			dom_html_collection_unref(t_bodies);
 			return exp;
 		}
 
@@ -683,9 +741,17 @@ dom_exception dom_html_table_element_insert_row(
 	dom_exception exp;
 	dom_html_collection* rows;
 	uint32_t len;
-	dom_html_document *doc = (dom_html_document *) 
+	dom_html_document *doc = (dom_html_document *)
 		((dom_node_internal *) element)->owner;
 
+	struct dom_html_element_create_params params = {
+		.type = DOM_HTML_ELEMENT_TYPE_TR,
+		.doc = doc,
+		.name = doc->elements[DOM_HTML_ELEMENT_TYPE_TR],
+		.namespace = ((dom_node_internal *)element)->namespace,
+		.prefix = ((dom_node_internal *)element)->prefix
+	};
+
 	exp = dom_html_table_element_get_rows(element,
 			&rows);
 	if(exp != DOM_NO_ERR) {
@@ -698,9 +764,7 @@ dom_exception dom_html_table_element_insert_row(
 		dom_html_collection_unref(rows);
 		return exp;
 	}
-	exp = _dom_html_table_row_element_create(doc,
-			((dom_node_internal *)element)->namespace,
-			((dom_node_internal *)element)->prefix,
+	exp = _dom_html_table_row_element_create(&params,
 			(dom_html_table_row_element **)row);
 	if(exp != DOM_NO_ERR) {
 		dom_node_unref(*row);
@@ -728,14 +792,15 @@ dom_exception dom_html_table_element_insert_row(
 			*row = (dom_html_element *)new_row;
 		}
 	} else {
+		uint32_t window_len = 0, section_len;
+		dom_html_table_section_element *t_head;
+		dom_html_table_section_element *t_foot;
+		dom_node_internal *n;
+
 		if(index ==-1) {
 			index = (int32_t)len;
 		}
 
-		dom_html_table_section_element *t_head;
-		dom_html_table_section_element *t_foot;
-		uint32_t window_len = 0, section_len;
-
 		exp = dom_html_table_element_get_t_head(element, &t_head);
 		if (exp != DOM_NO_ERR)
 			return exp;
@@ -763,13 +828,15 @@ dom_exception dom_html_table_element_insert_row(
 
 		window_len += section_len;
 
-		dom_node_internal *n = (dom_node_internal *)element;
+		n = (dom_node_internal *)element;
 
 		dom_html_collection_unref(rows);
 
 		for (n = n->first_child; n != NULL; n = n->next) {
 			if((n->type == DOM_ELEMENT_NODE) &&
-					dom_string_caseless_isequal(doc->memoised[hds_TBODY],n->name)) {
+					dom_string_caseless_isequal(
+					doc->elements[DOM_HTML_ELEMENT_TYPE_TBODY],
+					n->name)) {
 
 				exp = dom_html_table_section_element_get_rows((dom_html_table_section_element *)n, &rows);
 				exp = dom_html_collection_get_length(rows, &section_len);
@@ -846,15 +913,17 @@ dom_exception dom_html_table_element_delete_row(
 		dom_html_collection_unref(rows);
 		return DOM_INDEX_SIZE_ERR;
 	} else {
+		uint32_t window_len = 0, section_len;
+		dom_html_table_section_element *t_head;
+		dom_html_table_section_element *t_foot;
+		dom_node_internal *n;
+
 		if(index ==-1) {
 			index = (int32_t)len-1;
 		}
 
 		dom_html_collection_unref(rows);
 
-		dom_html_table_section_element *t_head;
-		dom_html_table_section_element *t_foot;
-		uint32_t window_len = 0, section_len;
 		exp = dom_html_table_element_get_t_head(element, &t_head);
 		if(exp != DOM_NO_ERR)
 			return exp;
@@ -876,11 +945,13 @@ dom_exception dom_html_table_element_delete_row(
 					index-window_len);
 		}
 		window_len += section_len;
-		dom_node_internal *n = (dom_node_internal *)element;
+		n = (dom_node_internal *)element;
 
 		for (n = n->first_child; n != NULL; n = n->next) {
 			if((n->type == DOM_ELEMENT_NODE) &&
-					dom_string_caseless_isequal(doc->memoised[hds_TBODY],n->name)) {
+					dom_string_caseless_isequal(
+					doc->elements[DOM_HTML_ELEMENT_TYPE_TBODY],
+					n->name)) {
 				exp = dom_html_table_section_element_get_rows
 					((dom_html_table_section_element *)n, &rows);
 				if(exp != DOM_NO_ERR) {
