@@ -31,7 +31,7 @@ our %special_type = (
 	# Some of the type are not defined now!
 	boolean => "bool ",
 	int => "int32_t ",
-	"unsigned long" => "uint32_t ",
+	"unsigned long" => "dom_ulong ",
 	DOMString => "dom_string *",
 	List => "list *",
 	Collection => "list *",
@@ -66,6 +66,8 @@ our %special_prefix = (
 	HTMLTableSectionElement => "dom_html_table_section_element",
 	HTMLIsIndexElement => "dom_html_isindex_element",
         HTMLIFrameElement => "dom_html_iframe_element",
+        HTMLOListElement => "dom_html_olist_element",
+        HTMLDListElement => "dom_html_dlist_element",
 	caption => "dom_html_table_caption_element *",
 	section => "dom_html_table_section_element *",
 	createCaption => "dom_html_element *",
@@ -99,6 +101,7 @@ our %no_unref = (
 	"boolean" => 1,
 	"int" => 1,
 	"unsigned int" => 1,
+	"unsigned long" => 1,
 	"List" => 1,
 	"Collection" => 1,
 );
