@@ -54,8 +54,52 @@ static void svgtiny_parse_transform_attributes(dom_element *node,
 static svgtiny_code svgtiny_add_path(float *p, unsigned int n,
 		struct svgtiny_parse_state *state);
 static void _svgtiny_parse_color(const char *s, svgtiny_colour *c,
+		struct svgtiny_parse_state_gradient *grad,
 		struct svgtiny_parse_state *state);
 
+/**
+ * Call this to ref the strings in a gradient state.
+ */
+static void svgtiny_grad_string_ref(struct svgtiny_parse_state_gradient *grad)
+{
+	if (grad->gradient_x1 != NULL) {
+		dom_string_ref(grad->gradient_x1);
+	}
+	if (grad->gradient_y1 != NULL) {
+		dom_string_ref(grad->gradient_y1);
+	}
+	if (grad->gradient_x2 != NULL) {
+		dom_string_ref(grad->gradient_x2);
+	}
+	if (grad->gradient_y2 != NULL) {
+		dom_string_ref(grad->gradient_y2);
+	}
+}
+
+/**
+ * Call this to clean up the strings in a gradient state.
+ */
+static void svgtiny_grad_string_cleanup(
+		struct svgtiny_parse_state_gradient *grad)
+{
+	if (grad->gradient_x1 != NULL) {
+		dom_string_unref(grad->gradient_x1);
+		grad->gradient_x1 = NULL;
+	}
+	if (grad->gradient_y1 != NULL) {
+		dom_string_unref(grad->gradient_y1);
+		grad->gradient_y1 = NULL;
+	}
+	if (grad->gradient_x2 != NULL) {
+		dom_string_unref(grad->gradient_x2);
+		grad->gradient_x2 = NULL;
+	}
+	if (grad->gradient_y2 != NULL) {
+		dom_string_unref(grad->gradient_y2);
+		grad->gradient_y2 = NULL;
+	}
+}
+
 /**
  * Set the local externally-stored parts of a parse state.
  * Call this in functions that made a new state on the stack.
@@ -63,18 +107,8 @@ static void _svgtiny_parse_color(const char *s, svgtiny_colour *c,
  */
 static void svgtiny_setup_state_local(struct svgtiny_parse_state *state)
 {
-	if (state->gradient_x1 != NULL) {
-		dom_string_ref(state->gradient_x1);
-	}
-	if (state->gradient_y1 != NULL) {
-		dom_string_ref(state->gradient_y1);
-	}
-	if (state->gradient_x2 != NULL) {
-		dom_string_ref(state->gradient_x2);
-	}
-	if (state->gradient_y2 != NULL) {
-		dom_string_ref(state->gradient_y2);
-	}
+	svgtiny_grad_string_ref(&(state->fill_grad));
+	svgtiny_grad_string_ref(&(state->stroke_grad));
 }
 
 /**
@@ -84,22 +118,8 @@ static void svgtiny_setup_state_local(struct svgtiny_parse_state *state)
  */
 static void svgtiny_cleanup_state_local(struct svgtiny_parse_state *state)
 {
-	if (state->gradient_x1 != NULL) {
-		dom_string_unref(state->gradient_x1);
-		state->gradient_x1 = NULL;
-	}
-	if (state->gradient_y1 != NULL) {
-		dom_string_unref(state->gradient_y1);
-		state->gradient_y1 = NULL;
-	}
-	if (state->gradient_x2 != NULL) {
-		dom_string_unref(state->gradient_x2);
-		state->gradient_x2 = NULL;
-	}
-	if (state->gradient_y2 != NULL) {
-		dom_string_unref(state->gradient_y2);
-		state->gradient_y2 = NULL;
-	}
+	svgtiny_grad_string_cleanup(&(state->fill_grad));
+	svgtiny_grad_string_cleanup(&(state->stroke_grad));
 }
 
 
@@ -152,11 +172,6 @@ svgtiny_code svgtiny_parse(struct svgtiny_diagram *diagram,
 
 	UNUSED(url);
 
-	state.gradient_x1 = NULL;
-	state.gradient_y1 = NULL;
-	state.gradient_x2 = NULL;
-	state.gradient_y2 = NULL;
-
 	parser = dom_xml_parser_create(NULL, NULL,
 				       ignore_msg, NULL, &document);
 
@@ -188,6 +203,12 @@ svgtiny_code svgtiny_parse(struct svgtiny_diagram *diagram,
 		dom_node_unref(document);
 		return svgtiny_LIBDOM_ERROR;
 	}
+        if (svg == NULL) {
+                /* no root svg element */
+                dom_node_unref(document);
+		return svgtiny_SVG_ERROR;
+        }
+
 	exc = dom_node_get_node_name(svg, &svg_name);
 	if (exc != DOM_NO_ERR) {
 		dom_node_unref(svg);
@@ -247,7 +268,6 @@ svgtiny_code svgtiny_parse(struct svgtiny_diagram *diagram,
 	state.fill = 0x000000;
 	state.stroke = svgtiny_TRANSPARENT;
 	state.stroke_width = 1;
-	state.linear_gradient_stop_count = 0;
 
 	/* parse tree */
 	code = svgtiny_parse_svg(svg, state);
@@ -402,11 +422,13 @@ svgtiny_code svgtiny_parse_path(dom_element *path,
 	dom_string *path_d_str;
 	dom_exception exc;
 	char *s, *path_d;
-	float *p;
+	float *p; /* path elemets */
+        unsigned int palloc; /* number of path elements allocated */
 	unsigned int i;
 	float last_x = 0, last_y = 0;
 	float last_cubic_x = 0, last_cubic_y = 0;
 	float last_quad_x = 0, last_quad_y = 0;
+	float subpath_first_x = 0, subpath_first_y = 0;
 
 	svgtiny_setup_state_local(&state);
 
@@ -429,16 +451,31 @@ svgtiny_code svgtiny_parse_path(dom_element *path,
 		return svgtiny_SVG_ERROR;
 	}
 
-	s = path_d = strndup(dom_string_data(path_d_str),
-			     dom_string_byte_length(path_d_str));
+        /* empty path is permitted it just disables the path */
+        palloc = dom_string_byte_length(path_d_str);
+        if (palloc == 0) {
+		svgtiny_cleanup_state_local(&state);
+		return svgtiny_OK;
+        }
+
+        /* local copy of the path data allowing in-place modification */
+	s = path_d = strndup(dom_string_data(path_d_str), palloc);
 	dom_string_unref(path_d_str);
 	if (s == NULL) {
 		svgtiny_cleanup_state_local(&state);
 		return svgtiny_OUT_OF_MEMORY;
 	}
-	/* allocate space for path: it will never have more elements than d */
-	p = malloc(sizeof p[0] * strlen(s));
-	if (!p) {
+
+        /* ensure path element allocation is sensibly bounded */
+        if (palloc < 8) {
+            palloc = 8;
+        } else if (palloc > 64) {
+            palloc = palloc / 2;
+        }
+
+	/* allocate initial space for path elements */
+	p = malloc(sizeof p[0] * palloc);
+	if (p == NULL) {
 		free(path_d);
 		svgtiny_cleanup_state_local(&state);
 		return svgtiny_OUT_OF_MEMORY;
@@ -455,6 +492,24 @@ svgtiny_code svgtiny_parse_path(dom_element *path,
 		float x, y, x1, y1, x2, y2, rx, ry, rotation, large_arc, sweep;
 		int n;
 
+                /* Ensure there is sufficient space for path elements */
+#define ALLOC_PATH_ELEMENTS(NUM_ELEMENTS)                               \
+                do {                                                    \
+                        if ((palloc - i) < NUM_ELEMENTS) {              \
+                                float *tp;                              \
+                                palloc = (palloc * 2) + (palloc / 2);   \
+                                tp = realloc(p, sizeof p[0] * palloc);  \
+                                if (tp == NULL) {                       \
+                                        free(p);                        \
+                                        free(path_d);                   \
+                                        svgtiny_cleanup_state_local(&state); \
+                                        return svgtiny_OUT_OF_MEMORY;   \
+                                }                                       \
+                                p = tp;                                 \
+                        }                                               \
+                } while(0)
+
+
 		/* moveto (M, m), lineto (L, l) (2 arguments) */
 		if (sscanf(s, " %1[MmLl] %f %f %n", command, &x, &y, &n) == 3) {
 			/*LOG(("moveto or lineto"));*/
@@ -463,11 +518,16 @@ svgtiny_code svgtiny_parse_path(dom_element *path,
 			else
 				plot_command = svgtiny_PATH_LINE;
 			do {
+                                ALLOC_PATH_ELEMENTS(3);
 				p[i++] = plot_command;
 				if ('a' <= *command) {
 					x += last_x;
 					y += last_y;
 				}
+				if (plot_command == svgtiny_PATH_MOVE) {
+					subpath_first_x = x;
+					subpath_first_y = y;
+				}
 				p[i++] = last_cubic_x = last_quad_x = last_x
 						= x;
 				p[i++] = last_cubic_y = last_quad_y = last_y
@@ -479,13 +539,19 @@ svgtiny_code svgtiny_parse_path(dom_element *path,
 		/* closepath (Z, z) (no arguments) */
 		} else if (sscanf(s, " %1[Zz] %n", command, &n) == 1) {
 			/*LOG(("closepath"));*/
+                        ALLOC_PATH_ELEMENTS(1);
+
 			p[i++] = svgtiny_PATH_CLOSE;
 			s += n;
+			last_cubic_x = last_quad_x = last_x = subpath_first_x;
+			last_cubic_y = last_quad_y = last_y = subpath_first_y;
 
 		/* horizontal lineto (H, h) (1 argument) */
 		} else if (sscanf(s, " %1[Hh] %f %n", command, &x, &n) == 2) {
 			/*LOG(("horizontal lineto"));*/
 			do {
+                                ALLOC_PATH_ELEMENTS(3);
+
 				p[i++] = svgtiny_PATH_LINE;
 				if (*command == 'h')
 					x += last_x;
@@ -499,6 +565,8 @@ svgtiny_code svgtiny_parse_path(dom_element *path,
 		} else if (sscanf(s, " %1[Vv] %f %n", command, &y, &n) == 2) {
 			/*LOG(("vertical lineto"));*/
 			do {
+                                ALLOC_PATH_ELEMENTS(3);
+
 				p[i++] = svgtiny_PATH_LINE;
 				if (*command == 'v')
 					y += last_y;
@@ -513,6 +581,8 @@ svgtiny_code svgtiny_parse_path(dom_element *path,
 				&x1, &y1, &x2, &y2, &x, &y, &n) == 7) {
 			/*LOG(("curveto"));*/
 			do {
+                                ALLOC_PATH_ELEMENTS(7);
+
 				p[i++] = svgtiny_PATH_BEZIER;
 				if (*command == 'c') {
 					x1 += last_x;
@@ -537,6 +607,8 @@ svgtiny_code svgtiny_parse_path(dom_element *path,
 				&x2, &y2, &x, &y, &n) == 5) {
 			/*LOG(("shorthand/smooth curveto"));*/
 			do {
+                                ALLOC_PATH_ELEMENTS(7);
+
 				p[i++] = svgtiny_PATH_BEZIER;
 				x1 = last_x + (last_x - last_cubic_x);
 				y1 = last_y + (last_y - last_cubic_y);
@@ -561,6 +633,8 @@ svgtiny_code svgtiny_parse_path(dom_element *path,
 				&x1, &y1, &x, &y, &n) == 5) {
 			/*LOG(("quadratic Bezier curveto"));*/
 			do {
+                                ALLOC_PATH_ELEMENTS(7);
+
 				p[i++] = svgtiny_PATH_BEZIER;
 				last_quad_x = x1;
 				last_quad_y = y1;
@@ -586,6 +660,8 @@ svgtiny_code svgtiny_parse_path(dom_element *path,
 				&x, &y, &n) == 3) {
 			/*LOG(("shorthand/smooth quadratic Bezier curveto"));*/
 			do {
+                                ALLOC_PATH_ELEMENTS(7);
+
 				p[i++] = svgtiny_PATH_BEZIER;
 				x1 = last_x + (last_x - last_quad_x);
 				y1 = last_y + (last_y - last_quad_y);
@@ -612,6 +688,8 @@ svgtiny_code svgtiny_parse_path(dom_element *path,
 				&rx, &ry, &rotation, &large_arc, &sweep,
 				&x, &y, &n) == 8) {
 			do {
+                                ALLOC_PATH_ELEMENTS(3);
+
 				p[i++] = svgtiny_PATH_LINE;
 				if (*command == 'a') {
 					x += last_x;
@@ -641,6 +719,19 @@ svgtiny_code svgtiny_parse_path(dom_element *path,
 		return svgtiny_OK;
 	}
 
+        /* resize path element array to not be over allocated */
+        if (palloc != i) {
+                float *tp;
+
+                /* try the resize, if it fails just continue to use the old
+                 * allocation
+                 */
+                tp = realloc(p, sizeof p[0] * i);
+                if (tp != NULL) {
+                        p = tp;
+                }
+        }
+
 	err = svgtiny_add_path(p, i, &state);
 
 	svgtiny_cleanup_state_local(&state);
@@ -1293,13 +1384,13 @@ void svgtiny_parse_paint_attributes(dom_element *node,
 	
 	exc = dom_element_get_attribute(node, state->interned_fill, &attr);
 	if (exc == DOM_NO_ERR && attr != NULL) {
-		svgtiny_parse_color(attr, &state->fill, state);
+		svgtiny_parse_color(attr, &state->fill, &state->fill_grad, state);
 		dom_string_unref(attr);
 	}
 
 	exc = dom_element_get_attribute(node, state->interned_stroke, &attr);
 	if (exc == DOM_NO_ERR && attr != NULL) {
-		svgtiny_parse_color(attr, &state->stroke, state);
+		svgtiny_parse_color(attr, &state->stroke, &state->stroke_grad, state);
 		dom_string_unref(attr);
 	}
 
@@ -1321,7 +1412,7 @@ void svgtiny_parse_paint_attributes(dom_element *node,
 			while (*s == ' ')
 				s++;
 			value = strndup(s, strcspn(s, "; "));
-			_svgtiny_parse_color(value, &state->fill, state);
+			_svgtiny_parse_color(value, &state->fill, &state->fill_grad, state);
 			free(value);
 		}
 		if ((s = strstr(style, "stroke:"))) {
@@ -1329,7 +1420,7 @@ void svgtiny_parse_paint_attributes(dom_element *node,
 			while (*s == ' ')
 				s++;
 			value = strndup(s, strcspn(s, "; "));
-			_svgtiny_parse_color(value, &state->stroke, state);
+			_svgtiny_parse_color(value, &state->stroke, &state->stroke_grad, state);
 			free(value);
 		}
 		if ((s = strstr(style, "stroke-width:"))) {
@@ -1352,6 +1443,7 @@ void svgtiny_parse_paint_attributes(dom_element *node,
  */
 
 static void _svgtiny_parse_color(const char *s, svgtiny_colour *c,
+		struct svgtiny_parse_state_gradient *grad,
 		struct svgtiny_parse_state *state)
 {
 	unsigned int r, g, b;
@@ -1383,19 +1475,21 @@ static void _svgtiny_parse_color(const char *s, svgtiny_colour *c,
 
 	} else if (5 < len && s[0] == 'u' && s[1] == 'r' && s[2] == 'l' &&
 			s[3] == '(') {
-		if (s[4] == '#') {
+		if (grad == NULL) {
+			*c = svgtiny_RGB(0, 0, 0);
+		} else if (s[4] == '#') {
 			id = strdup(s + 5);
 			if (!id)
 				return;
 			rparen = strchr(id, ')');
 			if (rparen)
 				*rparen = 0;
-			svgtiny_find_gradient(id, state);
+			svgtiny_find_gradient(id, grad, state);
 			free(id);
-			if (state->linear_gradient_stop_count == 0)
+			if (grad->linear_gradient_stop_count == 0)
 				*c = svgtiny_TRANSPARENT;
-			else if (state->linear_gradient_stop_count == 1)
-				*c = state->gradient_stop[0].color;
+			else if (grad->linear_gradient_stop_count == 1)
+				*c = grad->gradient_stop[0].color;
 			else
 				*c = svgtiny_LINEAR_GRADIENT;
 		}
@@ -1409,11 +1503,12 @@ static void _svgtiny_parse_color(const char *s, svgtiny_colour *c,
 }
 
 void svgtiny_parse_color(dom_string *s, svgtiny_colour *c,
+		struct svgtiny_parse_state_gradient *grad,
 		struct svgtiny_parse_state *state)
 {
-	char *ss = strndup(dom_string_data(s), dom_string_byte_length(s));
-	_svgtiny_parse_color(ss, c, state);
-	free(ss);
+	dom_string_ref(s);
+	_svgtiny_parse_color(dom_string_data(s), c, grad, state);
+	dom_string_unref(s);
 }
 
 /**
@@ -1494,23 +1589,24 @@ void svgtiny_parse_transform(char *s, float *ma, float *mb,
 		a = d = 1;
 		b = c = 0;
 		e = f = 0;
-		if (sscanf(s, "matrix (%f %f %f %f %f %f) %n",
-					&a, &b, &c, &d, &e, &f, &n) == 6)
+		n = 0;
+		if ((sscanf(s, " matrix (%f %f %f %f %f %f ) %n",
+                            &a, &b, &c, &d, &e, &f, &n) == 6) && (n > 0))
 			;
-		else if (sscanf(s, "translate (%f %f) %n",
-					&e, &f, &n) == 2)
+		else if ((sscanf(s, " translate (%f %f ) %n",
+                                 &e, &f, &n) == 2) && (n > 0))
 			;
-		else if (sscanf(s, "translate (%f) %n",
-					&e, &n) == 1)
+		else if ((sscanf(s, " translate (%f ) %n",
+                                 &e, &n) == 1) && (n > 0))
 			;
-		else if (sscanf(s, "scale (%f %f) %n",
-					&a, &d, &n) == 2)
+		else if ((sscanf(s, " scale (%f %f ) %n",
+                                 &a, &d, &n) == 2) && (n > 0))
 			;
-		else if (sscanf(s, "scale (%f) %n",
-					&a, &n) == 1)
+		else if ((sscanf(s, " scale (%f ) %n",
+                                 &a, &n) == 1) && (n > 0))
 			d = a;
-		else if (sscanf(s, "rotate (%f %f %f) %n",
-					&angle, &x, &y, &n) == 3) {
+		else if ((sscanf(s, " rotate (%f %f %f ) %n",
+                                 &angle, &x, &y, &n) == 3) && (n > 0)) {
 			angle = angle / 180 * M_PI;
 			a = cos(angle);
 			b = sin(angle);
@@ -1518,19 +1614,19 @@ void svgtiny_parse_transform(char *s, float *ma, float *mb,
 			d = cos(angle);
 			e = -x * cos(angle) + y * sin(angle) + x;
 			f = -x * sin(angle) - y * cos(angle) + y;
-		} else if (sscanf(s, "rotate (%f) %n",
-					&angle, &n) == 1) {
+		} else if ((sscanf(s, " rotate (%f ) %n",
+                                   &angle, &n) == 1) && (n > 0)) {
 			angle = angle / 180 * M_PI;
 			a = cos(angle);
 			b = sin(angle);
 			c = -sin(angle);
 			d = cos(angle);
-		} else if (sscanf(s, "skewX (%f) %n",
-					&angle, &n) == 1) {
+		} else if ((sscanf(s, " skewX (%f ) %n",
+                                   &angle, &n) == 1) && (n > 0)) {
 			angle = angle / 180 * M_PI;
 			c = tan(angle);
-		} else if (sscanf(s, "skewY (%f) %n",
-					&angle, &n) == 1) {
+		} else if ((sscanf(s, " skewY (%f ) %n",
+                                   &angle, &n) == 1) && (n > 0)) {
 			angle = angle / 180 * M_PI;
 			b = tan(angle);
 		} else
