@@ -16,33 +16,27 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/** \file
- * Content handling (implementation).
- *
- * This implementation is based on the ::handler_map array, which maps
- * ::content_type to the functions which implement that type.
+/**
+ * \file
+ * Content handling implementation.
  */
 
-#include <assert.h>
 #include <inttypes.h>
-#include <stdarg.h>
-#include <stdio.h>
 #include <stdlib.h>
-#include <string.h>
-#include <strings.h>
-#include <time.h>
+#include <nsutils/time.h>
 
-#include "utils/config.h"
-#include "content/content_protected.h"
-#include "content/hlcache.h"
-#include "image/bitmap.h"
-#include "desktop/browser.h"
-#include "utils/nsoption.h"
-
-#include "utils/http.h"
+#include "utils/utils.h"
 #include "utils/log.h"
 #include "utils/messages.h"
-#include "utils/utils.h"
+#include "netsurf/browser_window.h"
+#include "netsurf/bitmap.h"
+#include "netsurf/content.h"
+#include "desktop/knockout.h"
+#include "desktop/gui_internal.h"
+
+#include "content/content_protected.h"
+#include "content/content_debug.h"
+#include "content/hlcache.h"
 
 #define URL_FMT_SPC "%.140s"
 
@@ -72,15 +66,14 @@ static void content_convert(struct content *c);
  */
 
 nserror content__init(struct content *c, const content_handler *handler,
-		lwc_string *imime_type, const http_parameter *params,
+		lwc_string *imime_type, const struct http_parameter *params,
 		llcache_handle *llcache, const char *fallback_charset, 
 		bool quirks)
 {
 	struct content_user *user_sentinel;
 	nserror error;
 	
-	LOG(("url "URL_FMT_SPC" -> %p", 
-	     nsurl_access(llcache_handle_get_url(llcache)), c));
+	LOG("url "URL_FMT_SPC" -> %p", nsurl_access(llcache_handle_get_url(llcache)), c);
 
 	user_sentinel = calloc(1, sizeof(struct content_user));
 	if (user_sentinel == NULL) {
@@ -104,7 +97,7 @@ nserror content__init(struct content *c, const content_handler *handler,
 	c->available_width = 0;
 	c->quirks = quirks;
 	c->refresh = 0;
-	c->time = wallclock();
+	nsu_getmonotonic_ms(&c->time);
 	c->size = 0;
 	c->title = NULL;
 	c->active = 0;
@@ -223,10 +216,9 @@ static void content_update_status(struct content *c)
 				c->sub_status[0] != '\0' ? ", " : " ",
 				c->sub_status);
 	} else {
-		unsigned int time = c->time;
 		snprintf(c->status_message, sizeof (c->status_message),
 				"%s (%.1fs)", messages_get("Done"),
-				(float) time / 100);
+				(float) c->time / 1000);
 	}
 }
 
@@ -236,7 +228,8 @@ static void content_update_status(struct content *c)
  *
  * The textual status contained in the content is updated with given string.
  *
- * \param status_message  new textual status
+ * \param c The content to set status in.
+ * \param status_message new textual status
  */
 
 void content_set_status(struct content *c, const char *status_message)
@@ -279,8 +272,7 @@ void content_convert(struct content *c)
 	if (c->locked == true)
 		return;
 	
-	LOG(("content "URL_FMT_SPC" (%p)",
-			nsurl_access(llcache_handle_get_url(c->llcache)), c));
+	LOG("content "URL_FMT_SPC" (%p)", nsurl_access(llcache_handle_get_url(c->llcache)), c);
 
 	if (c->handler->data_complete != NULL) {
 		c->locked = true;
@@ -319,9 +311,12 @@ void content_set_ready(struct content *c)
 void content_set_done(struct content *c)
 {
 	union content_msg_data msg_data;
+	uint64_t now_ms;
+
+	nsu_getmonotonic_ms(&now_ms);
 
 	c->status = CONTENT_STATUS_DONE;
-	c->time = wallclock() - c->time;
+	c->time = now_ms - c->time;
 	content_update_status(c);
 	content_broadcast(c, CONTENT_MSG_DONE, msg_data);
 }
@@ -359,7 +354,7 @@ void content__reformat(struct content *c, bool background,
 	assert(c->status == CONTENT_STATUS_READY ||
 			c->status == CONTENT_STATUS_DONE);
 	assert(c->locked == false);
-	LOG(("%p %s", c, nsurl_access(llcache_handle_get_url(c->llcache))));
+
 	c->available_width = width;
 	if (c->handler->reformat != NULL) {
 
@@ -384,8 +379,7 @@ void content_destroy(struct content *c)
 	struct content_rfc5988_link *link;
 
 	assert(c);
-	LOG(("content %p %s", c,
-			nsurl_access(llcache_handle_get_url(c->llcache))));
+	LOG("content %p %s", c, nsurl_access(llcache_handle_get_url(c->llcache)));
 	assert(c->locked == false);
 
 	if (c->handler->destroy != NULL)
@@ -549,13 +543,8 @@ void content__request_redraw(struct content *c,
 	content_broadcast(c, CONTENT_MSG_REDRAW, data);
 }
 
-/**
- * Display content on screen with optional tiling.
- *
- * Calls the redraw_tile function for the content, or emulates it with the
- * redraw function if it doesn't exist.
- */
 
+/* exported interface, documented in content/content.h */
 bool content_redraw(hlcache_handle *h, struct content_redraw_data *data,
 		const struct rect *clip, const struct redraw_context *ctx)
 {
@@ -577,6 +566,74 @@ bool content_redraw(hlcache_handle *h, struct content_redraw_data *data,
 }
 
 
+/* exported interface, documented in content/content.h */
+bool content_scaled_redraw(struct hlcache_handle *h,
+		int width, int height, const struct redraw_context *ctx)
+{
+	struct content *c = hlcache_handle_get_content(h);
+	struct redraw_context new_ctx = *ctx;
+	struct rect clip;
+	struct content_redraw_data data;
+	bool plot_ok = true;
+
+	assert(c != NULL);
+
+	/* ensure it is safe to attempt redraw */
+	if (c->locked) {
+		return true;
+	}
+
+	/* ensure we have a redrawable content */
+	if (c->handler->redraw == NULL) {
+		return true;
+	}
+
+	LOG("Content %p %dx%d ctx:%p", c, width, height, ctx);
+
+	if (ctx->plot->option_knockout) {
+		knockout_plot_start(ctx, &new_ctx);
+	}
+
+	/* Set clip rectangle to required thumbnail size */
+	clip.x0 = 0;
+	clip.y0 = 0;
+	clip.x1 = width;
+	clip.y1 = height;
+
+	new_ctx.plot->clip(&clip);
+
+	/* Plot white background */
+	plot_ok &= new_ctx.plot->rectangle(clip.x0, clip.y0, clip.x1, clip.y1,
+			plot_style_fill_white);
+
+
+	/* Set up content redraw data */
+	data.x = 0;
+	data.y = 0;
+	data.width = width;
+	data.height = height;
+
+	data.background_colour = 0xFFFFFF;
+	data.repeat_x = false;
+	data.repeat_y = false;
+
+	/* Find the scale factor to use if the content has a width */
+	if (c->width) {
+		data.scale = (float)width / (float)c->width;
+	} else {
+		data.scale = 1.0;
+	}
+
+	/* Render the content */
+	plot_ok &= c->handler->redraw(c, &data, &clip, &new_ctx);
+
+	if (ctx->plot->option_knockout) {
+		knockout_plot_end();
+	}
+
+	return plot_ok;
+}
+
 /**
  * Register a user for callbacks.
  *
@@ -596,9 +653,7 @@ bool content_add_user(struct content *c,
 {
 	struct content_user *user;
 
-	LOG(("content "URL_FMT_SPC" (%p), user %p %p",
-			nsurl_access(llcache_handle_get_url(c->llcache)),
-			c, callback, pw));
+	LOG("content "URL_FMT_SPC" (%p), user %p %p", nsurl_access(llcache_handle_get_url(c->llcache)), c, callback, pw);
 	user = malloc(sizeof(struct content_user));
 	if (!user)
 		return false;
@@ -607,6 +662,9 @@ bool content_add_user(struct content *c,
 	user->next = c->user_list->next;
 	c->user_list->next = user;
 
+	if (c->handler->add_user != NULL)
+		c->handler->add_user(c);
+
 	return true;
 }
 
@@ -624,9 +682,7 @@ void content_remove_user(struct content *c,
 		void *pw)
 {
 	struct content_user *user, *next;
-	LOG(("content "URL_FMT_SPC" (%p), user %p %p",
-			nsurl_access(llcache_handle_get_url(c->llcache)), c,
-			callback, pw));
+	LOG("content "URL_FMT_SPC" (%p), user %p %p", nsurl_access(llcache_handle_get_url(c->llcache)), c, callback, pw);
 
 	/* user_list starts with a sentinel */
 	for (user = c->user_list; user->next != 0 &&
@@ -634,10 +690,14 @@ void content_remove_user(struct content *c,
 				user->next->pw == pw); user = user->next)
 		;
 	if (user->next == 0) {
-		LOG(("user not found in list"));
+		LOG("user not found in list");
 		assert(0);
 		return;
 	}
+
+	if (c->handler->remove_user != NULL)
+		c->handler->remove_user(c);
+
 	next = user->next;
 	user->next = next->next;
 	free(next);
@@ -697,7 +757,7 @@ void content_broadcast(struct content *c, content_msg msg,
 {
 	struct content_user *user, *next;
 	assert(c);
-//	LOG(("%p %s -> %d", c, c->url, msg));
+//	LOG("%p %s -> %d", c, c->url, msg);
 	for (user = c->user_list->next; user != 0; user = next) {
 		next = user->next;  /* user may be destroyed during callback */
 		if (user->callback != 0)
@@ -726,11 +786,11 @@ void content_broadcast_errorcode(struct content *c, nserror errorcode)
 /**
  * A window containing the content has been opened.
  *
- * \param  c	   content that has been opened
- * \param  bw	   browser window containing the content
- * \param  page	   content of type CONTENT_HTML containing c, or 0 if not an
+ * \param h	 handle to content that has been opened
+ * \param bw	 browser window containing the content
+ * \param page   content of type CONTENT_HTML containing h, or 0 if not an
  *		   object within a page
- * \param  params  object parameters, or 0 if not an object
+ * \param params object parameters, or 0 if not an object
  *
  * Calls the open function for the content.
  */
@@ -740,8 +800,7 @@ void content_open(hlcache_handle *h, struct browser_window *bw,
 {
 	struct content *c = hlcache_handle_get_content(h);
 	assert(c != 0);
-	LOG(("content %p %s", c,
-			nsurl_access(llcache_handle_get_url(c->llcache))));
+	LOG("content %p %s", c, nsurl_access(llcache_handle_get_url(c->llcache)));
 	if (c->handler->open != NULL)
 		c->handler->open(c, bw, page, params);
 }
@@ -757,8 +816,7 @@ void content_close(hlcache_handle *h)
 {
 	struct content *c = hlcache_handle_get_content(h);
 	assert(c != 0);
-	LOG(("content %p %s", c,
-			nsurl_access(llcache_handle_get_url(c->llcache))));
+	LOG("content %p %s", c, nsurl_access(llcache_handle_get_url(c->llcache)));
 	if (c->handler->close != NULL)
 		c->handler->close(c);
 }
@@ -795,20 +853,19 @@ char * content_get_selection(hlcache_handle *h)
 		return NULL;
 }
 
-
-void content_get_contextual_content(struct hlcache_handle *h,
-		int x, int y, struct contextual_content *data)
+/* exported interface documented in content/content.h */
+nserror content_get_contextual_content(struct hlcache_handle *h,
+		int x, int y, struct browser_window_features *data)
 {
 	struct content *c = hlcache_handle_get_content(h);
 	assert(c != 0);
 
 	if (c->handler->get_contextual_content != NULL) {
-		c->handler->get_contextual_content(c, x, y, data);
-		return;
-	} else {
-		data->object = h;
-		return;
+		return c->handler->get_contextual_content(c, x, y, data);
 	}
+
+	data->object = h;
+	return NSERROR_OK;
 }
 
 
@@ -873,26 +930,30 @@ nserror content_debug_dump(struct hlcache_handle *h, FILE *f, enum content_debug
 	return c->handler->debug_dump(c, f, op);
 }
 
+/* exported interface documented in content/content.h */
+nserror content_debug(struct hlcache_handle *h, enum content_debug op)
+{
+	struct content *c = hlcache_handle_get_content(h);
+
+	if (c == NULL) {
+		return NSERROR_BAD_PARAMETER;
+	}
+
+	if (c->handler->debug == NULL) {
+		return NSERROR_NOT_IMPLEMENTED;
+	}
+
+	return c->handler->debug(c, op);
+}
+
 
 void content_add_error(struct content *c, const char *token,
 		unsigned int line)
 {
 }
 
-bool content__set_title(struct content *c, const char *title)
-{
-	char *new_title = strdup(title);
-	if (new_title == NULL)
-		return false;
-
-	if (c->title != NULL)
-		free(c->title);
-
-	c->title = new_title;
-
-	return true;
-}
 
+/* exported interface documented in content/content.h */
 struct content_rfc5988_link *
 content_find_rfc5988_link(hlcache_handle *h, lwc_string *rel)
 {
@@ -984,12 +1045,19 @@ bool content__add_rfc5988_link(struct content *c,
 	return true;
 }
 
-/**
- * Retrieve computed type of content
- *
- * \param c  Content to retrieve type of
- * \return Computed content type
- */
+
+
+/* exported interface documented in content/content.h */
+nsurl *content_get_url(struct content *c)
+{
+	if (c == NULL)
+		return NULL;
+
+	return llcache_handle_get_url(c->llcache);
+}
+
+
+/* exported interface documented in content/content.h */
 content_type content_get_type(hlcache_handle *h)
 {
 	struct content *c = hlcache_handle_get_content(h);
@@ -1000,17 +1068,14 @@ content_type content_get_type(hlcache_handle *h)
 	return c->handler->type();
 }
 
-/**
- * Retrieve mime-type of content
- *
- * \param c  Content to retrieve mime-type of
- * \return Pointer to referenced mime-type, or NULL if not found.
- */
+
+/* exported interface documented in content/content.h */
 lwc_string *content_get_mime_type(hlcache_handle *h)
 {
 	return content__get_mime_type(hlcache_handle_get_content(h));
 }
 
+/* exported interface documented in content/content_protected.h */
 lwc_string *content__get_mime_type(struct content *c)
 {
 	if (c == NULL)
@@ -1019,31 +1084,30 @@ lwc_string *content__get_mime_type(struct content *c)
 	return lwc_string_ref(c->mime_type);
 }
 
-/**
- * Retrieve URL associated with content
- *
- * \param c  Content to retrieve URL from
- * \return Pointer to URL, or NULL if not found.
- */
-nsurl *content_get_url(struct content *c)
-{
-	if (c == NULL)
-		return NULL;
 
-	return llcache_handle_get_url(c->llcache);
+/* exported interface documented in content/content_protected.h */
+bool content__set_title(struct content *c, const char *title)
+{
+	char *new_title = strdup(title);
+	if (new_title == NULL)
+		return false;
+
+	if (c->title != NULL)
+		free(c->title);
+
+	c->title = new_title;
+
+	return true;
 }
 
-/**
- * Retrieve title associated with content
- *
- * \param c  Content to retrieve title from
- * \return Pointer to title, or NULL if not found.
- */
+
+/* exported interface documented in content/content.h */
 const char *content_get_title(hlcache_handle *h)
 {
 	return content__get_title(hlcache_handle_get_content(h));
 }
 
+/* exported interface documented in content/content_protected.h */
 const char *content__get_title(struct content *c)
 {
 	if (c == NULL)
@@ -1053,17 +1117,14 @@ const char *content__get_title(struct content *c)
 			nsurl_access(llcache_handle_get_url(c->llcache));
 }
 
-/**
- * Retrieve status of content
- *
- * \param c  Content to retrieve status of
- * \return Content status
- */
+
+/* exported interface documented in content/content.h */
 content_status content_get_status(hlcache_handle *h)
 {
 	return content__get_status(hlcache_handle_get_content(h));
 }
 
+/* exported interface documented in content/content_protected.h */
 content_status content__get_status(struct content *c)
 {
 	if (c == NULL)
@@ -1072,17 +1133,14 @@ content_status content__get_status(struct content *c)
 	return c->status;
 }
 
-/**
- * Retrieve status message associated with content
- *
- * \param c  Content to retrieve status message from
- * \return Pointer to status message, or NULL if not found.
- */
+
+/* exported interface documented in content/content.h */
 const char *content_get_status_message(hlcache_handle *h)
 {
 	return content__get_status_message(hlcache_handle_get_content(h));
 }
 
+/* exported interface documented in content/content_protected.h */
 const char *content__get_status_message(struct content *c)
 {
 	if (c == NULL)
@@ -1091,17 +1149,14 @@ const char *content__get_status_message(struct content *c)
 	return c->status_message;
 }
 
-/**
- * Retrieve width of content
- *
- * \param c  Content to retrieve width of
- * \return Content width
- */
+
+/* exported interface documented in content/content.h */
 int content_get_width(hlcache_handle *h)
 {
 	return content__get_width(hlcache_handle_get_content(h));
 }
 
+/* exported interface documented in content/content_protected.h */
 int content__get_width(struct content *c)
 {
 	if (c == NULL)
@@ -1110,17 +1165,14 @@ int content__get_width(struct content *c)
 	return c->width;
 }
 
-/**
- * Retrieve height of content
- *
- * \param c  Content to retrieve height of
- * \return Content height
- */
+
+/* exported interface documented in content/content.h */
 int content_get_height(hlcache_handle *h)
 {
 	return content__get_height(hlcache_handle_get_content(h));
 }
 
+/* exported interface documented in content/content_protected.h */
 int content__get_height(struct content *c)
 {
 	if (c == NULL)
@@ -1129,17 +1181,14 @@ int content__get_height(struct content *c)
 	return c->height;
 }
 
-/**
- * Retrieve available width of content
- *
- * \param c  Content to retrieve available width of
- * \return Available width of content
- */
+
+/* exported interface documented in content/content.h */
 int content_get_available_width(hlcache_handle *h)
 {
 	return content__get_available_width(hlcache_handle_get_content(h));
 }
 
+/* exported interface documented in content/content_protected.h */
 int content__get_available_width(struct content *c)
 {
 	if (c == NULL)
@@ -1149,18 +1198,13 @@ int content__get_available_width(struct content *c)
 }
 
 
-/**
- * Retrieve source of content
- *
- * \param c	Content to retrieve source of
- * \param size	Pointer to location to receive byte size of source
- * \return Pointer to source data
- */
+/* exported interface documented in content/content.h */
 const char *content_get_source_data(hlcache_handle *h, unsigned long *size)
 {
 	return content__get_source_data(hlcache_handle_get_content(h), size);
 }
 
+/* exported interface documented in content/content_protected.h */
 const char *content__get_source_data(struct content *c, unsigned long *size)
 {
 	const uint8_t *data;
@@ -1178,18 +1222,13 @@ const char *content__get_source_data(struct content *c, unsigned long *size)
 	return (const char *) data;
 }
 
-/**
- * Invalidate content reuse data: causes subsequent requests for content URL 
- * to query server to determine if content can be reused. This is required 
- * behaviour for forced reloads etc.
- *
- * \param c  Content to invalidate
- */
+/* exported interface documented in content/content.h */
 void content_invalidate_reuse_data(hlcache_handle *h)
 {
 	content__invalidate_reuse_data(hlcache_handle_get_content(h));
 }
 
+/* exported interface documented in content/content_protected.h */
 void content__invalidate_reuse_data(struct content *c)
 {
 	if (c == NULL || c->llcache == NULL)
@@ -1199,17 +1238,13 @@ void content__invalidate_reuse_data(struct content *c)
 	llcache_handle_invalidate_cache_data(c->llcache);
 }
 
-/**
- * Retrieve the refresh URL for a content
- *
- * \param c  Content to retrieve refresh URL from
- * \return Pointer to URL, or NULL if none
- */
+/* exported interface documented in content/content.h */
 nsurl *content_get_refresh_url(hlcache_handle *h)
 {
 	return content__get_refresh_url(hlcache_handle_get_content(h));
 }
 
+/* exported interface documented in content/content_protected.h */
 nsurl *content__get_refresh_url(struct content *c)
 {
 	if (c == NULL)
@@ -1219,17 +1254,14 @@ nsurl *content__get_refresh_url(struct content *c)
 }
 
 
-/**
- * Retrieve the bitmap contained in an image content
- *
- * \param c  Content to retrieve bitmap from
- * \return Pointer to bitmap, or NULL if none.
- */
+/* exported interface documented in content/content.h */
 struct bitmap *content_get_bitmap(hlcache_handle *h)
 {
 	return content__get_bitmap(hlcache_handle_get_content(h));
 }
 
+
+/* exported interface documented in content/content_protected.h */
 struct bitmap *content__get_bitmap(struct content *c)
 {
 	struct bitmap *bitmap = NULL;
@@ -1246,25 +1278,14 @@ struct bitmap *content__get_bitmap(struct content *c)
 }
 
 
-/**
- * Determine if a content is opaque from handle
- *
- * \param h high level cache handle to retrieve opacity from.
- * \return false if the content is not opaque or information is not
- *         known else true.
- */
+/* exported interface documented in content/content.h */
 bool content_get_opaque(hlcache_handle *h)
 {
 	return content__get_opaque(hlcache_handle_get_content(h));
 }
 
-/**
- * Determine if a content is opaque
- *
- * \param c Content to retrieve opacity from
- * \return false if the content is not opaque or information is not
- *         known else true.
- */
+
+/* exported interface documented in content/content_protected.h */
 bool content__get_opaque(struct content *c)
 {
 	bool opaque = false;
@@ -1277,7 +1298,7 @@ bool content__get_opaque(struct content *c)
 		struct bitmap *bitmap = NULL;
 		bitmap = c->handler->get_internal(c, NULL);
 		if (bitmap != NULL) { 
-			opaque = bitmap_get_opaque(bitmap);
+			opaque = guit->bitmap->get_opaque(bitmap);
 		}
 	}
 
@@ -1285,12 +1306,7 @@ bool content__get_opaque(struct content *c)
 }
 
 
-/**
- * Retrieve quirkiness of a content
- *
- * \param h  Content to examine
- * \return True if content is quirky, false otherwise
- */
+/* exported interface documented in content/content.h */
 bool content_get_quirks(hlcache_handle *h)
 {
 	struct content *c = hlcache_handle_get_content(h);
@@ -1302,18 +1318,36 @@ bool content_get_quirks(hlcache_handle *h)
 }
 
 
-/**
- * Return whether a content is currently locked
- *
- * \param c  Content to test
- * \return true iff locked, else false
- */
+/* exported interface documented in content/content.h */
+const char *content_get_encoding(hlcache_handle *h, enum content_encoding_type op)
+{
+	return content__get_encoding(hlcache_handle_get_content(h), op);
+}
 
+
+/* exported interface documented in content/content_protected.h */
+const char *content__get_encoding(struct content *c, enum content_encoding_type op)
+{
+	const char *encoding = NULL;
+
+	if ((c != NULL) &&
+	    (c->handler != NULL) &&
+	    (c->handler->get_encoding != NULL) ) {
+		encoding = c->handler->get_encoding(c, op);
+	}
+
+	return encoding;
+}
+
+
+/* exported interface documented in content/content.h */
 bool content_is_locked(hlcache_handle *h)
 {
 	return content__is_locked(hlcache_handle_get_content(h));
 }
 
+
+/* exported interface documented in content/content_protected.h */
 bool content__is_locked(struct content *c)
 {
 	return c->locked;
@@ -1322,7 +1356,7 @@ bool content__is_locked(struct content *c)
 /**
  * Retrieve the low-level cache handle for a content
  *
- * \param h  Content to retrieve from
+ * \param c Content to retrieve from
  * \return Low-level cache handle
  */
 const llcache_handle *content_get_llcache_handle(struct content *c)
@@ -1430,7 +1464,7 @@ nserror content__clone(const struct content *c, struct content *nc)
  */
 nserror content_abort(struct content *c)
 {
-	LOG(("Aborting %p", c));
+	LOG("Aborting %p", c);
 	
 	if (c->handler->stop != NULL)
 		c->handler->stop(c);
