@@ -18,7 +18,8 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/** \file
+/**
+ * \file
  * Implementation of fetching of data from a URL.
  *
  * The implementation is the fetch factory and the generic operations
@@ -30,6 +31,7 @@
  * fetches are stored in the ::queue_ring waiting for use.
  */
 
+#include <stdlib.h>
 #include <assert.h>
 #include <errno.h>
 #include <stdbool.h>
@@ -37,11 +39,8 @@
 #include <strings.h>
 #include <time.h>
 #include <libwapcaplet/libwapcaplet.h>
-#include <curl/curl.h>
 
 #include "utils/config.h"
-#include "desktop/netsurf.h"
-#include "desktop/gui_factory.h"
 #include "utils/corestrings.h"
 #include "utils/nsoption.h"
 #include "utils/log.h"
@@ -49,6 +48,8 @@
 #include "utils/nsurl.h"
 #include "utils/utils.h"
 #include "utils/ring.h"
+#include "netsurf/misc.h"
+#include "desktop/gui_internal.h"
 
 #include "content/fetch.h"
 #include "content/fetchers.h"
@@ -65,13 +66,13 @@
 
 /** Verbose fetcher logging */
 #ifdef DEBUG_FETCH_VERBOSE
-#define FETCH_LOG(x) LOG(x)
+#define FETCH_LOG(x...) LOG(x...)
 #else
-#define FETCH_LOG(x)
+#define FETCH_LOG(x...)
 #endif
 
 /** The maximum number of fetchers that can be added */
-#define MAX_FETCHERS 8
+#define MAX_FETCHERS 10
 
 /** The time in ms between polling the fetchers.
  *
@@ -158,8 +159,8 @@ static int get_fetcher_for_scheme(lwc_string *scheme)
 static bool fetch_dispatch_job(struct fetch *fetch)
 {
 	RING_REMOVE(queue_ring, fetch);
-	FETCH_LOG(("Attempting to start fetch %p, fetcher %p, url %s", fetch,
-	     fetch->fetcher_handle, nsurl_access(fetch->url)));
+	FETCH_LOG("Attempting to start fetch %p, fetcher %p, url %s", fetch,
+	     fetch->fetcher_handle, nsurl_access(fetch->url));
 
 	if (!fetchers[fetch->fetcherd].ops.start(fetch->fetcher_handle)) {
 		RING_INSERT(queue_ring, fetch); /* Put it back on the end of the queue */
@@ -217,14 +218,14 @@ static void dump_rings(void)
 	q = queue_ring;
 	if (q) {
 		do {
-			LOG(("queue_ring: %s", nsurl_access(q->url)));
+			LOG("queue_ring: %s", nsurl_access(q->url));
 			q = q->r_next;
 		} while (q != queue_ring);
 	}
 	f = fetch_ring;
 	if (f) {
 		do {
-			LOG(("fetch_ring: %s", nsurl_access(f->url)));
+			LOG("fetch_ring: %s", nsurl_access(f->url));
 			f = f->r_next;
 		} while (f != fetch_ring);
 	}
@@ -244,7 +245,7 @@ static bool fetch_dispatch_jobs(void)
 	RING_GETSIZE(struct fetch, queue_ring, all_queued);
 	RING_GETSIZE(struct fetch, fetch_ring, all_active);
 
-	FETCH_LOG(("queue_ring %i, fetch_ring %i", all_queued, all_active));
+	FETCH_LOG("queue_ring %i, fetch_ring %i", all_queued, all_active);
 	dump_rings();
 
 	while ((all_queued != 0) &&
@@ -252,12 +253,12 @@ static bool fetch_dispatch_jobs(void)
 	       fetch_choose_and_dispatch()) {
 			all_queued--;
 			all_active++;
-			FETCH_LOG(("%d queued, %d fetching",
-				   all_queued, all_active));
+			FETCH_LOG("%d queued, %d fetching",
+				   all_queued, all_active);
 	}
 
-	FETCH_LOG(("Fetch ring is now %d elements.", all_active));
-	FETCH_LOG(("Queue ring is now %d elements.", all_queued));
+	FETCH_LOG("Fetch ring is now %d elements.", all_active);
+	FETCH_LOG("Queue ring is now %d elements.", all_queued);
 
 	return (all_active > 0);
 }
@@ -267,7 +268,7 @@ static void fetcher_poll(void *unused)
 	int fetcherd;
 
 	if (fetch_dispatch_jobs()) {
-		FETCH_LOG(("Polling fetchers"));
+		FETCH_LOG("Polling fetchers");
 		for (fetcherd = 0; fetcherd < MAX_FETCHERS; fetcherd++) {
 			if (fetchers[fetcherd].refcount > 0) {
 				/* fetcher present */
@@ -276,7 +277,7 @@ static void fetcher_poll(void *unused)
 		}
 
 		/* schedule active fetchers to run again in 10ms */
-		guit->browser->schedule(SCHEDULE_TIME, fetcher_poll, NULL);
+		guit->misc->schedule(SCHEDULE_TIME, fetcher_poll, NULL);
 	}
 }
 
@@ -287,14 +288,38 @@ static void fetcher_poll(void *unused)
 /* exported interface documented in content/fetch.h */
 nserror fetcher_init(void)
 {
-	fetch_curl_register();
-	fetch_data_register();
-	fetch_file_register();
-	fetch_resource_register();
-	fetch_about_register();
-	fetch_javascript_register();
+	nserror ret;
 
-	return NSERROR_OK;
+#ifdef WITH_CURL
+	ret = fetch_curl_register();
+	if (ret != NSERROR_OK) {
+		return ret;
+	}
+#endif
+
+	ret = fetch_data_register();
+	if (ret != NSERROR_OK) {
+		return ret;
+	}
+
+	ret = fetch_file_register();
+	if (ret != NSERROR_OK) {
+		return ret;
+	}
+
+	ret = fetch_resource_register();
+	if (ret != NSERROR_OK) {
+		return ret;
+	}
+
+	ret = fetch_about_register();
+	if (ret != NSERROR_OK) {
+		return ret;
+	}
+
+	ret = fetch_javascript_register();
+
+	return ret;
 }
 
 /* exported interface documented in content/fetchers.h */
@@ -317,9 +342,7 @@ void fetcher_quit(void)
 			 * the reference count to allow the fetcher to
 			 * be stopped.
 			 */
-			LOG(("Fetcher for scheme %s still has %d active users at quit.",
-			     lwc_string_data(fetchers[fetcherd].scheme),
-			     fetchers[fetcherd].refcount));
+			LOG("Fetcher for scheme %s still has %d active users at quit.", lwc_string_data(fetchers[fetcherd].scheme), fetchers[fetcherd].refcount);
 
 			fetchers[fetcherd].refcount = 1;
 		}
@@ -359,22 +382,22 @@ fetcher_add(lwc_string *scheme, const struct fetcher_operation_table *ops)
 }
 
 /* exported interface documented in content/fetch.h */
-nserror fetcher_fdset(fd_set *read_fd_set,
+nserror
+fetch_fdset(fd_set *read_fd_set,
 		      fd_set *write_fd_set,
 		      fd_set *except_fd_set,
 		      int *maxfd_out)
 {
-	CURLMcode code;
-	int maxfd;
+	int maxfd = -1;
 	int fetcherd; /* fetcher index */
 
 	if (!fetch_dispatch_jobs()) {
-		FETCH_LOG(("No jobs"));
+		FETCH_LOG("No jobs");
 		*maxfd_out = -1;
 		return NSERROR_OK;
 	}
 
-	FETCH_LOG(("Polling fetchers"));
+	FETCH_LOG("Polling fetchers");
 
 	for (fetcherd = 0; fetcherd < MAX_FETCHERS; fetcherd++) {
 		if (fetchers[fetcherd].refcount > 0) {
@@ -386,12 +409,19 @@ nserror fetcher_fdset(fd_set *read_fd_set,
 	FD_ZERO(read_fd_set);
 	FD_ZERO(write_fd_set);
 	FD_ZERO(except_fd_set);
-	code = curl_multi_fdset(fetch_curl_multi,
-				read_fd_set,
-				write_fd_set,
-				except_fd_set,
-				&maxfd);
-	assert(code == CURLM_OK);
+
+	for (fetcherd = 0; fetcherd < MAX_FETCHERS; fetcherd++) {
+		if ((fetchers[fetcherd].refcount > 0) &&
+		    (fetchers[fetcherd].ops.fdset != NULL)) {
+			/* fetcher present */
+			int fetcher_maxfd;
+			fetcher_maxfd = fetchers[fetcherd].ops.fdset(
+				fetchers[fetcherd].scheme, read_fd_set,
+				write_fd_set, except_fd_set);
+			if (fetcher_maxfd > maxfd)
+				maxfd = fetcher_maxfd;
+		}
+	}
 
 	if (maxfd >= 0) {
 		/* change the scheduled poll to happen is a 1000ms as
@@ -407,7 +437,7 @@ nserror fetcher_fdset(fd_set *read_fd_set,
 		 * select on. All the other fetchers continue to need
 		 * polling frequently.
 		 */
-		guit->browser->schedule(FDSET_TIMEOUT, fetcher_poll, NULL);
+		guit->misc->schedule(FDSET_TIMEOUT, fetcher_poll, NULL);
 	}
 
 	*maxfd_out = maxfd;
@@ -416,7 +446,7 @@ nserror fetcher_fdset(fd_set *read_fd_set,
 }
 
 /* exported interface documented in content/fetch.h */
-struct fetch *
+nserror
 fetch_start(nsurl *url,
 	    nsurl *referer,
 	    fetch_callback callback,
@@ -426,7 +456,8 @@ fetch_start(nsurl *url,
 	    const struct fetch_multipart_data *post_multipart,
 	    bool verifiable,
 	    bool downgrade_tls,
-	    const char *headers[])
+	    const char *headers[],
+	    struct fetch **fetch_out)
 {
 	struct fetch *fetch;
 	lwc_string *scheme;
@@ -434,7 +465,7 @@ fetch_start(nsurl *url,
 
 	fetch = malloc(sizeof (*fetch));
 	if (fetch == NULL) {
-		return NULL;
+		return NSERROR_NOMEM;
 	}
 
 	/* The URL we're fetching must have a scheme */
@@ -446,10 +477,10 @@ fetch_start(nsurl *url,
 	if (fetch->fetcherd == -1) {
 		lwc_string_unref(scheme);
 		free(fetch);
-		return NULL;
+		return NSERROR_NO_FETCH_HANDLER;
 	}
 
-	FETCH_LOG(("fetch %p, url '%s'", fetch, nsurl_access(url)));
+	FETCH_LOG("fetch %p, url '%s'", fetch, nsurl_access(url));
 
 	/* construct a new fetch structure */
 	fetch->callback = callback;
@@ -526,7 +557,11 @@ fetch_start(nsurl *url,
 
 		free(fetch);
 
-		return NULL;
+
+	/** \todo The fetchers setup should return nserror and that be
+	 * passed back rather than assuming a bad url
+	 */
+		return NSERROR_BAD_URL;
 	}
 
 	/* Rah, got it, so ref the fetcher. */
@@ -537,27 +572,28 @@ fetch_start(nsurl *url,
 
 	/* Ask the queue to run. */
 	if (fetch_dispatch_jobs()) {
-		FETCH_LOG(("scheduling poll"));
+		FETCH_LOG("scheduling poll");
 		/* schedule active fetchers to run again in 10ms */
-		guit->browser->schedule(10, fetcher_poll, NULL);
+		guit->misc->schedule(10, fetcher_poll, NULL);
 	}
 
-	return fetch;
+	*fetch_out = fetch;
+	return NSERROR_OK;
 }
 
 /* exported interface documented in content/fetch.h */
 void fetch_abort(struct fetch *f)
 {
 	assert(f);
-	FETCH_LOG(("fetch %p, fetcher %p, url '%s'", f, f->fetcher_handle,
-	     nsurl_access(f->url)));
+	FETCH_LOG("fetch %p, fetcher %p, url '%s'", f, f->fetcher_handle,
+	     nsurl_access(f->url));
 	fetchers[f->fetcherd].ops.abort(f->fetcher_handle);
 }
 
 /* exported interface documented in content/fetch.h */
 void fetch_free(struct fetch *f)
 {
-	FETCH_LOG(("Freeing fetch %p, fetcher %p", f, f->fetcher_handle));
+	FETCH_LOG("Freeing fetch %p, fetcher %p", f, f->fetcher_handle);
 
 	fetchers[f->fetcherd].ops.free(f->fetcher_handle);
 
@@ -690,7 +726,7 @@ void fetch_multipart_data_destroy(struct fetch_multipart_data *list)
 		free(list->name);
 		free(list->value);
 		if (list->file) {
-			FETCH_LOG(("Freeing rawfile: %s", list->rawfile));
+			FETCH_LOG("Freeing rawfile: %s", list->rawfile);
 			free(list->rawfile);
 		}
 		free(list);
@@ -708,8 +744,8 @@ fetch_send_callback(const fetch_msg *msg, struct fetch *fetch)
 /* exported interface documented in content/fetch.h */
 void fetch_remove_from_queues(struct fetch *fetch)
 {
-	FETCH_LOG(("Fetch %p, fetcher %p can be freed",
-		   fetch, fetch->fetcher_handle));
+	FETCH_LOG("Fetch %p, fetcher %p can be freed",
+		   fetch, fetch->fetcher_handle);
 
 	/* Go ahead and free the fetch properly now */
 	if (fetch->fetch_is_active) {
@@ -725,9 +761,9 @@ void fetch_remove_from_queues(struct fetch *fetch)
 	RING_GETSIZE(struct fetch, fetch_ring, all_active);
 	RING_GETSIZE(struct fetch, queue_ring, all_queued);
 
-	LOG(("Fetch ring is now %d elements.", all_active));
+	LOG("Fetch ring is now %d elements.", all_active);
 
-	LOG(("Queue ring is now %d elements.", all_queued));
+	LOG("Queue ring is now %d elements.", all_queued);
 #endif
 }
 
@@ -735,7 +771,7 @@ void fetch_remove_from_queues(struct fetch *fetch)
 /* exported interface documented in content/fetch.h */
 void fetch_set_http_code(struct fetch *fetch, long http_code)
 {
-	FETCH_LOG(("Setting HTTP code to %ld", http_code));
+	FETCH_LOG("Setting HTTP code to %ld", http_code);
 
 	fetch->http_code = http_code;
 }
