@@ -24,14 +24,15 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "desktop/gui_factory.h"
 #include "utils/http.h"
 #include "utils/log.h"
 #include "utils/messages.h"
 #include "utils/ring.h"
 #include "utils/utils.h"
+#include "netsurf/misc.h"
+#include "netsurf/content.h"
+#include "desktop/gui_internal.h"
 
-#include "content/content.h"
 #include "content/mimesniff.h"
 #include "content/hlcache.h"
 
@@ -112,8 +113,7 @@ static void hlcache_clean(void *ignored)
 		if (entry->content == NULL)
 			continue;
 
-		if (content__get_status(entry->content) ==
-				CONTENT_STATUS_LOADING)
+		if (content__get_status(entry->content) == CONTENT_STATUS_LOADING)
 			continue;
 
 		if (content_count_users(entry->content) != 0)
@@ -146,7 +146,7 @@ static void hlcache_clean(void *ignored)
 	llcache_clean(false);
 
 	/* Re-schedule ourselves */
-	guit->browser->schedule(hlcache->params.bg_clean_time, hlcache_clean, NULL);
+	guit->misc->schedule(hlcache->params.bg_clean_time, hlcache_clean, NULL);
 }
 
 /**
@@ -191,7 +191,7 @@ static void hlcache_content_callback(struct content *c, content_msg msg,
 		error = handle->cb(handle, &event, handle->pw);
 
 	if (error != NSERROR_OK)
-		LOG(("Error in callback: %d", error));
+		LOG("Error in callback: %d", error);
 }
 
 /**
@@ -464,10 +464,12 @@ static nserror hlcache_llcache_callback(llcache_handle *handle,
 		 */
 		error = mimesniff_compute_effective_type(handle,
 				NULL, 0, false, false, &effective_type);
-		if (error == NSERROR_OK) {
+		if (error == NSERROR_OK || error == NSERROR_NOT_FOUND) {
 			error = hlcache_migrate_ctx(ctx, effective_type);
 
-			lwc_string_unref(effective_type);
+			if (effective_type != NULL) {
+				lwc_string_unref(effective_type);
+			}
 
 			return error;
 		}
@@ -475,8 +477,8 @@ static nserror hlcache_llcache_callback(llcache_handle *handle,
 		if (ctx->handle->cb != NULL) {
 			hlcache_event hlevent;
 
-			hlevent.type = CONTENT_MSG_ERROR;
-			hlevent.data.error = messages_get("BadType");
+			hlevent.type = CONTENT_MSG_ERRORCODE;
+			hlevent.data.errorcode = error;
 
 			ctx->handle->cb(ctx->handle, &hlevent, ctx->handle->pw);
 		}
@@ -535,7 +537,7 @@ hlcache_initialise(const struct hlcache_parameters *hlcache_parameters)
 	hlcache->params = *hlcache_parameters;
 
 	/* Schedule the cache cleanup */
-	guit->browser->schedule(hlcache->params.bg_clean_time, hlcache_clean, NULL);
+	guit->misc->schedule(hlcache->params.bg_clean_time, hlcache_clean, NULL);
 
 	return NSERROR_OK;
 }
@@ -544,7 +546,7 @@ hlcache_initialise(const struct hlcache_parameters *hlcache_parameters)
 void hlcache_stop(void)
 {
 	/* Remove the hlcache_clean schedule */
-	guit->browser->schedule(-1, hlcache_clean, NULL);
+	guit->misc->schedule(-1, hlcache_clean, NULL);
 }
 
 /* See hlcache.h for documentation */
@@ -560,7 +562,7 @@ void hlcache_finalise(void)
 		num_contents++;
 	}
 
-	LOG(("%d contents remain before cache drain", num_contents));
+	LOG("%d contents remain before cache drain", num_contents);
 
 	/* Drain cache */
 	do {
@@ -574,17 +576,14 @@ void hlcache_finalise(void)
 		}
 	} while (num_contents > 0 && num_contents != prev_contents);
 
-	LOG(("%d contents remaining:", num_contents));
+	LOG("%d contents remaining:", num_contents);
 	for (entry = hlcache->content_list; entry != NULL; entry = entry->next) {
 		hlcache_handle entry_handle = { entry, NULL, NULL };
 
 		if (entry->content != NULL) {
-			LOG(("	%p : %s (%d users)", entry,
-					nsurl_access(
-					hlcache_handle_get_url(&entry_handle)),
-					content_count_users(entry->content)));
+			LOG("	%p : %s (%d users)", entry, nsurl_access(hlcache_handle_get_url(&entry_handle)), content_count_users(entry->content));
 		} else {
-			LOG(("	%p", entry));
+			LOG("	%p", entry);
 		}
 	}
 
@@ -612,12 +611,12 @@ void hlcache_finalise(void)
 		hlcache->retrieval_ctx_ring = NULL;
 	}
 
-	LOG(("hit/miss %d/%d", hlcache->hit_count, hlcache->miss_count));
+	LOG("hit/miss %d/%d", hlcache->hit_count, hlcache->miss_count);
 
 	free(hlcache);
 	hlcache = NULL;
 
-	LOG(("Finalising low-level cache"));
+	LOG("Finalising low-level cache");
 	llcache_finalise();
 }
 
@@ -634,8 +633,9 @@ nserror hlcache_handle_retrieve(nsurl *url, uint32_t flags,
 	assert(cb != NULL);
 
 	ctx = calloc(1, sizeof(hlcache_retrieval_ctx));
-	if (ctx == NULL)
+	if (ctx == NULL) {
 		return NSERROR_NOMEM;
+	}
 
 	ctx->handle = calloc(1, sizeof(hlcache_handle));
 	if (ctx->handle == NULL) {
@@ -665,17 +665,17 @@ nserror hlcache_handle_retrieve(nsurl *url, uint32_t flags,
 			hlcache_llcache_callback, ctx,
 			&ctx->llcache);
 	if (error != NSERROR_OK) {
+		/* error retriving handle so free context and return error */
 		free((char *) ctx->child.charset);
 		free(ctx->handle);
 		free(ctx);
-		return error;
+	} else {
+		/* successfuly started fetch so add new context to list */
+		RING_INSERT(hlcache->retrieval_ctx_ring, ctx);
+
+		*result = ctx->handle;
 	}
-
-	RING_INSERT(hlcache->retrieval_ctx_ring, ctx);
-
-	*result = ctx->handle;
-
-	return NSERROR_OK;
+	return error;
 }
 
 /* See hlcache.h for documentation */
@@ -717,10 +717,9 @@ nserror hlcache_handle_release(hlcache_handle *handle)
 /* See hlcache.h for documentation */
 struct content *hlcache_handle_get_content(const hlcache_handle *handle)
 {
-	assert(handle != NULL);
-
-	if (handle->entry != NULL)
+	if ((handle != NULL) && (handle->entry != NULL)) {
 		return handle->entry->content;
+	}
 
 	return NULL;
 }
