@@ -17,8 +17,9 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/** \file
- * Browser history tree (implementation).
+/**
+ * \file
+ * Browser history tree implementation.
  */
 
 #include <assert.h>
@@ -27,20 +28,18 @@
 #include <string.h>
 #include <time.h>
 
-#include "content/content.h"
+#include "utils/log.h"
+#include "utils/utils.h"
+#include "netsurf/layout.h"
+#include "netsurf/plotters.h"
+#include "netsurf/content.h"
 #include "content/hlcache.h"
 #include "content/urldb.h"
-#include "css/css.h"
+#include "netsurf/bitmap.h"
+
+#include "desktop/gui_internal.h"
 #include "desktop/browser_history.h"
 #include "desktop/browser_private.h"
-#include "desktop/plotters.h"
-#include "desktop/thumbnail.h"
-#include "image/bitmap.h"
-#include "render/font.h"
-#include "utils/log.h"
-#include "utils/nsurl.h"
-#include "utils/utils.h"
-
 
 #define WIDTH 100
 #define HEIGHT 86
@@ -84,15 +83,13 @@ struct history {
 /**
  * Clone a history entry
  *
- * \param  history  opaque history structure, as returned by history_create()
- * \param  start    entry to clone
- *
- * \return  a cloned history entry, or 0 on error
+ * \param history opaque history structure, as returned by history_create()
+ * \param entry   entry to clone
+ * \return A cloned history entry or NULL on error
  */
-
-static struct history_entry *browser_window_history__clone_entry(
-		struct history *history,
-		struct history_entry *entry)
+static struct history_entry *
+browser_window_history__clone_entry(struct history *history,
+				    struct history_entry *entry)
 {
 	struct history_entry *child;
 	struct history_entry *new_child;
@@ -179,12 +176,11 @@ static void browser_window_history__free_entry(struct history_entry *entry)
  * \param  entry    subtree to position
  * \param  x        x position for entry
  * \param  y        smallest available y
- * \param  shuffle  shuffle layout
  * \return  greatest y used by subtree
  */
 
 static int browser_window_history__layout_subtree(struct history *history,
-		struct history_entry *entry, int x, int y, bool shuffle)
+		struct history_entry *entry, int x, int y)
 {
 	struct history_entry *child;
 	int y1 = y;
@@ -195,17 +191,13 @@ static int browser_window_history__layout_subtree(struct history *history,
 	if (!entry->forward) {
 		entry->x = x;
 		entry->y = y;
-		if (shuffle) {
-			entry->x = rand() % 600;
-			entry->y = rand() % 400;
-		}
 		return y + HEIGHT;
 	}
 
 	/* layout child subtrees below each other */
 	for (child = entry->forward; child; child = child->next) {
 		y1 = browser_window_history__layout_subtree(history, child,
-				x + WIDTH + RIGHT_MARGIN, y1, shuffle);
+				x + WIDTH + RIGHT_MARGIN, y1);
 		if (child->next)
 			y1 += BOTTOM_MARGIN;
 	}
@@ -213,10 +205,6 @@ static int browser_window_history__layout_subtree(struct history *history,
 	/* place ourselves in the middle */
 	entry->x = x;
 	entry->y = (y + y1) / 2 - HEIGHT / 2;
-	if (shuffle) {
-		entry->x = rand() % 600;
-		entry->y = rand() % 400;
-	}
 
 	return y1;
 }
@@ -232,10 +220,6 @@ static int browser_window_history__layout_subtree(struct history *history,
 
 static void browser_window_history__layout(struct history *history)
 {
-	time_t t = time(0);
-	struct tm *tp = localtime(&t);
-	bool shuffle = tp->tm_mon == 3 && tp->tm_mday == 1;
-
 	if (!history)
 		return;
 
@@ -243,14 +227,10 @@ static void browser_window_history__layout(struct history *history)
 	if (history->start)
 		history->height = browser_window_history__layout_subtree(
 				history, history->start,
-				RIGHT_MARGIN / 2, BOTTOM_MARGIN / 2,
-				shuffle);
+				RIGHT_MARGIN / 2, BOTTOM_MARGIN / 2);
 	else
 		history->height = 0;
-	if (shuffle) {
-		history->width = 600 + WIDTH;
-		history->height = 400 + HEIGHT;
-	}
+
 	history->width += RIGHT_MARGIN / 2;
 	history->height += BOTTOM_MARGIN / 2;
 }
@@ -258,12 +238,19 @@ static void browser_window_history__layout(struct history *history)
 /**
  * Recursively redraw a history_entry.
  *
- * \param  history        history containing the entry
- * \param  history_entry  entry to render
- * \param ctx		  current redraw context
+ * \param history history containing the entry
+ * \param entry entry to render
+ * \param x0 area top left x coordinate
+ * \param y0 area top left y coordinate
+ * \param x1 area bottom right x coordinate
+ * \param y1 area bottom right y coordinate
+ * \param x window x offset
+ * \param y window y offset
+ * \param clip clip redraw
+ * \param ctx     current redraw context
  */
-
-static bool browser_window_history__redraw_entry(struct history *history,
+static bool
+browser_window_history__redraw_entry(struct history *history,
 		struct history_entry *entry,
 		int x0, int y0, int x1, int y1,
 		int x, int y, bool clip,
@@ -284,6 +271,7 @@ static bool browser_window_history__redraw_entry(struct history *history,
             .stroke_width = entry == history->current ? 3 : 1,
         };
 	plot_font_style_t fstyle = *plot_style_font;
+	nserror res;
 
 	if (clip) {
 		struct rect rect;
@@ -291,24 +279,33 @@ static bool browser_window_history__redraw_entry(struct history *history,
 		rect.y0 = y0 + yoffset;
 		rect.x1 = x1 + xoffset;
 		rect.y1 = y1 + yoffset;
-		if(!plot->clip(&rect))
+		if (!plot->clip(&rect)) {
 			return false;
+		}
+	}
+
+	/* Only attempt to plot bitmap if it is present */
+	if (entry->bitmap != NULL) {
+		plot->bitmap(entry->x + xoffset,
+			     entry->y + yoffset,
+			     WIDTH, HEIGHT,
+			     entry->bitmap, 0xffffff, 0);
 	}
 
-	if (!plot->bitmap(entry->x + xoffset, entry->y + yoffset, WIDTH, HEIGHT,
-			entry->bitmap, 0xffffff, 0))
-		return false;
 	if (!plot->rectangle(entry->x - 1 + xoffset, 
                             entry->y - 1 + yoffset,
                             entry->x + xoffset + WIDTH, 
                             entry->y + yoffset + HEIGHT,
-                            &pstyle_history_rect))
+			     &pstyle_history_rect)) {
 		return false;
+	}
 
-	if (!nsfont.font_position_in_string(plot_style_font, entry->page.title,
-			strlen(entry->page.title), WIDTH,
-			&char_offset, &actual_x))
+	res = guit->layout->position(plot_style_font, entry->page.title,
+				     strlen(entry->page.title), WIDTH,
+				     &char_offset, &actual_x);
+	if (res != NSERROR_OK) {
 		return false;
+	}
 
 	fstyle.background = HISTORY_COLOUR_BACKGROUND;
 	fstyle.foreground = c;
@@ -413,14 +410,7 @@ static bool browser_window_history__enumerate_entry(
 /* -------------------------------------------------------------------------- */
 
 
-/**
- * Create a new history tree for a browser window window.
- *
- * \param  bw	browser window to create history for.
- *
- * \return  NSERROR_OK or appropriate error otherwise
- */
-
+/* exported interface documented in desktop/browser_history.h */
 nserror browser_window_history_create(struct browser_window *bw)
 {
 	struct history *history;
@@ -428,10 +418,10 @@ nserror browser_window_history_create(struct browser_window *bw)
 	bw->history = NULL;
 
 	history = calloc(1, sizeof *history);
-	if (!history) {
-		warn_user("NoMemory", 0);
+	if (history == NULL) {
 		return NSERROR_NOMEM;
 	}
+
 	history->width = RIGHT_MARGIN / 2;
 	history->height = BOTTOM_MARGIN / 2;
 
@@ -440,15 +430,7 @@ nserror browser_window_history_create(struct browser_window *bw)
 }
 
 
-/**
- * Clone a bw's history tree for new bw
- *
- * \param  existing	browser window with history to clone.
- * \param  clone	browser window to make cloned history for.
- *
- * \return  NSERROR_OK or appropriate error otherwise
- */
-
+/* exported interface documented in desktop/browser_history.h */
 nserror browser_window_history_clone(const struct browser_window *existing,
 		struct browser_window *clone)
 {
@@ -472,8 +454,7 @@ nserror browser_window_history_clone(const struct browser_window *existing,
 	new_history->start = browser_window_history__clone_entry(new_history,
 			new_history->start);
 	if (!new_history->start) {
-		LOG(("Insufficient memory to clone history"));
-		warn_user("NoMemory", 0);
+		LOG("Insufficient memory to clone history");
 		browser_window_history_destroy(clone);
 		clone->history = NULL;
 		return NSERROR_NOMEM;
@@ -483,18 +464,8 @@ nserror browser_window_history_clone(const struct browser_window *existing,
 }
 
 
-/**
- * Insert a url into the history tree.
- *
- * \param  bw       browser window with history object
- * \param  content  content to add to history
- * \param  frag_id  fragment identifier, or NULL.
- *
- * The page is added after the current entry and becomes current.
- */
-
-
-void browser_window_history_add(struct browser_window *bw,
+/* exported interface documented in desktop/browser_history.h */
+nserror browser_window_history_add(struct browser_window *bw,
 		struct hlcache_handle *content, lwc_string *frag_id)
 {
 	struct history *history;
@@ -502,6 +473,7 @@ void browser_window_history_add(struct browser_window *bw,
 	nsurl *nsurl = hlcache_handle_get_url(content);
 	char *title;
 	struct bitmap *bitmap;
+	nserror ret;
 
 	assert(bw);
 	assert(bw->history);
@@ -511,14 +483,14 @@ void browser_window_history_add(struct browser_window *bw,
 
 	/* allocate space */
 	entry = malloc(sizeof *entry);
-	if (entry == NULL)
-		return;
+	if (entry == NULL) {
+		return NSERROR_NOMEM;
+	}
 
 	title = strdup(content_get_title(content));
 	if (title == NULL) {
-		warn_user("NoMemory", 0);
 		free(entry);
-		return;
+		return NSERROR_NOMEM;
 	}
 
 	entry->page.url = nsurl_ref(nsurl);
@@ -546,35 +518,38 @@ void browser_window_history_add(struct browser_window *bw,
 	/* if we have a thumbnail, don't update until the page has finished
 	 * loading */
 	bitmap = urldb_get_thumbnail(nsurl);
-	if (!bitmap) {
-		LOG(("Creating thumbnail for %s", nsurl_access(nsurl)));
-		bitmap = bitmap_create(WIDTH, HEIGHT,
-				BITMAP_NEW | BITMAP_CLEAR_MEMORY |
-				BITMAP_OPAQUE);
-		if (!bitmap) {
-			warn_user("NoMemory", 0);
-			return;
-		}
-		if (thumbnail_create(content, bitmap, nsurl) == false) {
-			/* Thumbnailing failed. Ignore it silently */
-			bitmap_destroy(bitmap);
-			bitmap = NULL;
+	if (bitmap == NULL) {
+		LOG("Creating thumbnail for %s", nsurl_access(nsurl));
+		bitmap = guit->bitmap->create(WIDTH, HEIGHT,
+					      BITMAP_NEW | BITMAP_CLEAR_MEMORY |
+					      BITMAP_OPAQUE);
+		if (bitmap != NULL) {
+			ret = guit->bitmap->render(bitmap, content);
+			if (ret == NSERROR_OK) {
+				/* Successful thumbnail so register it
+				 * with the url.
+				 */
+				urldb_set_thumbnail(nsurl, bitmap);
+			} else {
+				/* Thumbnailing failed. Ignore it
+				 * silently but clean up bitmap.
+				 */
+				LOG("Thumbnail renderfailed");
+				guit->bitmap->destroy(bitmap);
+				bitmap = NULL;
+			}
 		}
 	}
 	entry->bitmap = bitmap;
 
 	browser_window_history__layout(history);
+
+	return NSERROR_OK;
 }
 
 
-/**
- * Update the thumbnail for the current entry.
- *
- * \param  history  opaque history structure, as returned by history_create()
- * \param  content  content for current entry
- */
-
-void browser_window_history_update(struct browser_window *bw,
+/* exported interface documented in desktop/browser_history.h */
+nserror browser_window_history_update(struct browser_window *bw,
 		struct hlcache_handle *content)
 {
 	struct history *history;
@@ -584,32 +559,29 @@ void browser_window_history_update(struct browser_window *bw,
 
 	history = bw->history;
 
-	if (!history || !history->current || !history->current->bitmap)
-		return;
+	if (!history || !history->current || !history->current->bitmap) {
+		return NSERROR_INVALID;
+	}
 
 	assert(history->current->page.url);
 	assert(history->current->page.title);
 
 	title = strdup(content_get_title(content));
-	if (!title) {
-		warn_user("NoMemory", 0);
-		return;
+	if (title == NULL) {
+		return NSERROR_NOMEM;
 	}
 
-	assert(title);
 	free(history->current->page.title);
 	history->current->page.title = title;
 
-	thumbnail_create(content, history->current->bitmap, NULL);
+	guit->bitmap->render(history->current->bitmap, content);
+
+	return NSERROR_OK;
 }
 
 
-/**
- * Free a history structure.
- *
- * \param  history  opaque history structure, as returned by history_create()
- */
 
+/* exported interface documented in desktop/browser_history.h */
 void browser_window_history_destroy(struct browser_window *bw)
 {
 	assert(bw != NULL);
@@ -624,48 +596,34 @@ void browser_window_history_destroy(struct browser_window *bw)
 }
 
 
-/**
- * Go back in the history.
- *
- * \param  bw       browser window
- * \param  history  history of the window
- * \param  new_window  whether to open in new window
- */
 
-void browser_window_history_back(struct browser_window *bw, bool new_window)
+/* exported interface documented in desktop/browser_history.h */
+nserror browser_window_history_back(struct browser_window *bw, bool new_window)
 {
 	if (!bw || !bw->history || !bw->history->current ||
-			!bw->history->current->back)
-		return;
-	browser_window_history_go(bw, bw->history->current->back, new_window);
+	    !bw->history->current->back) {
+		return NSERROR_BAD_PARAMETER;
+	}
+	return browser_window_history_go(bw, bw->history->current->back,
+					 new_window);
 }
 
 
-/**
- * Go forward in the history.
- *
- * \param  bw       browser window
- * \param  history  history of the window
- * \param  new_window  whether to open in new window
- */
 
-void browser_window_history_forward(struct browser_window *bw, bool new_window)
+/* exported interface documented in desktop/browser_history.h */
+nserror browser_window_history_forward(struct browser_window *bw,
+				       bool new_window)
 {
 	if (!bw || !bw->history || !bw->history->current ||
-			!bw->history->current->forward_pref)
-		return;
-	browser_window_history_go(bw, bw->history->current->forward_pref,
-			new_window);
+	    !bw->history->current->forward_pref) {
+		return NSERROR_BAD_PARAMETER;
+	}
+	return browser_window_history_go(bw, bw->history->current->forward_pref,
+					 new_window);
 }
 
 
-/**
- * Check whether it is pssible to go back in the history.
- *
- * \param  history  history of the window
- * \return true if the history can go back, false otherwise
- */
-
+/* exported interface documented in desktop/browser_history.h */
 bool browser_window_history_back_available(struct browser_window *bw)
 {
 	return (bw && bw->history && bw->history->current &&
@@ -673,13 +631,7 @@ bool browser_window_history_back_available(struct browser_window *bw)
 }
 
 
-/**
- * Check whether it is pssible to go forwards in the history.
- *
- * \param  history  history of the window
- * \return true if the history can go forwards, false otherwise
- */
-
+/* exported interface documented in desktop/browser_history.h */
 bool browser_window_history_forward_available(struct browser_window *bw)
 {
 	return (bw && bw->history && bw->history->current &&
@@ -687,8 +639,8 @@ bool browser_window_history_forward_available(struct browser_window *bw)
 }
 
 
-/* Documented in local_history.h */
-void browser_window_history_go(struct browser_window *bw,
+/* exported interface documented in desktop/browser_history.h */
+nserror browser_window_history_go(struct browser_window *bw,
 		struct history_entry *entry, bool new_window)
 {
 	struct history *history;
@@ -704,8 +656,7 @@ void browser_window_history_go(struct browser_window *bw,
 				entry->page.frag_id, &url);
 
 		if (error != NSERROR_OK) {
-			warn_user("NoMemory", 0);
-			return;
+			return error;
 		}
 	} else {
 		url = nsurl_ref(entry->page.url);
@@ -718,28 +669,19 @@ void browser_window_history_go(struct browser_window *bw,
 		error = browser_window_create(BW_CREATE_CLONE,
 				url, NULL, bw, NULL);
 		history->current = current;
-		if (error != NSERROR_OK) {
-			nsurl_unref(url);
-			return;
-		}
 	} else {
 		history->current = entry;
-		browser_window_navigate(bw, url, NULL,
+		error = browser_window_navigate(bw, url, NULL,
 				BW_NAVIGATE_NONE, NULL, NULL, NULL);
 	}
 
 	nsurl_unref(url);
+
+	return error;
 }
 
 
-/**
- * Get the dimensions of a history.
- *
- * \param  history  history to measure
- * \param  width    updated to width
- * \param  height   updated to height
- */
-
+/* exported interface documented in desktop/browser_history.h */
 void browser_window_history_size(struct browser_window *bw,
 		int *width, int *height)
 {
@@ -751,13 +693,7 @@ void browser_window_history_size(struct browser_window *bw,
 }
 
 
-/**
- * Redraw a history.
- *
- * \param history	history to render
- * \param ctx		current redraw context
- */
-
+/* exported interface documented in desktop/browser_history.h */
 bool browser_window_history_redraw(struct browser_window *bw,
 		const struct redraw_context *ctx)
 {
@@ -766,25 +702,19 @@ bool browser_window_history_redraw(struct browser_window *bw,
 	assert(bw != NULL);
 	history = bw->history;
 
+	if (history == NULL) {
+		LOG("Attempt to draw NULL history.");
+		return false;
+	}
+
 	if (!history->start)
 		return true;
 	return browser_window_history__redraw_entry(history, history->start,
 			0, 0, 0, 0, 0, 0, false, ctx);
 }
 
-/**
- * Redraw part of a history.
- *
- * \param  history	history to render
- * \param  x0		left X co-ordinate of redraw area
- * \param  y0		top Y co-ordinate of redraw area
- * \param  x1		right X co-ordinate of redraw area
- * \param  y1		lower Y co-ordinate of redraw area
- * \param  x		start X co-ordinate on plot canvas
- * \param  y		start Y co-ordinate on plot canvas
- * \param ctx		current redraw context
- */
 
+/* exported interface documented in desktop/browser_history.h */
 bool browser_window_history_redraw_rectangle(struct browser_window *bw,
 	int x0, int y0, int x1, int y1,
 	int x, int y, const struct redraw_context *ctx)
@@ -801,16 +731,7 @@ bool browser_window_history_redraw_rectangle(struct browser_window *bw,
 }
 
 
-/**
- * Handle a mouse click in a history.
- *
- * \param  bw          browser window containing history
- * \param  x           click coordinate
- * \param  y           click coordinate
- * \param  new_window  open a new window instead of using bw
- * \return  true if action was taken, false if click was not on an entry
- */
-
+/* exported interface documented in desktop/browser_history.h */
 bool browser_window_history_click(struct browser_window *bw,
 		int x, int y, bool new_window)
 {
@@ -832,15 +753,7 @@ bool browser_window_history_click(struct browser_window *bw,
 }
 
 
-/**
- * Determine the URL of the entry at a position.
- *
- * \param  history  history to search
- * \param  x        coordinate
- * \param  y        coordinate
- * \return  URL, or 0 if no entry at (x, y)
- */
-
+/* exported interface documented in desktop/browser_history.h */
 const char *browser_window_history_position_url(struct browser_window *bw,
 		int x, int y)
 {
@@ -857,7 +770,8 @@ const char *browser_window_history_position_url(struct browser_window *bw,
 	return nsurl_access(entry->page.url);
 }
 
-/* Documented in local_history.h */
+
+/* exported interface documented in desktop/browser_history.h */
 void browser_window_history_enumerate_forward(const struct browser_window *bw, 
 		browser_window_history_enumerate_cb cb, void *user_data)
 {
@@ -874,7 +788,8 @@ void browser_window_history_enumerate_forward(const struct browser_window *bw,
 	}
 }
 
-/* Documented in local_history.h */
+
+/* exported interface documented in desktop/browser_history.h */
 void browser_window_history_enumerate_back(const struct browser_window *bw, 
 		browser_window_history_enumerate_cb cb, void *user_data)
 {
@@ -890,7 +805,8 @@ void browser_window_history_enumerate_back(const struct browser_window *bw,
 	}
 }
 
-/* Documented in local_history.h */
+
+/* exported interface documented in desktop/browser_history.h */
 void browser_window_history_enumerate(const struct browser_window *bw,
 		browser_window_history_enumerate_cb cb, void *user_data)
 {
@@ -900,21 +816,24 @@ void browser_window_history_enumerate(const struct browser_window *bw,
 			bw->history->start, cb, user_data);
 }
 
-/* Documented in local_history.h */
+
+/* exported interface documented in desktop/browser_history.h */
 const char *browser_window_history_entry_get_url(
 		const struct history_entry *entry)
 {
 	return nsurl_access(entry->page.url);
 }
 
-/* Documented in local_history.h */
+
+/* exported interface documented in desktop/browser_history.h */
 const char *browser_window_history_entry_get_fragment_id(
 		const struct history_entry *entry)
 {
 	return (entry->page.frag_id) ? lwc_string_data(entry->page.frag_id) : 0;
 }
 
-/* Documented in local_history.h */
+
+/* exported interface documented in desktop/browser_history.h */
 const char *browser_window_history_entry_get_title(
 		const struct history_entry *entry)
 {
