@@ -19,17 +19,19 @@
 
 #include <stdlib.h>
 #include <string.h>
+#include <time.h>
 
 #include "utils/messages.h"
 #include "utils/utils.h"
 #include "utils/utf8.h"
 #include "utils/libdom.h"
 #include "utils/log.h"
+#include "utils/nsurl.h"
 #include "content/urldb.h"
 
 #include "desktop/global_history.h"
 #include "desktop/treeview.h"
-#include "desktop/browser.h"
+#include "netsurf/browser_window.h"
 
 #define N_DAYS 28
 #define N_SEC_PER_DAY (60 * 60 * 24)
@@ -135,36 +137,45 @@ static nserror global_history_create_dir(enum global_history_folders f)
 	case GH_TODAY:
 		label = "DateToday";
 		break;
+
 	case GH_YESTERDAY:
 		label = "DateYesterday";
 		break;
+
 	case GH_2_DAYS_AGO:
 		label = "Date2Days";
 		break;
+
 	case GH_3_DAYS_AGO:
 		label = "Date3Days";
 		break;
+
 	case GH_4_DAYS_AGO:
 		label = "Date4Days";
 		break;
+
 	case GH_5_DAYS_AGO:
 		label = "Date5Days";
 		break;
+
 	case GH_6_DAYS_AGO:
 		label = "Date6Days";
 		break;
+
 	case GH_LAST_WEEK:
 		label = "Date1Week";
 		break;
+
 	case GH_2_WEEKS_AGO:
 		label = "Date2Week";
 		break;
+
 	case GH_3_WEEKS_AGO:
 		label = "Date3Week";
 		break;
+
 	default:
-		assert(0);
-		break;
+		return NSERROR_BAD_PARAMETER;
 	}
 
 	label = messages_get(label);
@@ -245,7 +256,7 @@ static inline nserror global_history_get_parent_treeview_node(
  * Set a global history entry's data from the url_data.
  *
  * \param e		Global history entry to set up
- * \param url_data	Data associated with entry's URL
+ * \param data	Data associated with entry's URL
  * \return NSERROR_OK on success, appropriate error otherwise
  */
 static nserror global_history_create_treeview_field_data(
@@ -525,7 +536,7 @@ static nserror global_history_initialise_entry_fields(void)
 		goto error;
 	}
 
-	gh_ctx.fields[GH_URL].flags = TREE_FLAG_NONE;
+	gh_ctx.fields[GH_URL].flags = TREE_FLAG_COPY_TEXT;
 	label = "TreeviewLabelURL";
 	label = messages_get(label);
 	if (lwc_intern_string(label, strlen(label),
@@ -585,7 +596,7 @@ static nserror global_history_initialise_time(void)
 	/* get the current time */
 	t = time(NULL);
 	if (t == -1) {
-		LOG(("time info unaviable"));
+		LOG("time info unaviable");
 		return NSERROR_UNKNOWN;
 	}
 
@@ -596,7 +607,7 @@ static nserror global_history_initialise_time(void)
 	full_time->tm_hour = 0;
 	t = mktime(full_time);
 	if (t == -1) {
-		LOG(("mktime failed"));
+		LOG("mktime failed");
 		return NSERROR_UNKNOWN;
 	}
 
@@ -661,10 +672,12 @@ static nserror global_history_tree_node_folder_cb(
 
 	return NSERROR_OK;
 }
-static nserror global_history_tree_node_entry_cb(
-		struct treeview_node_msg msg, void *data)
+
+static nserror
+global_history_tree_node_entry_cb(struct treeview_node_msg msg, void *data)
 {
 	struct global_history_entry *e = data;
+	nserror ret = NSERROR_OK;
 
 	switch (msg.msg) {
 	case TREE_MSG_NODE_DELETE:
@@ -678,10 +691,8 @@ static nserror global_history_tree_node_entry_cb(
 
 	case TREE_MSG_NODE_LAUNCH:
 	{
-		nserror error;
 		struct browser_window *existing = NULL;
-		enum browser_window_create_flags flags =
-				BW_CREATE_HISTORY;
+		enum browser_window_create_flags flags = BW_CREATE_HISTORY;
 
 		/* TODO: Set existing to window that new tab appears in */
 
@@ -693,16 +704,14 @@ static nserror global_history_tree_node_entry_cb(
 			/* TODO: flags ^= BW_CREATE_TAB; */
 		}
 
-		error = browser_window_create(flags, e->url, NULL,
+		ret = browser_window_create(flags, e->url, NULL,
 				existing, NULL);
-		if (error != NSERROR_OK) {
-			warn_user(messages_get_errorcode(error), 0);
-		}
 	}
 		break;
 	}
-	return NSERROR_OK;
+	return ret;
 }
+
 struct treeview_callback_table gh_tree_cb_t = {
 	.folder = global_history_tree_node_folder_cb,
 	.entry = global_history_tree_node_entry_cb
@@ -715,7 +724,12 @@ nserror global_history_init(struct core_window_callback_table *cw_t,
 {
 	nserror err;
 
-	LOG(("Loading global history"));
+	err = treeview_init();
+	if (err != NSERROR_OK) {
+		return err;
+	}
+
+	LOG("Loading global history");
 
 	/* Init. global history treeview time */
 	err = global_history_initialise_time();
@@ -771,7 +785,7 @@ nserror global_history_init(struct core_window_callback_table *cw_t,
 	/* Inform client of window height */
 	treeview_get_height(gh_ctx.tree);
 
-	LOG(("Loaded global history"));
+	LOG("Loaded global history");
 
 	return NSERROR_OK;
 }
@@ -783,7 +797,7 @@ nserror global_history_fini(void)
 	int i;
 	nserror err;
 
-	LOG(("Finalising global history"));
+	LOG("Finalising global history");
 
 	gh_ctx.built = false;
 
@@ -795,7 +809,12 @@ nserror global_history_fini(void)
 		if (gh_ctx.fields[i].field != NULL)
 			lwc_string_unref(gh_ctx.fields[i].field);
 
-	LOG(("Finalised global history"));
+	err = treeview_fini();
+	if (err != NSERROR_OK) {
+		return err;
+	}
+
+	LOG("Finalised global history");
 
 	return err;
 }
@@ -812,7 +831,7 @@ nserror global_history_add(nsurl *url)
 
 	data = urldb_get_url_data(url);
 	if (data == NULL) {
-		LOG(("Can't add URL to history that's not present in urldb."));
+		LOG("Can't add URL to history that's not present in urldb.");
 		return NSERROR_BAD_PARAMETER;
 	}
 
@@ -938,9 +957,9 @@ void global_history_mouse_action(browser_mouse_state mouse, int x, int y)
 
 
 /* Exported interface, documented in global_history.h */
-void global_history_keypress(uint32_t key)
+bool global_history_keypress(uint32_t key)
 {
-	treeview_keypress(gh_ctx.tree, key);
+	return treeview_keypress(gh_ctx.tree, key);
 }
 
 
