@@ -16,15 +16,33 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include <stdlib.h>
+#include <stdint.h>
+#include <stdbool.h>
+
+#include "utils/config.h"
+#include "utils/errors.h"
+#include "utils/file.h"
+#include "netsurf/bitmap.h"
 #include "content/hlcache.h"
 #include "content/backing_store.h"
 
+#include "desktop/save_pdf.h"
 #include "desktop/download.h"
 #include "desktop/searchweb.h"
-#include "desktop/gui_factory.h"
-#include "utils/file.h"
+#include "netsurf/download.h"
+#include "netsurf/fetch.h"
+#include "netsurf/misc.h"
+#include "netsurf/window.h"
+#include "netsurf/search.h"
+#include "netsurf/clipboard.h"
+#include "netsurf/utf8.h"
+#include "netsurf/layout.h"
+#include "netsurf/netsurf.h"
 
-/** The global interface table */
+/**
+ * The global interface table.
+ */
 struct netsurf_table *guit = NULL;
 
 
@@ -32,8 +50,9 @@ static void gui_default_window_set_title(struct gui_window *g, const char *title
 {
 }
 
-static void gui_default_window_set_url(struct gui_window *g, const char *url)
+static nserror gui_default_window_set_url(struct gui_window *g, struct nsurl *url)
 {
+	return NSERROR_OK;
 }
 
 static void gui_default_window_start_throbber(struct gui_window *g)
@@ -51,10 +70,11 @@ static bool gui_default_window_drag_start(struct gui_window *g,
 	return true;
 }
 
-static void gui_default_window_save_link(struct gui_window *g,
-					 const char *url,
+static nserror gui_default_window_save_link(struct gui_window *g,
+					 nsurl *url,
 					 const char *title)
 {
+	return NSERROR_OK;
 }
 
 static void gui_default_window_set_icon(struct gui_window *g,
@@ -99,6 +119,11 @@ static void gui_default_window_remove_caret(struct gui_window *g)
 {
 }
 
+static void gui_default_window_create_form_select_menu(struct gui_window *g,
+						struct form_control *control)
+{
+}
+
 static void gui_default_window_file_gadget_open(struct gui_window *g,
 						hlcache_handle *hl,
 						struct form_control *gadget)
@@ -202,6 +227,10 @@ static nserror verify_window_register(struct gui_window_table *gwt)
 	if (gwt->scroll_start == NULL) {
 		gwt->scroll_start = gui_default_window_scroll_start;
 	}
+	if (gwt->create_form_select_menu == NULL) {
+		gwt->create_form_select_menu =
+				gui_default_window_create_form_select_menu;
+	}
 	if (gwt->file_gadget_open == NULL) {
 		gwt->file_gadget_open = gui_default_window_file_gadget_open;
 	}
@@ -241,6 +270,12 @@ static void gui_default_download_done(struct gui_download_window *dw)
 {
 }
 
+static struct gui_download_table default_download_table = {
+	.create = gui_default_download_create,
+	.data = gui_default_download_data,
+	.error = gui_default_download_error,
+	.done = gui_default_download_done,
+};
 
 /** verify download window table is valid */
 static nserror verify_download_register(struct gui_download_table *gdt)
@@ -450,6 +485,9 @@ static nserror verify_llcache_register(struct gui_llcache_table *glt)
 	if (glt->invalidate == NULL) {
 		return NSERROR_BAD_PARAMETER;
 	}
+	if (glt->release == NULL) {
+		return NSERROR_BAD_PARAMETER;
+	}
 	if (glt->initialise == NULL) {
 		return NSERROR_BAD_PARAMETER;
 	}
@@ -465,6 +503,16 @@ static nsurl *gui_default_get_resource_url(const char *path)
 	return NULL;
 }
 
+static nserror gui_default_get_resource_data(const char *path, const uint8_t **data, size_t *data_len)
+{
+	return NSERROR_NOT_FOUND;
+}
+
+static nserror gui_default_release_resource_data(const uint8_t *data)
+{
+	return NSERROR_OK;
+}
+
 static char *gui_default_mimetype(const char *path)
 {
 	return strdup(guit->fetch->filetype(path));
@@ -487,6 +535,12 @@ static nserror verify_fetch_register(struct gui_fetch_table *gft)
 	if (gft->get_resource_url == NULL) {
 		gft->get_resource_url = gui_default_get_resource_url;
 	}
+	if (gft->get_resource_data == NULL) {
+		gft->get_resource_data = gui_default_get_resource_data;
+	}
+	if (gft->release_resource_data == NULL) {
+		gft->release_resource_data = gui_default_release_resource_data;
+	}
 	if (gft->mimetype == NULL) {
 		gft->mimetype = gui_default_mimetype;
 	}
@@ -522,6 +576,104 @@ static nserror verify_file_register(struct gui_file_table *gft)
 	return NSERROR_OK;
 }
 
+/**
+ * verify bitmap table is valid
+ *
+ * \param gbt The bitmap table to verify.
+ * \return NSERROR_OK if teh table is valid else NSERROR_BAD_PARAMETER.
+ */
+static nserror verify_bitmap_register(struct gui_bitmap_table *gbt)
+{
+	/* check table is present */
+	if (gbt == NULL) {
+		return NSERROR_BAD_PARAMETER;
+	}
+
+	/* check the mandantory fields are set */
+	if (gbt->create == NULL) {
+		return NSERROR_BAD_PARAMETER;
+	}
+
+	if (gbt->destroy == NULL) {
+		return NSERROR_BAD_PARAMETER;
+	}
+
+	if (gbt->set_opaque == NULL) {
+		return NSERROR_BAD_PARAMETER;
+	}
+
+	if (gbt->get_opaque == NULL) {
+		return NSERROR_BAD_PARAMETER;
+	}
+
+	if (gbt->test_opaque == NULL) {
+		return NSERROR_BAD_PARAMETER;
+	}
+
+	if (gbt->get_buffer == NULL) {
+		return NSERROR_BAD_PARAMETER;
+	}
+
+	if (gbt->get_rowstride == NULL) {
+		return NSERROR_BAD_PARAMETER;
+	}
+
+	if (gbt->get_width == NULL) {
+		return NSERROR_BAD_PARAMETER;
+	}
+
+	if (gbt->get_height == NULL) {
+		return NSERROR_BAD_PARAMETER;
+	}
+
+	if (gbt->get_bpp == NULL) {
+		return NSERROR_BAD_PARAMETER;
+	}
+
+	if (gbt->save == NULL) {
+		return NSERROR_BAD_PARAMETER;
+	}
+
+	if (gbt->modified == NULL) {
+		return NSERROR_BAD_PARAMETER;
+	}
+
+	if (gbt->render == NULL) {
+		return NSERROR_BAD_PARAMETER;
+	}
+
+	return NSERROR_OK;
+}
+
+/**
+ * verify layout table is valid
+ *
+ * \param glt The layout table to verify.
+ * \return NSERROR_OK if the table is valid else NSERROR_BAD_PARAMETER.
+ */
+static nserror verify_layout_register(struct gui_layout_table *glt)
+{
+	/* check table is present */
+	if (glt == NULL) {
+		return NSERROR_BAD_PARAMETER;
+	}
+
+	/* check the mandantory fields are set */
+	if (glt->width == NULL) {
+		return NSERROR_BAD_PARAMETER;
+	}
+
+	if (glt->position == NULL) {
+		return NSERROR_BAD_PARAMETER;
+	}
+
+	if (glt->split == NULL) {
+		return NSERROR_BAD_PARAMETER;
+	}
+
+	return NSERROR_OK;
+}
+
 static void gui_default_quit(void)
 {
 }
@@ -532,19 +684,14 @@ static nserror gui_default_launch_url(struct nsurl *url)
 	return NSERROR_NO_FETCH_HANDLER;
 }
 
-static void gui_default_create_form_select_menu(struct browser_window *bw,
-						struct form_control *control)
-{
-}
 
-
-static void gui_default_cert_verify(nsurl *url,
+static nserror gui_default_cert_verify(nsurl *url,
 				    const struct ssl_cert_info *certs,
 				    unsigned long num,
 				    nserror (*cb)(bool proceed, void *pw),
 				    void *cbpw)
 {
-	cb(false, cbpw);
+	return NSERROR_NOT_IMPLEMENTED;
 }
 
 static void gui_default_401login_open(nsurl *url, const char *realm,
@@ -553,52 +700,51 @@ static void gui_default_401login_open(nsurl *url, const char *realm,
 	cb(false, cbpw);
 }
 
-static struct gui_download_table default_download_table = {
-	.create = gui_default_download_create,
-	.data = gui_default_download_data,
-	.error = gui_default_download_error,
-	.done = gui_default_download_done,
-};
+static void
+gui_default_pdf_password(char **owner_pass, char **user_pass, char *path)
+{
+	*owner_pass = NULL;
+	save_pdf(path);
+}
 
-/** verify browser table is valid */
-static nserror verify_browser_register(struct gui_browser_table *gbt)
+/** verify misc table is valid */
+static nserror verify_misc_register(struct gui_misc_table *gmt)
 {
 	/* check table is present */
-	if (gbt == NULL) {
+	if (gmt == NULL) {
 		return NSERROR_BAD_PARAMETER;
 	}
 
 	/* check the mandantory fields are set */
-	if (gbt->poll == NULL) {
+	if (gmt->schedule == NULL) {
 		return NSERROR_BAD_PARAMETER;
 	}
-
-	if (gbt->schedule == NULL) {
+	if (gmt->warning == NULL) {
 		return NSERROR_BAD_PARAMETER;
 	}
 
 	/* fill in the optional entries with defaults */
-	if (gbt->quit == NULL) {
-		gbt->quit = gui_default_quit;
+	if (gmt->quit == NULL) {
+		gmt->quit = gui_default_quit;
 	}
-	if (gbt->launch_url == NULL) {
-		gbt->launch_url = gui_default_launch_url;
+	if (gmt->launch_url == NULL) {
+		gmt->launch_url = gui_default_launch_url;
 	}
-	if (gbt->create_form_select_menu == NULL) {
-		gbt->create_form_select_menu = gui_default_create_form_select_menu;
+	if (gmt->cert_verify == NULL) {
+		gmt->cert_verify = gui_default_cert_verify;
 	}
-	if (gbt->cert_verify == NULL) {
-		gbt->cert_verify = gui_default_cert_verify;
+	if (gmt->login == NULL) {
+		gmt->login = gui_default_401login_open;
 	}
-	if (gbt->login == NULL) {
-		gbt->login = gui_default_401login_open;
+	if (gmt->pdf_password == NULL) {
+		gmt->pdf_password = gui_default_pdf_password;
 	}
 	return NSERROR_OK;
 }
 
 
-/* exported interface documented in desktop/gui_factory.h */
-nserror gui_factory_register(struct netsurf_table *gt)
+/* exported interface documented in netsurf/netsurf.h */
+nserror netsurf_register(struct netsurf_table *gt)
 {
 	nserror err;
 
@@ -612,8 +758,10 @@ nserror gui_factory_register(struct netsurf_table *gt)
 		return NSERROR_BAD_PARAMETER;
 	}
 
-	/* browser table */
-	err = verify_browser_register(gt->browser);
+	/* mandantory tables */
+
+	/* miscellaneous table */
+	err = verify_misc_register(gt->misc);
 	if (err != NSERROR_OK) {
 		return err;
 	}
@@ -630,6 +778,20 @@ nserror gui_factory_register(struct netsurf_table *gt)
 		return err;
 	}
 
+	/* bitmap table */
+	err = verify_bitmap_register(gt->bitmap);
+	if (err != NSERROR_OK) {
+		return err;
+	}
+
+	/* layout table */
+	err = verify_layout_register(gt->layout);
+	if (err != NSERROR_OK) {
+		return err;
+	}
+
+	/* optional tables */
+
 	/* file table */
 	if (gt->file == NULL) {
 		gt->file = default_file_table;
