@@ -17,6 +17,7 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include <errno.h>
 #include <assert.h>
 #include <stdlib.h>
 #include <string.h>
@@ -24,16 +25,20 @@
 #include <dom/dom.h>
 #include <dom/bindings/hubbub/parser.h>
 
-#include "content/urldb.h"
-#include "desktop/hotlist.h"
-#include "desktop/treeview.h"
-#include "desktop/browser.h"
 #include "utils/corestrings.h"
 #include "utils/messages.h"
 #include "utils/utils.h"
 #include "utils/utf8.h"
 #include "utils/libdom.h"
 #include "utils/log.h"
+#include "utils/nsurl.h"
+#include "content/urldb.h"
+
+#include "netsurf/misc.h"
+#include "desktop/gui_internal.h"
+#include "desktop/hotlist.h"
+#include "desktop/treeview.h"
+#include "netsurf/browser_window.h"
 
 #define N_DAYS 28
 #define N_SEC_PER_DAY (60 * 60 * 24)
@@ -71,9 +76,9 @@ struct hotlist_entry {
 /**
  * Set a hotlist entry's data from the url_data.
  *
- * \param e		hotlist entry to set up
- * \param url_data	Data associated with entry's URL
- * \return NSERROR_OK on success, appropriate error otherwise
+ * \param e    hotlist entry to set up.
+ * \param data Data associated with entry's URL.
+ * \return NSERROR_OK on success, appropriate error otherwise.
  */
 static nserror hotlist_create_treeview_field_visits_data(
 		struct hotlist_entry *e, const struct url_data *data)
@@ -126,9 +131,9 @@ static nserror hotlist_create_treeview_field_visits_data(
 /**
  * Set a hotlist entry's data from the url_data.
  *
- * \param e		hotlist entry to set up
- * \param title		Title for entry, or NULL if using title from data
- * \param url_data	Data associated with entry's URL
+ * \param e     hotlist entry to set up
+ * \param title Title for entry, or NULL if using title from data
+ * \param data  Data associated with entry's URL
  * \return NSERROR_OK on success, appropriate error otherwise
  */
 static nserror hotlist_create_treeview_field_data(
@@ -320,6 +325,7 @@ static nserror hotlist_add_entry_internal(nsurl *url, const char *title,
  * \param relation	Existing node to insert as relation of, or NULL
  * \param rel		Folder's relationship to relation
  * \param folder	Updated to new hotlist folder data
+ * \param default_folder Add to the default folder.
  * \return NSERROR_OK on success, or appropriate error otherwise
  */
 static nserror hotlist_add_folder_internal(
@@ -412,14 +418,18 @@ static nserror hotlist_tree_node_folder_cb(
 
 	return NSERROR_OK;
 }
-static nserror hotlist_tree_node_entry_cb(
-		struct treeview_node_msg msg, void *data)
+
+/**
+ * callback for hotlist treeview entry manipulation.
+ */
+static nserror
+hotlist_tree_node_entry_cb(struct treeview_node_msg msg, void *data)
 {
 	struct hotlist_entry *e = data;
 	const char *old_text;
 	nsurl *old_url;
 	nsurl *url;
-	nserror err;
+	nserror err = NSERROR_OK;
 	bool match;
 
 	switch (msg.msg) {
@@ -456,27 +466,24 @@ static nserror hotlist_tree_node_entry_cb(
 				msg.data.node_edit.text[0] != '\0') {
 			/* Requst to change the entry URL text */
 			err = nsurl_create(msg.data.node_edit.text, &url);
-			if (err != NSERROR_OK)
-				return err;
+			if (err == NSERROR_OK) {
+				old_url = e->url;
 
-			old_url = e->url;
+				e->url = url;
+				e->data[HL_URL].value = nsurl_access(url);
+				e->data[HL_URL].value_len = nsurl_length(e->url);
 
-			e->url = url;
-			e->data[HL_URL].value = nsurl_access(url);
-			e->data[HL_URL].value_len = nsurl_length(e->url);
-
-			treeview_update_node_entry(hl_ctx.tree,
-					e->entry, e->data, e);
-			nsurl_unref(old_url);
+				treeview_update_node_entry(hl_ctx.tree,
+						   e->entry, e->data, e);
+				nsurl_unref(old_url);
+			}
 		}
 		break;
 
 	case TREE_MSG_NODE_LAUNCH:
 	{
-		nserror error;
 		struct browser_window *existing = NULL;
-		enum browser_window_create_flags flags =
-				BW_CREATE_HISTORY;
+		enum browser_window_create_flags flags = BW_CREATE_HISTORY;
 
 		/* TODO: Set existing to window that new tab appears in */
 
@@ -488,16 +495,14 @@ static nserror hotlist_tree_node_entry_cb(
 			/* TODO: flags ^= BW_CREATE_TAB; */
 		}
 
-		error = browser_window_create(flags, e->url, NULL,
+		err = browser_window_create(flags, e->url, NULL,
 				existing, NULL);
-		if (error != NSERROR_OK) {
-			warn_user(messages_get_errorcode(error), 0);
-		}
 	}
 		break;
 	}
-	return NSERROR_OK;
+	return err;
 }
+
 struct treeview_callback_table hl_tree_cb_t = {
 	.folder = hotlist_tree_node_folder_cb,
 	.entry = hotlist_tree_node_entry_cb
@@ -517,8 +522,8 @@ typedef struct {
 /**
  * Parse an entry represented as a li.
  *
- * \param li		DOM node for parsed li
- * \param ctx		Our hotlist loading context.
+ * \param li DOM node for parsed li
+ * \param ctx Our hotlist loading context.
  * \return NSERROR_OK on success, or appropriate error otherwise
  */
 static nserror hotlist_load_entry(dom_node *li, hotlist_load_ctx *ctx)
@@ -533,23 +538,23 @@ static nserror hotlist_load_entry(dom_node *li, hotlist_load_ctx *ctx)
 	/* The li must contain an "a" element */
 	a = libdom_find_first_element(li, corestring_lwc_a);
 	if (a == NULL) {
-		warn_user("TreeLoadError", "(Missing <a> in <li>)");
-		return NSERROR_NOMEM;
+		LOG("Missing <a> in <li>");
+		return NSERROR_INVALID;
 	}
 
 	derror = dom_node_get_text_content(a, &title1);
 	if (derror != DOM_NO_ERR) {
-		warn_user("TreeLoadError", "(No title)");
+		LOG("No title");
 		dom_node_unref(a);
-		return NSERROR_NOMEM;
+		return NSERROR_INVALID;
 	}
 
 	derror = dom_element_get_attribute(a, corestring_dom_href, &url1);
 	if (derror != DOM_NO_ERR || url1 == NULL) {
-		warn_user("TreeLoadError", "(No URL)");
+		LOG("No URL");
 		dom_string_unref(title1);
 		dom_node_unref(a);
-		return NSERROR_NOMEM;
+		return NSERROR_INVALID;
 	}
 	dom_node_unref(a);
 
@@ -564,12 +569,11 @@ static nserror hotlist_load_entry(dom_node *li, hotlist_load_ctx *ctx)
 	dom_string_unref(url1);
 
 	if (err != NSERROR_OK) {
-		LOG(("Failed normalising '%s'", dom_string_data(url1)));
+		LOG("Failed normalising '%s'", dom_string_data(url1));
 
-		warn_user(messages_get_errorcode(err), NULL);
-
-		if (title1 != NULL)
+		if (title1 != NULL) {
 			dom_string_unref(title1);
+		}
 
 		return err;
 	}
@@ -578,8 +582,9 @@ static nserror hotlist_load_entry(dom_node *li, hotlist_load_ctx *ctx)
 	err = hotlist_add_entry_internal(url, title, NULL, ctx->rel,
 			ctx->relshp, &ctx->rel);
 	nsurl_unref(url);
-	if (title1 != NULL)
+	if (title1 != NULL) {
 		dom_string_unref(title1);
+	}
 	ctx->relshp = TREE_REL_NEXT_SIBLING;
 
 	if (err != NSERROR_OK) {
@@ -602,8 +607,8 @@ static nserror hotlist_load_directory_cb(dom_node *node, void *ctx);
 /**
  * Parse a directory represented as a ul.
  *
- * \param  ul		DOM node for parsed ul
- * \param  directory	directory to add this directory to
+ * \param ul DOM node for parsed ul.
+ * \param ctx The hotlist context.
  * \return NSERROR_OK on success, or appropriate error otherwise
  */
 static nserror hotlist_load_directory(dom_node *ul, hotlist_load_ctx *ctx)
@@ -631,7 +636,7 @@ nserror hotlist_load_directory_cb(dom_node *node, void *ctx)
 
 	error = dom_node_get_node_name(node, &name);
 	if (error != DOM_NO_ERR || name == NULL)
-		return NSERROR_NOMEM;
+		return NSERROR_DOM;
 
 	if (dom_string_caseless_lwc_isequal(name, corestring_lwc_li)) {
 		/* Entry handling */
@@ -644,10 +649,9 @@ nserror hotlist_load_directory_cb(dom_node *node, void *ctx)
 
 		error = dom_node_get_text_content(node, &title);
 		if (error != DOM_NO_ERR || title == NULL) {
-			warn_user("TreeLoadError", "(Empty <h4> "
-					"or memory exhausted.)");
+			LOG("Empty <h4> or memory exhausted.");
 			dom_string_unref(name);
-			return NSERROR_NOMEM;
+			return NSERROR_DOM;
 		}
 
 		if (current_ctx->title != NULL)
@@ -724,6 +728,36 @@ nserror hotlist_load_directory_cb(dom_node *node, void *ctx)
 }
 
 
+/*
+ * Get path for writing hotlist to
+ *
+ * \param path		The final path of the hotlist
+ * \param loaded	Updated to the path to write the holist to
+ * \return NSERROR_OK on success, or appropriate error otherwise
+ */
+static nserror hotlist_get_temp_path(const char *path, char **temp_path)
+{
+	const char *extension = "-bk";
+	char *joined;
+	int len;
+
+	len = strlen(path) + strlen(extension);
+
+	joined = malloc(len + 1);
+	if (joined == NULL) {
+		return NSERROR_NOMEM;
+	}
+
+	if (snprintf(joined, len + 1, "%s%s", path, extension) != len) {
+		free(joined);
+		return NSERROR_UNKNOWN;
+	}
+
+	*temp_path = joined;
+	return NSERROR_OK;
+}
+
+
 /*
  * Load the hotlist data from file
  *
@@ -736,28 +770,40 @@ static nserror hotlist_load(const char *path, bool *loaded)
 	dom_document *document;
 	dom_node *html, *body, *ul;
 	hotlist_load_ctx ctx;
+	char *temp_path;
 	nserror err;
 
 	*loaded = false;
 
 	/* Handle no path */
 	if (path == NULL) {
-		LOG(("No hotlist file path provided."));
+		LOG("No hotlist file path provided.");
 		return NSERROR_OK;
 	}
 
+	/* Get temp hotlist write path */
+	err = hotlist_get_temp_path(path, &temp_path);
+	if (err != NSERROR_OK) {
+		return err;
+	}
+
 	/* Load hotlist file */
 	err = libdom_parse_file(path, "iso-8859-1", &document);
 	if (err != NSERROR_OK) {
-		return err;
+		err = libdom_parse_file(temp_path, "iso-8859-1", &document);
+		if (err != NSERROR_OK) {
+			free(temp_path);
+			return err;
+		}
 	}
+	free(temp_path);
 
 	/* Find HTML element */
 	html = libdom_find_first_element((dom_node *) document,
 			corestring_lwc_html);
 	if (html == NULL) {
 		dom_node_unref(document);
-		warn_user("TreeLoadError", "(<html> not found)");
+		guit->misc->warning("TreeLoadError", "(<html> not found)");
 		return NSERROR_OK;
 	}
 
@@ -766,7 +812,7 @@ static nserror hotlist_load(const char *path, bool *loaded)
 	if (body == NULL) {
 		dom_node_unref(html);
 		dom_node_unref(document);
-		warn_user("TreeLoadError", "(<html>...<body> not found)");
+		guit->misc->warning("TreeLoadError", "(<html>...<body> not found)");
 		return NSERROR_OK;
 	}
 
@@ -776,7 +822,7 @@ static nserror hotlist_load(const char *path, bool *loaded)
 		dom_node_unref(body);
 		dom_node_unref(html);
 		dom_node_unref(document);
-		warn_user("TreeLoadError",
+		guit->misc->warning("TreeLoadError",
 					"(<html>...<body>...<ul> not found.)");
 		return NSERROR_OK;
 	}
@@ -801,7 +847,7 @@ static nserror hotlist_load(const char *path, bool *loaded)
 	dom_node_unref(document);
 
 	if (err != NSERROR_OK) {
-		warn_user("TreeLoadError", "(Failed building tree.)");
+		guit->misc->warning("TreeLoadError", "(Failed building tree.)");
 		return NSERROR_OK;
 	}
 
@@ -872,9 +918,49 @@ static nserror hotlist_generate(void)
 }
 
 
+/* Save the hotlist to to a file at the given path
+ *
+ * \param path  Path to save hostlist file to.
+ * \return NSERROR_OK on success, or appropriate error otherwise
+ */
+static nserror hotlist_save(const char *path)
+{
+	nserror res = NSERROR_OK;
+	char *temp_path;
+
+	/* Get path to export to */
+	res = hotlist_get_temp_path(path, &temp_path);
+	if (res != NSERROR_OK) {
+		return res;
+	}
+
+	/* Export to temp path */
+	res = hotlist_export(temp_path, NULL);
+	if (res != NSERROR_OK) {
+		goto cleanup;
+	}
+
+	/* Remove old hotlist to handle non-POSIX rename() implementations. */
+	(void)remove(path);
+
+	/* Replace any old hotlist file with the one we just saved */
+	if (rename(temp_path, path) != 0) {
+		res = NSERROR_SAVE_FAILED;
+		LOG("Error renaming hotlist: %s.", strerror(errno));
+		goto cleanup;
+	}
+
+cleanup:
+	free(temp_path);
+
+	return res;
+}
+
+
 struct treeview_export_walk_ctx {
 	FILE *fp;
 };
+
 /** Callback for treeview_walk node entering */
 static nserror hotlist_export_enter_cb(void *ctx, void *node_data,
 		enum treeview_node_type type, bool *abort)
@@ -1064,7 +1150,8 @@ static nserror hotlist_initialise_entry_fields(void)
 		goto error;
 	}
 
-	hl_ctx.fields[HL_URL].flags = TREE_FLAG_ALLOW_EDIT;
+	hl_ctx.fields[HL_URL].flags = TREE_FLAG_ALLOW_EDIT |
+			TREE_FLAG_COPY_TEXT;
 	label = "TreeviewLabelURL";
 	label = messages_get(label);
 	if (lwc_intern_string(label, strlen(label),
@@ -1145,7 +1232,12 @@ nserror hotlist_init(struct core_window_callback_table *cw_t,
 {
 	nserror err;
 
-	LOG(("Loading hotlist"));
+	err = treeview_init();
+	if (err != NSERROR_OK) {
+		return err;
+	}
+
+	LOG("Loading hotlist");
 
 	hl_ctx.tree = NULL;
 	hl_ctx.built = false;
@@ -1182,7 +1274,7 @@ nserror hotlist_init(struct core_window_callback_table *cw_t,
 	/* Inform client of window height */
 	treeview_get_height(hl_ctx.tree);
 
-	LOG(("Loaded hotlist"));
+	LOG("Loaded hotlist");
 
 	return NSERROR_OK;
 }
@@ -1194,12 +1286,12 @@ nserror hotlist_fini(const char *path)
 	int i;
 	nserror err;
 
-	LOG(("Finalising hotlist"));
+	LOG("Finalising hotlist");
 
 	/* Save the hotlist */
-	err = hotlist_export(path, NULL);
+	err = hotlist_save(path);
 	if (err != NSERROR_OK) {
-		warn_user("Couldn't save the hotlist.", 0);
+		LOG("Problem saving the hotlist.");
 	}
 
 	/* Destroy the hotlist treeview */
@@ -1211,7 +1303,12 @@ nserror hotlist_fini(const char *path)
 		if (hl_ctx.fields[i].field != NULL)
 			lwc_string_unref(hl_ctx.fields[i].field);
 
-	LOG(("Finalised hotlist"));
+	err = treeview_fini();
+	if (err != NSERROR_OK) {
+		return err;
+	}
+
+	LOG("Finalised hotlist");
 
 	return err;
 }
@@ -1485,9 +1582,9 @@ void hotlist_mouse_action(browser_mouse_state mouse, int x, int y)
 
 
 /* Exported interface, documented in hotlist.h */
-void hotlist_keypress(uint32_t key)
+bool hotlist_keypress(uint32_t key)
 {
-	treeview_keypress(hl_ctx.tree, key);
+	return treeview_keypress(hl_ctx.tree, key);
 }
 
 
