@@ -66,11 +66,15 @@
 #include <string.h>
 #include <stdio.h>
 
-#include "image/bitmap.h"
+#include "utils/utils.h"
 #include "utils/log.h"
+#include "utils/errors.h"
+#include "netsurf/bitmap.h"
 #include "content/content.h"
+
+#include "desktop/gui_internal.h"
 #include "desktop/knockout.h"
-#include "desktop/plotters.h"
+#include "netsurf/plotters.h"
 
 /* Define to enable knockout debug */
 #undef KNOCKOUT_DEBUG
@@ -288,10 +292,7 @@ bool knockout_plot_flush(void)
 
 	/* debugging information */
 #ifdef KNOCKOUT_DEBUG
-	LOG(("Entries are %i/%i, %i/%i, %i/%i",
-			knockout_entry_cur, KNOCKOUT_ENTRIES,
-			knockout_box_cur, KNOCKOUT_BOXES,
-			knockout_polygon_cur, KNOCKOUT_POLYGONS));
+	LOG("Entries are %i/%i, %i/%i, %i/%i", knockout_entry_cur, KNOCKOUT_ENTRIES, knockout_box_cur, KNOCKOUT_BOXES, knockout_polygon_cur, KNOCKOUT_POLYGONS);
 #endif
 
 	for (i = 0; i < knockout_entry_cur; i++) {
@@ -404,11 +405,11 @@ bool knockout_plot_flush(void)
 /**
  * Knockout a section of previous rendering
  *
- * \param  x0	the left edge of the removal box
- * \param  y0	the bottom edge of the removal box
- * \param  x1	the right edge of the removal box
- * \param  y1	the top edge of the removal box
- * \param  box  the parent box set to consider, or NULL for top level
+ * \param x0    The left edge of the removal box
+ * \param y0    The bottom edge of the removal box
+ * \param x1    The right edge of the removal box
+ * \param y1    The top edge of the removal box
+ * \param owner The parent box set to consider, or NULL for top level
 */
 void knockout_calculate(int x0, int y0, int x1, int y1, struct knockout_box *owner)
 {
@@ -683,8 +684,7 @@ bool knockout_plot_clip(const struct rect *clip)
 {
 	if (clip->x1 < clip->x0 || clip->y0 > clip->y1) {
 #ifdef KNOCKOUT_DEBUG
-		LOG(("bad clip rectangle %i %i %i %i",
-				clip->x0, clip->y0, clip->x1, clip->y1));
+		LOG("bad clip rectangle %i %i %i %i", clip->x0, clip->y0, clip->x1, clip->y1);
 #endif
 		return false;
 	}
@@ -772,8 +772,9 @@ bool knockout_plot_bitmap(int x, int y, int width, int height,
 	}
 
 	/* tiled bitmaps both knock out and get knocked out */
-	if (bitmap_get_opaque(bitmap))
+	if (guit->bitmap->get_opaque(bitmap)) {
 		knockout_calculate(kx0, ky0, kx1, ky1, NULL);
+	}
 	knockout_boxes[knockout_box_cur].bbox.x0 = kx0;
 	knockout_boxes[knockout_box_cur].bbox.y0 = ky0;
 	knockout_boxes[knockout_box_cur].bbox.x1 = kx1;
