@@ -17,15 +17,16 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/** \file
- * Save HTML document with dependencies (implementation).
+/**
+ * \file
+ * Save HTML document with dependencies implementation.
  */
 
 #include <assert.h>
-#include <ctype.h>
 #include <errno.h>
 #include <stdio.h>
 #include <string.h>
+#include <strings.h>
 #include <sys/types.h>
 #include <regex.h>
 #include <dom/dom.h>
@@ -38,20 +39,22 @@
 #include "utils/utils.h"
 #include "utils/file.h"
 #include "utils/messages.h"
-#include "content/content.h"
+#include "utils/ascii.h"
+#include "netsurf/content.h"
 #include "content/hlcache.h"
 #include "css/css.h"
 #include "render/box.h"
 #include "render/html.h"
 
-#include "desktop/gui_factory.h"
+#include "netsurf/misc.h"
+#include "desktop/gui_internal.h"
 #include "desktop/save_complete.h"
 
 regex_t save_complete_import_re;
 
 /** An entry in save_complete_list. */
 typedef struct save_complete_entry {
-	hlcache_handle *content;
+	struct hlcache_handle *content;
 	struct save_complete_entry *next; /**< Next entry in list */
 } save_complete_entry;
 
@@ -71,7 +74,7 @@ typedef enum {
 } save_complete_event_type;
 
 
-static bool save_complete_save_html(save_complete_ctx *ctx, hlcache_handle *c,
+static bool save_complete_save_html(save_complete_ctx *ctx, struct hlcache_handle *c,
 		bool index);
 static bool save_complete_save_imported_sheets(save_complete_ctx *ctx,
 		struct nscss_import *imports, uint32_t import_count);
@@ -97,7 +100,7 @@ static void save_complete_ctx_finalise(save_complete_ctx *ctx)
 }
 
 static bool save_complete_ctx_add_content(save_complete_ctx *ctx,
-		hlcache_handle *content)
+		struct hlcache_handle *content)
 {
 	save_complete_entry *entry;
 
@@ -112,24 +115,32 @@ static bool save_complete_ctx_add_content(save_complete_ctx *ctx,
 	return true;
 }
 
-
-static hlcache_handle *save_complete_ctx_find_content(save_complete_ctx *ctx,
-		const nsurl *url)
+/**
+ * find handle to content for url
+ *
+ * \param ctx The save context
+ * \param url The url to find content handle for
+ * \return The content handle or NULL if not found.
+ */
+static struct hlcache_handle *
+save_complete_ctx_find_content(save_complete_ctx *ctx, const nsurl *url)
 {
 	save_complete_entry *entry;
 
-	for (entry = ctx->list; entry != NULL; entry = entry->next)
+	for (entry = ctx->list; entry != NULL; entry = entry->next) {
 		if (nsurl_compare(url,
-				hlcache_handle_get_url(entry->content),
-				NSURL_COMPLETE))
+				  hlcache_handle_get_url(entry->content),
+				  NSURL_COMPLETE)) {
 			return entry->content;
+		}
+	}
 
 	return NULL;
 }
 
 
 static bool save_complete_ctx_has_content(save_complete_ctx *ctx,
-		hlcache_handle *content)
+		struct hlcache_handle *content)
 {
 	save_complete_entry *entry;
 
@@ -150,15 +161,15 @@ static bool save_complete_save_buffer(save_complete_ctx *ctx,
 
 	ret = netsurf_mkpath(&fname, NULL, 2, ctx->path, leafname);
 	if (ret != NSERROR_OK) {
-		warn_user(messages_get_errorcode(ret), 0);
+		guit->misc->warning(messages_get_errorcode(ret), 0);
 		return false;
 	}
 
 	fp = fopen(fname, "wb");
 	if (fp == NULL) {
 		free(fname);
-		LOG(("fopen(): errno = %i", errno));
-		warn_user("SaveError", strerror(errno));
+		LOG("fopen(): errno = %i", errno);
+		guit->misc->warning("SaveError", strerror(errno));
 		return false;
 	}
 
@@ -177,11 +188,12 @@ static bool save_complete_save_buffer(save_complete_ctx *ctx,
 /**
  * Rewrite stylesheet \@import rules for save complete.
  *
- * \param  source  stylesheet source
- * \param  size    size of source
- * \param  base    url of stylesheet
- * \param  osize   updated with the size of the result
- * \return  converted source, or NULL on out of memory
+ * \param ctx Save complete context.
+ * \param source stylesheet source.
+ * \param size size of source.
+ * \param base url of stylesheet.
+ * \param osize updated with the size of the result.
+ * \return converted source, or NULL on out of memory.
  */
 
 static char *save_complete_rewrite_stylesheet_urls(save_complete_ctx *ctx,
@@ -198,13 +210,14 @@ static char *save_complete_rewrite_stylesheet_urls(save_complete_ctx *ctx,
 	for (offset = 0; SLEN("@import") < size &&
 			offset <= size - SLEN("@import"); offset++) {
 		if (source[offset] == '@' &&
-				tolower(source[offset + 1]) == 'i' &&
-				tolower(source[offset + 2]) == 'm' &&
-				tolower(source[offset + 3]) == 'p' &&
-				tolower(source[offset + 4]) == 'o' &&
-				tolower(source[offset + 5]) == 'r' &&
-				tolower(source[offset + 6]) == 't')
+		    ascii_to_lower(source[offset + 1]) == 'i' &&
+		    ascii_to_lower(source[offset + 2]) == 'm' &&
+		    ascii_to_lower(source[offset + 3]) == 'p' &&
+		    ascii_to_lower(source[offset + 4]) == 'o' &&
+		    ascii_to_lower(source[offset + 5]) == 'r' &&
+		    ascii_to_lower(source[offset + 6]) == 't') {
 			imports++;
+		}
 	}
 
 	rewritten = malloc(size + imports * 20);
@@ -315,7 +328,7 @@ static bool save_complete_save_stylesheet(save_complete_ctx *ctx,
 		return true;
 
 	if (save_complete_ctx_add_content(ctx, css) == false) {
-		warn_user("NoMemory", 0);
+		guit->misc->warning("NoMemory", 0);
 		return false;
 	}
 
@@ -328,7 +341,7 @@ static bool save_complete_save_stylesheet(save_complete_ctx *ctx,
 	source = save_complete_rewrite_stylesheet_urls(ctx, css_data, css_size,
 			hlcache_handle_get_url(css), &source_len);
 	if (source == NULL) {
-		warn_user("NoMemory", 0);
+		guit->misc->warning("NoMemory", 0);
 		return false;
 	}
 
@@ -411,7 +424,7 @@ static bool save_complete_save_html_object(save_complete_ctx *ctx,
 		return true;
 
 	if (save_complete_ctx_add_content(ctx, obj) == false) {
-		warn_user("NoMemory", 0);
+		guit->misc->warning("NoMemory", 0);
 		return false;
 	}
 
@@ -1031,7 +1044,7 @@ static bool save_complete_node_handler(dom_node *node,
 	} else if (type == DOM_DOCUMENT_NODE) {
 		/* Do nothing */
 	} else {
-		LOG(("Unhandled node type: %d", type));
+		LOG("Unhandled node type: %d", type);
 	}
 
 	return true;
@@ -1055,15 +1068,15 @@ static bool save_complete_save_html_document(save_complete_ctx *ctx,
 
 	ret = netsurf_mkpath(&fname, NULL, 2, ctx->path, filename);
 	if (ret != NSERROR_OK) {
-		warn_user(messages_get_errorcode(ret), NULL);
+		guit->misc->warning(messages_get_errorcode(ret), NULL);
 		return false;
 	}
 
 	fp = fopen(fname, "wb");
 	if (fp == NULL) {
 		free(fname);
-		LOG(("fopen(): errno = %i", errno));
-		warn_user("SaveError", strerror(errno));
+		LOG("fopen(): errno = %i", errno);
+		guit->misc->warning("SaveError", strerror(errno));
 		return false;
 	}
 
@@ -1076,7 +1089,7 @@ static bool save_complete_save_html_document(save_complete_ctx *ctx,
 	if (save_complete_libdom_treewalk((dom_node *) doc,
 			save_complete_node_handler, ctx) == false) {
 		free(fname);
-		warn_user("NoMemory", 0);
+		guit->misc->warning("NoMemory", 0);
 		fclose(fp);
 		return false;
 	}
@@ -1141,8 +1154,8 @@ static bool save_complete_inventory(save_complete_ctx *ctx)
 	fp = fopen(fname, "w");
 	free(fname);
 	if (fp == NULL) {
-		LOG(("fopen(): errno = %i", errno));
-		warn_user("SaveError", strerror(errno));
+		LOG("fopen(): errno = %i", errno);
+		guit->misc->warning("SaveError", strerror(errno));
 		return false;
 	}
 
@@ -1157,6 +1170,25 @@ static bool save_complete_inventory(save_complete_ctx *ctx)
 	return true;
 }
 
+/**
+ * Compile a regular expression, handling errors.
+ *
+ * Parameters as for regcomp(), see man regex.
+ */
+static nserror regcomp_wrapper(regex_t *preg, const char *regex, int cflags)
+{
+	int r;
+	r = regcomp(preg, regex, cflags);
+	if (r) {
+		char errbuf[200];
+		regerror(r, preg, errbuf, sizeof errbuf);
+		LOG("Failed to compile regexp '%s': %s\n", regex, errbuf);
+		return NSERROR_INIT_FAILED;
+	}
+	return NSERROR_OK;
+}
+
+
 /* Documented in save_complete.h */
 void save_complete_init(void)
 {
