@@ -25,15 +25,17 @@
 #include <stdbool.h>
 #include <stdlib.h>
 
-#include "desktop/system_colour.h"
-#include "desktop/mouse.h"
-#include "desktop/scrollbar.h"
-#include "utils/nsoption.h"
-#include "desktop/plotters.h"
-#include "desktop/plot_style.h"
 #include "utils/log.h"
 #include "utils/messages.h"
 #include "utils/utils.h"
+#include "utils/nsoption.h"
+
+#include "netsurf/browser_window.h"
+#include "desktop/system_colour.h"
+#include "netsurf/mouse.h"
+#include "desktop/scrollbar.h"
+#include "netsurf/plotters.h"
+#include "desktop/plot_style.h"
 
 
 struct scrollbar {
@@ -68,9 +70,9 @@ struct scrollbar {
 
 
 /*
- * Exported function.  Documented in scrollbar.h
+ * Exported function.  Documented in desktop/scrollbar.h
  */
-bool scrollbar_create(bool horizontal, int length, int full_size,
+nserror scrollbar_create(bool horizontal, int length, int full_size,
 		int visible_size, void *client_data,
 		scrollbar_client_callback client_callback,
 		struct scrollbar **s)
@@ -80,10 +82,8 @@ bool scrollbar_create(bool horizontal, int length, int full_size,
 
 	scrollbar = malloc(sizeof(struct scrollbar));
 	if (scrollbar == NULL) {
-		LOG(("malloc failed"));
-		warn_user("NoMemory", 0);
 		*s = NULL;
-		return false;
+		return NSERROR_NOMEM;
 	}
 
 	scrollbar->horizontal = horizontal;
@@ -107,7 +107,7 @@ bool scrollbar_create(bool horizontal, int length, int full_size,
 
 	*s = scrollbar;
 
-	return true;
+	return NSERROR_OK;
 }
 
 
@@ -510,8 +510,15 @@ int scrollbar_get_offset(struct scrollbar *s)
 void scrollbar_set_extents(struct scrollbar *s, int length,
 		int visible_size, int full_size)
 {
-	int well_length;
 	int cur_excess = s->full_size - s->visible_size;
+	int well_length;
+	struct scrollbar_msg_data msg;
+
+	if (length == s->length && visible_size == s->visible_size &&
+			full_size == s->full_size) {
+		/* Nothing's changed. */
+		return;
+	}
 
 	if (length != -1)
 		s->length = length;
@@ -524,9 +531,11 @@ void scrollbar_set_extents(struct scrollbar *s, int length,
 		s->full_size = s->visible_size;
 
 	/* Update scroll offset (scaled in proportion with change in excess) */
-	s->offset = (cur_excess < 1) ? 0 :
-			((s->full_size - s->visible_size) * s->offset /
-			cur_excess);
+	if (cur_excess <= 0) {
+		s->offset = 0;
+	} else {
+		s->offset = (full_size - visible_size) * s->offset / cur_excess;
+	}
 
 	well_length = s->length - 2 * SCROLLBAR_WIDTH;
 
@@ -537,6 +546,11 @@ void scrollbar_set_extents(struct scrollbar *s, int length,
 		s->bar_len = (well_length * s->visible_size) / s->full_size;
 		s->bar_pos = (well_length * s->offset) / s->full_size;
 	}
+
+	msg.scrollbar = s;
+	msg.msg = SCROLLBAR_MSG_MOVED;
+	msg.scroll_offset = s->offset;
+	s->client_callback(s->client_data, &msg);
 }
 
 
