@@ -16,27 +16,36 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/** \file
- * Treeview handling (implementation).
+/**
+ * \file
+ *
+ * Treeview handling implementation.
  */
 
 #include "utils/log.h"
-#include "css/utils.h"
-#include "image/bitmap.h"
-#include "render/font.h"
+#include "utils/nsurl.h"
+#include "utils/nsoption.h"
+#include "netsurf/bitmap.h"
+#include "netsurf/content.h"
+#include "netsurf/plotters.h"
+#include "netsurf/clipboard.h"
+#include "netsurf/layout.h"
+#include "netsurf/keypress.h"
+#include "netsurf/core_window.h"
 #include "content/hlcache.h"
+#include "css/utils.h"
 
 #include "desktop/system_colour.h"
 #include "desktop/knockout.h"
-#include "desktop/plotters.h"
 #include "desktop/textarea.h"
 #include "desktop/treeview.h"
+#include "desktop/gui_internal.h"
 
-/* TODO: get rid of REDRAW_MAX -- need to be able to know window size */
+/** @todo get rid of REDRAW_MAX -- need to be able to know window size */
 #define REDRAW_MAX 8000
 
 struct treeview_globals {
-	bool initialised;
+	unsigned initialised;
 	int line_height;
 	int furniture_width;
 	int step_width;
@@ -54,7 +63,7 @@ enum treeview_node_part {
 
 struct treeview_text {
 	const char *data;	/**< Text string */
-	uint32_t len;		/**< Lenfth of string in bytes */
+	uint32_t len;		/**< Length of string in bytes */
 	int width;		/**< Width of text in px */
 };
 
@@ -210,6 +219,29 @@ static struct treeview_resource treeview_res[TREE_RES_LAST] = {
 }; /**< Treeview content resources */
 
 
+/* Helper function to access the given field of a node
+ *
+ * \param tree		Treeview that node belongs to
+ * \param n		Node to get field from
+ * \param i		Index of field of interest
+ * \return text entry for field or NULL.
+ */
+static inline struct treeview_text * treeview_get_text_for_field(
+		treeview *tree, treeview_node *n, int i)
+{
+	if (i == 0) {
+		return &n->text;
+
+	} else if (i < tree->n_fields && n->type == TREE_NODE_ENTRY) {
+		struct treeview_node_entry *e = (struct treeview_node_entry *)n;
+		return &e->fields[i - 1].value;
+	}
+
+	assert(0 && "Bad field index for node");
+	return NULL;
+}
+
+
 /* Find the next node in depth first tree order
  *
  * \param node		Start node
@@ -513,10 +545,10 @@ static inline void treeview_insert_node(treeview_node *a,
 		/* Parent is expanded, so inserted node will be visible and
 		 * affect layout */
 		if (a->text.width == 0) {
-			nsfont.font_width(&plot_style_odd.text,
-					a->text.data,
-					a->text.len,
-					&(a->text.width));
+			guit->layout->width(&plot_style_odd.text,
+					    a->text.data,
+					    a->text.len,
+					    &(a->text.width));
 		}
 
 		do {
@@ -617,7 +649,7 @@ nserror treeview_update_node_folder(treeview *tree,
 
 	if (folder->parent->flags & TV_NFLAGS_EXPANDED) {
 		/* Text will be seen, get its width */
-		nsfont.font_width(&plot_style_odd.text,
+		guit->layout->width(&plot_style_odd.text,
 				folder->text.data,
 				folder->text.len,
 				&(folder->text.width));
@@ -667,7 +699,7 @@ nserror treeview_update_node_entry(treeview *tree,
 
 	if (entry->parent->flags & TV_NFLAGS_EXPANDED) {
 		/* Text will be seen, get its width */
-		nsfont.font_width(&plot_style_odd.text,
+		guit->layout->width(&plot_style_odd.text,
 				entry->text.data,
 				entry->text.len,
 				&(entry->text.width));
@@ -687,7 +719,7 @@ nserror treeview_update_node_entry(treeview *tree,
 
 		if (entry->flags & TV_NFLAGS_EXPANDED) {
 			/* Text will be seen, get its width */
-			nsfont.font_width(&plot_style_odd.text,
+			guit->layout->width(&plot_style_odd.text,
 					e->fields[i - 1].value.data,
 					e->fields[i - 1].value.len,
 					&(e->fields[i - 1].value.width));
@@ -1284,7 +1316,7 @@ nserror treeview_create(treeview **tree,
 		f->value.data = lwc_string_data(fields[i].field);
 		f->value.len = lwc_string_length(fields[i].field);
 
-		nsfont.font_width(&plot_style_odd.text, f->value.data,
+		guit->layout->width(&plot_style_odd.text, f->value.data,
 				f->value.len, &(f->value.width));
 
 		if (f->flags & TREE_FLAG_SHOW_NAME)
@@ -1352,9 +1384,9 @@ nserror treeview_destroy(treeview *tree)
 /**
  * Expand a treeview's nodes
  *
- * \param tree		Treeview object to expand nodes in
- * \param only_folders	Iff true, only folders are expanded.
- * \return NSERROR_OK on success, appropriate error otherwise
+ * \param tree Treeview object to expand nodes in
+ * \param node The node to expand.
+ * \return NSERROR_OK on success, appropriate error otherwise.
  */
 static nserror treeview_node_expand_internal(treeview *tree,
 		treeview_node *node)
@@ -1369,7 +1401,7 @@ static nserror treeview_node_expand_internal(treeview *tree,
 
 	if (node->flags & TV_NFLAGS_EXPANDED) {
 		/* What madness is this? */
-		LOG(("Tried to expand an expanded node."));
+		LOG("Tried to expand an expanded node.");
 		return NSERROR_OK;
 	}
 
@@ -1384,7 +1416,7 @@ static nserror treeview_node_expand_internal(treeview *tree,
 		do {
 			assert((child->flags & TV_NFLAGS_EXPANDED) == false);
 			if (child->text.width == 0) {
-				nsfont.font_width(&plot_style_odd.text,
+				guit->layout->width(&plot_style_odd.text,
 						child->text.data,
 						child->text.len,
 						&(child->text.width));
@@ -1405,7 +1437,7 @@ static nserror treeview_node_expand_internal(treeview *tree,
 		for (i = 0; i < tree->n_fields - 1; i++) {
 
 			if (e->fields[i].value.width == 0) {
-				nsfont.font_width(&plot_style_odd.text,
+				guit->layout->width(&plot_style_odd.text,
 						e->fields[i].value.data,
 						e->fields[i].value.len,
 						&(e->fields[i].value.width));
@@ -1513,7 +1545,7 @@ static nserror treeview_node_contract_internal(treeview *tree,
 
 	if ((node->flags & TV_NFLAGS_EXPANDED) == false) {
 		/* What madness is this? */
-		LOG(("Tried to contract a contracted node."));
+		LOG("Tried to contract a contracted node.");
 		return NSERROR_OK;
 	}
 
@@ -1917,7 +1949,8 @@ struct treeview_selection_walk_data {
 		TREEVIEW_WALK_COMMIT_SELECT_DRAG,
 		TREEVIEW_WALK_DELETE_SELECTION,
 		TREEVIEW_WALK_PROPAGATE_SELECTION,
-		TREEVIEW_WALK_YANK_SELECTION
+		TREEVIEW_WALK_YANK_SELECTION,
+		TREEVIEW_WALK_COPY_SELECTION
 	} purpose;
 	union {
 		bool has_selection;
@@ -1935,6 +1968,10 @@ struct treeview_selection_walk_data {
 		struct {
 			treeview_node *n;
 		} first;
+		struct {
+			char *text;
+			uint32_t len;
+		} copy;
 	} data;
 	int current_y;
 	treeview *tree;
@@ -2040,6 +2077,47 @@ static nserror treeview_node_selection_walk_cb(treeview_node *n,
 			*skip_children = true;
 		}
 		break;
+
+	case TREEVIEW_WALK_COPY_SELECTION:
+		if (n->flags & TV_NFLAGS_SELECTED &&
+				n->type == TREE_NODE_ENTRY) {
+			int i;
+			char *temp;
+			uint32_t len;
+			const char *text;
+			struct treeview_field *ef;
+			struct treeview_text *val;
+
+			for (i = 0; i < sw->tree->n_fields; i++) {
+				ef = &(sw->tree->fields[i]);
+
+				if (!(ef->flags & TREE_FLAG_COPY_TEXT)) {
+					continue;
+				}
+				val = treeview_get_text_for_field(sw->tree,
+						n, i);
+				text = val->data;
+				len  = val->len;
+
+				temp = realloc(sw->data.copy.text,
+						sw->data.copy.len + len + 1);
+				if (temp == NULL) {
+					free(sw->data.copy.text);
+					sw->data.copy.text = NULL;
+					sw->data.copy.len = 0;
+					return NSERROR_NOMEM;
+				}
+
+				if (sw->data.copy.len != 0) {
+					temp[sw->data.copy.len - 1] = '\n';
+				}
+				memcpy(temp + sw->data.copy.len, text, len);
+				temp[sw->data.copy.len + len] = '\0';
+				sw->data.copy.len += len + 1;
+				sw->data.copy.text = temp;
+			}
+		}
+		break;
 	}
 
 	if (changed) {
@@ -2211,6 +2289,35 @@ static void treeview_move_yank_selection(treeview *tree)
 }
 
 
+/**
+ * Copy a selection to the clipboard.
+ *
+ * \param tree		Treeview object to yank selection from
+ */
+static void treeview_copy_selection(treeview *tree)
+{
+	struct treeview_selection_walk_data sw;
+	nserror err;
+
+	sw.purpose = TREEVIEW_WALK_COPY_SELECTION;
+	sw.data.copy.text = NULL;
+	sw.data.copy.len = 0;
+	sw.tree = tree;
+
+	err = treeview_walk_internal(tree->root, false, NULL,
+			treeview_node_selection_walk_cb, &sw);
+	if (err != NSERROR_OK) {
+		return;
+	}
+
+	if (sw.data.copy.text != NULL) {
+		guit->clipboard->set(sw.data.copy.text,
+				sw.data.copy.len - 1, NULL, 0);
+		free(sw.data.copy.text);
+	}
+}
+
+
 /**
  * Delete a selection.
  *
@@ -2332,7 +2439,7 @@ static nserror treeview_move_selection(treeview *tree, struct rect *rect)
 		break;
 
 	default:
-		LOG(("Bad drop target for move."));
+		LOG("Bad drop target for move.");
 		return NSERROR_BAD_PARAMETER;
 	}
 
@@ -2400,6 +2507,7 @@ static nserror treeview_node_launch_walk_fwd_cb(treeview_node *n, void *ctx,
 		bool *skip_children, bool *end)
 {
 	struct treeview_launch_walk_data *lw = ctx;
+	nserror ret = NSERROR_OK;
 
 	if (n->type == TREE_NODE_FOLDER && n->flags & TV_NFLAGS_SELECTED) {
 		lw->selected_depth++;
@@ -2410,10 +2518,10 @@ static nserror treeview_node_launch_walk_fwd_cb(treeview_node *n, void *ctx,
 		struct treeview_node_msg msg;
 		msg.msg = TREE_MSG_NODE_LAUNCH;
 		msg.data.node_launch.mouse = BROWSER_MOUSE_HOVER;
-		lw->tree->callbacks->entry(msg, n->client_data);
+		ret = lw->tree->callbacks->entry(msg, n->client_data);
 	}
 
-	return NSERROR_OK;
+	return ret;
 }
 /**
  * Launch a selection.
@@ -2560,7 +2668,7 @@ static bool treeview_keyboard_navigation(treeview *tree, uint32_t key,
 	redraw = treeview_clear_selection(tree, rect);
 
 	switch (key) {
-	case KEY_LEFT:
+	case NS_KEY_LEFT:
 		if (ns.curr != NULL &&
 				ns.curr->parent != NULL &&
 				ns.curr->parent->type != TREE_NODE_ROOT) {
@@ -2573,7 +2681,7 @@ static bool treeview_keyboard_navigation(treeview *tree, uint32_t key,
 		}
 		break;
 
-	case KEY_RIGHT:
+	case NS_KEY_RIGHT:
 		if (ns.curr != NULL) {
 			if (!(ns.curr->flags & TV_NFLAGS_EXPANDED)) {
 				/* Toggle node to expanded */
@@ -2599,14 +2707,14 @@ static bool treeview_keyboard_navigation(treeview *tree, uint32_t key,
 		}
 		break;
 
-	case KEY_UP:
+	case NS_KEY_UP:
 		if (ns.prev != NULL) {
 			/* Step to previous node */
 			ns.prev->flags |= TV_NFLAGS_SELECTED;
 		}
 		break;
 
-	case KEY_DOWN:
+	case NS_KEY_DOWN:
 		if (ns.next != NULL) {
 			/* Step to next node */
 			ns.next->flags |= TV_NFLAGS_SELECTED;
@@ -2642,11 +2750,11 @@ bool treeview_keypress(treeview *tree, uint32_t key)
 	/* Pass to textarea, if editing in progress */
 	if (tree->edit.textarea != NULL) {
 		switch (key) {
-		case KEY_ESCAPE:
+		case NS_KEY_ESCAPE:
 			treeview_edit_cancel(tree, true);
 			return true;
-		case KEY_NL:
-		case KEY_CR:
+		case NS_KEY_NL:
+		case NS_KEY_CR:
 			treeview_edit_done(tree);
 			return true;
 		default:
@@ -2656,14 +2764,14 @@ bool treeview_keypress(treeview *tree, uint32_t key)
 
 	/* Keypress to be handled by treeview */
 	switch (key) {
-	case KEY_SELECT_ALL:
+	case NS_KEY_SELECT_ALL:
 		redraw = treeview_select_all(tree, &r);
 		break;
-	case KEY_COPY_SELECTION:
-		/* TODO: Copy selection as text */
+	case NS_KEY_COPY_SELECTION:
+		treeview_copy_selection(tree);
 		break;
-	case KEY_DELETE_LEFT:
-	case KEY_DELETE_RIGHT:
+	case NS_KEY_DELETE_LEFT:
+	case NS_KEY_DELETE_RIGHT:
 		redraw = treeview_delete_selection(tree, &r);
 
 		if (tree->flags & TREEVIEW_DEL_EMPTY_DIRS) {
@@ -2679,18 +2787,18 @@ bool treeview_keypress(treeview *tree, uint32_t key)
 			}
 		}
 		break;
-	case KEY_CR:
-	case KEY_NL:
+	case NS_KEY_CR:
+	case NS_KEY_NL:
 		treeview_launch_selection(tree);
 		break;
-	case KEY_ESCAPE:
-	case KEY_CLEAR_SELECTION:
+	case NS_KEY_ESCAPE:
+	case NS_KEY_CLEAR_SELECTION:
 		redraw = treeview_clear_selection(tree, &r);
 		break;
-	case KEY_LEFT:
-	case KEY_RIGHT:
-	case KEY_UP:
-	case KEY_DOWN:
+	case NS_KEY_LEFT:
+	case NS_KEY_RIGHT:
+	case NS_KEY_UP:
+	case NS_KEY_DOWN:
 		redraw = treeview_keyboard_navigation(tree, key, &r);
 		break;
 	default:
@@ -3481,7 +3589,7 @@ static void treeview_init_plot_styles(int font_pt_size)
 
 	/* Text colour */
 	plot_style_even.text.family = PLOT_FONT_FAMILY_SANS_SERIF;
-	plot_style_even.text.size = font_pt_size * FONT_SIZE_SCALE;
+	plot_style_even.text.size = font_pt_size;
 	plot_style_even.text.weight = 400;
 	plot_style_even.text.flags = FONTF_NONE;
 	plot_style_even.text.foreground = ns_system_colour_char("WindowText");
@@ -3531,8 +3639,10 @@ static void treeview_init_plot_styles(int font_pt_size)
 /**
  * Callback for hlcache.
  */
-static nserror treeview_res_cb(hlcache_handle *handle,
-		const hlcache_event *event, void *pw)
+static nserror
+treeview_res_cb(struct hlcache_handle *handle,
+		const hlcache_event *event,
+		void *pw)
 {
 	struct treeview_resource *r = pw;
 
@@ -3595,12 +3705,12 @@ static struct bitmap * treeview_generate_triangle_bitmap(
 	colour colour4 = fg;
 
 	/* Create the bitmap */
-	b = bitmap_create(size, size, BITMAP_NEW | BITMAP_OPAQUE);
+	b = guit->bitmap->create(size, size, BITMAP_NEW | BITMAP_OPAQUE);
 	if (b == NULL)
 		return NULL;
 
-	rpos = bitmap_get_buffer(b);
-	stride = bitmap_get_rowstride(b);
+	rpos = guit->bitmap->get_buffer(b);
+	stride = guit->bitmap->get_rowstride(b);
 
 	/* Draw the triangle */
 	for (y = 0; y < size; y++) {
@@ -3667,7 +3777,7 @@ static struct bitmap * treeview_generate_triangle_bitmap(
 		rpos += stride;
 	}
 
-	bitmap_modified(b);
+	guit->bitmap->modified(b);
 
 	return b;
 }
@@ -3690,29 +3800,29 @@ static struct bitmap * treeview_generate_copy_bitmap(
 	if (orig == NULL)
 		return NULL;
 
-	assert(size == bitmap_get_width(orig));
-	assert(size == bitmap_get_height(orig));
+	assert(size == guit->bitmap->get_width(orig));
+	assert(size == guit->bitmap->get_height(orig));
 
 	/* Create the bitmap */
-	b = bitmap_create(size, size, BITMAP_NEW | BITMAP_OPAQUE);
+	b = guit->bitmap->create(size, size, BITMAP_NEW | BITMAP_OPAQUE);
 	if (b == NULL)
 		return NULL;
 
-	stride = bitmap_get_rowstride(b);
-	assert(stride == bitmap_get_rowstride(orig));
+	stride = guit->bitmap->get_rowstride(b);
+	assert(stride == guit->bitmap->get_rowstride(orig));
 
-	data = bitmap_get_buffer(b);
-	orig_data = bitmap_get_buffer(orig);
+	data = guit->bitmap->get_buffer(b);
+	orig_data = guit->bitmap->get_buffer(orig);
 
 	/* Copy the bitmap */
 	memcpy(data, orig_data, stride * size);
 
-	bitmap_modified(b);
+	guit->bitmap->modified(b);
 
 	/* We've not modified the original image, but we called
 	 * bitmap_get_buffer(), so we need to pair that with a
 	 * bitmap_modified() call to appease certain front ends. */
-	bitmap_modified(orig);
+	guit->bitmap->modified(orig);
 
 	return b;
 }
@@ -3738,19 +3848,19 @@ static struct bitmap * treeview_generate_rotate_bitmap(
 	if (orig == NULL)
 		return NULL;
 
-	assert(size == bitmap_get_width(orig));
-	assert(size == bitmap_get_height(orig));
+	assert(size == guit->bitmap->get_width(orig));
+	assert(size == guit->bitmap->get_height(orig));
 
 	/* Create the bitmap */
-	b = bitmap_create(size, size, BITMAP_NEW | BITMAP_OPAQUE);
+	b = guit->bitmap->create(size, size, BITMAP_NEW | BITMAP_OPAQUE);
 	if (b == NULL)
 		return NULL;
 
-	stride = bitmap_get_rowstride(b);
-	assert(stride == bitmap_get_rowstride(orig));
+	stride = guit->bitmap->get_rowstride(b);
+	assert(stride == guit->bitmap->get_rowstride(orig));
 
-	rpos = bitmap_get_buffer(b);
-	orig_data = bitmap_get_buffer(orig);
+	rpos = guit->bitmap->get_buffer(b);
+	orig_data = guit->bitmap->get_buffer(orig);
 
 	/* Copy the rotated bitmap */
 	for (y = 0; y < size; y++) {
@@ -3762,18 +3872,19 @@ static struct bitmap * treeview_generate_rotate_bitmap(
 			*(pos++) = *(orig_pos++);
 			*(pos++) = *(orig_pos);
 			*(pos++) = 0xff;
-			
+		
 		}
 
 		rpos += stride;
 	}
 
-	bitmap_modified(b);
+	guit->bitmap->modified(b);
 
 	/* We've not modified the original image, but we called
 	 * bitmap_get_buffer(), so we need to pair that with a
-	 * bitmap_modified() call to appease certain front ends. */
-	bitmap_modified(orig);
+	 * bitmap_modified() call to appease certain front ends.
+	 */
+	guit->bitmap->modified(orig);
 
 	return b;
 }
@@ -3838,23 +3949,29 @@ static nserror treeview_init_furniture(void)
 
 
 /* Exported interface, documented in treeview.h */
-nserror treeview_init(int font_pt_size)
+nserror treeview_init(void)
 {
-	int font_px_size;
+	long long font_px_size;
+	long long font_pt_size;
 	nserror err;
 
-	if (tree_g.initialised == true)
+	if (tree_g.initialised > 0) {
+		tree_g.initialised++;
 		return NSERROR_OK;
+	}
 
-	LOG(("Initialising treeview module"));
+	LOG("Initialising treeview module");
 
-	if (font_pt_size <= 0)
-		font_pt_size = 11;
+	font_pt_size = nsoption_int(treeview_font_size);
+	if (font_pt_size <= 0) {
+		font_pt_size = 11 * 10;
+	}
 
-	font_px_size = (font_pt_size * FIXTOINT(nscss_screen_dpi) + 36) / 72;
+	font_px_size = (font_pt_size * FIXTOINT(nscss_screen_dpi) /
+			10 + 36) / 72;
 	tree_g.line_height = (font_px_size * 8 + 3) / 6;
 
-	treeview_init_plot_styles(font_pt_size);
+	treeview_init_plot_styles(font_pt_size * FONT_SIZE_SCALE / 10);
 	treeview_init_resources();
 	err = treeview_init_furniture();
 	if (err != NSERROR_OK)
@@ -3866,9 +3983,9 @@ nserror treeview_init(int font_pt_size)
 	tree_g.icon_step = 23;
 	tree_g.move_offset = 18;
 
-	tree_g.initialised = true;
+	tree_g.initialised++;
 
-	LOG(("Initialised treeview module"));
+	LOG("Initialised treeview module");
 
 	return NSERROR_OK;
 }
@@ -3879,24 +3996,33 @@ nserror treeview_fini(void)
 {
 	int i;
 
-	LOG(("Finalising treeview module"));
+	if (tree_g.initialised > 1) {
+		tree_g.initialised--;
+		return NSERROR_OK;
+
+	} else if (tree_g.initialised == 0) {
+		LOG("Warning: tried to finalise uninitialised treeview module");
+		return NSERROR_OK;
+	}
+
+	LOG("Finalising treeview module");
 
 	for (i = 0; i < TREE_RES_LAST; i++) {
 		hlcache_handle_release(treeview_res[i].c);
 	}
 
-	bitmap_destroy(plot_style_odd.furn[TREE_FURN_EXPAND].bmp);
-	bitmap_destroy(plot_style_odd.furn[TREE_FURN_EXPAND].sel);
-	bitmap_destroy(plot_style_even.furn[TREE_FURN_EXPAND].bmp);
-	bitmap_destroy(plot_style_even.furn[TREE_FURN_EXPAND].sel);
-	bitmap_destroy(plot_style_odd.furn[TREE_FURN_CONTRACT].bmp);
-	bitmap_destroy(plot_style_odd.furn[TREE_FURN_CONTRACT].sel);
-	bitmap_destroy(plot_style_even.furn[TREE_FURN_CONTRACT].bmp);
-	bitmap_destroy(plot_style_even.furn[TREE_FURN_CONTRACT].sel);
+	guit->bitmap->destroy(plot_style_odd.furn[TREE_FURN_EXPAND].bmp);
+	guit->bitmap->destroy(plot_style_odd.furn[TREE_FURN_EXPAND].sel);
+	guit->bitmap->destroy(plot_style_even.furn[TREE_FURN_EXPAND].bmp);
+	guit->bitmap->destroy(plot_style_even.furn[TREE_FURN_EXPAND].sel);
+	guit->bitmap->destroy(plot_style_odd.furn[TREE_FURN_CONTRACT].bmp);
+	guit->bitmap->destroy(plot_style_odd.furn[TREE_FURN_CONTRACT].sel);
+	guit->bitmap->destroy(plot_style_even.furn[TREE_FURN_CONTRACT].bmp);
+	guit->bitmap->destroy(plot_style_even.furn[TREE_FURN_CONTRACT].sel);
 
-	tree_g.initialised = false;
+	tree_g.initialised--;
 
-	LOG(("Finalised treeview module"));
+	LOG("Finalised treeview module");
 
 	return NSERROR_OK;
 }
