@@ -26,12 +26,14 @@
 #include <assert.h>
 #include <stdbool.h>
 #include <string.h>
-#include "css/css.h"
+
+#include "utils/log.h"
+#include "utils/errors.h"
 #include "css/select.h"
+
 #include "render/box.h"
 #include "render/html_internal.h"
 #include "render/table.h"
-#include "utils/log.h"
 
 /* Define to enable box normalise debug */
 #undef BOX_NORMALISE_DEBUG
@@ -80,9 +82,9 @@ static bool box_normalise_inline_container(struct box *cont, html_content *c);
 /**
  * Ensure the box tree is correctly nested by adding and removing nodes.
  *
- * \param  block     box of type BLOCK, INLINE_BLOCK, or TABLE_CELL
- * \param  box_pool  pool to allocate new boxes in
- * \return  true on success, false on memory exhaustion
+ * \param block  box of type BLOCK, INLINE_BLOCK, or TABLE_CELL
+ * \param c      content of boxes
+ * \return true on success, false on memory exhaustion
  *
  * The tree is modified to satisfy the following:
  * \code
@@ -109,7 +111,7 @@ bool box_normalise_block(struct box *block, html_content *c)
 	assert(block != NULL);
 
 #ifdef BOX_NORMALISE_DEBUG
-	LOG(("block %p, block->type %u", block, block->type));
+	LOG("block %p, block->type %u", block, block->type);
 #endif
 
 	assert(block->type == BOX_BLOCK || block->type == BOX_INLINE_BLOCK ||
@@ -117,7 +119,7 @@ bool box_normalise_block(struct box *block, html_content *c)
 
 	for (child = block->children; child != NULL; child = next_child) {
 #ifdef BOX_NORMALISE_DEBUG
-		LOG(("child %p, child->type = %d", child, child->type));
+		LOG("child %p, child->type = %d", child, child->type);
 #endif
 
 		next_child = child->next;	/* child may be destroyed */
@@ -221,7 +223,7 @@ bool box_normalise_table(struct box *table, html_content * c)
 	assert(table->type == BOX_TABLE);
 
 #ifdef BOX_NORMALISE_DEBUG
-	LOG(("table %p", table));
+	LOG("table %p", table);
 #endif
 
 	col_info.num_columns = 1;
@@ -335,7 +337,7 @@ bool box_normalise_table(struct box *table, html_content * c)
 		struct box *row;
 
 #ifdef BOX_NORMALISE_DEBUG
-		LOG(("table->children == 0, creating implied row"));
+		LOG("table->children == 0, creating implied row");
 #endif
 
 		assert(table->style != NULL);
@@ -397,7 +399,7 @@ bool box_normalise_table(struct box *table, html_content * c)
 		return false;
 
 #ifdef BOX_NORMALISE_DEBUG
-	LOG(("table %p done", table));
+	LOG("table %p done", table);
 #endif
 
 	return true;
@@ -583,7 +585,7 @@ bool box_normalise_table_row_group(struct box *row_group,
 	assert(row_group->type == BOX_TABLE_ROW_GROUP);
 
 #ifdef BOX_NORMALISE_DEBUG
-	LOG(("row_group %p", row_group));
+	LOG("row_group %p", row_group);
 #endif
 
 	for (child = row_group->children; child != NULL; child = next_child) {
@@ -675,7 +677,7 @@ bool box_normalise_table_row_group(struct box *row_group,
 
 	if (row_group->children == NULL) {
 #ifdef BOX_NORMALISE_DEBUG
-		LOG(("row_group->children == 0, inserting implied row"));
+		LOG("row_group->children == 0, inserting implied row");
 #endif
 
 		assert(row_group->style != NULL);
@@ -710,7 +712,7 @@ bool box_normalise_table_row_group(struct box *row_group,
 	row_group->rows = group_row_count;
 
 #ifdef BOX_NORMALISE_DEBUG
-	LOG(("row_group %p done", row_group));
+	LOG("row_group %p done", row_group);
 #endif
 
 	return true;
@@ -732,7 +734,7 @@ bool box_normalise_table_row(struct box *row,
 	assert(row->type == BOX_TABLE_ROW);
 
 #ifdef BOX_NORMALISE_DEBUG
-	LOG(("row %p", row));
+	LOG("row %p", row);
 #endif
 
 	for (child = row->children; child != NULL; child = next_child) {
@@ -841,7 +843,7 @@ bool box_normalise_table_row(struct box *row,
 	col_info->num_rows++;
 
 #ifdef BOX_NORMALISE_DEBUG
-	LOG(("row %p done", row));
+	LOG("row %p done", row);
 #endif
 
 	return true;
@@ -932,7 +934,7 @@ bool box_normalise_inline_container(struct box *cont, html_content * c)
 	assert(cont->type == BOX_INLINE_CONTAINER);
 
 #ifdef BOX_NORMALISE_DEBUG
-	LOG(("cont %p", cont));
+	LOG("cont %p", cont);
 #endif
 
 	for (child = cont->children; child != NULL; child = next_child) {
@@ -995,7 +997,7 @@ bool box_normalise_inline_container(struct box *cont, html_content * c)
 	}
 
 #ifdef BOX_NORMALISE_DEBUG
-	LOG(("cont %p done", cont));
+	LOG("cont %p done", cont);
 #endif
 
 	return true;
