@@ -26,29 +26,12 @@
  */
 
 #include <assert.h>
-#include <ctype.h>
 #include <limits.h>
 #include <stdbool.h>
 #include <stdio.h>
 #include <string.h>
 #include <dom/dom.h>
 
-#include "content/fetch.h"
-#include "content/hlcache.h"
-#include "css/css.h"
-#include "css/utils.h"
-#include "desktop/mouse.h"
-#include "desktop/knockout.h"
-#include "desktop/plot_style.h"
-#include "desktop/plotters.h"
-#include "desktop/scrollbar.h"
-#include "desktop/textarea.h"
-#include "render/box.h"
-#include "render/font.h"
-#include "render/form.h"
-#include "render/html.h"
-#include "render/html_internal.h"
-#include "render/layout.h"
 #include "utils/corestrings.h"
 #include "utils/log.h"
 #include "utils/messages.h"
@@ -56,6 +39,26 @@
 #include "utils/url.h"
 #include "utils/utf8.h"
 #include "utils/utils.h"
+#include "utils/ascii.h"
+#include "content/fetch.h"
+#include "content/hlcache.h"
+#include "css/utils.h"
+#include "netsurf/browser_window.h"
+#include "netsurf/mouse.h"
+#include "desktop/knockout.h"
+#include "desktop/plot_style.h"
+#include "netsurf/plotters.h"
+#include "desktop/scrollbar.h"
+#include "desktop/textarea.h"
+#include "netsurf/misc.h"
+#include "desktop/gui_internal.h"
+
+#include "render/box.h"
+#include "render/font.h"
+#include "render/form_internal.h"
+#include "render/html.h"
+#include "render/html_internal.h"
+#include "render/layout.h"
 
 #define MAX_SELECT_HEIGHT 210
 #define SELECT_LINE_SPACING 0.2
@@ -94,18 +97,7 @@ static void form_select_menu_clicked(struct form_control *control,
 static void form_select_menu_scroll_callback(void *client_data,
 		struct scrollbar_msg_data *scrollbar_data);
 
-/**
- * Create a struct form.
- *
- * \param  node    DOM node associated with form
- * \param  action  URL to submit form to, or NULL for default
- * \param  target  Target frame of form, or NULL for default
- * \param  method  method and enctype
- * \param  charset acceptable encodings for form submission, or NULL
- * \param  doc_charset  encoding of containing document, or NULL
- * \param  html  HTML content containing form
- * \return  a new structure, or NULL on memory exhaustion
- */
+/* exported interface documented in render/form_internal.h */
 struct form *form_new(void *node, const char *action, const char *target, 
 		form_method method, const char *charset, 
 		const char *doc_charset)
@@ -155,17 +147,7 @@ struct form *form_new(void *node, const char *action, const char *target,
 }
 
 
-/**
- * Free a form, and any controls it owns.
- *
- * \param form  The form to free
- *
- * \note There may exist controls attached to box tree nodes which are not
- * associated with any form. These will leak at present. Ideally, they will
- * be cleaned up when the box tree is destroyed. As that currently happens
- * via talloc, this won't happen. These controls are distinguishable, as their
- * form field will be NULL.
- */
+/* exported interface documented in render/form_internal.h */
 void form_free(struct form *form)
 {
 	struct form_control *c, *d;
@@ -184,13 +166,7 @@ void form_free(struct form *form)
 	free(form);
 }
 
-/**
- * Create a struct form_control.
- *
- * \param  node  Associated DOM node
- * \param  type  control type
- * \return  a new structure, or NULL on memory exhaustion
- */
+/* exported interface documented in render/form_internal.h */
 struct form_control *form_new_control(void *node, form_control_type type)
 {
 	struct form_control *control;
@@ -240,6 +216,10 @@ void form_add_control(struct form *form, struct form_control *control)
  */
 void form_free_control(struct form_control *control)
 {
+	struct form_control *c;
+	assert(control != NULL);
+
+	LOG("Control:%p name:%p value:%p initial:%p", control, control->name, control->value, control->initial_value);
 	free(control->name);
 	free(control->value);
 	free(control->initial_value);
@@ -250,23 +230,47 @@ void form_free_control(struct form_control *control)
 		for (option = control->data.select.items; option;
 				option = next) {
 			next = option->next;
+			LOG("select option:%p text:%p value:%p", option, option->text, option->value);
 			free(option->text);
 			free(option->value);
 			free(option);
 		}
-		if (control->data.select.menu != NULL)
+		if (control->data.select.menu != NULL) {
 			form_free_select_menu(control);
+		}
 	}
 
 	if (control->type == GADGET_TEXTAREA ||
 			control->type == GADGET_TEXTBOX ||
 			control->type == GADGET_PASSWORD) {
 
-		if (control->data.text.initial != NULL)
+		if (control->data.text.initial != NULL) {
 			dom_string_unref(control->data.text.initial);
+		}
 
-		if (control->data.text.ta != NULL)
+		if (control->data.text.ta != NULL) {
 			textarea_destroy(control->data.text.ta);
+		}
+	}
+
+	/* unlink the control from the form */
+	if (control->form != NULL) {
+		for (c = control->form->controls; c != NULL; c = c->next) {
+			if (c->next == control) {
+				c->next = control->next;
+				if (control->form->last_control == control)
+					control->form->last_control = c;
+				break;
+			}
+			if (c == control) {
+				/* can only happen if control was first control */
+				control->form->controls = control->next;
+				if (control->form->last_control == control)
+					control->form->controls =
+						control->form->last_control = NULL;
+				break;
+			}
+		}
 	}
 
 	free(control);
@@ -321,24 +325,7 @@ bool form_add_option(struct form_control *control, char *value, char *text,
 }
 
 
-/**
- * Identify 'successful' controls via the DOM.
- *
- * All text strings in the successful controls list will be in the charset most
- * appropriate for submission. Therefore, no utf8_to_* processing should be
- * performed upon them.
- *
- * \todo The chosen charset needs to be made available such that it can be
- * included in the submission request (e.g. in the fetch's Content-Type header)
- *
- * \param  form           form to search for successful controls
- * \param  submit_button  control used to submit the form, if any
- * \param  successful_controls  updated to point to linked list of
- *                        fetch_multipart_data, 0 if no controls
- * \return  true on success, false on memory exhaustion
- *
- * See HTML 4.01 section 17.13.2.
- */
+/* exported interface documented in render/form_internal.h */
 bool form_successful_controls_dom(struct form *_form,
 				  struct form_control *_submit_button,
 				  struct fetch_multipart_data **successful_controls)
@@ -359,12 +346,10 @@ bool form_successful_controls_dom(struct form *_form,
 	last_success = &sentinel;
 	sentinel.next = NULL;
 	
-	LOG(("XYZZY: Yay, let's look for a form"));
-	
 	/** \todo Replace this call with something DOMish */
 	charset = form_acceptable_charset(_form);
 	if (charset == NULL) {
-		LOG(("failed to find charset"));
+		LOG("failed to find charset");
 		return false;
 	}
 
@@ -378,22 +363,20 @@ bool form_successful_controls_dom(struct form *_form,
 	err = dom_html_form_element_get_elements(form, &form_elements);
 	
 	if (err != DOM_NO_ERR) {
-		LOG(("Could not get form elements"));
+		LOG("Could not get form elements");
 		goto dom_no_memory;
 	}
 	
-	LOG(("Reffed %p", form_elements));
 	
 	err = dom_html_collection_get_length(form_elements, &element_count);
 	
 	if (err != DOM_NO_ERR) {
-		LOG(("Could not get form element count"));
+		LOG("Could not get form element count");
 		goto dom_no_memory;
 	}
 	
 	for (index = 0; index < element_count; index++) {
 		if (form_element != NULL) {
-			LOG(("Unreffed %p", form_element));
 			dom_node_unref(form_element);
 			form_element = NULL;
 		}
@@ -420,36 +403,35 @@ bool form_successful_controls_dom(struct form *_form,
 		err = dom_html_collection_item(form_elements,
 					       index, &form_element);
 		if (err != DOM_NO_ERR) {
-			LOG(("Could not retrieve form element %d", index));
+			LOG("Could not retrieve form element %d", index);
 			goto dom_no_memory;
 		}
-		LOG(("Reffed %p", form_element));
+
 		/* Form elements are one of:
+		 *   HTMLButtonElement
 		 *   HTMLInputElement
 		 *   HTMLTextAreaElement
 		 *   HTMLSelectElement
 		 */
 		err = dom_node_get_node_name(form_element, &nodename);
 		if (err != DOM_NO_ERR) {
-			LOG(("Could not get node name"));
+			LOG("Could not get node name");
 			goto dom_no_memory;
 		}
-		LOG(("Found a node(%p): `%*s`", nodename,
-		     dom_string_byte_length(nodename),
-		     dom_string_data(nodename)));
+
 		if (dom_string_isequal(nodename, corestring_dom_TEXTAREA)) {
 			err = dom_html_text_area_element_get_disabled(
 				(dom_html_text_area_element *)form_element,
 				&element_disabled);
 			if (err != DOM_NO_ERR) {
-				LOG(("Could not get text area disabled property"));
+				LOG("Could not get text area disabled property");
 				goto dom_no_memory;
 			}
 			err = dom_html_text_area_element_get_name(
 				(dom_html_text_area_element *)form_element,
 				&inputname);
 			if (err != DOM_NO_ERR) {
-				LOG(("Could not get text area name property"));
+				LOG("Could not get text area name property");
 				goto dom_no_memory;
 			}
 		} else if (dom_string_isequal(nodename, corestring_dom_SELECT)) {
@@ -457,14 +439,14 @@ bool form_successful_controls_dom(struct form *_form,
 				(dom_html_select_element *)form_element,
 				&element_disabled);
 			if (err != DOM_NO_ERR) {
-				LOG(("Could not get select disabled property"));
+				LOG("Could not get select disabled property");
 				goto dom_no_memory;
 			}
 			err = dom_html_select_element_get_name(
 				(dom_html_select_element *)form_element,
 				&inputname);
 			if (err != DOM_NO_ERR) {
-				LOG(("Could not get select name property"));
+				LOG("Could not get select name property");
 				goto dom_no_memory;
 			}
 		} else if (dom_string_isequal(nodename, corestring_dom_INPUT)) {
@@ -472,24 +454,36 @@ bool form_successful_controls_dom(struct form *_form,
 				(dom_html_input_element *)form_element,
 				&element_disabled);
 			if (err != DOM_NO_ERR) {
-				LOG(("Could not get input disabled property"));
+				LOG("Could not get input disabled property");
 				goto dom_no_memory;
 			}
 			err = dom_html_input_element_get_name(
 				(dom_html_input_element *)form_element,
 				&inputname);
 			if (err != DOM_NO_ERR) {
-				LOG(("Could not get input name property"));
+				LOG("Could not get input name property");
 				goto dom_no_memory;
 			}
 		} else if (dom_string_isequal(nodename, corestring_dom_BUTTON)) {
-			/* It was a button, no fair */
-			continue;
+			err = dom_html_button_element_get_disabled(
+				(dom_html_button_element *)form_element,
+				&element_disabled);
+			if (err != DOM_NO_ERR) {
+				LOG("Could not get button disabled property");
+				goto dom_no_memory;
+			}
+			err = dom_html_button_element_get_name(
+				(dom_html_button_element *)form_element,
+				&inputname);
+			if (err != DOM_NO_ERR) {
+				LOG("Could not get button name property");
+				goto dom_no_memory;
+			}
 		} else {
 			/* Unknown element type came through! */
-			LOG(("Unknown element type: %*s",
-			     dom_string_byte_length(nodename),
-			     dom_string_data(nodename)));
+			LOG("Unknown element type: %*s",
+			    (int)dom_string_byte_length(nodename),
+			    dom_string_data(nodename));
 			goto dom_no_memory;
 		}
 		if (element_disabled)
@@ -502,7 +496,7 @@ bool form_successful_controls_dom(struct form *_form,
 				(dom_html_text_area_element *)form_element,
 				&inputvalue);
 			if (err != DOM_NO_ERR) {
-				LOG(("Could not get text area content"));
+				LOG("Could not get text area content");
 				goto dom_no_memory;
 			}
 		} else if (dom_string_isequal(nodename, corestring_dom_SELECT)) {
@@ -511,13 +505,13 @@ bool form_successful_controls_dom(struct form *_form,
 				(dom_html_select_element *)form_element,
 				&options);
 			if (err != DOM_NO_ERR) {
-				LOG(("Could not get select options collection"));
+				LOG("Could not get select options collection");
 				goto dom_no_memory;
 			}
 			err = dom_html_options_collection_get_length(
 				options, &options_count);
 			if (err != DOM_NO_ERR) {
-				LOG(("Could not get select options collection length"));
+				LOG("Could not get select options collection length");
 				goto dom_no_memory;
 			}
 			for(option_index = 0; option_index < options_count;
@@ -534,14 +528,14 @@ bool form_successful_controls_dom(struct form *_form,
 				err = dom_html_options_collection_item(
 					options, option_index, &option_element);
 				if (err != DOM_NO_ERR) {
-					LOG(("Could not get options item %d", option_index));
+					LOG("Could not get options item %d", option_index);
 					goto dom_no_memory;
 				}
 				err = dom_html_option_element_get_selected(
 					(dom_html_option_element *)option_element,
 					&selected);
 				if (err != DOM_NO_ERR) {
-					LOG(("Could not get option selected property"));
+					LOG("Could not get option selected property");
 					goto dom_no_memory;
 				}
 				if (!selected)
@@ -550,13 +544,13 @@ bool form_successful_controls_dom(struct form *_form,
 					(dom_html_option_element *)option_element,
 					&inputvalue);
 				if (err != DOM_NO_ERR) {
-					LOG(("Could not get option value"));
+					LOG("Could not get option value");
 					goto dom_no_memory;
 				}
 				
 				success_new = calloc(1, sizeof(*success_new));
 				if (success_new == NULL) {
-					LOG(("Could not allocate data for option"));
+					LOG("Could not allocate data for option");
 					goto dom_no_memory;
 				}
 		
@@ -565,16 +559,48 @@ bool form_successful_controls_dom(struct form *_form,
 		
 				success_new->name = ENCODE_ITEM(inputname);
 				if (success_new->name == NULL) {
-					LOG(("Could not encode name for option"));
+					LOG("Could not encode name for option");
 					goto dom_no_memory;
 				}
 				success_new->value = ENCODE_ITEM(inputvalue);
 				if (success_new->value == NULL) {
-					LOG(("Could not encode value for option"));
+					LOG("Could not encode value for option");
 					goto dom_no_memory;
 				}
 			}
 			continue;
+		} else if (dom_string_isequal(nodename, corestring_dom_BUTTON)) {
+			err = dom_html_button_element_get_type(
+				(dom_html_button_element *) form_element,
+				&inputtype);
+			if (err != DOM_NO_ERR) {
+				LOG("Could not get button element type");
+				goto dom_no_memory;
+			}
+			if (dom_string_caseless_isequal(
+				    inputtype, corestring_dom_submit)) {
+
+				if (submit_button == NULL && !had_submit) {
+					/* no button used, and first submit
+					 * node found, so use it
+					 */
+					had_submit = true;
+				} else if ((dom_node *)submit_button !=
+					   (dom_node *)form_element) {
+					continue;
+				}
+
+				err = dom_html_button_element_get_value(
+					(dom_html_button_element *)form_element,
+					&inputvalue);
+				if (err != DOM_NO_ERR) {
+					LOG("Could not get submit button value");
+					goto dom_no_memory;
+				}
+				/* Drop through to report successful button */
+			} else {
+				continue;
+			}
 		} else if (dom_string_isequal(nodename, corestring_dom_INPUT)) {
 			/* Things to consider here */
 			/* Buttons -- only if the successful control */
@@ -585,25 +611,27 @@ bool form_successful_controls_dom(struct form *_form,
 				(dom_html_input_element *) form_element,
 				&inputtype);
 			if (err != DOM_NO_ERR) {
-				LOG(("Could not get input element type"));
+				LOG("Could not get input element type");
 				goto dom_no_memory;
 			}
 			if (dom_string_caseless_isequal(
 				    inputtype, corestring_dom_submit)) {
-				LOG(("Examining submit button"));
-				if (submit_button == NULL && !had_submit)
+
+				if (submit_button == NULL && !had_submit) {
 					/* no button used, and first submit
 					 * node found, so use it
 					 */
 					had_submit = true;
-				else if ((dom_node *)submit_button !=
-					 (dom_node *)form_element)
+				} else if ((dom_node *)submit_button !=
+					   (dom_node *)form_element) {
 					continue;
+				}
+
 				err = dom_html_input_element_get_value(
 					(dom_html_input_element *)form_element,
 					&inputvalue);
 				if (err != DOM_NO_ERR) {
-					LOG(("Could not get submit button value"));
+					LOG("Could not get submit button value");
 					goto dom_no_memory;
 				}
 				/* Drop through to report the successful button */
@@ -612,7 +640,6 @@ bool form_successful_controls_dom(struct form *_form,
 				/* We *ONLY* use an image input if it was the
 				 * thing which activated us
 				 */
-				LOG(("Examining image button"));
 				 if ((dom_node *)submit_button !=
 				     (dom_node *)form_element)
 					 continue;
@@ -622,11 +649,11 @@ bool form_successful_controls_dom(struct form *_form,
 					 corestring_dom___ns_key_image_coords_node_data,
 					 &coords);
 				 if (err != DOM_NO_ERR) {
-					 LOG(("Could not get image XY data"));
+					 LOG("Could not get image XY data");
 					 goto dom_no_memory;
 				 }
 				 if (coords == NULL) {
-					 LOG(("No XY data on the image input"));
+					 LOG("No XY data on the image input");
 					 goto dom_no_memory;
 				 }
 				 
@@ -635,7 +662,7 @@ bool form_successful_controls_dom(struct form *_form,
 				 success_new = calloc(1, sizeof(*success_new));
 				 if (success_new == NULL) {
 					 free(basename);
-					 LOG(("Could not allocate data for image.x"));
+					 LOG("Could not allocate data for image.x");
 					 goto dom_no_memory;
 				 }
 				 
@@ -645,13 +672,13 @@ bool form_successful_controls_dom(struct form *_form,
 				 success_new->name = malloc(strlen(basename) + 3);
 				 if (success_new->name == NULL) {
 					 free(basename);
-					 LOG(("Could not allocate name for image.x"));
+					 LOG("Could not allocate name for image.x");
 					 goto dom_no_memory;
 				 }
 				 success_new->value = malloc(20);
 				 if (success_new->value == NULL) {
 					 free(basename);
-					 LOG(("Could not allocate value for image.x"));
+					 LOG("Could not allocate value for image.x");
 					 goto dom_no_memory;
 				 }
 				 sprintf(success_new->name, "%s.x", basename);
@@ -660,7 +687,7 @@ bool form_successful_controls_dom(struct form *_form,
 				 success_new = calloc(1, sizeof(*success_new));
 				 if (success_new == NULL) {
 					 free(basename);
-					 LOG(("Could not allocate data for image.y"));
+					 LOG("Could not allocate data for image.y");
 					 goto dom_no_memory;
 				 }
 				 
@@ -670,13 +697,13 @@ bool form_successful_controls_dom(struct form *_form,
 				 success_new->name = malloc(strlen(basename) + 3);
 				 if (success_new->name == NULL) {
 					 free(basename);
-					 LOG(("Could not allocate name for image.y"));
+					 LOG("Could not allocate name for image.y");
 					 goto dom_no_memory;
 				 }
 				 success_new->value = malloc(20);
 				 if (success_new->value == NULL) {
 					 free(basename);
-					 LOG(("Could not allocate value for image.y"));
+					 LOG("Could not allocate value for image.y");
 					 goto dom_no_memory;
 				 }
 				 sprintf(success_new->name, "%s.y", basename);
@@ -687,12 +714,11 @@ bool form_successful_controls_dom(struct form *_form,
 					   inputtype, corestring_dom_radio) ||
 				   dom_string_caseless_isequal(
 					   inputtype, corestring_dom_checkbox)) {
-				LOG(("Examining radio or checkbox"));
 				err = dom_html_input_element_get_checked(
 					(dom_html_input_element *)form_element,
 					&checked);
 				if (err != DOM_NO_ERR) {
-					LOG(("Could not get input element checked"));
+					LOG("Could not get input element checked");
 					goto dom_no_memory;
 				}
 				if (!checked)
@@ -701,21 +727,22 @@ bool form_successful_controls_dom(struct form *_form,
 					(dom_html_input_element *)form_element,
 					&inputvalue);
 				if (err != DOM_NO_ERR) {
-					LOG(("Could not get input element value"));
+					LOG("Could not get input element value");
 					goto dom_no_memory;
 				}
-				if (inputvalue == NULL)
+				if (inputvalue == NULL) {
 					inputvalue = dom_string_ref(
 						corestring_dom_on);
+				}
 				/* Fall through to simple allocation */
 			} else if (dom_string_caseless_isequal(
 					   inputtype, corestring_dom_file)) {
-				LOG(("Examining file input"));
+
 				err = dom_html_input_element_get_value(
 					(dom_html_input_element *)form_element,
 					&inputvalue);
 				if (err != DOM_NO_ERR) {
-					LOG(("Could not get file value"));
+					LOG("Could not get file value");
 					goto dom_no_memory;
 				}
 				err = dom_node_get_user_data(
@@ -723,14 +750,14 @@ bool form_successful_controls_dom(struct form *_form,
 					corestring_dom___ns_key_file_name_node_data,
 					&rawfile_temp);
 				if (err != DOM_NO_ERR) {
-					LOG(("Could not get file rawname"));
+					LOG("Could not get file rawname");
 					goto dom_no_memory;
 				}
 				rawfile_temp = strdup(rawfile_temp != NULL ?
 						      rawfile_temp :
 						      "");
 				if (rawfile_temp == NULL) {
-					LOG(("Could not copy file rawname"));
+					LOG("Could not copy file rawname");
 					goto dom_no_memory;
 				}
 				/* Fall out to the allocation */
@@ -739,16 +766,15 @@ bool form_successful_controls_dom(struct form *_form,
 				   dom_string_caseless_isequal(
 					   inputtype, corestring_dom_button)) {
 				/* Skip these */
-				LOG(("Skipping RESET and BUTTON"));
+				LOG("Skipping RESET and BUTTON");
 				continue;
 			} else {
 				/* Everything else is treated as text values */
-				LOG(("Retrieving generic input text"));
 				err = dom_html_input_element_get_value(
 					(dom_html_input_element *)form_element,
 					&inputvalue);
 				if (err != DOM_NO_ERR) {
-					LOG(("Could not get input value"));
+					LOG("Could not get input value");
 					goto dom_no_memory;
 				}
 				/* Fall out to the allocation */
@@ -757,7 +783,7 @@ bool form_successful_controls_dom(struct form *_form,
 		
 		success_new = calloc(1, sizeof(*success_new));
 		if (success_new == NULL) {
-			LOG(("Could not allocate data for generic"));
+			LOG("Could not allocate data for generic");
 			goto dom_no_memory;
 		}
 		
@@ -766,12 +792,12 @@ bool form_successful_controls_dom(struct form *_form,
 		
 		success_new->name = ENCODE_ITEM(inputname);
 		if (success_new->name == NULL) {
-			LOG(("Could not encode name for generic"));
+			LOG("Could not encode name for generic");
 			goto dom_no_memory;
 		}
 		success_new->value = ENCODE_ITEM(inputvalue);
 		if (success_new->value == NULL) {
-			LOG(("Could not encode value for generic"));
+			LOG("Could not encode value for generic");
 			goto dom_no_memory;
 		}
 		if (rawfile_temp != NULL) {
@@ -782,36 +808,45 @@ bool form_successful_controls_dom(struct form *_form,
 	}
 	
 	free(charset);
+
 	if (form_element != NULL) {
-		LOG(("Unreffed %p", form_element));
 		dom_node_unref(form_element);
 	}
+
 	if (form_elements != NULL) {
-		LOG(("Unreffed %p", form_elements));
 		dom_html_collection_unref(form_elements);
 	}
-	if (nodename != NULL)
+
+	if (nodename != NULL) {
 		dom_string_unref(nodename);
-	if (inputname != NULL)
+	}
+
+	if (inputname != NULL) {
 		dom_string_unref(inputname);
-	if (inputvalue != NULL)
+	}
+
+	if (inputvalue != NULL) {
 		dom_string_unref(inputvalue);
-	if (options != NULL)
+	}
+
+	if (options != NULL) {
 		dom_html_options_collection_unref(options);
-	if (option_element != NULL)
+	}
+
+	if (option_element != NULL) {
 		dom_node_unref(option_element);
-	if (inputtype != NULL)
+	}
+
+	if (inputtype != NULL) {
 		dom_string_unref(inputtype);
-	if (rawfile_temp != NULL)
+	}
+
+	if (rawfile_temp != NULL) {
 		free(rawfile_temp);
+	}
+
 	*successful_controls = sentinel.next;
 	
-	for (success_new = *successful_controls; success_new != NULL;
-	     success_new = success_new->next) {
-		LOG(("%p -> %s=%s", success_new, success_new->name, success_new->value));
-		LOG(("%p -> file=%s rawfile=%s", success_new,
-		     success_new->file ? "yes" : "no", success_new->rawfile));
-	}
 	return true;
 	
 dom_no_memory:
@@ -877,7 +912,7 @@ static char *form_url_encode(struct form *form,
 	}
 
 	for (; control; control = control->next) {
-		url_err = url_escape(control->name, 0, true, NULL, &name);
+		url_err = url_escape(control->name, true, NULL, &name);
 		if (url_err == NSERROR_NOMEM) {
 			free(s);
 			return NULL;
@@ -885,7 +920,7 @@ static char *form_url_encode(struct form *form,
 
 		assert(url_err == NSERROR_OK);
 
-		url_err = url_escape(control->value, 0, true, NULL, &value);
+		url_err = url_escape(control->value, true, NULL, &value);
 		if (url_err == NSERROR_NOMEM) {
 			free(name);
 			free(s);
@@ -909,9 +944,10 @@ static char *form_url_encode(struct form *form,
 		free(value);
 	}
 
-	if (len > len_init)
+	if (len > len_init) {
 		/* Replace trailing '&' */
 		s[len - 1] = '\0';
+	}
 	return s;
 }
 
@@ -944,8 +980,9 @@ char *form_acceptable_charset(struct form *form)
 		return NULL;
 
 	/* make it upper case */
-	for (c = temp; *c; c++)
-		*c = toupper(*c);
+	for (c = temp; *c; c++) {
+		*c = ascii_to_upper(*c);
+	}
 
 	/* is UTF-8 specified? */
 	c = strstr(temp, "UTF-8");
@@ -961,14 +998,14 @@ char *form_acceptable_charset(struct form *form)
 	 * form element contains a space and/or comma separated list */
 	c = form->accept_charsets;
 
-	/* What would be an improvement would be to choose an encoding
+	/** \todo an improvement would be to choose an encoding
 	 * acceptable to the server which covers as much of the input
-	 * values as possible. Additionally, we need to handle the case
-	 * where none of the acceptable encodings cover all the textual
-	 * input values.
-	 * For now, we just extract the first element of the charset list
+	 * values as possible. Additionally, we need to handle the
+	 * case where none of the acceptable encodings cover all the
+	 * textual input values.  For now, we just extract the first
+	 * element of the charset list
 	 */
-	while (*c && !isspace(*c)) {
+	while (*c && !ascii_is_space(*c)) {
 		if (*c == ',')
 			break;
 		c++;
@@ -1042,17 +1079,7 @@ char *form_encode_item(const char *item, uint32_t len, const char *charset,
 	return ret;
 }
 
-/**
- * Open a select menu for a select form control, creating it if necessary.
- *
- * \param client_data	data passed to the redraw callback
- * \param control	the select form control for which the menu is being
- * 			opened
- * \param callback	redraw callback for the select menu
- * \param bw		the browser window in which the select menu is being
- * 			opened
- * \return		false on memory exhaustion, true otherwise
- */
+/* exported interface documented in render/form_internal.h */
 bool form_open_select_menu(void *client_data,
 		struct form_control *control,
 		select_menu_redraw_callback callback,
@@ -1070,7 +1097,7 @@ bool form_open_select_menu(void *client_data,
 
 		menu = calloc(1, sizeof (struct form_select_menu));
 		if (menu == NULL) {
-			warn_user("NoMemory", 0);
+			guit->misc->warning("NoMemory", 0);
 			return false;
 		}
 
@@ -1106,13 +1133,13 @@ bool form_open_select_menu(void *client_data,
 		}
 		menu->client_data = client_data;
 		menu->callback = callback;
-		if (!scrollbar_create(false,
+		if (scrollbar_create(false,
 				menu->height,
     				total_height,
 				menu->height,
 				control,
 				form_select_menu_scroll_callback,
-				&(menu->scrollbar))) {
+				&(menu->scrollbar)) != NSERROR_OK) {
 			free(menu);
 			return false;
 		}
@@ -1126,12 +1153,7 @@ bool form_open_select_menu(void *client_data,
 }
 
 
-/**
- * Destroy a select menu and free allocated memory.
- * 
- * \param control	the select form control owning the select menu being
- * 			destroyed
- */
+/* exported interface documented in render/form_internal.h */
 void form_free_select_menu(struct form_control *control)
 {
 	if (control->data.select.menu->scrollbar != NULL)
@@ -1140,17 +1162,8 @@ void form_free_select_menu(struct form_control *control)
 	control->data.select.menu = NULL;
 }
 
-/**
- * Redraw an opened select menu.
- * 
- * \param control	the select menu being redrawn
- * \param x		the X coordinate to draw the menu at
- * \param x		the Y coordinate to draw the menu at
- * \param scale		current redraw scale
- * \param clip		clipping rectangle
- * \param ctx		current redraw context
- * \return		true on success, false otherwise
- */
+
+/* exported interface documented in render/form_internal.h */
 bool form_redraw_select_menu(struct form_control *control, int x, int y,
 		float scale, const struct rect *clip,
 		const struct redraw_context *ctx)
@@ -1277,13 +1290,10 @@ bool form_redraw_select_menu(struct form_control *control, int x, int y,
  * Check whether a clipping rectangle is completely contained in the
  * select menu.
  *
- * \param control	the select menu to check the clipping rectangle for
- * \param scale		the current browser window scale
- * \param clip_x0	minimum x of clipping rectangle
- * \param clip_y0	minimum y of clipping rectangle
- * \param clip_x1	maximum x of clipping rectangle
- * \param clip_y1	maximum y of clipping rectangle
- * \return		true if inside false otherwise
+ * \param control  the select menu to check the clipping rectangle for
+ * \param scale    the current browser window scale
+ * \param clip     the clipping rectangle
+ * \return true if inside false otherwise
  */
 bool form_clip_inside_select_menu(struct form_control *control, float scale,
 		const struct rect *clip)
@@ -1311,23 +1321,25 @@ bool form_clip_inside_select_menu(struct form_control *control, float scale,
 /**
  * Process a selection from a form select menu.
  *
- * \param  bw	    browser window with menu
+ * \param  html The html content handle for the form
  * \param  control  form control with menu
  * \param  item	    index of item selected from the menu
+ * \return NSERROR_OK or appropriate error code.
  */
-
-static void form__select_process_selection(html_content *html,
+static nserror form__select_process_selection(html_content *html,
 		struct form_control *control, int item)
 {
 	struct box *inline_box;
 	struct form_option *o;
 	int count;
+	nserror ret = NSERROR_OK;
 
 	assert(control != NULL);
 	assert(html != NULL);
 
 	/** \todo Even though the form code is effectively part of the html
-	 *        content handler, poking around inside contents is not good */
+	 *        content handler, poking around inside contents is not good
+	 */
 
 	inline_box = control->box->children->children;
 
@@ -1338,6 +1350,7 @@ static void form__select_process_selection(html_content *html,
 			o->selected = false;
 			dom_html_option_element_set_selected(o->node, false);
 		}
+
 		if (count == item) {
 			if (control->data.select.multiple) {
 				if (o->selected) {
@@ -1357,39 +1370,75 @@ static void form__select_process_selection(html_content *html,
 				o->selected = true;
 			}
 		}
-		if (o->selected)
+
+		if (o->selected) {
 			control->data.select.current = o;
+		}
 	}
 
 	talloc_free(inline_box->text);
 	inline_box->text = 0;
-	if (control->data.select.num_selected == 0)
+
+	if (control->data.select.num_selected == 0) {
 		inline_box->text = talloc_strdup(html->bctx,
 				messages_get("Form_None"));
-	else if (control->data.select.num_selected == 1)
+	} else if (control->data.select.num_selected == 1) {
 		inline_box->text = talloc_strdup(html->bctx,
 				control->data.select.current->text);
-	else
+	} else {
 		inline_box->text = talloc_strdup(html->bctx,
 				messages_get("Form_Many"));
+	}
+
 	if (!inline_box->text) {
-		warn_user("NoMemory", 0);
+		ret = NSERROR_NOMEM;
 		inline_box->length = 0;
-	} else
+	} else {
 		inline_box->length = strlen(inline_box->text);
+	}
 	inline_box->width = control->box->width;
 
 	html__redraw_a_box(html, control->box);
+
+	return ret;
 }
 
-
-void form_select_process_selection(struct form_control *control, int item)
+/* exported interface documented in netsurf/form.h */
+nserror form_select_process_selection(struct form_control *control, int item)
 {
 	assert(control != NULL);
 
-	form__select_process_selection(control->html, control, item);
+	return form__select_process_selection(control->html, control, item);
 }
 
+/* exported interface documented in netsurf/form.h */
+struct form_option *
+form_select_get_option(struct form_control *control, int item)
+{
+	struct form_option *opt;
+
+	opt = control->data.select.items;
+	while ((opt != NULL) && (item > 0)) {
+		opt = opt->next;
+		item--;
+	}
+	return opt;
+}
+
+/* exported interface documented in netsurf/form.h */
+char *form_control_get_name(struct form_control *control)
+{
+	return control->name;
+}
+
+/* exported interface documented in netsurf/form.h */
+nserror form_control_bounding_rect(struct form_control *control, struct rect *r)
+{
+	box_bounds( control->box, r );
+	return NSERROR_OK;
+}
+
+
 /**
  * Handle a click on the area of the currently opened select menu.
  * 
@@ -1611,8 +1660,7 @@ void form_select_menu_callback(void *client_data,
 /**
  * Set a radio form control and clear the others in the group.
  *
- * \param  content  content containing the form, of type CONTENT_TYPE
- * \param  radio    form control of type GADGET_RADIO
+ * \param radio form control of type GADGET_RADIO
  */
 
 void form_radio_set(struct form_control *radio)
@@ -1664,7 +1712,7 @@ void form_submit(nsurl *page_url, struct browser_window *target,
 	assert(form != NULL);
 
 	if (form_successful_controls_dom(form, submit_button, &success) == false) {
-		warn_user("NoMemory", 0);
+		guit->misc->warning("NoMemory", 0);
 		return;
 	}
 
@@ -1672,7 +1720,7 @@ void form_submit(nsurl *page_url, struct browser_window *target,
 	if (nsurl_create(form->action, &action_url) != NSERROR_OK) {
 		free(data);
 		fetch_multipart_data_destroy(success);
-		warn_user("NoMemory", 0);
+		guit->misc->warning("NoMemory", 0);
 		return;
 	}
 
@@ -1681,7 +1729,7 @@ void form_submit(nsurl *page_url, struct browser_window *target,
 		data = form_url_encode(form, success, true);
 		if (data == NULL) {
 			fetch_multipart_data_destroy(success);
-			warn_user("NoMemory", 0);
+			guit->misc->warning("NoMemory", 0);
 			return;
 		}
 
@@ -1691,7 +1739,7 @@ void form_submit(nsurl *page_url, struct browser_window *target,
 			nsurl_unref(action_query);
 			free(data);
 			fetch_multipart_data_destroy(success);
-			warn_user(messages_get_errorcode(error), 0);
+			guit->misc->warning(messages_get_errorcode(error), 0);
 			return;
 		}
 
@@ -1711,7 +1759,7 @@ void form_submit(nsurl *page_url, struct browser_window *target,
 		data = form_url_encode(form, success, false);
 		if (data == NULL) {
 			fetch_multipart_data_destroy(success);
-			warn_user("NoMemory", 0);
+			guit->misc->warning("NoMemory", 0);
 			nsurl_unref(action_url);
 			return;
 		}
