@@ -17,16 +17,17 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/** \file
+/**
+ * \file
  * Content for text/html (implementation).
  */
 
 #include <assert.h>
-#include <ctype.h>
 #include <stdint.h>
 #include <string.h>
 #include <strings.h>
 #include <stdlib.h>
+#include <nsutils/time.h>
 
 #include "utils/config.h"
 #include "utils/corestrings.h"
@@ -36,21 +37,24 @@
 #include "utils/messages.h"
 #include "utils/talloc.h"
 #include "utils/utf8.h"
-#include "utils/utils.h"
 #include "utils/nsoption.h"
-#include "content/content_protected.h"
-#include "content/fetch.h"
+#include "utils/string.h"
+#include "utils/ascii.h"
+#include "netsurf/content.h"
+#include "netsurf/browser_window.h"
+#include "netsurf/utf8.h"
+#include "netsurf/layout.h"
+#include "netsurf/misc.h"
 #include "content/hlcache.h"
 #include "desktop/selection.h"
 #include "desktop/scrollbar.h"
 #include "desktop/textarea.h"
-#include "image/bitmap.h"
+#include "netsurf/bitmap.h"
 #include "javascript/js.h"
-#include "desktop/gui_factory.h"
+#include "desktop/gui_internal.h"
 
 #include "render/box.h"
-#include "render/font.h"
-#include "render/form.h"
+#include "render/form_internal.h"
 #include "render/html_internal.h"
 #include "render/imagemap.h"
 #include "render/layout.h"
@@ -69,6 +73,30 @@ static const char *html_types[] = {
 	"text/html"
 };
 
+/* Exported interface, see html_internal.h */
+bool fire_dom_event(dom_string *type, dom_node *target,
+		    bool bubbles, bool cancelable)
+{
+	dom_exception exc;
+	dom_event *evt;
+	bool result;
+
+	exc = dom_event_create(&evt);
+	if (exc != DOM_NO_ERR) return false;
+	exc = dom_event_init(evt, type, bubbles, cancelable);
+	if (exc != DOM_NO_ERR) {
+		dom_event_unref(evt);
+		return false;
+	}
+	LOG("Dispatching '%*s' against %p",
+	    dom_string_length(type), dom_string_data(type), target);
+	exc = dom_event_target_dispatch_event(target, evt, &result);
+	if (exc != DOM_NO_ERR) {
+		result = false;
+	}
+	dom_event_unref(evt);
+	return result;
+}
 
 /**
  * Perform post-box-creation conversion of a document
@@ -82,7 +110,7 @@ static void html_box_convert_done(html_content *c, bool success)
 	dom_exception exc; /* returned by libdom functions */
 	dom_node *html;
 
-	LOG(("Done XML to box (%p)", c));
+	LOG("Done XML to box (%p)", c);
 
 	/* Clean up and report error if unsuccessful or aborted */
 	if ((success == false) || (c->aborted)) {
@@ -112,7 +140,7 @@ static void html_box_convert_done(html_content *c, bool success)
 		/** @todo should this call html_object_free_objects(c);
 		 * like the other error paths 
 		 */
-		LOG(("error retrieving html element from dom"));
+		LOG("error retrieving html element from dom");
 		content_broadcast_errorcode(&c->base, NSERROR_DOM);
 		content_set_error(&c->base);
 		return;
@@ -121,7 +149,7 @@ static void html_box_convert_done(html_content *c, bool success)
 	/* extract image maps - can't do this sensibly in dom_to_box */
 	err = imagemap_extract(c);
 	if (err != NSERROR_OK) {
-		LOG(("imagemap extraction failed"));
+		LOG("imagemap extraction failed");
 		html_object_free_objects(c);
 		content_broadcast_errorcode(&c->base, err);
 		content_set_error(&c->base);
@@ -140,7 +168,6 @@ static void html_box_convert_done(html_content *c, bool success)
 		content_set_done(&c->base);
 	}
 
-	html_set_status(c, "");
 	dom_node_unref(html);
 }
 
@@ -367,7 +394,7 @@ static nserror html_meta_refresh_process_element(html_content *c, dom_node *n)
 	url = dom_string_data(content);
 
 	/* *LWS */
-	while (url < end && isspace(*url)) {
+	while (url < end && ascii_is_space(*url)) {
 		url++;
 	}
 
@@ -395,7 +422,7 @@ static nserror html_meta_refresh_process_element(html_content *c, dom_node *n)
 	}
 
 	/* *LWS */
-	while (url < end && isspace(*url)) {
+	while (url < end && ascii_is_space(*url)) {
 		url++;
 	}
 
@@ -404,7 +431,7 @@ static nserror html_meta_refresh_process_element(html_content *c, dom_node *n)
 		url++;
 
 	/* *LWS */
-	while (url < end && isspace(*url)) {
+	while (url < end && ascii_is_space(*url)) {
 		url++;
 	}
 
@@ -436,7 +463,7 @@ static nserror html_meta_refresh_process_element(html_content *c, dom_node *n)
 	}
 
 	/* *LWS */
-	while (url < end && isspace(*url)) {
+	while (url < end && ascii_is_space(*url)) {
 		url++;
 	}
 
@@ -456,7 +483,7 @@ static nserror html_meta_refresh_process_element(html_content *c, dom_node *n)
 	}
 
 	/* *LWS */
-	while (url < end && isspace(*url)) {
+	while (url < end && ascii_is_space(*url)) {
 		url++;
 	}
 
@@ -475,7 +502,7 @@ static nserror html_meta_refresh_process_element(html_content *c, dom_node *n)
 			url++;
 	} else {
 		/* url-nq */
-		while (url < end && !isspace(*url))
+		while (url < end && !ascii_is_space(*url))
 			url++;
 	}
 
@@ -539,12 +566,8 @@ static bool html_process_img(html_content *c, dom_node *node)
 	return success;
 }
 
-/**
- * Complete conversion of an HTML document
- *
- * \param c  Content to convert
- */
-void html_finish_conversion(html_content *c)
+/* exported function documented in render/html_internal.h */
+void html_finish_conversion(html_content *htmlc)
 {
 	union content_msg_data msg_data;
 	dom_exception exc; /* returned by libdom functions */
@@ -552,17 +575,17 @@ void html_finish_conversion(html_content *c)
 	nserror error;
 
 	/* Bail out if we've been aborted */
-	if (c->aborted) {
-		content_broadcast_errorcode(&c->base, NSERROR_STOPPED);
-		content_set_error(&c->base);
+	if (htmlc->aborted) {
+		content_broadcast_errorcode(&htmlc->base, NSERROR_STOPPED);
+		content_set_error(&htmlc->base);
 		return;
 	}
 
 	/* create new css selection context */
-	error = html_css_new_selection_context(c, &c->select_ctx);
+	error = html_css_new_selection_context(htmlc, &htmlc->select_ctx);
 	if (error != NSERROR_OK) {
-		content_broadcast_errorcode(&c->base, error);
-		content_set_error(&c->base);
+		content_broadcast_errorcode(&htmlc->base, error);
+		content_set_error(&htmlc->base);
 		return;
 	}
 
@@ -571,29 +594,31 @@ void html_finish_conversion(html_content *c)
 	 * object, but with its target set to the Document object (and
 	 * the currentTarget set to the Window object)
 	 */
-	js_fire_event(c->jscontext, "load", c->document, NULL);
+	if (htmlc->jscontext != NULL) {
+		js_fire_event(htmlc->jscontext, "load", htmlc->document, NULL);
+	}
 
 	/* convert dom tree to box tree */
-	LOG(("DOM to box (%p)", c));
-	content_set_status(&c->base, messages_get("Processing"));
+	LOG("DOM to box (%p)", htmlc);
+	content_set_status(&htmlc->base, messages_get("Processing"));
 	msg_data.explicit_status_text = NULL;
-	content_broadcast(&c->base, CONTENT_MSG_STATUS, msg_data);
+	content_broadcast(&htmlc->base, CONTENT_MSG_STATUS, msg_data);
 
-	exc = dom_document_get_document_element(c->document, (void *) &html);
+	exc = dom_document_get_document_element(htmlc->document, (void *) &html);
 	if ((exc != DOM_NO_ERR) || (html == NULL)) {
-		LOG(("error retrieving html element from dom"));
-		content_broadcast_errorcode(&c->base, NSERROR_DOM);
-		content_set_error(&c->base);
+		LOG("error retrieving html element from dom");
+		content_broadcast_errorcode(&htmlc->base, NSERROR_DOM);
+		content_set_error(&htmlc->base);
 		return;
 	}
 
-	error = dom_to_box(html, c, html_box_convert_done);
+	error = dom_to_box(html, htmlc, html_box_convert_done);
 	if (error != NSERROR_OK) {
-		LOG(("box conversion failed"));
+		LOG("box conversion failed");
 		dom_node_unref(html);
-		html_object_free_objects(c);
-		content_broadcast_errorcode(&c->base, error);
-		content_set_error(&c->base);
+		html_object_free_objects(htmlc);
+		content_broadcast_errorcode(&htmlc->base, error);
+		content_set_error(&htmlc->base);
 		return;
 	}
 
@@ -606,7 +631,6 @@ dom_default_action_DOMNodeInserted_cb(struct dom_event *evt, void *pw)
 {
 	dom_event_target *node;
 	dom_node_type type;
-	dom_string *name;
 	dom_exception exc;
 	html_content *htmlc = pw;
 
@@ -615,38 +639,59 @@ dom_default_action_DOMNodeInserted_cb(struct dom_event *evt, void *pw)
 		exc = dom_node_get_node_type(node, &type);
 		if ((exc == DOM_NO_ERR) && (type == DOM_ELEMENT_NODE)) {
 			/* an element node has been inserted */
-			exc = dom_node_get_node_name(node, &name);
-			if ((exc == DOM_NO_ERR) && (name != NULL)) {
+			dom_html_element_type tag_type;
 
-				if (dom_string_caseless_isequal(name,
-						corestring_dom_link)) {
-					/* Handle stylesheet loading */
-					html_css_process_link(htmlc,
-							(dom_node *)node);
-					/* Generic link handling */
-					html_process_link(htmlc,
-							(dom_node *)node);
+			exc = dom_html_element_get_tag_type(node, &tag_type);
+			if (exc != DOM_NO_ERR) {
+				tag_type = DOM_HTML_ELEMENT_TYPE__UNKNOWN;
+			}
 
-				} else if (dom_string_caseless_lwc_isequal(name,
-						corestring_lwc_meta) &&
-						htmlc->refresh == false) {
-					html_meta_refresh_process_element(htmlc,
-							(dom_node *)node);
-				} else if (dom_string_caseless_lwc_isequal(
-						name, corestring_lwc_base)) {
-					html_process_base(htmlc,
-							(dom_node *)node);
-				} else if (dom_string_caseless_lwc_isequal(
-						name, corestring_lwc_title) &&
-						htmlc->title == NULL) {
-					htmlc->title = dom_node_ref(node);
-				} else if (dom_string_caseless_lwc_isequal(
-						name, corestring_lwc_img)) {
-					html_process_img(htmlc,
-							(dom_node *) node);
+			switch (tag_type) {
+			case DOM_HTML_ELEMENT_TYPE_LINK:
+				/* Handle stylesheet loading */
+				html_css_process_link(htmlc, (dom_node *)node);
+				/* Generic link handling */
+				html_process_link(htmlc, (dom_node *)node);
+				break;
+			case DOM_HTML_ELEMENT_TYPE_META:
+				if (htmlc->refresh)
+					break;
+				html_meta_refresh_process_element(htmlc,
+						(dom_node *)node);
+				break;
+			case DOM_HTML_ELEMENT_TYPE_TITLE:
+				if (htmlc->title != NULL)
+					break;
+				htmlc->title = dom_node_ref(node);
+				break;
+			case DOM_HTML_ELEMENT_TYPE_BASE:
+				html_process_base(htmlc, (dom_node *)node);
+				break;
+			case DOM_HTML_ELEMENT_TYPE_IMG:
+				html_process_img(htmlc, (dom_node *) node);
+				break;
+			case DOM_HTML_ELEMENT_TYPE_STYLE:
+				html_css_process_style(htmlc, (dom_node *) node);
+				break;
+			default:
+				break;
+			}
+			if (htmlc->enable_scripting) {
+				/* ensure javascript context is available */
+				if (htmlc->jscontext == NULL) {
+					union content_msg_data msg_data;
+
+					msg_data.jscontext = &htmlc->jscontext;
+					content_broadcast(&htmlc->base,
+							CONTENT_MSG_GETCTX,
+							msg_data);
+					LOG("javascript context: %p",
+							htmlc->jscontext);
+				}
+				if (htmlc->jscontext != NULL) {
+					js_handle_new_element(htmlc->jscontext,
+							(dom_element *) node);
 				}
-
-				dom_string_unref(name);
 			}
 		}
 		dom_node_unref(node);
@@ -691,6 +736,15 @@ dom_default_action_DOMSubtreeModified_cb(struct dom_event *evt, void *pw)
 	}
 }
 
+static void
+dom_default_action_finished_cb(struct dom_event *evt, void *pw)
+{
+	html_content *htmlc = pw;
+
+	if (htmlc->jscontext != NULL)
+		js_event_cleanup(htmlc->jscontext, evt);
+}
+
 /* callback function selector
  *
  * selects a callback function for libdom to call based on the type and phase.
@@ -709,7 +763,7 @@ dom_event_fetcher(dom_string *type,
 		  dom_default_action_phase phase,
 		  void **pw)
 {
-	//LOG(("type:%s", dom_string_data(type)));
+	//LOG("type:%s", dom_string_data(type));
 
 	if (phase == DOM_DEFAULT_ACTION_END) {
 		if (dom_string_isequal(type, corestring_dom_DOMNodeInserted)) {
@@ -717,10 +771,46 @@ dom_event_fetcher(dom_string *type,
 		} else if (dom_string_isequal(type, corestring_dom_DOMSubtreeModified)) {
 			return dom_default_action_DOMSubtreeModified_cb;
 		}
+	} else if (phase == DOM_DEFAULT_ACTION_FINISHED) {
+		return dom_default_action_finished_cb;
 	}
 	return NULL;
 }
 
+static void
+html_document_user_data_handler(dom_node_operation operation,
+				dom_string *key, void *data,
+				struct dom_node *src,
+				struct dom_node *dst)
+{
+	if (dom_string_isequal(corestring_dom___ns_key_html_content_data,
+			       key) == false || data == NULL) {
+		return;
+	}
+
+	switch (operation) {
+	case DOM_NODE_CLONED:
+		LOG("Cloned");
+		break;
+	case DOM_NODE_RENAMED:
+		LOG("Renamed");
+		break;
+	case DOM_NODE_IMPORTED:
+		LOG("imported");
+		break;
+	case DOM_NODE_ADOPTED:
+		LOG("Adopted");
+		break;
+	case DOM_NODE_DELETED:
+		/* This is the only path I expect */
+		break;
+	default:
+		LOG("User data operation not handled.");
+		assert(0);
+	}
+}
+
+
 static nserror
 html_create_html_data(html_content *c, const http_parameter *params)
 {
@@ -728,6 +818,8 @@ html_create_html_data(html_content *c, const http_parameter *params)
 	nserror nerror;
 	dom_hubbub_parser_params parse_params;
 	dom_hubbub_error error;
+	dom_exception err;
+	void *old_node_data;
 
 	c->parser = NULL;
 	c->parse_completed = false;
@@ -738,6 +830,7 @@ html_create_html_data(html_content *c, const http_parameter *params)
 	c->base_target = NULL;
 	c->aborted = false;
 	c->refresh = false;
+	c->reflowing = false;
 	c->title = NULL;
 	c->bctx = NULL;
 	c->layout = NULL;
@@ -754,7 +847,7 @@ html_create_html_data(html_content *c, const http_parameter *params)
 	c->frameset = NULL;
 	c->iframe = NULL;
 	c->page = NULL;
-	c->font_func = &nsfont;
+	c->font_func = guit->layout;
 	c->drag_type = HTML_DRAG_NONE;
 	c->drag_owner.no_owner = true;
 	c->selection_type = HTML_SELECTION_NONE;
@@ -767,6 +860,7 @@ html_create_html_data(html_content *c, const http_parameter *params)
 	c->scripts = NULL;
 	c->jscontext = NULL;
 
+	c->enable_scripting = nsoption_bool(enable_javascript);
 	c->base.active = 1; /* The html content itself is active */
 
 	if (lwc_intern_string("*", SLEN("*"), &c->universal) != lwc_error_ok) {
@@ -793,7 +887,7 @@ html_create_html_data(html_content *c, const http_parameter *params)
 	/* Create the parser binding */
 	parse_params.enc = c->encoding;
 	parse_params.fix_enc = true;
-	parse_params.enable_script = nsoption_bool(enable_javascript);
+	parse_params.enable_script = c->enable_scripting;
 	parse_params.msg = NULL;
 	parse_params.script = html_process_script;
 	parse_params.ctx = c;
@@ -824,6 +918,24 @@ html_create_html_data(html_content *c, const http_parameter *params)
 		return libdom_hubbub_error_to_nserror(error);
 	}
 
+	err = dom_node_set_user_data(c->document,
+				     corestring_dom___ns_key_html_content_data,
+				     c, html_document_user_data_handler,
+				     (void *) &old_node_data);
+	if (err != DOM_NO_ERR) {
+		dom_hubbub_parser_destroy(c->parser);
+		nsurl_unref(c->base_url);
+		c->base_url = NULL;
+
+		lwc_string_unref(c->universal);
+		c->universal = NULL;
+
+		LOG("Unable to set user data.");
+		return NSERROR_DOM;
+	}
+
+	assert(old_node_data == NULL);
+
 	return NSERROR_OK;
 
 }
@@ -918,7 +1030,7 @@ html_process_encoding_change(struct content *c,
 
 	parse_params.enc = html->encoding;
 	parse_params.fix_enc = true;
-	parse_params.enable_script = nsoption_bool(enable_javascript);
+	parse_params.enable_script = html->enable_scripting;
 	parse_params.msg = NULL;
 	parse_params.script = html_process_script;
 	parse_params.ctx = html;
@@ -1025,11 +1137,11 @@ static bool html_convert(struct content *c)
 	exc = dom_document_get_quirks_mode(htmlc->document, &htmlc->quirks);
 	if (exc == DOM_NO_ERR) {
 		html_css_quirks_stylesheets(htmlc);
-		LOG(("quirks set to %d", htmlc->quirks));
+		LOG("quirks set to %d", htmlc->quirks);
 	}
 
 	htmlc->base.active--; /* the html fetch is no longer active */
-	LOG(("%d fetches active", htmlc->base.active));
+	LOG("%d fetches active", htmlc->base.active);
 
 	/* The parse cannot be completed here because it may be paused
 	 * untill all the resources being fetched have completed.
@@ -1082,11 +1194,11 @@ html_begin_conversion(html_content *htmlc)
 	 * complete to avoid repeating the completion pointlessly.
 	 */
 	if (htmlc->parse_completed == false) {
-		LOG(("Completing parse"));
+		LOG("Completing parse");
 		/* complete parsing */
 		error = dom_hubbub_parser_completed(htmlc->parser);
 		if (error != DOM_HUBBUB_OK) {
-			LOG(("Parsing failed"));
+			LOG("Parsing failed");
 	
 			content_broadcast_errorcode(&htmlc->base, 
 						    libdom_hubbub_error_to_nserror(error));
@@ -1108,7 +1220,7 @@ html_begin_conversion(html_content *htmlc)
 	}
 
 	/* complete script execution */
-	html_scripts_exec(htmlc);
+	html_script_exec(htmlc);
 
 	/* fire a simple event that bubbles named DOMContentLoaded at
 	 * the Document.
@@ -1137,7 +1249,7 @@ html_begin_conversion(html_content *htmlc)
 	/* locate root element and ensure it is html */
 	exc = dom_document_get_document_element(htmlc->document, (void *) &html);
 	if ((exc != DOM_NO_ERR) || (html == NULL)) {
-		LOG(("error retrieving html element from dom"));
+		LOG("error retrieving html element from dom");
 		content_broadcast_errorcode(&htmlc->base, NSERROR_DOM);
 		return false;
 	}
@@ -1147,7 +1259,7 @@ html_begin_conversion(html_content *htmlc)
 	    (node_name == NULL) ||
 	    (!dom_string_caseless_lwc_isequal(node_name,
 	    		corestring_lwc_html))) {
-		LOG(("root element not html"));
+		LOG("root element not html");
 		content_broadcast_errorcode(&htmlc->base, NSERROR_DOM);
 		dom_node_unref(html);
 		return false;
@@ -1215,12 +1327,21 @@ html_begin_conversion(html_content *htmlc)
 
 /**
  * Stop loading a CONTENT_HTML.
+ *
+ * called when the content is aborted. This must clean up any state
+ * created during the fetch.
  */
 
 static void html_stop(struct content *c)
 {
 	html_content *htmlc = (html_content *) c;
 
+	/* invalidate the html content reference to the javascript context
+	 * as it is about to become invalid and must not be used any
+	 * more.
+	 */
+	html_script_invalidate_ctx(htmlc);
+
 	switch (c->status) {
 	case CONTENT_STATUS_LOADING:
 		/* Still loading; simply flag that we've been aborted
@@ -1234,7 +1355,6 @@ static void html_stop(struct content *c)
 		/* If there are no further active fetches and we're still
  		 * in the READY state, transition to the DONE state. */
 		if (c->status == CONTENT_STATUS_READY && c->active == 0) {
-			html_set_status(htmlc, "");
 			content_set_done(c);
 		}
 
@@ -1245,7 +1365,7 @@ static void html_stop(struct content *c)
 		break;
 
 	default:
-		LOG(("Unexpected status %d", c->status));
+		LOG("Unexpected status %d", c->status);
 		assert(0);
 	}
 }
@@ -1259,20 +1379,24 @@ static void html_reformat(struct content *c, int width, int height)
 {
 	html_content *htmlc = (html_content *) c;
 	struct box *layout;
-	unsigned int time_before, time_taken;
+	uint64_t ms_before;
+	uint64_t ms_after;
+	uint64_t ms_interval;
 
-	time_before = wallclock();
+	nsu_getmonotonic_ms(&ms_before);
+
+	htmlc->reflowing = true;
 
 	layout_document(htmlc, width, height);
 	layout = htmlc->layout;
 
 	/* width and height are at least margin box of document */
 	c->width = layout->x + layout->padding[LEFT] + layout->width +
-			layout->padding[RIGHT] + layout->border[RIGHT].width +
-			layout->margin[RIGHT];
+		layout->padding[RIGHT] + layout->border[RIGHT].width +
+		layout->margin[RIGHT];
 	c->height = layout->y + layout->padding[TOP] + layout->height +
-			layout->padding[BOTTOM] + layout->border[BOTTOM].width +
-			layout->margin[BOTTOM];
+		layout->padding[BOTTOM] + layout->border[BOTTOM].width +
+		layout->margin[BOTTOM];
 
 	/* if boxes overflow right or bottom edge, expand to contain it */
 	if (c->width < layout->x + layout->descendant_x1)
@@ -1282,10 +1406,16 @@ static void html_reformat(struct content *c, int width, int height)
 
 	selection_reinit(&htmlc->sel, htmlc->layout);
 
-	time_taken = wallclock() - time_before;
-	c->reformat_time = wallclock() +
-		((time_taken * 3 < nsoption_uint(min_reflow_period) ?
-		  nsoption_uint(min_reflow_period) : time_taken * 3));
+	htmlc->reflowing = false;
+
+	/* calculate next reflow time at three times what it took to reflow */
+	nsu_getmonotonic_ms(&ms_after);
+
+	ms_interval = (ms_before - ms_after) * 3;
+	if (ms_interval < (nsoption_uint(min_reflow_period) * 10)) {
+		ms_interval = nsoption_uint(min_reflow_period) * 10;
+	}
+	c->reformat_time = ms_after + ms_interval;
 }
 
 
@@ -1311,8 +1441,8 @@ void html_redraw_a_box(hlcache_handle *h, struct box *box)
 /**
  * Redraw a box.
  *
- * \param  h	content containing the box, of type CONTENT_HTML
- * \param  box  box to redraw
+ * \param html  content containing the box, of type CONTENT_HTML
+ * \param box  box to redraw.
  */
 
 void html__redraw_a_box(struct html_content *html, struct box *box)
@@ -1392,7 +1522,7 @@ static void html_destroy(struct content *c)
 	html_content *html = (html_content *) c;
 	struct form *f, *g;
 
-	LOG(("content %p", c));
+	LOG("content %p", c);
 
 	/* Destroy forms */
 	for (f = html->forms; f != NULL; f = g) {
@@ -1464,7 +1594,7 @@ static void html_destroy(struct content *c)
 	html_css_free_stylesheets(html);
 
 	/* Free scripts */
-	html_free_scripts(html);
+	html_script_free(html);
 
 	/* Free objects */
 	html_object_free_objects(html);
@@ -1486,15 +1616,6 @@ static nserror html_clone(const struct content *old, struct content **newc)
 	return true;
 }
 
-/**
- * Set the content status.
- */
-
-void html_set_status(html_content *c, const char *extra)
-{
-	content_set_status(&c->base, extra);
-}
-
 
 /**
  * Handle a window containing a CONTENT_HTML being opened.
@@ -1529,16 +1650,25 @@ html_open(struct content *c,
 
 static void html_close(struct content *c)
 {
-	html_content *html = (html_content *) c;
+	html_content *htmlc = (html_content *) c;
 
-	selection_clear(&html->sel, false);
+	selection_clear(&htmlc->sel, false);
 
-	if (html->search != NULL)
-		search_destroy_context(html->search);
+	if (htmlc->search != NULL) {
+		search_destroy_context(htmlc->search);
+	}
 
-	html->bw = NULL;
+	/* clear the html content reference to the browser window */
+	htmlc->bw = NULL;
 
-	html_object_close_objects(html);
+	/* invalidate the html content reference to the javascript context
+	 * as it is about to become invalid and must not be used any
+	 * more.
+	 */
+	html_script_invalidate_ctx(htmlc);
+
+	/* remove all object references from the html content */
+	html_object_close_objects(htmlc);
 }
 
 
@@ -1610,17 +1740,16 @@ static char *html_get_selection(struct content *c)
  * Get access to any content, link URLs and objects (images) currently
  * at the given (x, y) coordinates.
  *
- * \param c	html content to look inside
- * \param x	x-coordinate of point of interest
- * \param y	y-coordinate of point of interest
- * \param data	pointer to contextual_content struct.  Its fields are updated
- *		with pointers to any relevent content, or set to NULL if none.
+ * \param[in] c html content to look inside
+ * \param[in] x x-coordinate of point of interest
+ * \param[in] y y-coordinate of point of interest
+ * \param[out] data Positional features struct to be updated with any
+ *             relevent content, or set to NULL if none.
+ * \return NSERROR_OK on success else appropriate error code.
  */
-static void
-html_get_contextual_content(struct content *c,
-			    int x,
-			    int y,
-			    struct contextual_content *data)
+static nserror
+html_get_contextual_content(struct content *c, int x, int y,
+			    struct browser_window_features *data)
 {
 	html_content *html = (html_content *) c;
 
@@ -1631,13 +1760,16 @@ html_get_contextual_content(struct content *c,
 	while ((next = box_at_point(box, x, y, &box_x, &box_y)) != NULL) {
 		box = next;
 
-		if (box->style && css_computed_visibility(box->style) ==
-				CSS_VISIBILITY_HIDDEN)
+		/* hidden boxes are ignored */
+		if ((box->style != NULL) &&
+		    css_computed_visibility(box->style) == CSS_VISIBILITY_HIDDEN) {
 			continue;
+		}
 
-		if (box->iframe)
-			browser_window_get_contextual_content(box->iframe,
+		if (box->iframe) {
+			browser_window_get_features(box->iframe,
 					x - box_x, y - box_y, data);
+		}
 
 		if (box->object)
 			content_get_contextual_content(box->object,
@@ -1647,7 +1779,7 @@ html_get_contextual_content(struct content *c,
 			data->object = box->object;
 
 		if (box->href)
-			data->link_url = nsurl_access(box->href);
+			data->link = box->href;
 
 		if (box->usemap) {
 			const char *target = NULL;
@@ -1656,7 +1788,7 @@ html_get_contextual_content(struct content *c,
 			/* Box might have imagemap, but no actual link area
 			 * at point */
 			if (url != NULL)
-				data->link_url = nsurl_access(url);
+				data->link = url;
 		}
 		if (box->gadget) {
 			switch (box->gadget->type) {
@@ -1676,6 +1808,7 @@ html_get_contextual_content(struct content *c,
 			}
 		}
 	}
+	return NSERROR_OK;
 }
 
 
@@ -1778,7 +1911,7 @@ static void html__dom_user_data_handler(dom_node_operation operation,
 		free(data);
 		break;
 	default:
-		LOG(("User data operation not handled."));
+		LOG("User data operation not handled.");
 		assert(0);
 	}
 }
@@ -1794,7 +1927,7 @@ static void html__set_file_gadget_filename(struct content *c,
 	ret = guit->utf8->local_to_utf8(fn, 0, &utf8_fn);
 	if (ret != NSERROR_OK) {
 		assert(ret != NSERROR_BAD_ENCODING);
-		LOG(("utf8 to local encoding conversion failed"));
+		LOG("utf8 to local encoding conversion failed");
 		/* Load was for us - just no memory */
 		return;		
 	}
@@ -1911,6 +2044,12 @@ static bool html_drop_file_at_point(struct content *c, int x, int y, char *file)
 		file_len = ftell(fp);
 		fseek(fp, 0, SEEK_SET);
 
+		if ((long)file_len == -1) {
+			/* unable to get file length, but drop was for us */
+			fclose(fp);
+			return true;
+		}
+
 		/* Allocate buffer for file data */
 		buffer = malloc(file_len + 1);
 		if (buffer == NULL) {
@@ -1940,9 +2079,9 @@ static bool html_drop_file_at_point(struct content *c, int x, int y, char *file)
 		if (ret != NSERROR_OK) {
 			/* bad encoding shouldn't happen */
 			assert(ret != NSERROR_BAD_ENCODING);
-			LOG(("local to utf8 encoding failed"));
+			LOG("local to utf8 encoding failed");
 			free(buffer);
-			warn_user("NoMemory", NULL);
+			guit->misc->warning("NoMemory", NULL);
 			return true;
 		}
 
@@ -1968,11 +2107,27 @@ static bool html_drop_file_at_point(struct content *c, int x, int y, char *file)
 }
 
 
+/**
+ * set debug status.
+ *
+ * \param c The content to debug
+ * \param op The debug operation type
+ */
+static nserror
+html_debug(struct content *c, enum content_debug op)
+{
+	html_redraw_debug = !html_redraw_debug;
+
+	return NSERROR_OK;
+}
+
+
 /**
  * Dump debug info concerning the html_content
  *
- * \param bw The browser window
+ * \param c The content to debug
  * \param f The file to dump to
+ * \param op The debug dump type
  */
 static nserror
 html_debug_dump(struct content *c, FILE *f, enum content_debug op)
@@ -1990,19 +2145,19 @@ html_debug_dump(struct content *c, FILE *f, enum content_debug op)
 		ret = NSERROR_OK;
 	} else {
 		if (htmlc->document == NULL) {
-			LOG(("No document to dump"));
+			LOG("No document to dump");
 			return NSERROR_DOM;
 		}
 
 		exc = dom_document_get_document_element(htmlc->document, (void *) &html);
 		if ((exc != DOM_NO_ERR) || (html == NULL)) {
-			LOG(("Unable to obtain root node"));
+			LOG("Unable to obtain root node");
 			return NSERROR_DOM;
 		}
 
 		ret = libdom_dump_structure(html, f, 0);
 
-		LOG(("DOM structure dump returning %d", ret));
+		LOG("DOM structure dump returning %d", ret);
 
 		dom_node_unref(html);
 	}
@@ -2100,32 +2255,25 @@ struct box *html_get_box_tree(hlcache_handle *h)
 /**
  * Retrieve the charset of an HTML document
  *
- * \param h  Content to retrieve charset from
+ * \param c Content to retrieve charset from
+ * \param op The content encoding operation to perform.
  * \return Pointer to charset, or NULL
  */
-const char *html_get_encoding(hlcache_handle *h)
+static const char *html_encoding(const struct content *c, enum content_encoding_type op)
 {
-	html_content *c = (html_content *) hlcache_handle_get_content(h);
+	html_content *html = (html_content *) c;
+	static char enc_token[10] = "Encoding0";
 
-	assert(c != NULL);
+	assert(html != NULL);
 
-	return c->encoding;
+	if (op == CONTENT_ENCODING_SOURCE) {
+		enc_token[8] = '0' + html->encoding_source;
+		return messages_get(enc_token);
+	}
+
+	return html->encoding;
 }
 
-/**
- * Retrieve the charset of an HTML document
- *
- * \param h  Content to retrieve charset from
- * \return Pointer to charset, or NULL
- */
-dom_hubbub_encoding_source html_get_encoding_source(hlcache_handle *h)
-{
-	html_content *c = (html_content *) hlcache_handle_get_content(h);
-
-	assert(c != NULL);
-
-	return c->encoding_source;
-}
 
 /**
  * Retrieve framesets used in an HTML document
@@ -2252,7 +2400,9 @@ static const content_handler html_content_handler = {
 	.search = html_search,
 	.search_clear = html_search_clear,
 	.debug_dump = html_debug_dump,
+	.debug = html_debug,
 	.clone = html_clone,
+	.get_encoding = html_encoding,
 	.type = html_content_type,
 	.no_share = true,
 };
