@@ -19,7 +19,8 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/** \file
+/**
+ * \file
  * User interaction with a CONTENT_HTML (implementation).
  */
 
@@ -28,30 +29,32 @@
 
 #include <dom/dom.h>
 
-#include "content/content.h"
-#include "content/hlcache.h"
-#include "desktop/browser.h"
-#include "desktop/gui_factory.h"
-#include "desktop/frames.h"
-#include "desktop/mouse.h"
-#include "utils/nsoption.h"
-#include "desktop/scrollbar.h"
-#include "desktop/selection.h"
-#include "desktop/textarea.h"
-#include "desktop/textinput.h"
-#include "render/box.h"
-#include "render/box_textarea.h"
-#include "render/font.h"
-#include "render/form.h"
-#include "render/html_internal.h"
-#include "render/imagemap.h"
-#include "render/search.h"
-#include "javascript/js.h"
 #include "utils/corestrings.h"
 #include "utils/messages.h"
 #include "utils/utils.h"
 #include "utils/log.h"
+#include "utils/nsoption.h"
+#include "netsurf/content.h"
+#include "netsurf/browser_window.h"
+#include "netsurf/mouse.h"
+#include "netsurf/misc.h"
+#include "netsurf/layout.h"
+#include "netsurf/keypress.h"
+#include "content/hlcache.h"
+#include "desktop/frames.h"
+#include "desktop/scrollbar.h"
+#include "desktop/selection.h"
+#include "desktop/textarea.h"
+#include "javascript/js.h"
+#include "desktop/gui_internal.h"
 
+#include "render/box.h"
+#include "render/box_textarea.h"
+#include "render/font.h"
+#include "render/form_internal.h"
+#include "render/html_internal.h"
+#include "render/imagemap.h"
+#include "render/search.h"
 
 /**
  * Get pointer shape for given box
@@ -187,10 +190,11 @@ static void html_box_drag_start(struct box *box, int x, int y)
 /**
  * End overflow scroll scrollbar drags
  *
- * \param  h      html content's high level cache entry
- * \param  mouse  state of mouse buttons and modifier keys
- * \param  x	  coordinate of mouse
- * \param  y	  coordinate of mouse
+ * \param html   html content
+ * \param mouse  state of mouse buttons and modifier keys
+ * \param x	 coordinate of mouse
+ * \param y	 coordinate of mouse
+ * \param dir    Direction of drag
  */
 static size_t html_selection_drag_end(struct html_content *html,
 		browser_mouse_state mouse, int x, int y, int dir)
@@ -206,8 +210,8 @@ static size_t html_selection_drag_end(struct html_content *html,
 
 		font_plot_style_from_css(box->style, &fstyle);
 
-		nsfont.font_position_in_string(&fstyle, box->text, box->length,
-				dx, &idx, &pixel_offset);
+		guit->layout->position(&fstyle, box->text, box->length,
+				       dx, &idx, &pixel_offset);
 
 		idx += box->byte_offset;
 	}
@@ -232,14 +236,26 @@ void html_mouse_track(struct content *c, struct browser_window *bw,
 	html_mouse_action(c, bw, mouse, x, y);
 }
 
-/** Helper for file gadgets to store their filename unencoded on the
- * dom node associated with the gadget.
+/**
+ * Helper for file gadgets to store their filename.
+ *
+ * Stores the filename unencoded on the dom node associated with the
+ * gadget.
  *
  * \todo Get rid of this crap eventually
+ *
+ * \param operation DOM operation
+ * \param key DOM node key being considerd
+ * \param _data The data assocated with the key
+ * \param src The source DOM node.
+ * \param dst The destination DOM node.
  */
-static void html__image_coords_dom_user_data_handler(dom_node_operation operation,
-		dom_string *key, void *_data, struct dom_node *src,
-		struct dom_node *dst)
+static void
+html__image_coords_dom_user_data_handler(dom_node_operation operation,
+					 dom_string *key,
+					 void *_data,
+					 struct dom_node *src,
+					 struct dom_node *dst)
 {
 	struct image_input_coords *oldcoords, *coords = _data, *newcoords;
 
@@ -251,25 +267,29 @@ static void html__image_coords_dom_user_data_handler(dom_node_operation operatio
 	switch (operation) {
 	case DOM_NODE_CLONED:
 		newcoords = calloc(1, sizeof(*newcoords));
-		*newcoords = *coords;
-		if (dom_node_set_user_data(dst,
-					   corestring_dom___ns_key_image_coords_node_data,
-					   newcoords, html__image_coords_dom_user_data_handler,
-					   &oldcoords) == DOM_NO_ERR) {
-			free(oldcoords);
+		if (newcoords != NULL) {
+			*newcoords = *coords;
+			if (dom_node_set_user_data(dst,
+				 corestring_dom___ns_key_image_coords_node_data,
+				 newcoords,
+				 html__image_coords_dom_user_data_handler,
+				 &oldcoords) == DOM_NO_ERR) {
+				free(oldcoords);
+			}
 		}
 		break;
 
+	case DOM_NODE_DELETED:
+		free(coords);
+		break;
+
 	case DOM_NODE_RENAMED:
 	case DOM_NODE_IMPORTED:
 	case DOM_NODE_ADOPTED:
 		break;
 
-	case DOM_NODE_DELETED:
-		free(coords);
-		break;
 	default:
-		LOG(("User data operation not handled."));
+		LOG("User data operation not handled.");
 		assert(0);
 	}
 }
@@ -297,6 +317,8 @@ void html_mouse_action(struct content *c, struct browser_window *bw,
 	enum { ACTION_NONE, ACTION_SUBMIT, ACTION_GO } action = ACTION_NONE;
 	const char *title = 0;
 	nsurl *url = 0;
+	char *url_s = NULL;
+	size_t url_l = 0;
 	const char *target = 0;
 	char status_buffer[200];
 	const char *status = 0;
@@ -314,7 +336,6 @@ void html_mouse_action(struct content *c, struct browser_window *bw,
 	hlcache_handle *object = NULL;
 	struct box *html_object_box = NULL;
 	struct browser_window *iframe = NULL;
-	struct box *next_box;
 	struct box *drag_candidate = NULL;
 	struct scrollbar *scrollbar = NULL;
 	plot_font_style_t fstyle;
@@ -396,9 +417,9 @@ void html_mouse_action(struct content *c, struct browser_window *bw,
 
 			font_plot_style_from_css(box->style, &fstyle);
 
-			nsfont.font_position_in_string(&fstyle,
-					box->text, box->length,
-					dx, &idx, &pixel_offset);
+			guit->layout->position(&fstyle,
+					       box->text, box->length,
+					       dx, &idx, &pixel_offset);
 
 			selection_track(&html->sel, mouse,
 					box->byte_offset + idx);
@@ -522,9 +543,7 @@ void html_mouse_action(struct content *c, struct browser_window *bw,
 	 * text_box - text box
 	 * text_box_x - text_box
 	 */
-	while ((next_box = box_at_point(box, x, y, &box_x, &box_y)) != NULL) {
-		box = next_box;
-
+	do {
 		if ((box->style != NULL) && 
 		    (css_computed_visibility(box->style) == 
 		     CSS_VISIBILITY_HIDDEN)) {
@@ -630,10 +649,11 @@ void html_mouse_action(struct content *c, struct browser_window *bw,
 			text_box = box;
 			text_box_x = box_x;
 		}
-	}
+	} while ((box = box_at_point(box, x, y, &box_x, &box_y)) != NULL);
 
 	/* use of box_x, box_y, or content below this point is probably a
 	 * mistake; they will refer to the last box returned by box_at_point */
+	assert(node != NULL);
 
 	if (scrollbar) {
 		status = scrollbar_mouse_status_to_message(
@@ -655,8 +675,11 @@ void html_mouse_action(struct content *c, struct browser_window *bw,
 						form_select_menu_callback,
 						c);
 				pointer = BROWSER_POINTER_DEFAULT;
-			} else if (mouse & BROWSER_MOUSE_CLICK_1)
-				guit->browser->create_form_select_menu(bw, gadget);
+			} else if (mouse & BROWSER_MOUSE_CLICK_1) {
+				msg_data.select_menu.gadget = gadget;
+				content_broadcast(c, CONTENT_MSG_SELECTMENU,
+						msg_data);
+			}
 			break;
 		case GADGET_CHECKBOX:
 			status = messages_get("FormCheckbox");
@@ -808,12 +831,26 @@ void html_mouse_action(struct content *c, struct browser_window *bw,
 					y - html_object_pos_y);
 		}
 	} else if (url) {
+		if (nsoption_bool(display_decoded_idn) == true) {
+			if (nsurl_get_utf8(url, &url_s, &url_l) != NSERROR_OK) {
+				/* Unable to obtain a decoded IDN.  This is not a fatal error.
+				 * Ensure the string pointer is NULL so we use the encoded version. */
+				url_s = NULL;
+			}
+		}
+
 		if (title) {
 			snprintf(status_buffer, sizeof status_buffer, "%s: %s",
-					nsurl_access(url), title);
-			status = status_buffer;
-		} else
-			status = nsurl_access(url);
+					url_s ? url_s : nsurl_access(url), title);
+		} else {
+			snprintf(status_buffer, sizeof status_buffer, "%s",
+					url_s ? url_s : nsurl_access(url));
+		}
+
+		status = status_buffer;
+
+		if (url_s != NULL)
+			free(url_s);
 
 		pointer = get_pointer_shape(url_box, imagemap);
 
@@ -830,7 +867,7 @@ void html_mouse_action(struct content *c, struct browser_window *bw,
 
 		} else if (mouse & BROWSER_MOUSE_CLICK_2 &&
 				mouse & BROWSER_MOUSE_MOD_1) {
-			msg_data.savelink.url = nsurl_access(url);
+			msg_data.savelink.url = url;
 			msg_data.savelink.title = title;
 			content_broadcast(c, CONTENT_MSG_SAVELINK, msg_data);
 
@@ -874,12 +911,12 @@ void html_mouse_action(struct content *c, struct browser_window *bw,
 				font_plot_style_from_css(text_box->style,
 						&fstyle);
 
-				nsfont.font_position_in_string(&fstyle,
-					text_box->text,
-					text_box->length,
-					x - text_box_x,
-					&idx,
-					&pixel_offset);
+				guit->layout->position(&fstyle,
+						       text_box->text,
+						       text_box->length,
+						       x - text_box_x,
+						       &idx,
+						       &pixel_offset);
 
 				if (selection_click(&html->sel, mouse,
 						text_box->byte_offset + idx)) {
@@ -904,7 +941,6 @@ void html_mouse_action(struct content *c, struct browser_window *bw,
 				}
 
 			} else if (mouse & BROWSER_MOUSE_PRESS_1) {
-				union html_selection_owner sel_owner;
 				sel_owner.none = true;
 				selection_clear(&html->sel, true);
 			}
@@ -984,9 +1020,8 @@ void html_mouse_action(struct content *c, struct browser_window *bw,
 	}
 
 	/* fire dom click event */
-	if ((mouse & BROWSER_MOUSE_CLICK_1) ||
-	    (mouse & BROWSER_MOUSE_CLICK_2)) {
-		js_fire_event(html->jscontext, "click", html->document, node);
+	if (mouse & BROWSER_MOUSE_CLICK_1) {
+		fire_dom_event(corestring_dom_click, node, true, true);
 	}
 
 	/* deferred actions that can cause this browser_window to be destroyed
@@ -1042,19 +1077,19 @@ bool html_keypress(struct content *c, uint32_t key)
 	}
 
 	switch (key) {
-	case KEY_COPY_SELECTION:
+	case NS_KEY_COPY_SELECTION:
 		selection_copy_to_clipboard(sel);
 		return true;
 
-	case KEY_CLEAR_SELECTION:
+	case NS_KEY_CLEAR_SELECTION:
 		selection_clear(sel, true);
 		return true;
 
-	case KEY_SELECT_ALL:
+	case NS_KEY_SELECT_ALL:
 		selection_select_all(sel);
 		return true;
 
-	case KEY_ESCAPE:
+	case NS_KEY_ESCAPE:
 		if (selection_defined(sel)) {
 			selection_clear(sel, true);
 			return true;
@@ -1154,6 +1189,13 @@ void html_overflow_scroll_callback(void *client_data,
 	
 	switch(scrollbar_data->msg) {
 	case SCROLLBAR_MSG_MOVED:
+
+		if (html->reflowing == true) {
+			/* Can't redraw during layout, and it will
+			 * be redrawn after layout anyway. */
+			break;
+		}
+
 		html__redraw_a_box(html, box);
 		break;
 	case SCROLLBAR_MSG_SCROLL_START:
@@ -1184,7 +1226,7 @@ void html_overflow_scroll_callback(void *client_data,
 /**
  * End overflow scroll scrollbar drags
  *
- * \param  scroll  scrollbar widget
+ * \param  scrollbar  scrollbar widget
  * \param  mouse   state of mouse buttons and modifier keys
  * \param  x	   coordinate of mouse
  * \param  y	   coordinate of mouse
