@@ -26,15 +26,18 @@
 #include <string.h>
 #include <strings.h>
 #include <stdlib.h>
+#include <nsutils/time.h>
 
-#include "content/hlcache.h"
-#include "css/utils.h"
-#include "utils/nsoption.h"
-#include "desktop/scrollbar.h"
-#include "desktop/gui_factory.h"
 #include "utils/corestrings.h"
 #include "utils/config.h"
 #include "utils/log.h"
+#include "utils/nsoption.h"
+#include "netsurf/content.h"
+#include "netsurf/misc.h"
+#include "content/hlcache.h"
+#include "css/utils.h"
+#include "desktop/scrollbar.h"
+#include "desktop/gui_internal.h"
 
 #include "render/box.h"
 #include "render/html_internal.h"
@@ -157,7 +160,7 @@ html_object_callback(hlcache_handle *object,
 
 	case CONTENT_MSG_DONE:
 		c->base.active--;
-		LOG(("%d fetches active", c->base.active));
+		LOG("%d fetches active", c->base.active);
 
 		html_object_done(box, object, o->background);
 
@@ -187,28 +190,13 @@ html_object_callback(hlcache_handle *object,
 
 		if (box != NULL) {
 			c->base.active--;
-			LOG(("%d fetches active", c->base.active));
+			LOG("%d fetches active", c->base.active);
 
 			content_add_error(&c->base, "?", 0);
 			html_object_failed(box, c, o->background);
 		}
 		break;
 
-	case CONTENT_MSG_STATUS:
-		if (event->data.explicit_status_text == NULL) {
-			/* Object content's status text updated */
-			union content_msg_data data;
-			data.explicit_status_text =
-					content_get_status_message(object);
-			html_set_status(c, data.explicit_status_text);
-			content_broadcast(&c->base, CONTENT_MSG_STATUS, data);
-		} else {
-			/* Object content wants to set explicit message */
-			content_broadcast(&c->base, CONTENT_MSG_STATUS,
-					event->data);
-		}
-		break;
-
 	case CONTENT_MSG_REDRAW:
 		if (c->base.status != CONTENT_STATUS_LOADING) {
 			union content_msg_data data = event->data;
@@ -336,7 +324,7 @@ html_object_callback(hlcache_handle *object,
 	case CONTENT_MSG_REFRESH:
 		if (content_get_type(object) == CONTENT_HTML) {
 			/* only for HTML objects */
-			guit->browser->schedule(event->data.delay * 1000,
+			guit->misc->schedule(event->data.delay * 1000,
 					html_object_refresh, o);
 		}
 
@@ -374,6 +362,7 @@ html_object_callback(hlcache_handle *object,
 
 	case CONTENT_MSG_SAVELINK:
 	case CONTENT_MSG_POINTER:
+	case CONTENT_MSG_SELECTMENU:
 	case CONTENT_MSG_GADGETCLICK:
 		/* These messages are for browser window layer.
 		 * we're not interested, so pass them on. */
@@ -446,33 +435,40 @@ html_object_callback(hlcache_handle *object,
 		break;
 	}
 
-	if (c->base.status == CONTENT_STATUS_READY && c->base.active == 0 &&
-			(event->type == CONTENT_MSG_LOADING ||
-			event->type == CONTENT_MSG_DONE ||
-			event->type == CONTENT_MSG_ERROR)) {
+	if (c->base.status == CONTENT_STATUS_READY &&
+            c->base.active == 0 &&
+            (event->type == CONTENT_MSG_LOADING ||
+             event->type == CONTENT_MSG_DONE ||
+             event->type == CONTENT_MSG_ERROR)) {
 		/* all objects have arrived */
 		content__reformat(&c->base, false, c->base.available_width,
 				c->base.height);
-		html_set_status(c, "");
 		content_set_done(&c->base);
-	}
-
-	/* If  1) the configuration option to reflow pages while objects are
-	 *        fetched is set
-	 *     2) an object is newly fetched & converted,
-	 *     3) the box's dimensions need to change due to being replaced
-	 *     4) the object's parent HTML is ready for reformat,
-	 *     5) the time since the previous reformat is more than the
-	 *        configured minimum time between reformats
-	 * then reformat the page to display newly fetched objects */
-	else if (nsoption_bool(incremental_reflow) &&
-			event->type == CONTENT_MSG_DONE &&
-			box != NULL && !(box->flags & REPLACE_DIM) &&
-			(c->base.status == CONTENT_STATUS_READY ||
-			 c->base.status == CONTENT_STATUS_DONE) &&
-			(wallclock() > c->base.reformat_time)) {
-		content__reformat(&c->base, false, c->base.available_width,
-				c->base.height);
+	} else if (nsoption_bool(incremental_reflow) &&
+                   event->type == CONTENT_MSG_DONE &&
+                   box != NULL &&
+                   !(box->flags & REPLACE_DIM) &&
+                   (c->base.status == CONTENT_STATUS_READY ||
+                    c->base.status == CONTENT_STATUS_DONE)) {
+                /* 1) the configuration option to reflow pages while
+                 *      objects are fetched is set
+                 * 2) an object is newly fetched & converted,
+                 * 3) the box's dimensions need to change due to being replaced
+                 * 4) the object's parent HTML is ready for reformat,
+                 */
+                uint64_t ms_now;
+                nsu_getmonotonic_ms(&ms_now);
+                if (ms_now > c->base.reformat_time) {
+                        /* The time since the previous reformat is
+                         *  more than the configured minimum time
+                         *  between reformats so reformat the page to
+                         *  display newly fetched objects
+                         */
+                        content__reformat(&c->base,
+                                          false,
+                                          c->base.available_width,
+                                          c->base.height);
+                }
 	}
 
 	return NSERROR_OK;
@@ -505,7 +501,7 @@ static bool html_replace_object(struct content_html_object *object, nsurl *url)
 		/* remove existing object */
 		if (content_get_status(object->content) != CONTENT_STATUS_DONE) {
 			c->base.active--;
-			LOG(("%d fetches active", c->base.active));
+			LOG("%d fetches active", c->base.active);
 		}
 
 		hlcache_handle_release(object->content);
@@ -526,7 +522,7 @@ static bool html_replace_object(struct content_html_object *object, nsurl *url)
 
 	for (page = c; page != NULL; page = page->page) {
 		page->base.active++;
-		LOG(("%d fetches active", c->base.active));
+		LOG("%d fetches active", c->base.active);
 
 		page->base.status = CONTENT_STATUS_READY;
 	}
@@ -607,9 +603,10 @@ nserror html_object_abort_objects(html_content *htmlc)
 			hlcache_handle_abort(object->content);
 			hlcache_handle_release(object->content);
 			object->content = NULL;
-
-			htmlc->base.active--;
-			LOG(("%d fetches active", htmlc->base.active));
+			if (object->box != NULL) {
+				htmlc->base.active--;
+				LOG("%d fetches active", htmlc->base.active);
+			}
 			break;
 
 		}
@@ -632,7 +629,7 @@ nserror html_object_close_objects(html_content *html)
 			continue;
 
 		if (content_get_type(object->content) == CONTENT_HTML) {
-			guit->browser->schedule(-1, html_object_refresh, object);
+			guit->misc->schedule(-1, html_object_refresh, object);
 		}
 
 		content_close(object->content);
@@ -646,10 +643,10 @@ nserror html_object_free_objects(html_content *html)
 		struct content_html_object *victim = html->object_list;
 
 		if (victim->content != NULL) {
-			LOG(("object %p", victim->content));
+			LOG("object %p", victim->content);
 
 			if (content_get_type(victim->content) == CONTENT_HTML) {
-				guit->browser->schedule(-1, html_object_refresh, victim);
+				guit->misc->schedule(-1, html_object_refresh, victim);
 			}
 			hlcache_handle_release(victim->content);
 		}
@@ -708,7 +705,7 @@ bool html_fetch_object(html_content *c, nsurl *url, struct box *box,
 	c->num_objects++;
 	if (box != NULL) {
 		c->base.active++;
-		LOG(("%d fetches active", c->base.active));
+		LOG("%d fetches active", c->base.active);
 	}
 
 	return true;
