@@ -22,38 +22,43 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/** \file
- * Redraw of a CONTENT_HTML (implementation).
+/**
+ * \file
+ *
+ * Redrawing CONTENT_HTML implementation.
  */
 
+#include "utils/config.h"
 #include <assert.h>
 #include <stdbool.h>
 #include <stdlib.h>
 #include <string.h>
 #include <math.h>
 #include <dom/dom.h>
-#include "utils/config.h"
-#include "content/content_protected.h"
-#include "css/css.h"
-#include "css/utils.h"
-#include "desktop/plotters.h"
-#include "desktop/selection.h"
-#include "utils/nsoption.h"
-#include "desktop/print.h"
-#include "desktop/scrollbar.h"
-#include "desktop/textarea.h"
-#include "image/bitmap.h"
-#include "render/box.h"
-#include "render/font.h"
-#include "render/form.h"
-#include "render/html_internal.h"
-#include "render/layout.h"
-#include "render/search.h"
+
 #include "utils/log.h"
 #include "utils/messages.h"
 #include "utils/utils.h"
+#include "utils/nsoption.h"
+#include "netsurf/content.h"
+#include "netsurf/browser_window.h"
+#include "netsurf/plotters.h"
+#include "netsurf/bitmap.h"
+#include "netsurf/layout.h"
+#include "content/content_protected.h"
+#include "css/utils.h"
+#include "desktop/selection.h"
+#include "desktop/print.h"
+#include "desktop/scrollbar.h"
+#include "desktop/textarea.h"
+#include "desktop/gui_internal.h"
 
-
+#include "render/box.h"
+#include "render/font.h"
+#include "render/form_internal.h"
+#include "render/html_internal.h"
+#include "render/layout.h"
+#include "render/search.h"
 
 
 bool html_redraw_debug = false;
@@ -127,18 +132,21 @@ static struct box *html_redraw_find_bg_box(struct box *box)
  * Redraw a short text string, complete with highlighting
  * (for selection/search)
  *
- * \param  utf8_text  pointer to UTF-8 text string
- * \param  utf8_len   length of string, in bytes
- * \param  offset     byte offset within textual representation
- * \param  space      width of space that follows string (0 = no space)
- * \param  fstyle     text style to use (pass text size unscaled)
- * \param  x          x ordinate at which to plot text
- * \param  y          y ordinate at which to plot text
- * \param  clip       pointer to current clip rectangle
- * \param  height     height of text string
- * \param  scale      current display scale (1.0 = 100%)
- * \param  excluded   exclude this text string from the selection
- * \param  ctx	      current redraw context
+ * \param utf8_text pointer to UTF-8 text string
+ * \param utf8_len  length of string, in bytes
+ * \param offset    byte offset within textual representation
+ * \param space     width of space that follows string (0 = no space)
+ * \param fstyle    text style to use (pass text size unscaled)
+ * \param x         x ordinate at which to plot text
+ * \param y         y ordinate at which to plot text
+ * \param clip      pointer to current clip rectangle
+ * \param height    height of text string
+ * \param scale     current display scale (1.0 = 100%)
+ * \param excluded  exclude this text string from the selection
+ * \param c         Content being redrawn.
+ * \param sel       Selection context
+ * \param search    Search context
+ * \param ctx	    current redraw context
  * \return true iff successful and redraw should proceed
  */
 
@@ -187,6 +195,7 @@ bool text_redraw(const char *utf8_text, size_t utf8_len,
 			int startx, endx;
 			plot_style_t pstyle_fill_hback = *plot_style_fill_white;
 			plot_font_style_t fstyle_hback = plot_fstyle;
+			nserror res;
 
 			if (end_idx > utf8_len) {
 				/* adjust for trailing space, not present in
@@ -195,13 +204,19 @@ bool text_redraw(const char *utf8_text, size_t utf8_len,
 				endtxt_idx = utf8_len;
 			}
 
-			if (!nsfont.font_width(fstyle, utf8_text, start_idx,
-					&startx))
+			res = guit->layout->width(fstyle,
+						  utf8_text, start_idx,
+						  &startx);
+			if (res != NSERROR_OK) {
 				startx = 0;
+			}
 
-			if (!nsfont.font_width(fstyle, utf8_text, endtxt_idx,
-					&endx))
+			res = guit->layout->width(fstyle,
+						  utf8_text, endtxt_idx,
+						  &endx);
+			if (res != NSERROR_OK) {
 				endx = 0;
+			}
 
 			/* is there a trailing space that should be highlighted
 			 * as well? */
@@ -322,6 +337,7 @@ static plot_style_t plot_style_fillbdr_dlight = {
  * \param  style	border line style
  * \param  thickness	border thickness
  * \param  rectangular	whether border is rectangular
+ * \param clip cliping area for redrawing border.
  * \param  ctx		current redraw context
  * \return true if successful, false otherwise
  */
@@ -683,6 +699,7 @@ static bool html_redraw_border_plot(const int side, const int *p, colour c,
  * \param  y_parent	coordinate of top padding edge of parent of box
  * \param  p_width	width of padding box
  * \param  p_height	height of padding box
+ * \param clip cliping area for redrawing border.
  * \param  scale	scale for redraw
  * \param  ctx		current redraw context
  * \return true if successful, false otherwise
@@ -916,6 +933,7 @@ static bool html_redraw_borders(struct box *box, int x_parent, int y_parent,
  *
  * \param  box	  BOX_INLINE which created the border
  * \param  b	  coordinates of border edge rectangle
+ * \param clip cliping area for redrawing border.
  * \param  scale  scale for redraw
  * \param  first  true if this is the first rectangle associated with the inline
  * \param  last	  true if this is the last rectangle associated with the inline
@@ -1252,6 +1270,7 @@ static bool html_redraw_file(int x, int y, int width, int height,
 	const char *text;
 	size_t length;
 	plot_font_style_t fstyle;
+	nserror res;
 
 	font_plot_style_from_css(box->style, &fstyle);
 	fstyle.background = background_colour;
@@ -1262,13 +1281,16 @@ static bool html_redraw_file(int x, int y, int width, int height,
 		text = messages_get("Form_Drop");
 	length = strlen(text);
 
-	if (!nsfont.font_width(&fstyle, text, length, &text_width))
+	res = guit->layout->width(&fstyle, text, length, &text_width);
+	if (res != NSERROR_OK) {
 		return false;
+	}
 	text_width *= scale;
-	if (width < text_width + 8)
+	if (width < text_width + 8) {
 		x = x + width - text_width - 4;
-	else
+	} else {
 		x = x + 4;
+	}
 
 	return ctx->plot->text(x, y + height * 0.75, text, length, &fstyle);
 }
@@ -1283,13 +1305,13 @@ static bool html_redraw_file(int x, int y, int width, int height,
  * \param  scale  scale for redraw
  * \param  clip   current clip rectangle
  * \param  background_colour  current background colour
- * \param  background  box containing background details (usually ::box)
+ * \param  background  box containing background details (usually \a box)
  * \param  ctx	  current redraw context
  * \return true if successful, false otherwise
  *
  * The reason for the presence of ::background is the backwards compatibility
- * mess that is backgrounds on <body>. The background will be drawn relative
- * to ::box, using the background information contained within ::background.
+ * mess that is backgrounds on &lt;body&gt;. The background will be drawn relative
+ * to \a box, using the background information contained within \a background.
  */
 
 static bool html_redraw_background(int x, int y, struct box *box, float scale,
@@ -1789,6 +1811,7 @@ static bool html_redraw_text_decoration(struct box *box,
  * Redraw the text content of a box, possibly partially highlighted
  * because the text has been selected, or matches a search operation.
  *
+ * \param html The html content to redraw text within.
  * \param  box      box with text content
  * \param  x        x co-ord of box
  * \param  y        y co-ord of box
@@ -2414,17 +2437,25 @@ bool html_redraw_box(const html_content *html, struct box *box,
 			const char *obj = "\xef\xbf\xbc";
 			int obj_width;
 			int obj_x = x + padding_left;
+			nserror res;
+
 			if (!plot->rectangle(x + padding_left,
 					y + padding_top,
 					x + padding_left + width - 1,
 					y + padding_top + height - 1,
-					plot_style_broken_object))
+					     plot_style_broken_object)) {
 				return false;
-			if (!nsfont.font_width(plot_fstyle_broken_object, obj,
-					sizeof(obj) - 1, &obj_width))
+			}
+
+			res = guit->layout->width(plot_fstyle_broken_object,
+						  obj,
+						  sizeof(obj) - 1,
+						  &obj_width);
+			if (res != NSERROR_OK) {
 				obj_x += 1;
-			else
+			} else {
 				obj_x += width / 2 - obj_width / 2;
+			}
 
 			if (!plot->text(obj_x, y + padding_top + (int)
 						(height * 0.75),
