@@ -241,32 +241,21 @@ void imagemap_dump(html_content *c)
 
 		map = c->imagemaps[i];
 		while (map != NULL) {
-			LOG(("Imagemap: %s", map->key));
+			LOG("Imagemap: %s", map->key);
 
 			for (entry = map->list; entry; entry = entry->next) {
 				switch (entry->type) {
 				case IMAGEMAP_DEFAULT:
-					LOG(("\tDefault: %s", nsurl_access(
-							entry->url)));
+					LOG("\tDefault: %s", nsurl_access(entry->url));
 					break;
 				case IMAGEMAP_RECT:
-					LOG(("\tRectangle: %s: [(%d,%d),(%d,%d)]",
-						nsurl_access(entry->url),
-						entry->bounds.rect.x0,
-						entry->bounds.rect.y0,
-						entry->bounds.rect.x1,
-						entry->bounds.rect.y1));
+					LOG("\tRectangle: %s: [(%d,%d),(%d,%d)]", nsurl_access(entry->url), entry->bounds.rect.x0, entry->bounds.rect.y0, entry->bounds.rect.x1, entry->bounds.rect.y1);
 					break;
 				case IMAGEMAP_CIRCLE:
-					LOG(("\tCircle: %s: [(%d,%d),%d]",
-						nsurl_access(entry->url),
-						entry->bounds.circle.x,
-						entry->bounds.circle.y,
-						entry->bounds.circle.r));
+					LOG("\tCircle: %s: [(%d,%d),%d]", nsurl_access(entry->url), entry->bounds.circle.x, entry->bounds.circle.y, entry->bounds.circle.r);
 					break;
 				case IMAGEMAP_POLY:
-					LOG(("\tPolygon: %s:", nsurl_access(
-							entry->url)));
+					LOG("\tPolygon: %s:", nsurl_access(entry->url));
 					for (j = 0; j != entry->bounds.poly.num;
 							j++) {
 						fprintf(stderr, "(%d,%d) ",
@@ -285,13 +274,15 @@ void imagemap_dump(html_content *c)
 /**
  * Adds an imagemap entry to the list
  *
- * \param n     The xmlNode representing the entry to add
+ * \param c  The html content that the imagemap belongs to
+ * \param n  The xmlNode representing the entry to add
  * \param base_url  Base URL for resolving relative URLs
- * \param entry Pointer to list of entries
+ * \param entry  Pointer to list of entries
+ * \param tagtype  The type of tag
  * \return false on memory exhaustion, true otherwise
  */
 static bool
-imagemap_addtolist(dom_node *n, nsurl *base_url,
+imagemap_addtolist(const struct html_content *c, dom_node *n, nsurl *base_url,
 		   struct mapentry **entry, dom_string *tagtype)
 {
 	dom_exception exc;
@@ -356,8 +347,7 @@ imagemap_addtolist(dom_node *n, nsurl *base_url,
 	else
 		goto bad_out;
 
-	if (box_extract_link(dom_string_data(href),
-			     base_url, &new_map->url) == false)
+	if (box_extract_link(c, href, base_url, &new_map->url) == false)
 		goto bad_out;
 
 	if (new_map->url == NULL) {
@@ -547,7 +537,7 @@ imagemap_extract_map_entries(dom_node *node, html_content *c,
 			dom_nodelist_unref(nlist);
 			return false;
 		}
-		if (imagemap_addtolist(subnode, c->base_url,
+		if (imagemap_addtolist(c, subnode, c->base_url,
 				       entry, tname) == false) {
 			dom_node_unref(subnode);
 			dom_nodelist_unref(nlist);
@@ -582,8 +572,7 @@ static bool imagemap_extract_map(dom_node *node, html_content *c,
 /**
  * Extract all imagemaps from a document tree
  *
- * \param c The content
- * \param map_str A dom_string which is "map"
+ * \param c The content to extract imagemaps from.
  * \return false on memory exhaustion, true otherwise
  */
 nserror
@@ -712,7 +701,7 @@ imagemap_point_in_poly(int num, float *xpt, float *ypt, unsigned long x,
 /**
  * Retrieve url associated with imagemap entry
  *
- * \param h        The containing content
+ * \param c        The containing content
  * \param key      The map name to search for
  * \param x        The left edge of the containing box
  * \param y        The top edge of the containing box
