@@ -21,16 +21,15 @@
 
 #include <dom/dom.h>
 
-#include "utils/nsurl.h"
-
 struct html_content;
 struct hlcache_handle;
+struct nsurl;
 
 void imagemap_destroy(struct html_content *c);
 void imagemap_dump(struct html_content *c);
 nserror imagemap_extract(struct html_content *c);
 
-nsurl *imagemap_get(struct html_content *c, const char *key,
+struct nsurl *imagemap_get(struct html_content *c, const char *key,
 		unsigned long x, unsigned long y,
 		unsigned long click_x, unsigned long click_y,
 		const char **target);
