@@ -22,15 +22,14 @@
  */
 
 #include <assert.h>
-
 #include <dom/dom.h>
 
-#include "css/css.h"
-#include "css/utils.h"
-#include "render/box.h"
-#include "render/table.h"
 #include "utils/log.h"
 #include "utils/talloc.h"
+#include "css/utils.h"
+
+#include "render/box.h"
+#include "render/table.h"
 
 /* Define to enable verbose table debug */
 #undef TABLE_DEBUG
@@ -220,10 +219,13 @@ bool table_calculate_column_types(struct box *table)
 
 #ifdef TABLE_DEBUG
 	for (i = 0; i != table->columns; i++)
-		LOG(("table %p, column %u: type %s, width %i", table, i,
-				((const char *[]) {"UNKNOWN", "FIXED", "AUTO",
-				"PERCENT", "RELATIVE"})[col[i].type],
-				col[i].width));
+		LOG("table %p, column %u: type %s, width %i", table, i, ((const char *[]){
+			"UNKNOWN",
+			"FIXED",
+			"AUTO",
+			"PERCENT",
+			"RELATIVE"		
+		})[col[i].type], col[i].width);
 #endif
 
 	return true;
