@@ -31,27 +31,29 @@
 #include <parserutils/input/inputstream.h>
 #include <parserutils/charset/utf8.h>
 
-#include "content/content_protected.h"
-#include "content/hlcache.h"
-#include "css/css.h"
-#include "css/utils.h"
-#include "desktop/browser.h"
-#include "utils/nsoption.h"
-#include "desktop/plotters.h"
-#include "desktop/search.h"
-#include "desktop/selection.h"
-#include "desktop/textinput.h"
-#include "render/font.h"
-#include "render/search.h"
-#include "render/textplain.h"
-#include "render/html.h"
-#include "render/search.h"
 #include "utils/corestrings.h"
 #include "utils/http.h"
 #include "utils/log.h"
 #include "utils/messages.h"
 #include "utils/utils.h"
 #include "utils/utf8.h"
+#include "netsurf/content.h"
+#include "netsurf/keypress.h"
+#include "netsurf/browser_window.h"
+#include "netsurf/plotters.h"
+#include "netsurf/layout.h"
+#include "content/content_protected.h"
+#include "content/hlcache.h"
+#include "css/utils.h"
+#include "utils/nsoption.h"
+#include "desktop/search.h"
+#include "desktop/selection.h"
+#include "desktop/gui_internal.h"
+
+#include "render/search.h"
+#include "render/textplain.h"
+#include "render/html.h"
+#include "render/search.h"
 
 struct textplain_line {
 	size_t	start;
@@ -465,13 +467,20 @@ void textplain_reformat(struct content *c, int width, int height)
 	size_t columns = 80;
 	int character_width;
 	size_t line_start;
+	nserror res;
 
-	LOG(("content %p w:%d h:%d",c, width, height));
+	LOG("content %p w:%d h:%d", c, width, height);
 
 	/* compute available columns (assuming monospaced font) - use 8
-	 * characters for better accuracy */
-	if (!nsfont.font_width(&textplain_style, "ABCDEFGH", 8, &character_width))
+	 * characters for better accuracy
+	 */
+	res = guit->layout->width(&textplain_style,
+				  "ABCDEFGH", 8,
+				  &character_width);
+	if (res != NSERROR_OK) {
 		return;
+	}
+
 	columns = (width - MARGIN - MARGIN) * 8 / character_width;
 	textplain_tab_width = (TAB_WIDTH * character_width) / 8;
 
@@ -556,7 +565,7 @@ void textplain_reformat(struct content *c, int width, int height)
 	return;
 
 no_memory:
-	LOG(("out of memory (line_count %lu)", line_count));
+	LOG("out of memory (line_count %lu)", line_count);
 	return;
 }
 
@@ -687,7 +696,7 @@ void textplain_mouse_track(struct content *c, struct browser_window *bw,
  *
  * \param  c	  content of type textplain
  * \param  bw	  browser window
- * \param  click  type of mouse click
+ * \param  mouse  mouse state on action
  * \param  x	  coordinate of mouse
  * \param  y	  coordinate of mouse
  */
@@ -742,19 +751,19 @@ bool textplain_keypress(struct content *c, uint32_t key)
 	struct selection *sel = &text->sel;
 
 	switch (key) {
-	case KEY_COPY_SELECTION:
+	case NS_KEY_COPY_SELECTION:
 		selection_copy_to_clipboard(sel);
 		return true;
 
-	case KEY_CLEAR_SELECTION:
+	case NS_KEY_CLEAR_SELECTION:
 		selection_clear(sel, true);
 		return true;
 
-	case KEY_SELECT_ALL:
+	case NS_KEY_SELECT_ALL:
 		selection_select_all(sel);
 		return true;
 
-	case KEY_ESCAPE:
+	case NS_KEY_ESCAPE:
 		if (selection_defined(sel)) {
 			selection_clear(sel, true);
 			return true;
@@ -771,11 +780,10 @@ bool textplain_keypress(struct content *c, uint32_t key)
 /**
  * Handle search.
  *
- * \param  c			content of type text
- * \param  gui_callbacks	vtable for updating front end
- * \param  gui_data		front end private data
- * \param  flags		search flags
- * \param  string		search string
+ * \param c         content of type text
+ * \param gui_data  front end private data
+ * \param flags     search flags
+ * \param string    search string
  */
 void textplain_search(struct content *c, void *gui_data,
 		search_flags_t flags, const char *string)
@@ -916,6 +924,7 @@ bool textplain_redraw(struct content *c, struct content_redraw_data *data,
 			size_t next_offset = offset;
 			int width;
 			int ntx;
+			nserror res;
 
 			while (next_offset < length && text_d[next_offset] != '\t')
 				next_offset = utf8_next(text_d, length, next_offset);
@@ -932,10 +941,14 @@ bool textplain_redraw(struct content *c, struct content_redraw_data *data,
 			if (next_offset >= length)
 				break;
 
+			res = guit->layout->width(&textplain_style,
+						&text_d[offset],
+						next_offset - offset,
+						  &width);
 			/* locate end of string and align to next tab position */
-			if (nsfont.font_width(&textplain_style, &text_d[offset],
-					next_offset - offset, &width))
+			if (res == NSERROR_OK) {
 				tx += (int)(width * data->scale);
+			}
 
 			ntx = x + ((1 + (tx - x) / tab_width) * tab_width);
 
@@ -1028,7 +1041,7 @@ char *textplain_get_selection(struct content *c)
 /**
  * Retrieve number of lines in content
  *
- * \param h  Content to retrieve line count from
+ * \param c  Content to retrieve line count from
  * \return Number of lines
  */
 unsigned long textplain_line_count(struct content *c)
@@ -1043,7 +1056,7 @@ unsigned long textplain_line_count(struct content *c)
 /**
  * Retrieve the size (in bytes) of text data
  *
- * \param h  Content to retrieve size of
+ * \param c Content to retrieve size of
  * \return Size, in bytes, of data
  */
 size_t textplain_size(struct content *c)
@@ -1056,12 +1069,14 @@ size_t textplain_size(struct content *c)
 }
 
 /**
- * Return byte offset within UTF8 textplain content, given the co-ordinates
- * of a point within a textplain content. 'dir' specifies the direction in
- * which to search (-1 = above-left, +1 = below-right) if the co-ordinates are not
- * contained within a line.
+
+ * Return byte offset within UTF8 textplain content.
  *
- * \param  h     content of type CONTENT_TEXTPLAIN
+ * given the co-ordinates of a point within a textplain content. 'dir'
+ * specifies the direction in which to search (-1 = above-left, +1 =
+ * below-right) if the co-ordinates are not contained within a line.
+ *
+ * \param  c     content of type CONTENT_TEXTPLAIN
  * \param  x     x ordinate of point
  * \param  y     y ordinate of point
  * \param  dir   direction of search if not within line
@@ -1103,16 +1118,20 @@ size_t textplain_offset_from_coords(struct content *c, int x, int y, int dir)
 		while (next_offset < length && text[next_offset] != '\t')
 			next_offset = utf8_next(text, length, next_offset);
 
-		if (next_offset < length)
-			nsfont.font_width(&textplain_style, text, next_offset, &width);
+		if (next_offset < length) {
+			guit->layout->width(&textplain_style,
+					    text,
+					    next_offset,
+					    &width);
+		}
 
 		if (x <= width) {
 			int pixel_offset;
 			size_t char_offset;
 
-			nsfont.font_position_in_string(&textplain_style,
-				text, next_offset, x,
-				&char_offset, &pixel_offset);
+			guit->layout->position(&textplain_style,
+					       text, next_offset, x,
+					       &char_offset, &pixel_offset);
 
 			idx += char_offset;
 			break;
@@ -1190,10 +1209,12 @@ int textplain_coord_from_offset(const char *text, size_t offset, size_t length)
 		size_t next_offset = 0;
 		int tx;
 
-		while (next_offset < offset && text[next_offset] != '\t')
+		while (next_offset < offset && text[next_offset] != '\t') {
 			next_offset = utf8_next(text, length, next_offset);
+		}
+
+		guit->layout->width(&textplain_style, text, next_offset, &tx);
 
-		nsfont.font_width(&textplain_style, text, next_offset, &tx);
 		x += tx;
 
 		if (next_offset >= offset)
@@ -1215,7 +1236,7 @@ int textplain_coord_from_offset(const char *text, size_t offset, size_t length)
  * Given a range of byte offsets within a UTF8 textplain content,
  * return a box that fully encloses the text
  *
- * \param  h      content of type CONTENT_TEXTPLAIN
+ * \param  c      content of type CONTENT_TEXTPLAIN
  * \param  start  byte offset of start of text range
  * \param  end    byte offset of end
  * \param  r      rectangle to be completed
@@ -1272,7 +1293,7 @@ void textplain_coords_from_range(struct content *c, unsigned start,
 /**
  * Return a pointer to the requested line of text.
  *
- * \param  h		    content of type CONTENT_TEXTPLAIN
+ * \param  c		    content of type CONTENT_TEXTPLAIN
  * \param  lineno	    line number
  * \param  poffset	    receives byte offset of line start within text
  * \param  plen		    receives length of returned line
@@ -1302,7 +1323,7 @@ char *textplain_get_line(struct content *c, unsigned lineno,
  * text to fit the window width. Thus only hard newlines are preserved
  * in the saved/copied text of a selection.
  *
- * \param  h		    content of type CONTENT_TEXTPLAIN
+ * \param  c		    content of type CONTENT_TEXTPLAIN
  * \param  start	    starting byte offset within UTF-8 text
  * \param  end		    ending byte offset
  * \param  plen		    receives validated length
