@@ -23,6 +23,7 @@
 #include <dom/dom.h>
 
 #include "utils/corestrings.h"
+#include "utils/nsurl.h"
 #include "utils/utils.h"
 
 /* lwc_string strings */
@@ -74,6 +75,7 @@ lwc_string *corestring_lwc_justify;
 lwc_string *corestring_lwc_left;
 lwc_string *corestring_lwc_li;
 lwc_string *corestring_lwc_link;
+lwc_string *corestring_lwc_mailto;
 lwc_string *corestring_lwc_meta;
 lwc_string *corestring_lwc_middle;
 lwc_string *corestring_lwc_multipart_form_data;
@@ -121,6 +123,7 @@ lwc_string *corestring_lwc__blank;
 lwc_string *corestring_lwc__parent;
 lwc_string *corestring_lwc__self;
 lwc_string *corestring_lwc__top;
+lwc_string *corestring_lwc_slash_;
 
 /* dom_string strings */
 dom_string *corestring_dom_a;
@@ -207,6 +210,7 @@ dom_string *corestring_dom_mousewheel;
 dom_string *corestring_dom_name;
 dom_string *corestring_dom_nohref;
 dom_string *corestring_dom_noresize;
+dom_string *corestring_dom_nowrap;
 dom_string *corestring_dom_offline;
 dom_string *corestring_dom_online;
 dom_string *corestring_dom_pagehide;
@@ -258,16 +262,26 @@ dom_string *corestring_dom_BUTTON;
 dom_string *corestring_dom_INPUT;
 dom_string *corestring_dom_SELECT;
 dom_string *corestring_dom_TEXTAREA;
+dom_string *corestring_dom_BODY;
+dom_string *corestring_dom_HEAD;
+dom_string *corestring_dom_html_namespace;
 dom_string *corestring_dom_button;
 dom_string *corestring_dom_image;
 dom_string *corestring_dom_radio;
 dom_string *corestring_dom_checkbox;
 dom_string *corestring_dom_file;
 dom_string *corestring_dom_on;
+dom_string *corestring_dom_onblur;
+dom_string *corestring_dom_onerror;
+dom_string *corestring_dom_onfocus;
+dom_string *corestring_dom_onload;
+dom_string *corestring_dom_onresize;
+dom_string *corestring_dom_onscroll;
 dom_string *corestring_dom___ns_key_box_node_data;
 dom_string *corestring_dom___ns_key_libcss_node_data;
 dom_string *corestring_dom___ns_key_file_name_node_data;
 dom_string *corestring_dom___ns_key_image_coords_node_data;
+dom_string *corestring_dom___ns_key_html_content_data;
 
 /* nsurl URLs */
 nsurl *corestring_nsurl_about_blank;
@@ -333,6 +347,7 @@ void corestrings_fini(void)
 	CSS_LWC_STRING_UNREF(left);
 	CSS_LWC_STRING_UNREF(li);
 	CSS_LWC_STRING_UNREF(link);
+	CSS_LWC_STRING_UNREF(mailto);
 	CSS_LWC_STRING_UNREF(meta);
 	CSS_LWC_STRING_UNREF(middle);
 	CSS_LWC_STRING_UNREF(multipart_form_data);
@@ -380,6 +395,7 @@ void corestrings_fini(void)
 	CSS_LWC_STRING_UNREF(_parent);
 	CSS_LWC_STRING_UNREF(_self);
 	CSS_LWC_STRING_UNREF(_top);
+	CSS_LWC_STRING_UNREF(slash_);
 
 #undef CSS_LWC_STRING_UNREF
 
@@ -475,6 +491,7 @@ void corestrings_fini(void)
 	CSS_DOM_STRING_UNREF(name);
 	CSS_DOM_STRING_UNREF(nohref);
 	CSS_DOM_STRING_UNREF(noresize);
+	CSS_DOM_STRING_UNREF(nowrap);
 	CSS_DOM_STRING_UNREF(offline);
 	CSS_DOM_STRING_UNREF(online);
 	CSS_DOM_STRING_UNREF(pagehide);
@@ -527,23 +544,38 @@ void corestrings_fini(void)
 	CSS_DOM_STRING_UNREF(INPUT);
 	CSS_DOM_STRING_UNREF(SELECT);
 	CSS_DOM_STRING_UNREF(TEXTAREA);
+	CSS_DOM_STRING_UNREF(BODY);
+	CSS_DOM_STRING_UNREF(HEAD);
+	/* DOM namespaces, not really CSS */
+	CSS_DOM_STRING_UNREF(html_namespace);
 	/* DOM input types, not really CSS */
 	CSS_DOM_STRING_UNREF(button);
 	CSS_DOM_STRING_UNREF(image);
 	CSS_DOM_STRING_UNREF(radio);
 	CSS_DOM_STRING_UNREF(checkbox);
 	CSS_DOM_STRING_UNREF(file);
+	/* DOM event prefix */
 	CSS_DOM_STRING_UNREF(on);
+	/* DOM events forwarded from body to window */
+	CSS_DOM_STRING_UNREF(onblur);
+	CSS_DOM_STRING_UNREF(onerror);
+	CSS_DOM_STRING_UNREF(onfocus);
+	CSS_DOM_STRING_UNREF(onload);
+	CSS_DOM_STRING_UNREF(onresize);
+	CSS_DOM_STRING_UNREF(onscroll);
 	/* DOM userdata keys, not really CSS */
 	CSS_DOM_STRING_UNREF(__ns_key_box_node_data);
 	CSS_DOM_STRING_UNREF(__ns_key_libcss_node_data);
 	CSS_DOM_STRING_UNREF(__ns_key_file_name_node_data);
 	CSS_DOM_STRING_UNREF(__ns_key_image_coords_node_data);
+	CSS_DOM_STRING_UNREF(__ns_key_html_content_data);
 #undef CSS_DOM_STRING_UNREF
 
 	/* nsurl URLs */
-	if (corestring_nsurl_about_blank != NULL)
+	if (corestring_nsurl_about_blank != NULL) {
 		nsurl_unref(corestring_nsurl_about_blank);
+		corestring_nsurl_about_blank = NULL;
+	}
 }
 
 
@@ -617,6 +649,7 @@ nserror corestrings_init(void)
 	CSS_LWC_STRING_INTERN(left);
 	CSS_LWC_STRING_INTERN(li);
 	CSS_LWC_STRING_INTERN(link);
+	CSS_LWC_STRING_INTERN(mailto);
 	CSS_LWC_STRING_INTERN(meta);
 	CSS_LWC_STRING_INTERN(middle);
 	CSS_LWC_STRING_INTERN(no);
@@ -687,6 +720,13 @@ nserror corestrings_init(void)
 		goto error;
 	}
 
+	lerror = lwc_intern_string("/", SLEN("/"),
+			&corestring_lwc_slash_);
+	if ((lerror != lwc_error_ok) || (corestring_lwc_slash_ == NULL)) {
+		error = NSERROR_NOMEM;
+		goto error;
+	}
+
 
 #define CSS_DOM_STRING_INTERN(NAME)					\
 	do {								\
@@ -785,6 +825,7 @@ nserror corestrings_init(void)
 	CSS_DOM_STRING_INTERN(name);
 	CSS_DOM_STRING_INTERN(nohref);
 	CSS_DOM_STRING_INTERN(noresize);
+	CSS_DOM_STRING_INTERN(nowrap);
 	CSS_DOM_STRING_INTERN(offline);
 	CSS_DOM_STRING_INTERN(online);
 	CSS_DOM_STRING_INTERN(pagehide);
@@ -836,18 +877,29 @@ nserror corestrings_init(void)
 	CSS_DOM_STRING_INTERN(INPUT);
 	CSS_DOM_STRING_INTERN(SELECT);
 	CSS_DOM_STRING_INTERN(TEXTAREA);
+	CSS_DOM_STRING_INTERN(BODY);
+	CSS_DOM_STRING_INTERN(HEAD);
 	/* DOM input types, not really CSS */
 	CSS_DOM_STRING_INTERN(button);
 	CSS_DOM_STRING_INTERN(image);
 	CSS_DOM_STRING_INTERN(radio);
 	CSS_DOM_STRING_INTERN(checkbox);
 	CSS_DOM_STRING_INTERN(file);
+	/* DOM event prefix */
 	CSS_DOM_STRING_INTERN(on);
+	/* DOM events forwarded from body to window */
+	CSS_DOM_STRING_INTERN(onblur);
+	CSS_DOM_STRING_INTERN(onerror);
+	CSS_DOM_STRING_INTERN(onfocus);
+	CSS_DOM_STRING_INTERN(onload);
+	CSS_DOM_STRING_INTERN(onresize);
+	CSS_DOM_STRING_INTERN(onscroll);
 	/* DOM userdata keys, not really CSS */
 	CSS_DOM_STRING_INTERN(__ns_key_box_node_data);
 	CSS_DOM_STRING_INTERN(__ns_key_libcss_node_data);
 	CSS_DOM_STRING_INTERN(__ns_key_file_name_node_data);
 	CSS_DOM_STRING_INTERN(__ns_key_image_coords_node_data);
+	CSS_DOM_STRING_INTERN(__ns_key_html_content_data);
 #undef CSS_DOM_STRING_INTERN
 
 	exc = dom_string_create_interned((const uint8_t *) "text/javascript",
@@ -866,6 +918,14 @@ nserror corestrings_init(void)
 		goto error;
 	}
 
+	exc = dom_string_create_interned((const uint8_t *) "http://www.w3.org/1999/xhtml",
+					 SLEN("http://www.w3.org/1999/xhtml"),
+					 &corestring_dom_html_namespace);
+	if ((exc != DOM_NO_ERR) || (corestring_dom_html_namespace == NULL)) {
+		error = NSERROR_NOMEM;
+		goto error;
+	}
+
 	error = nsurl_create("about:blank", &corestring_nsurl_about_blank);
 	if (error != NSERROR_OK) {
 		goto error;
