@@ -16,14 +16,17 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/** \file
- * NetSurf international domain name handling (implementation).
+/**
+ * \file
+ * NetSurf international domain name handling implementation.
  */
 
+#include <assert.h>
 #include <stddef.h>
 #include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
+#include <libutf8proc/utf8proc.h>
 
 #include "utils/errors.h"
 #include "utils/idna.h"
@@ -31,7 +34,6 @@
 #include "utils/log.h"
 #include "utils/punycode.h"
 #include "utils/utf8.h"
-#include "utils/utf8proc.h"
 #include "utils/utils.h"
 
 
@@ -59,17 +61,17 @@ static nserror punycode_status_to_nserror(enum punycode_status status)
 		break;
 
 	case punycode_bad_input:
-		LOG(("Bad input"));
+		LOG("Bad input");
 		ret = NSERROR_BAD_ENCODING;
 		break;
 
 	case punycode_big_output:
-		LOG(("Output too big"));
+		LOG("Output too big");
 		ret = NSERROR_BAD_SIZE;
 		break;
 
 	case punycode_overflow:
-		LOG(("Overflow"));
+		LOG("Overflow");
 		ret = NSERROR_NOSPACE;
 		break;
 
@@ -146,8 +148,9 @@ static bool idna__contexto_rule(int32_t cp)
  * Check if a CONTEXTJ codepoint has a rule defined,
  * and conforms to that rule.
  *
- * \param string	UCS-4 string
+ * \param label UCS-4 string
  * \param index	character in the string which is CONTEXTJ
+ * \param len The length of the label
  * \return true if conforming
  */
 static bool idna__contextj_rule(int32_t *label, int index, size_t len)
@@ -166,7 +169,7 @@ static bool idna__contextj_rule(int32_t *label, int index, size_t len)
 			return false; /* No previous character */
 		}
 		unicode_props = utf8proc_get_property(label[index - 1]);
-		if (unicode_props->combining_class == UTF8PROC_CCC_VIRAMA) {
+		if (unicode_props->combining_class == IDNA_UNICODE_CCC_VIRAMA) {
 			return true;
 		}
 
@@ -205,7 +208,7 @@ static bool idna__contextj_rule(int32_t *label, int index, size_t len)
 			return false; /* No previous character */
 		}
 		unicode_props = utf8proc_get_property(label[index - 1]);
-		if (unicode_props->combining_class == UTF8PROC_CCC_VIRAMA) {
+		if (unicode_props->combining_class == IDNA_UNICODE_CCC_VIRAMA) {
 			return true;
 		}
 		return false;
@@ -432,7 +435,7 @@ static bool idna__is_valid(int32_t *label, size_t len)
 
 	/* 2. Check characters 3 and 4 are not '--'. */
 	if ((label[2] == 0x002d) && (label[3] == 0x002d)) {
-		LOG(("Check failed: characters 2 and 3 are '--'"));
+		LOG("Check failed: characters 2 and 3 are '--'");
 		return false;
 	}
 
@@ -442,7 +445,7 @@ static bool idna__is_valid(int32_t *label, size_t len)
 	if ((unicode_props->category == UTF8PROC_CATEGORY_MN) ||
 		(unicode_props->category == UTF8PROC_CATEGORY_MC) ||
 		(unicode_props->category == UTF8PROC_CATEGORY_ME)) {
-		LOG(("Check failed: character 0 is a combining mark"));
+		LOG("Check failed: character 0 is a combining mark");
 		return false;
 	}
 
@@ -451,14 +454,14 @@ static bool idna__is_valid(int32_t *label, size_t len)
 
 		/* 4. Check characters not DISALLOWED by RFC5892 */
 		if (idna_prop == IDNA_P_DISALLOWED) {
-			LOG(("Check failed: character %d (%x) is DISALLOWED", i, label[i]));
+			LOG("Check failed: character %" PRIsizet " (%x) is DISALLOWED", i, label[i]);
 			return false;
 		}
 
 		/* 5. Check CONTEXTJ characters conform to defined rules */
 		if (idna_prop == IDNA_P_CONTEXTJ) {
 			if (idna__contextj_rule(label, i, len) == false) {
-				LOG(("Check failed: character %d (%x) does not conform to CONTEXTJ rule", i, label[i]));
+				LOG("Check failed: character %" PRIsizet " (%x) does not conform to CONTEXTJ rule", i, label[i]);
 				return false;
 			}
 		}
@@ -467,14 +470,14 @@ static bool idna__is_valid(int32_t *label, size_t len)
 		/** \todo optionally we can check conformance to this rule */
 		if (idna_prop == IDNA_P_CONTEXTO) {
 			if (idna__contexto_rule(label[i]) == false) {
-				LOG(("Check failed: character %d (%x) has no CONTEXTO rule defined", i, label[i]));
+				LOG("Check failed: character %" PRIsizet " (%x) has no CONTEXTO rule defined", i, label[i]);
 				return false;
 			}
 		}
 
 		/* 7. Check characters are not UNASSIGNED */
 		if (idna_prop == IDNA_P_UNASSIGNED) {
-			LOG(("Check failed: character %d (%x) is UNASSIGNED", i, label[i]));
+			LOG("Check failed: character %" PRIsizet " (%x) is UNASSIGNED", i, label[i]);
 			return false;
 		}
 
@@ -583,7 +586,7 @@ static bool idna__verify(const char *label, size_t len)
 		return true;
 	}
 
-	LOG(("Re-encoded ACE label %s does not match input", ace));
+	LOG("Re-encoded ACE label %s does not match input", ace);
 	free(ace);
 
 	return false;
@@ -636,7 +639,7 @@ idna_encode(const char *host, size_t len, char **ace_host, size_t *ace_len)
 			/* This is already a DNS-valid ASCII string */
 			if ((idna__is_ace(host, label_len) == true) &&
 			    (idna__verify(host, label_len) == false)) {
-				LOG(("Cannot verify ACE label %s", host));
+				LOG("Cannot verify ACE label %s", host);
 				return NSERROR_BAD_URL;
 			}
 			strncpy(fqdn_p, host, label_len);
