@@ -16,7 +16,8 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/** \file
+/**
+ * \file
  * Option reading and saving (implementation).
  *
  * Options are stored in the format key:value, one per line.
@@ -29,6 +30,7 @@
 #include <stdbool.h>
 #include <stdlib.h>
 #include <string.h>
+#include <strings.h>
 
 #include "desktop/plot_style.h"
 #include "utils/errors.h"
@@ -36,6 +38,9 @@
 #include "utils/utils.h"
 #include "utils/nsoption.h"
 
+/** Length of buffer used to read lines from input file */
+#define NSOPTION_MAX_LINE_LEN 1024
+
 struct nsoption_s *nsoptions = NULL;
 struct nsoption_s *nsoptions_default = NULL;
 
@@ -72,6 +77,8 @@ static struct nsoption_s defaults[] = {
 #include "atari/options.h"
 #elif defined(nsmonkey)
 #include "monkey/options.h"
+#elif defined(nswin32)
+#include "windows/options.h"
 #endif
 	{ NULL, 0, OPTION_INTEGER, { 0 } }
 };
@@ -139,6 +146,14 @@ static void nsoption_validate(struct nsoption_s *opts, struct nsoption_s *defs)
 	int cloop;
 	bool black = true;
 
+	if (opts[NSOPTION_treeview_font_size].value.i  < 50) {
+		opts[NSOPTION_treeview_font_size].value.i = 50;
+	}
+
+	if (opts[NSOPTION_treeview_font_size].value.i > 1000) {
+		opts[NSOPTION_treeview_font_size].value.i = 1000;
+	}
+
 	if (opts[NSOPTION_font_size].value.i  < 50) {
 		opts[NSOPTION_font_size].value.i = 50;
 	}
@@ -179,6 +194,21 @@ static void nsoption_validate(struct nsoption_s *opts, struct nsoption_s *defs)
 			opts[cloop].value.c = defs[cloop].value.c;
 		}
 	}
+
+	/* To aid migration and ensure that timeouts don't go crazy,
+	 * ensure that (a) we allow at least 1 attempt and
+	 * (b) the total time that we spend should not exceed 60s
+	 */
+	if (opts[NSOPTION_max_retried_fetches].value.u == 0)
+		opts[NSOPTION_max_retried_fetches].value.u = 1;
+	if (opts[NSOPTION_curl_fetch_timeout].value.u < 5)
+		opts[NSOPTION_curl_fetch_timeout].value.u = 5;
+	if (opts[NSOPTION_curl_fetch_timeout].value.u > 60)
+		opts[NSOPTION_curl_fetch_timeout].value.u = 60;
+	while (((opts[NSOPTION_curl_fetch_timeout].value.u *
+		 opts[NSOPTION_max_retried_fetches].value.u) > 60) &&
+		(opts[NSOPTION_max_retried_fetches].value.u > 1))
+		opts[NSOPTION_max_retried_fetches].value.u--;
 }
 
 /**
@@ -231,6 +261,8 @@ nsoption_is_set(const struct nsoption_s *opts,
 		 */
 		if (((defs[entry].value.s == NULL) &&
 		     (opts[entry].value.s != NULL)) ||
+		    ((defs[entry].value.s != NULL) &&
+		     (opts[entry].value.s == NULL)) ||
 		    ((defs[entry].value.s != opts[entry].value.s) &&
 		     (strcmp(opts[entry].value.s, defs[entry].value.s) != 0))) {
 			ret = true;
@@ -442,7 +474,9 @@ nsoption_output_value_text(struct nsoption_s *option,
  *
  * Allocates a new option table and copies an existing one into it.
  *
- * @param src The source table to copy
+ * \param[in] src The source table to copy
+ * \param[out] pdst The output table
+ * \return NSERROR_OK on success or appropriate error code.
  */
 static nserror
 nsoption_dup(struct nsoption_s *src, struct nsoption_s **pdst)
@@ -559,28 +593,40 @@ nsoption_init(nsoption_set_default_t *set_defaults,
 /* exported interface documented in utils/nsoption.h */
 nserror nsoption_finalise(struct nsoption_s *opts, struct nsoption_s *defs)
 {
+	nserror res;
+
 	/* check to see if global table selected */
 	if (opts == NULL) {
-		opts = nsoptions;
+		res = nsoption_free(nsoptions);
+		if (res == NSERROR_OK) {
+			nsoptions = NULL;
+		}
+	} else {
+		res = nsoption_free(opts);
+	}
+	if (res != NSERROR_OK) {
+		return res;
 	}
-
-	nsoption_free(opts);
 
 	/* check to see if global table selected */
 	if (defs == NULL) {
-		defs = nsoptions_default;
+		res = nsoption_free(nsoptions_default);
+		if (res == NSERROR_OK) {
+			nsoptions_default = NULL;
+		}
+	} else {
+		res = nsoption_free(defs);
 	}
 
-	nsoption_free(defs);
-
-	return NSERROR_OK;
+	return res;
 }
 
+
 /* exported interface documented in utils/nsoption.h */
 nserror
 nsoption_read(const char *path, struct nsoption_s *opts)
 {
-	char s[100];
+	char s[NSOPTION_MAX_LINE_LEN];
 	FILE *fp;
 	struct nsoption_s *defs;
 
@@ -596,15 +642,19 @@ nsoption_read(const char *path, struct nsoption_s *opts)
 	/** @todo is this and API bug not being a parameter */
 	defs = nsoptions_default;
 
+	if ((opts == NULL) || (defs == NULL)) {
+		return NSERROR_BAD_PARAMETER;
+	}
+
 	fp = fopen(path, "r");
 	if (!fp) {
-		LOG(("Failed to open file '%s'", path));
+		LOG("Failed to open file '%s'", path);
 		return NSERROR_NOT_FOUND;
 	}
 
-	LOG(("Sucessfully opened '%s' for Options file", path));
+	LOG("Successfully opened '%s' for Options file", path);
 
-	while (fgets(s, 100, fp)) {
+	while (fgets(s, NSOPTION_MAX_LINE_LEN, fp)) {
 		char *colon, *value;
 		unsigned int idx;
 
@@ -662,9 +712,13 @@ nsoption_write(const char *path,
 		defs = nsoptions_default;
 	}
 
+	if ((opts == NULL) || (defs == NULL)) {
+		return NSERROR_BAD_PARAMETER;
+	}
+
 	fp = fopen(path, "w");
 	if (!fp) {
-		LOG(("failed to open file '%s' for writing", path));
+		LOG("failed to open file '%s' for writing", path);
 		return NSERROR_NOT_FOUND;
 	}
 
@@ -683,10 +737,13 @@ nsoption_dump(FILE *outf, struct nsoption_s *opts)
 		return NSERROR_BAD_PARAMETER;
 	}
 
-	/* check to see if global table selected */
+	/* check to see if global table selected and available */
 	if (opts == NULL) {
 		opts = nsoptions;
 	}
+	if (opts == NULL) {
+		return NSERROR_BAD_PARAMETER;
+	}
 
 	return nsoption_output(outf, opts, NULL, true);
 }
@@ -703,10 +760,17 @@ nsoption_commandline(int *pargc, char **argv, struct nsoption_s *opts)
 	int mv_loop;
 	unsigned int entry_loop;
 
-	/* check to see if global table selected */
+	if ((pargc == NULL) || (argv == NULL)) {
+		return NSERROR_BAD_PARAMETER;
+	}
+
+	/* check to see if global table selected and available */
 	if (opts == NULL) {
 		opts = nsoptions;
 	}
+	if (opts == NULL) {
+		return NSERROR_BAD_PARAMETER;
+	}
 
 	while (idx < *pargc) {
 		arg = argv[idx];
@@ -734,7 +798,7 @@ nsoption_commandline(int *pargc, char **argv, struct nsoption_s *opts)
 
 		/* arg+arglen is the option to set, val is the value */
 
-		LOG(("%.*s = %s", arglen, arg, val));
+		LOG("%.*s = %s", arglen, arg, val);
 
 		for (entry_loop = 0;
 		     entry_loop < NSOPTION_LISTEND;
@@ -768,14 +832,22 @@ nsoption_snoptionf(char *string,
 	int fmtc = 0; /* current index into format string */
 	struct nsoption_s *option;
 
+	if (fmt == NULL) {
+		return -1;
+	}
+
 	if (option_idx >= NSOPTION_LISTEND) {
 		return -1;
 	}
 
-	option = &nsoptions[option_idx]; /* assume the global table */
-	if (option == NULL || option->key == NULL)
+	if (nsoptions == NULL) {
 		return -1;
+	}
 
+	option = &nsoptions[option_idx]; /* assume the global table */
+	if (option == NULL || option->key == NULL) {
+		return -1;
+	}
 
 	while ((slen < size) && (fmt[fmtc] != 0)) {
 		if (fmt[fmtc] == '%') {
